/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */
   
// Globals
var globalPrefService;//defined in koGlobalOverlay.js
var globalPrefs = null;

const nsILocalFile = Components.interfaces.nsILocalFile;
const nsLocalFile_CONTRACTID = "@mozilla.org/file/local;1";
const nsIFile = Components.interfaces.nsIFile;

const koIFTPNode = Components.interfaces.koIFTPNode;
const koIFTPNodeFile = Components.interfaces.koIFTPNodeFile;
const koFTPNodeFile_CONTRACTID = "@activestate.com/koFTPNodeFile;1";

const nsIFilePicker     = Components.interfaces.nsIFilePicker;
const modeGetFolder = nsIFilePicker.modeGetFolder;
const modeOpen      = nsIFilePicker.modeOpen;
const modeSave      = nsIFilePicker.modeSave;
const returnCancel      = nsIFilePicker.returnCancel;
const returnOK      = nsIFilePicker.returnOK;
const returnReplace      = nsIFilePicker.returnReplace;

const nsIDirectoryServiceProvider = Components.interfaces.nsIDirectoryServiceProvider;
const nsIDirectoryServiceProvider_CONTRACTID = "@mozilla.org/file/directory_service;1";
const nsIOutlinerBoxObject = Components.interfaces.nsIOutlinerBoxObject;

// for now, we're only doing ftp browsing with this file browser
var FTPBrowserOnly = true;
const localDrives = "Local Drives";
var _fileListSep = "; ";
var sfile = null;
var retvals;
var filePickerMode;
var homeDir;
var outlinerView;

var textInput;
var okButton;

function localFile(path) {
    this._sfile = Components.classes[nsLocalFile_CONTRACTID].createInstance(nsILocalFile);
    this._haveChildren = 0;
    this._directories = [];
    this._filelist = [];
    this._parent = null;
    if (path) this.init(path);
}

localFile.prototype = {
    get URL() { 
        var url; 
        try {
            url = this._sfile.unicodeURL;
        } catch(e) {
            url = this._sfile.URL;
        }
        return url;
    },

    get path() { return this._sfile.unicodePath; },

    get parent() {
        if (!this._parent && this._sfile.parent) {
            this._parent = new localFile();
            this._parent._sfile = this._sfile.parent;
        }
        return this._parent;
    },

    get isDirectory() { return this._sfile.exists() && this._sfile.isDirectory(); },

    get isFile() { return this._sfile.exists() && this._sfile.isFile(); },

    get isSymlink() { return this._sfile.exists() && this._sfile.isSymlink(); },

    get fileSize() { return this._sfile.exists() && this._sfile.fileSize; },

    get lastModificationDate() { return this._sfile.exists() && this._sfile.lastModificationDate; },

    get target() { return this._sfile.unicodeTarget; },

    get leafName() { return this._sfile.unicodeLeafName; },
    
    set followLinks(v) { this._sfile.followLinks = v; },
    get followLinks() { return this._sfile.followLinks; },
    
    get directories() {
        if (!this._haveChildren) this.initChildren();
        return this._directories;
    },
    get filelist() {
        if (!this._haveChildren) this.initChildren();
        return this._filelist;
    },
    createDirectory: function(dirName) {
        // what are the correct permissions to create a directory with?
        if (!this.isDirectory) return false;
        try {
            //dump("current dir: "+this._sfile.path+", making "+dirName+"\n");
            // XXX funky nsIFile behaviour, you must append the new directory name
            // first, then call createUnique with the same directory name again!
            // I've duplicated this odd behaviour in koFTP.
            file = this.clone();
            file._sfile.appendUnicode(dirName);
            file._sfile.createUnique(dirName,file._sfile.DIRECTORY_TYPE,755);
        } catch (e) {
            alertDialog("Unable to create directory. "+this.lastError());
            return false;
        }
        return true;
    },
    remove: function(recursive) {
        try {
            this._sfile.remove(recursive);
            return true;
        } catch (e) {
            alertDialog("Unable to remove "+this.path+"\n "+this.lastError());
            return false;
        }
    },
    initChildren: function() {
        this._directories = [];
        this._filelist = [];
        var dirEntries = this._sfile.QueryInterface(nsIFile).directoryEntries;
        var nextFile = null;
        while (dirEntries.hasMoreElements()) {
          nextFile = dirEntries.getNext().QueryInterface(nsIFile);
          // XXXjag hack for bug 82355 till symlink handling is fixed (bug 57995)
          var isDir = false;
          try {
            isDir = nextFile.isDirectory();
          } catch (e) {
            // this here to fool the rest of the code into thinking
            // this is a nsIFile object
            //dump("nextfile : "+nextFile.unicodeLeafName+"\n");
            nextFile = { leafName : nextFile.unicodeLeafName,
                         fileSize : 0,
                         lastModificationDate : 0,
                         isHidden: function() { return false; } };
          }
          // end of hack
          fileobj = new Object();
          fileobj.file = new localFile();
          fileobj.file._parent = this;
          fileobj.file._sfile = nextFile;
          if (isDir) {
            this._directories[this._directories.length] = fileobj;
          } else {
            this._filelist[this._filelist.length] = fileobj;
          }
        }
        this._haveChildren = 1;
    },
    
    init: function(path) {
        this._sfile.initWithUnicodePath(path);
        this._sfile.normalize();
    },
    isHidden: function() {
        return this._sfile.isHidden();
    },
    exists: function() {
        return this._sfile.exists();
    },
    clone: function() {
        var newfile = new localFile();
        newfile._sfile = this._sfile.clone().QueryInterface(nsILocalFile);
        newfile._parent = this._parent;
        newfile._directories = this._directories;
        newfile._filelist = this._filelist;
        return newfile;
    },
    equals: function(file) {
        return this._sfile.equals(file._sfile);
    },
    getChild: function(URL) {
        var i;
        for (i = 0; i < this._directories.length; i++) {
            if (this._directories[i].file.URL == URL) return this._directories[i].file;
        }
        for (i = 0; i < this._filelist.length; i++) {
            if (this._filelist[i].file.URL == URL) return this._filelist[i].file;
        }
        return null;
    },
    appendRelativePath: function(input) {
        this._sfile.appendRelativeUnicodePath(input);
    },
    moveToUnicode: function(parent, newname) {
        try {
            this._sfile.moveToUnicode(parent, newname);
        } catch (e) {
            alertDialog("Unable to rename file. "+this.lastError());
        }
    },
    disconnect: function() {
    },
    refresh: function() {
    },
    lastError: function() {return "";}
}


function FTPFile(path, server, port, username, password) {
    this._haveChildren = 0;
    this._directories = [];
    this._filelist = [];
    // allow us to create an empty ftpfile
    this._sfile = Components.classes[koFTPNodeFile_CONTRACTID].createInstance(koIFTPNodeFile);
    if (typeof(path)=="undefined") return;
    
    //dump("FTPFile("+path+", "+server+", "+port+", "+username+", "+password+")\n");
    if (typeof(port) != "number") port = Number(port);
    if (server != "") {
        try {
            this._sfile.init(server, port, username, password);
        } catch (e) {
            return;
        }
    }
    if (path) this.init(path);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
FTPFile.prototype = new localFile();
FTPFile.prototype.constructor = FTPFile;

FTPFile.prototype.__defineGetter__("parent",
function() 
{
    if (!this._parent && this._sfile.parent) {
        this._parent = new FTPFile();
        this._parent._sfile = this._sfile.parent;
    }
    return this._parent;
});

FTPFile.prototype.initChildren = function() {
    this._directories = [];
    this._filelist = [];
    var dirEntries = this._sfile.QueryInterface(koIFTPNode).directoryEntries;
    var nextFile = null;
    while (dirEntries.hasMoreElements()) {
      nextFile = dirEntries.getNext().QueryInterface(koIFTPNode);
      // XXXjag hack for bug 82355 till symlink handling is fixed (bug 57995)
      var isDir = false;
      try {
        isDir = nextFile.isDirectory();
      } catch (e) {
        // this here to fool the rest of the code into thinking
        // this is a nsIFile object
        //dump("nextfile 2: "+nextFile.unicodeLeafName+"\n");
        nextFile = { leafName : nextFile.unicodeLeafName,
                     fileSize : 0,
                     lastModificationDate : 0,
                     isHidden: function() { return false; } };
      }
      // end of hack
      //dump("child is "+file.path+"\n");
      var fileobj = new Object();
      fileobj.file = new FTPFile();
      fileobj.file._parent = this;
      fileobj.file._sfile = nextFile;
      //dump("fileobj : "+nextFile.unicodeLeafName+"\n");
      fileobj.cachedName = nextFile.unicodeLeafName;
      fileobj.cachedDate = nextFile.lastModificationDate;
      fileobj.cachedDateText = formatDate(fileobj.cachedDate);
      fileobj.cachedSize = String(nextFile.fileSize);
      if (isDir) {
        this._directories[this._directories.length] = fileobj;
      } else {
        this._filelist[this._filelist.length] = fileobj;
      }
    }
    this._haveChildren = 1;
}
FTPFile.prototype.initWithConnection = function(conn, path) {
    this._sfile.initWithConnection(conn, path);
}
FTPFile.prototype.init = function(path) {
    this._sfile.initWithPath(path);
    //dump("ftpFile path is "+this._sfile.path+"\n");
}
FTPFile.prototype.clone = function() {
    var newfile = new FTPFile();
    newfile._sfile = this._sfile.clone().QueryInterface(koIFTPNodeFile);
    newfile._parent = this._parent;
    newfile._directories = this._directories;
    newfile._filelist = this._filelist;
    return newfile;
}
FTPFile.prototype.disconnect = function() {
    this._sfile.connection.close();
}
FTPFile.prototype.refresh = function() {
    //dump("refreshing "+this._sfile.path+"\n");
    this._sfile.connection.refresh(this._sfile.path);
}
FTPFile.prototype.lastError = function() {return this._sfile.connection.lastError;}

var gFilePickerBundle;

function filebrowserLoad() {
  globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
    getService(Components.interfaces.koIPrefService);
  globalPrefs = globalPrefService.prefs;
  gFilePickerBundle = document.getElementById("bundle_filepicker");

  textInput = document.getElementById("textInput");
  okButton = document.getElementById("ok");
  outlinerView = new filebrowseview();
  outlinerView.selectionCallback = onSelect;

  if (window.arguments) {
    var o = window.arguments[0];
    retvals = o.retvals; /* set this to a global var so we can set return values */
    const title = o.title;
    filePickerMode = o.mode;
    if (o.displayDirectory) {
      const directory = o.displayDirectory.path;
    }
    const initialText = o.defaultString;
    const filterTitles = o.filters.titles;
    const filterTypes = o.filters.types;
    const numFilters = filterTitles.length;

    window.title = title;

    if (initialText) {
      textInput.value = initialText;
    }
  }

  if ((filePickerMode == modeOpen) ||
      (filePickerMode == modeSave)) {

    if (filePickerMode == modeSave)
        outlinerView.singleSelect = true;

    outlinerView.setFilter(filterTypes[0]);

    /* build filter popup */
    var filterPopup = document.getElementById("filterMenu");//document.createElement("menupopup");

    for (var i = 0; i < numFilters; i++) {
      var menuItem = document.createElement("menuitem");
      menuItem.setAttribute("label", filterTitles[i] + " (" + filterTypes[i] + ")");
      menuItem.setAttribute("filters", filterTypes[i]);
      filterPopup.appendChild(menuItem);
    }

    var filterMenuList = document.getElementById("filterMenuList");
    //filterMenuList.appendChild(filterPopup);
    if (numFilters > 0)
      filterMenuList.selectedIndex = 0;
    var filterBox = document.getElementById("filterBox");
    filterBox.removeAttribute("hidden");
  } else if (filePickerMode == modeGetFolder) {
    outlinerView.singleSelect = true;
    outlinerView.showOnlyDirectories = true;
  }

  // start out with a filename sort
  handleColumnClick("FilenameColumn");

  try {
    var buttonLabel = getOKAction();
    okButton.setAttribute("label", buttonLabel);
  } catch (exception) {
    // keep it set to "OK"
  }

  // setup the dialogOverlay.xul button handlers
  koDoSetOKCancel(onOK, onCancel);
  retvals.buttonStatus = returnCancel;

  var outliner = document.getElementById("directoryOutliner");
  outliner.outlinerBoxObject.view = outlinerView;

  // Start out with the ok button disabled since nothing will be
  // selected and nothing will be in the text field.
  okButton.disabled = true;
  textInput.focus();

  // load the deployment server list
  if (directory.indexOf("ftp://") != 0) {
      //dump("loading server list\n");
      loadServerList();
  }
  
  // This allows the window to show onscreen before we begin
  // loading the file list
  setTimeout(setInitialDirectory, 0, directory);
}

function filebrowserClose() {
    closeFTP();
}

function getURIInfo(uri) {
    var uriparts = new Array();
    uriparts["host"]="";
    uriparts["port"]="";
    uriparts["username"]="";
    uriparts["password"]="";
    
    // we've received a string that can be contain any combination of parts of the following,
    // the only absolute is the host
    // user:email@host@host.com:port
    var parts = uri.split("@");
    var hparts;
    // we have 1-3 elements
    if (parts.length == 1) {
        // should have: host[:port]
        hparts = parts[0].split(":");
        uriparts["host"]=hparts[0];
        if (hparts.length > 1) uriparts["port"] = hparts[1];
    } else if (parts.length == 2) {
        // should have user[:password] and host[:port]
        hparts = parts[0].split(":");
        uriparts["username"]=hparts[0];
        if (hparts.length > 1) uriparts["password"] = hparts[1];
        hparts = parts[1].split(":");
        uriparts["host"]=hparts[0];
        if (hparts.length > 1) uriparts["port"] = hparts[1];
    } else if (parts.length == 3) {
        // should have user:email@host and host[:port]
        hparts = parts[0].split(":");
        uriparts["username"]=hparts[0];
        uriparts["password"] = hparts[1] +"@"+ parts[1]; // must be length==2
        hparts = parts[2].split(":");
        uriparts["host"]=hparts[0];
        if (hparts.length > 1) uriparts["port"] = hparts[1];
    } else return null; // error!
    //dump("parts: host["+uriparts["host"]+"] port["+uriparts["port"]+"] user["+uriparts["username"]+"] password["+uriparts["password"]+"]\n");
    return uriparts;
}

function FTPInfo() {
    this.username = null;
    this.password = null;
    this.host = null;
    this.port = null;
    this.connection = null;
}
FTPInfo.prototype.constructor = FTPInfo;

var currentServer = null;
var ftpBrowser = false;
var ftpInfo = new FTPInfo();
var lastLocalDirectory = null;
function newFile(directory)
{
    //dump("creating new file for "+directory+"\n");
    var file = null;

    if (directory.indexOf("ftp://") == 0) {
        var parts = directory.match(/ftp:\/\/(.*?)\/(.*)/);
        var site = parts[1];
        var path = parts[2];
        //dump('site after split is '+site+'\n');
        //dump('path after split is '+path+'\n');
        if (typeof(path) == "undefined" || path == "") {
            if (!ftpInfo.connection) {
                // first connection, go home
                path = "~";
            } else {
                path = "/";
            }
        }
        if (path != "~" && path[0] != "/") path = "/" + path;
        // this uri crashes moz release builds, so workaround using getURIInfo
        //var parts = site.split(/^(([^:]+(?=[:@]*\w*[@]))(:([^:]+(?=@)))?@)?([^:@]+)(:(\d+))?$/);
        parts = getURIInfo(site);
        ftpInfo.username = parts["username"];
        ftpInfo.password = parts["password"];
        ftpInfo.domain = parts["host"];
        ftpInfo.port = parts["port"];
        ftpBrowser = true; //once ftp, always ftp!
        // ask for username/password if it isn't provided
        if (!ftpInfo.username || !ftpInfo.password) {
            // XXX fix persist post-1.2
            var obj = authDialog(gFilePickerBundle.getString("authenticateTitle"),
                                 gFilePickerBundle.getString("authenticateMessage"), ftpInfo.username, true, false);
            if (!obj)
                return null;
            if (obj.retval) {
                ftpInfo.username = obj.user;
                ftpInfo.password = obj.pwd;
                if (obj.anonymous)  {
                    // attempt anonymous
                    if (!ftpInfo.username) ftpInfo.username = gFilePickerBundle.getString("anonymousUser");
                    if (!ftpInfo.password) ftpInfo.password = gFilePickerBundle.getString("anonymousPwd");
                }
            }
            else  {
                return null;
            }
        }

        //  pass exceptions, connection will have error message.
        try {
            if (path == "~" || path == "/~") {
                file = new FTPFile("", ftpInfo.domain, ftpInfo.port, ftpInfo.username, ftpInfo.password);
                var home = file._sfile.connection.getHomeDirectory();
                if (home) file.init(home);
            } else {
                file = new FTPFile("", ftpInfo.domain, ftpInfo.port, ftpInfo.username, ftpInfo.password);
                file.init(path);
            }
        } catch(e) { /* do nothing, we deal with the error later */ }
        
        ftpInfo.connection = file._sfile.connection;
        if (ftpInfo.connection.lastError != "") {
            alertDialog(ftpInfo.connection.lastError);
            file = null;
            return null;
        }
        return file;
    }
    if (ftpBrowser) {
        // maintain a single ftp connection by passing our connection around
        file = new FTPFile();
        file.initWithConnection(ftpInfo.connection, directory);
        return file;
    }
    if (FTPBrowserOnly) {
        return null;
    }
    ftpInfo.connection = null;
    lastLocalDirectory = directory;
    return new localFile(directory)
}

function closeFTP()
{
    if (ftpBrowser) {
        ftpBrowser = false;
        if (ftpInfo.connection) ftpInfo.connection.close();
        ftpInfo = new FTPInfo();
        ftpInfo.connection = null;
    }
}

function setInitialDirectory(directory)
{
  // get the home dir
  //dump("setInitialDirectory("+directory+")\n");
  clearHistoryList();
  var dirServiceProvider = Components.classes[nsIDirectoryServiceProvider_CONTRACTID]
                                     .getService(nsIDirectoryServiceProvider);
  var persistent = new Object();
  homeDir = dirServiceProvider.getFile("Home", persistent);
  sfile = null;
  if (directory) {
    sfile = newFile(directory);
    //dump("file: "+sfile.path+" exists: "+sfile.exists()+" isDir "+sfile.isDirectory+"\n");
  } else if (!FTPBrowserOnly && !(sfile.exists() && sfile.isDirectory)) {
    // Start in the user's home directory
    //dump("initialize new file with homeDir "+homeDir.unicodePath+"\n");
    sfile = newFile(homeDir.unicodePath);
  }
  if (!sfile) {
      outlinerView.clearList();
      return;
  }
  gotoDirectory(sfile);
}

function onFilterChanged(target)
{
  // Do this on a timeout callback so the filter list can roll up
  // and we don't keep the mouse grabbed while we are refiltering.

  setTimeout(changeFilter, 0, target.getAttribute("filters"));
}

function changeFilter(filterTypes)
{
  window.setCursor("wait");
  outlinerView.setFilter(filterTypes);
  window.setCursor("auto");
}

function cloneAppend(input) {
      var file = sfile.clone();
      if (!file) {
          //dump("unable to close!\n");
        return null;
      }

      /* XXX we need an XP way to test for an absolute path! */
      if (input[0] == '/' || input[1]==":")   /* an absolute path was entered */
        file.init(input);
      else if ((input.indexOf("/../") > 0) ||
               (input.substr(-3) == "/..") ||
               (input.substr(0,3) == "../") ||
               (input == "..")) {
        /* appendRelativePath doesn't allow .. */
        file.init(file.path + "/" + input);
      } else {
        try {
          file.appendRelativePath(input);
        } catch (e) {
          //dump("Can't append relative path '"+input+"':\n");
          return null;
        }
      }
      if (!file.exists()) {
          // check to see if this is a directory since python ftplib won't let us know
          var child = sfile.getChild(file.URL);
          if (child) file = child;
      }
      return file;
}

function onOK()
{
  var errorTitle, errorMessage, promptService;
  var ret = returnCancel;
  var file;
  var i=0;
  // get the files from the outliner
  var files = outlinerView.getSelectedFiles();
  var input = textInput.value;
  var inputFiles = input.split(_fileListSep);
  //dump("number of files: "+files.length+"\n");
  //dump("input is: "+input+"\n");
  //dump("first input "+inputFiles[0]+"\n");
  //dump("using basedir from: "+sfile.URL+"\n");

    if (filePickerMode == modeSave) {
      // dump any select files since we have an input filename
      files = new Array();
    }

  // we don't have files in the file picker, they typed something in, create
  // the files array
  if (inputFiles.length < 1) {
      dump ("no inputFiles!\n");
      return false;
  }
  
  for (i = 0; i < inputFiles.length; i++) {
      //dump("checking input "+inputFiles[i]+"\n");
      input = inputFiles[i];
      if (input[0] == '~') { // XXX XP?
        if (ftpBrowser) {
            var home = sfile._sfile.connection.getHomeDirectory();
            input = home + input.substring(1);
            //dump("FTP HOME: "+input+"\n");
        } else {
            input  = homeDir.unicodePath + input.substring(1);
        }
      }
      
      // look to see if we already have this file
      var haveFile = false;
      for (i = 0; i< files.length; i++) {
          var url = sfile.URL;
          
          // make sure we have a slash!
          if (url[url.length-1] != "/" && input[0] != "/") url += "/"+ input;
          else url += input;
          
          //dump("checking "+files[i].URL+" == "+url+"\n");
          if (files[i].URL == url) {
              haveFile = true;
              break;
          }
      }
      if (haveFile) continue;
      file = cloneAppend(input);
      //dump("adding file "+file.URL+" to list isDir:"+file.isDirectory+"\n");
      files.push(file);
  }

  // verify that the file(s) or directory exist
  for (i = 0; i < files.length; i++) {
      file = files[i];
      if (!file.exists() && (filePickerMode != modeSave)) {
        errorTitle = gFilePickerBundle.getFormattedString("errorOpenFileDoesntExistTitle",
                                                          [file.path]);
        errorMessage = gFilePickerBundle.getFormattedString("errorOpenFileDoesntExistMessage",
                                                            [file.path]);
        promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
        promptService.alert(window, errorTitle, errorMessage);
        return false;
      }
  }
  
  // unless we are in modeOpen, we will not have more than one file
  file = files[0];
  var filepaths = new Array();
  retvals.filepaths = filepaths; // set an empty return array
  // return the server object so we don't have to look it up again!
  var idx = getAliasIndex(currentServer);
  if (idx > -1) retvals.server = servers[idx];
  
  switch(filePickerMode) {
  case modeOpen:
    if (file.isFile) {
        // if the first file is a file, all files 'should' be files
        // if the first file is a file, all files 'should' be files
        for (i=0; i < files.length; i++) {
            // build an array of full file paths
            //dump("adding path "+files[i].URL+"\n");

            // munge the url into a koRemote url
            url = files[i].URL;
            if (url.substring(0, 7) != 'file://') {
                // retvals.server exists only for ftp aliases from preferences
                // typed in sites we skip that part
                if (retvals.server) {
                    url = "koremote://"+retvals.server.alias+"/"+url;
                } else {
                    url = "koremote:///"+url;
                }
            }
            filepaths.push(url);
        }
        retvals.filepaths = filepaths;
        // single selection support
        retvals.directory = file.parent.path;
        ret = returnOK;
    } else if (file.isDirectory) {
      if (!sfile.equals(file)) {
        gotoDirectory(file);
      }
      textInput.value = "";
      doEnabling();
      ret = returnCancel;
    }
    break;
  case modeSave:
    if (file.isFile) { // can only be true if file.exists()
      // we need to pop up a dialog asking if you want to save
      var message = gFilePickerBundle.getFormattedString("confirmFileReplacing",
                                                         [file.path]);
      var rv = window.confirm(message);
      if (rv) {
        ret = returnReplace;
        retvals.directory = file.parent.path;
      } else {
        ret = returnCancel;
      }
    } else if (file.isDirectory) {
      if (!sfile.equals(file)) {
        gotoDirectory(file);
      }
      textInput.value = "";
      doEnabling();
      ret = returnCancel;
    } else {
      var parent = file.parent;
      if (parent.exists() && parent.isDirectory) {
        ret = returnOK;
        retvals.directory = parent.path;
      } else {
        var oldParent = parent;
        while (!parent.exists() && parent != parent.parent) {
          oldParent = parent;
          parent = parent.parent;
        }
        errorTitle = gFilePickerBundle.getFormattedString("errorSavingFileTitle",
                                                          [file.path]);
        if (parent.isFile) {
          errorMessage = gFilePickerBundle.getFormattedString("saveParentIsFileMessage",
                                                              [parent.path, file.path]);
        } else {
          errorMessage = gFilePickerBundle.getFormattedString("saveParentDoesntExistMessage",
                                                              [oldParent.path, file.path]);
        }
        promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
        promptService.alert(window, errorTitle, errorMessage);
        ret = returnCancel;
      }
    }
    break;
  case modeGetFolder:
    if (file.isDirectory) {
      retvals.directory = file.path;
    } else { // if nothing selected, the current directory will be fine
      retvals.directory = sfile.path;
    }
    ret = returnOK;
    break;
  }

  //retvals.file = file.path;
    url = file.URL;
    if (url.substring(0, 7) != 'file://') {
        // retvals.server exists only for ftp aliases from preferences
        // typed in sites we skip that part
        if (retvals.server) {
            url = "koremote://"+retvals.server.alias+"/"+url;
        } else {
            url = "koremote:///"+url;
        }
    }
  retvals.file = url;
  retvals.buttonStatus = ret;
  
  
  if (ret == returnCancel)
    return false;
  else {
    closeFTP(); // in case it's open
    return true;
  }
}

function onCancel()
{
  closeFTP(); // in case it's open
  // Close the window.
  retvals.buttonStatus = returnCancel;
  retvals.file = null;
  return true;
}

function onDblClick(e) {
  var t = e.originalTarget;
  if (t.localName != "outlinerbody")
    return;

  openSelectedFile();
}

function openSelectedFile() {
  var files = outlinerView.getSelectedFiles();
  if (!files.length)
    return;
  var hasDirectory = false;
  for (var i=0; i< files.length; i++) {
      var file = files[i];
      if (file.isDirectory) {
        gotoDirectory(file);
        hasDirectory = true;
        break;
      }
      if (file.isSymlink)
        file.init(file.target);
  }

  if (hasDirectory) return;

  // finish up!
  doOKButton();
}

function onClick(e) {
  var t = e.originalTarget;
  if (t.localName == "outlinercol")
    handleColumnClick(t.id);
}

function convertColumnIDtoSortType(columnID) {
  var sortKey;
  
  switch (columnID) {
  case "FilenameColumn":
    sortKey = filebrowseview.SORTTYPE_NAME;
    break;
  case "FileSizeColumn":
    sortKey = filebrowseview.SORTTYPE_SIZE;
    break;
  case "LastModifiedColumn":
    sortKey = filebrowseview.SORTTYPE_DATE;
    break;
  default:
    dump("unsupported sort column: " + columnID + "\n");
    sortKey = 0;
    break;
  }
  
  return sortKey;
}

function handleColumnClick(columnID) {
  var sortType = convertColumnIDtoSortType(columnID);
  var sortOrder = (outlinerView.sortType == sortType) ? !outlinerView.reverseSort : false;
  outlinerView.sort(sortType, sortOrder, false);
  
  // set the sort indicator on the column we are sorted by
  var sortedColumn = document.getElementById(columnID);
  if (outlinerView.reverseSort) {
    sortedColumn.setAttribute("sortDirection", "descending");
  } else {
    sortedColumn.setAttribute("sortDirection", "ascending");
  }
  
  // remove the sort indicator from the rest of the columns
  var currCol = document.getElementById("directoryOutliner").firstChild;
  while (currCol) {
    while (currCol && currCol.localName != "outlinercol")
      currCol = currCol.nextSibling;
    if (currCol) {
      if (currCol != sortedColumn) {
        currCol.removeAttribute("sortDirection");
      }
      currCol = currCol.nextSibling;
    }
  }
}

function doSort(sortType) {
  outlinerView.sort(sortType, false);
}

function onKeypress(e) {
  if (e.keyCode == 8) /* backspace */
    goUp();
  else if (e.keyCode == 13) { /* enter */
    var file = outlinerView.getSelectedFile();
    if (file) {
      if (file.isDirectory) {
        gotoDirectory(file);
        e.preventDefault();
      }
    }
  }
}

function doEnabling() {
  // Maybe add check if textInput.value would resolve to an existing
  // file or directory in .modeOpen. Too costly I think.
  var enable = (textInput.value != "");

  okButton.disabled = !enable;
}

function onOutlinerFocus(event) {
  // Reset the button label and enabled/disabled state.
  onSelect(outlinerView.getSelectedFiles());
}

function getOKAction(file) {
  if (file && file.isDirectory && filePickerMode != modeGetFolder)
    return gFilePickerBundle.getString("openButtonLabel");

  switch(filePickerMode) {
  case modeGetFolder:
    return gFilePickerBundle.getString("selectFolderButtonLabel");
    break;
  case modeOpen:
    return gFilePickerBundle.getString("openButtonLabel");
    break;
  case modeSave:
    return gFilePickerBundle.getString("saveButtonLabel");
    break;
  }
  return ""; // js strict
}

function onSelect(files) {
    var filesText = "";
    var file = null;
    for (var i = 0; i < files.length; i++) {
        file = files[i];
        if (file) {
            var path = file.leafName;
            
            // break out on the first directory
            if (file.isDirectory) {
                if (filePickerMode == modeGetFolder) filesText = path;
                break;
            } else if (!(filePickerMode == modeGetFolder)) {
                if (path) {
                  if ((filePickerMode == modeGetFolder) || !file.isDirectory) {
                    if (filesText != "")
                        filesText += _fileListSep+path;
                    else
                        filesText = path;
                  }
                }
            }
        }
    }
    textInput.value = filesText;
    if (file && filesText != "") {
        var buttonLabel = getOKAction(file);
        okButton.setAttribute("label", buttonLabel);
        okButton.disabled = false;
        return;
    }

  okButton.disabled = true;
}

function onTextFieldFocus() {
  var buttonLabel = getOKAction(null);
  okButton.setAttribute("label", buttonLabel);
  doEnabling();
}

function onDirectoryChanged(target)
{
  var path = target.getAttribute("label");
  //dump("onDirectoryChanged("+path+")\n");
  var file = newFile(path);

  if (!sfile.equals(file)) {
    // Do this on a timeout callback so the directory list can roll up
    // and we don't keep the mouse grabbed while we are loading.

    setTimeout(gotoDirectory, 0, file);
  }
}

function addToHistory(directoryName) {
  var menu = document.getElementById("lookInMenu");
  var children = menu.childNodes;

  var i = 0;
  while (i < children.length) {
    if (children[i].getAttribute("label") == directoryName)
      break;

    ++i;
  }

  if (i < children.length) {
    if (i != 0) {
      var node = children[i];
      menu.removeChild(node);
      menu.insertBefore(node, children[0]);
    }
  } else {
    var menuItem = document.createElement("menuitem");
    menuItem.setAttribute("label", directoryName);
    menu.insertBefore(menuItem, children[0]);
  }

  var menuList = document.getElementById("lookInMenuList");
  menuList.selectedIndex = 0;
}

function goLast() {
  var menu = document.getElementById("lookInMenu");
  var children = menu.childNodes;
  var dir = null;
  if (children.length > 1) {
      dir = children[1];
      // move current history item to last history item
      var node = children[0];
      menu.appendChild(node, children[children.length-1]);
      onDirectoryChanged(dir);
  }
}

function doMKDIR() {
    if (!sfile) return false;
    var textInput = document.getElementById("textInput")
    var dirName = promptDialog("Enter the name for the new directory", textInput.value, "OK", "Cancel")
    if (!dirName) return false;
    
    //dump("current dir "+sfile.path+" isdir: "+sfile.isDirectory+"\n");
    //dump("doMKDIR: "+dirName+"\n");

    window.setCursor("wait");
    var e = new Object();
    var ret = false;
    if (sfile.createDirectory(dirName)) {
        //dump("name of new folder is "+sfile.path+"/"+dirName+"\n");
        sfile.refresh();
        gotoDirectory(sfile);
        // select the new folder in the outliner
        /* until we can edit cells in an ouliner, this does nothing
        if (outlinerView.selectEntryByNameType(dirName, 1)) {
            // make the cell editable, and edit it!
            doRename();
        }
        */
        ret = true;
    }
    window.setCursor("auto");
    return ret;
}

function doRename() {
    var textInput = document.getElementById("textInput")
    var newName = promptDialog("Enter the new name for the file or directory", textInput.value, "OK", "Cancel")
    if (!newName || newName == textInput.value) return false;

    file = outlinerView.getSelectedFile()
    //dump("doRename called on "+file.path+"\n");
    file.moveToUnicode(null, newName);
    
    sfile.refresh();
    gotoDirectory(sfile);
    return true;
}

function doDelete() {
    files = outlinerView.getSelectedFiles()
    if (yesNoDialog("Are you sure you want to delete the selected files?", "No") == "No") {
      return;
    }
    for (var i=0; i< files.length; i++) {
        //dump("doDelete called on "+files[i].path+"\n");
        var recurse = false;
        if (files[i].isDirectory) {
            if (yesNoDialog("The file '"+files[i].leafName+"' is a Directory, would you like to delete all sub-directories and folders?", "No") == "Yes") {
              recurse = true
            }
        }
        files[i].remove(recurse);
        // XXX TODO remove from history if its in there
    }
    var outliner = document.getElementById("directoryOutliner");
    outliner.outlinerBoxObject.scrollToRow(0);
    sfile.refresh();
    gotoDirectory(sfile);
}

function doRefresh() {
    if (!sfile) return;
    window.setCursor("wait");
    sfile.refresh();
    gotoDirectory(sfile);
    window.setCursor("auto");
}

function goUp() {
  try {
    var parent = sfile.parent;
  } catch(ex) { dump("can't get parent directory\n"); }

  if (parent) {
    gotoDirectory(parent);
  }
}

function gotoDirectory(directory) {
  addToHistory(directory.path);

  window.setCursor("wait");
  outlinerView.setDirectory(directory.path);
  window.setCursor("auto");

  sfile = directory;
}


var servers = null;
function Server(type, alias, hostname, port, path, username, password) {
  this.type = type;
  this.alias = alias;
  this.hostname = hostname;
  this.port = port;
  this.path = path;
  this.username = username;
  this.password = password;
  this.id = Server_generateId();
}

// Each Association object has a unique identifier.  This is used to
// establish a relationship between the association object and the
// items in the tree that represent it.

var Server_index = 0;  // used for generating ids
function Server_generateId() {
  return "server" + Server_index++;
}

/* XXX duplicated in pref-servers.js */
function decodePassword(password) {
    if (typeof(password) == 'undefined' || !password) return '';
    // XXX 2way encrypt the password somehow, to prevent plaintext in the prefs files
    var crypto = 
       Components.classes["@activestate.com/koPasswordCrypto;1"]
          .getService(Components.interfaces.koIPasswordCrypto);
    var text = password;
    try {
        text = crypto.decodePassword(password);    
    } catch(e) { 
        /* password is empty, that's ok, we'll ask for it later */ 
        logWarning("Dropping password that could not be decoded. See bug 18752.");
    }
    return text
}

function _strcmp(a,b) {
    if (a==b) return 0;
    if (a<b) return -1;
    return 1;
}

function getServerListFromPreference(prefset) {
  var list = new Array();  
  if (prefset.hasPref("deploymentServerList")) {
    var deploymentServerList = prefset.getPref("deploymentServerList");
    for (var i = 0; i < deploymentServerList.length; i++) {
      var tuple = deploymentServerList.getPref(i).QueryInterface(Components.interfaces.koIOrderedPreference);
      // invocations were added later
      var info = new String(tuple.getStringPref(1));
      var server_info = info.split(":")
      var password = decodePassword(server_info[5]);
      list.push(new Server(server_info[0], tuple.getStringPref(0), 
              server_info[1], server_info[2], server_info[3],
              server_info[4], password));
    }
  }
  list.sort(function (a,b) { return _strcmp(a.alias,b.alias); });
  return list;
}

function createServersMenuItem(server) {
  var cell = document.createElementNS(XUL_NS, 'menuitem');
  cell.setAttribute('id', server.id);
  cell.setAttribute('label', server.alias);
  cell.setAttribute('assocId', server.id);
  //cell.setAttribute("onclick", "onServerChanged('"+server.alias+"');");
  return cell;
}

function loadServerList()
{
    var menubox = document.getElementById("serversList");
    menubox.removeAttribute("collapsed");
    
    servers = getServerListFromPreference(globalPrefs);
    var menulist = document.getElementById("serverMenu");
    var menuitem;
    
    // add the default entry
    menuitem = document.createElementNS(XUL_NS, 'menuitem');
    if (FTPBrowserOnly) {
        menuitem.setAttribute('id', "blank_entry");
        menuitem.setAttribute('label', "");
        //menuitem.setAttribute("onclick", "onServerChanged('');");
    } else {
        menuitem.setAttribute('id', localDrives);
        menuitem.setAttribute('label', localDrives);
        //menuitem.setAttribute("onclick", "onServerChanged('Local Drives');");
    }
    menulist.appendChild(menuitem);
    
    // add the deployment servers
    for (var i = 0; i < servers.length; i++) {
        menuitem = createServersMenuItem(servers[i]);
        menulist.appendChild(menuitem);
    }
    if (servers.length > 0) document.getElementById("serverMenuList").selectedIndex = 0
        
}

function clearHistoryList()
{
    var menulist = document.getElementById("lookInMenu");
    var items = menulist.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        menulist.removeChild(items[i]);
    }
}

function clearServerList()
{
    var menulist = document.getElementById("serverMenu");
    var items = menulist.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        menulist.removeChild(items[i]);
    }
}

function getAliasIndex(alias) {
  //dump("getting the server object for "+alias+"\n");
  if (servers) {
  for (var i = 0; i < servers.length; i++) {
    if (alias == servers[i].alias) return i;
  }
  }
  return -1;
}

function onServerChanged(alias) {
    //dump("onServerChanged("+alias+")\n");
    closeFTP();
    // just to allow closing the connection...
    if (!alias) {
        setInitialDirectory(null);
        return;
    }
    currentServer = alias;
    var idx = getAliasIndex(alias);
    if (idx < 0) {
        if (alias == localDrives) {
            // clear the global ftp connection
            setInitialDirectory(lastLocalDirectory);
            return;
        }
        if (alias.indexOf("ftp://") == 0) {
            setInitialDirectory(alias);
            return;
        }
        if (alias.indexOf("ftp.") == 0) {
            var url = "ftp://"+alias;
            setInitialDirectory(url);
            return;
        }
        alertDialog("Unable to access URL\n");
        return;
    }
    var server = servers[idx];
    goServerConnect(server);
}

function goServerConnect(server) {
    // build url!
    var url = server.type.toLowerCase()+"://";
    if (server.username) {
        url += server.username;
        if (server.password) url += ":"+server.password;
        url += "@";
    }
    url += server.hostname;
    if (server.port) url += ":"+server.port;
    if (server.path[0] != "/") url += "/"+server.path;
    else url += server.path;
    setInitialDirectory(url);
}

function goServerPrefs() {
    // just load up prefs so they can add servers
    parent.doGlobalPrefs("serversItem");
    clearServerList();
    loadServerList();
}
