    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// NO SCIMOZ in this class!!!   
var gViewManagerList = [];

function ViewManager() {
    this._views = new Object();
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
          getService(Components.interfaces.nsIObserverService);
    gViewManagerList.push(this);
};

ViewManager.prototype.destructor = function() {
    for (var i in gViewManagerList) {
        if (typeof(gViewManagerList[i]) != 'undefined' &&
            gViewManagerList[i] == this) {
            gViewManagerList.splice(i,1);
        }
    }
}

// The public API to get the current view.  May return NULL if no view is current
// (generally because no views are open, or internally during the
// view change process).
ViewManager.prototype.getCurrentView = function() {
    throw "ViewManager.prototype.getCurrentView - Not Implemented"
}

ViewManager.prototype.getNextView = function() {
    throw "ViewManager.prototype.getNextView - Not Implemented"
}

ViewManager.prototype.getPreviousView = function() {
    throw "ViewManager.prototype.getPreviousView - Not Implemented"
}

ViewManager.prototype.destroyView = function() {
    throw "ViewManager.prototype.destroyView - Not Implemented"
}

ViewManager.prototype.createNewView = function(type) {
    throw "ViewManager.prototype.createNewView - Not Implemented"
}

ViewManager.prototype.setFocus = function() {
    throw "ViewManager.prototype.setFocus - Not Implemented"
}

ViewManager.prototype.getViewForFile = function(url) {
    if (!url) return null;
    var urlUtils = Components.classes["@activestate.com/koUrlUtils;1"].getService(Components.interfaces.koIUrlUtils);
    
    for(var url_look in this._views) {
        if (urlUtils.sameURL(url, url_look))
            return this._views[url_look];
    }
    return null;
}

ViewManager.prototype.renameView = function(view, url, leafName) {
    throw "ViewManager.prototype.renameView - Not Implemented"
}

ViewManager.prototype.doFileNew = function(language) {
    throw "ViewManager.prototype.doFileNew - Not Implemented"
}

ViewManager.prototype.doFileOpen = function(url) {
    //XXX This function is way too overloaded and needs to be split up.

    //XXX don't do an expensive fixup if we already have the correct url
    //XXX the fixup breaks untitled buffers anyway.  
    var view = this.getViewForFile(url);
    
    if (view == null) {
	url = fixupUrl(url);
	view = this.getViewForFile(url);
    }
    if (view != null) {
        this.setCurrentView(view);
    } else {
      var leafName = makeLeafName(url);
      view = this.createNewViewWithURI("scintilla", url, leafName);

      // If the view was successfully opened, add the file to the MRU list, which
      // is stored in the global preferences service.
      if(view) {
        // Some URLs we dont want added to the list!
        MRU_addURL("mruFileList", url);
        update_breakpoints(view);
        updateTitlebar(view);
        
        // notify a new file is opened
        try {
          this.observerSvc.Notify(view, 'view_opened', view.url);
        } catch (e) {
          ;
        }        
      }
    }

    return view;
}

ViewManager.prototype.doFileOpenAtLine = function(url, lineno) {
    var v = this.doFileOpen(url);
    if (v) {
        v.gotoPosition(lineno);
    }
    return v;
}

ViewManager.prototype.getDirtyURLs= function() {
    var endangeredURLs = new Array();
    for(viewURL in this._views) {
        view = this._views[viewURL];
        if (view.isDirty()) {
            endangeredURLs.push(viewURL);
        }
    }
    return endangeredURLs;
}

ViewManager.prototype.doFileCloseAll = function() {
    var view, i;
    var endangeredViews = this.getDirtyURLs(new Object())
    if (endangeredViews.length > 0) {
        var obj = new Object();
        obj.URLs = endangeredViews;
        obj.title = "Close All";
        obj.prompt = "Save changes to the selected documents?";
        openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                 "komodo_closeConfirm",
                                 "chrome,close=no,modal,resizable=true",
                                 obj);
        if (!obj || obj.ret == false) {
            // user pressed ESC or cancel
            return false;
        }
        var selectedURLs = obj.URLs;
        for (i=0 ; i < selectedURLs.length; i++) {
            var url = selectedURLs[i];
            this.ConfirmedSave(url);
        }
    }
    for (i in this._views) {
        view = this._views[i];
        this.setCurrentView(view);
        this._forcedCloseView(this._views[i]);
    }
    return true;
}

ViewManager.prototype.doSaveAll = function() {
  // walk the views for things we have in memory
  // for any that aren't associated, raise them as
  // a possibility.

  var initialView = this.getCurrentView();

  if (initialView == null) {
    logError("viewManager.getCurrentView() == null!");
  }

  for(viewURL in this._views) {
    //dump(flattenJSObject(viewURL)+"\n");
    var view = this._views[viewURL];
    if (view.isDirty()) {
        if (! this._SaveView(view)) {
            return false;
        }
    }
    // If we are already there, setCurrentView is smart :-)
    this.setCurrentView(initialView);
  }
  return true;
}

ViewManager.prototype._SaveView = function (view) {
    //XXX koEditorWrapper always acts on the current view,
    //so we must switch to it here or it will save the contents
    //of the foreground buffer on top of the file at view.url!

    // It would be very nice to remove this restriction as it causes massive flicker
    // and takes a long time.
    
    this.setCurrentView(view);

    var rc = view.doSave();
    if (!view.url) {
        // ignore failure - user may choose to cancel saving this
        // file, but want to save the rest
        // (indeed, MarkH believes these shouldnt be prompted!)
        return true;
    } else {
        return rc;
    }
}

ViewManager.prototype.ConfirmedSave = function(url) {
    return this._SaveView(this._views[url]);
}

ViewManager.prototype.doDirtyDocsExist = function() {
   // Find out whether there are any dirty views.
    var dirtyDocsExist = false;
    for(var viewURL in this._views) {
        var view = this._views[viewURL];
        dirtyDocsExist |= view.isDirty();
    }
    return dirtyDocsExist;
}

ViewManager.prototype.activate = function() {
    // do nothing
}

ViewManager.prototype.deactivate = function() {
    // do nothing
}

ViewManager.prototype.setCurrentView = function (view, forceview /*=false*/) {
    if (typeof(forceview) == "undefined") forceview = false;
    var currentView = this.getCurrentView()
    if (forceview || view != currentView) {
        // de-select the current tab.
        if (!forceview && currentView != null)
            currentView.deactivate();
        // select the new tab.
        if (view != null) {
            view.activate();
            updateTitlebar(view);
        }
        else {
            updateTitlebar(null); // no title passed
        }
        // And tell anyone listening we have changed.
        //XXX There are new 'post_activate_view' and
        //    'pre_deactivate_view' notifications that are more useful.
        //    Should convert to those.
        try {
          this.observerSvc.Notify(view, 'currentView', view);
        } catch (e) {
          ;
        }
    } else {
        updateTitlebar(view);
    }
    if (view)  {
        this.updateWindowList(view, 'existing');
        // XXX is this needed?
        this.setFocus();
    }
}

ViewManager.prototype.getViews = function() {
    return this._views;
}
 
ViewManager.prototype.closeViewForFile = function (file) {
    var view = this.getViewForFile(file);
    if (view) 
        return this.closeView(view);
    else 
        return true;
}

ViewManager.prototype._forcedCloseView = function (view) {
    // This function is called by closeAll after the user has specifically said they
    // don't mind throwing away the contents of the view.  So this code
    // will go through even if the file is dirty.
    
    // This function is also called by the normal closeView() call after the user has
    // been given a chance to save it.
    
    if (this.getCurrentView() == view) {
        // Next determine the new tab to get focus - we switch to it
        // while we self-destruct.  This deactivates and helps cleanup.
        var nextView = this.getNextView(false);
        if (nextView == null) {
            logWarning("nextView returned null");
            nextView = this.getPreviousView();
            //logWarning(nextView);
        }
        if (nextView == view) {
            nextView = null; // If I am the only view, don't try and re-select me!
        }
        this.setCurrentView(nextView); // handles view being NULL - still de-selects old one.
    }

    // Tell the view to clean itself up.
    view.close();

    // and finally wipe the view itself.
    var url = view.url;
    this.destroyView(view)
    
    try {
        this.observerSvc.Notify(this, 'view_closed', url);
    } catch (e) {
        ;
    }
    
    return true;
}

ViewManager.prototype.closeAllViewsEvenIfDirty = function () {
    for (var URL in this._views) {
        view = this._views[URL];
        this.setCurrentView(view);
        this._forcedCloseView(view);
    }
}

ViewManager.prototype.closeView = function (view) {
    // allow using this function to close the current view without params.
    if (typeof(view)=="undefined" || view == null) {
        view = this.getCurrentView();
        if (!view) return true;
    }
    
    // nicely and cleanly shut the view, saving if necessary.
    // returns true if OK to continue closing, else false.
    if (!view.canClose()) {
        return false;
    }
    return this._forcedCloseView(view)
}

ViewManager.prototype.updateWindowList = function(view, action, oldurl) {
    // XXX do nothing???
}

/****************************************************/
// status icon functions

ViewManager.prototype.updateFileStatusIcon = function(item, actionimg, statusimg) {
    var status = null;
    var readonly = null;
    for (var key in item.iconstyle) {
        switch (key) {
            case 'ok':
            case 'sync':
            case 'conflict':
                if (parseInt(item.iconstyle[key])) status=key;
            break;
            case 'readonly':
                if (parseInt(item.iconstyle[key])) readonly=key;
            break;
            default:
            break;
        }
        //dump('iconvalue '+readonly+status+' key '+pKeys.value[i]+' value '+pValues.value[i]+'\n');
    }
    var baseicon = "chrome://komodo/skin/images/tab_icon_";
    if (status && readonly) {
        statusimg.setAttribute("src",baseicon+readonly+"_"+status+".png");
    } else if (status) {
        statusimg.setAttribute("src",baseicon+status+".png");
    } else if (readonly) {
        statusimg.setAttribute("src",baseicon+readonly+".png");
    } else {
        statusimg.removeAttribute("src");
    }
    if ('action' in item.properties && item.properties['action'] != '') {
        actionimg.setAttribute("src",baseicon+item.properties['action']+".png");
    } else {
        actionimg.removeAttribute("src");
    }
}

ViewManager.prototype.revertViewsByURLs = function(urls) {
    var originalView = this.getCurrentView();
    var view;
    var openedurl;
    var i;
    
    for (i = 0; i < urls.length; i++) {
        view = this.getViewForFile(urls[i]); // This horrible hack is required because revert only works on the current view
        if (view) {
            view._observedFileChange = 1;  // This horrible hack is required to prevent setCurrentView to call activate to detect that the file is dirty and prompt a dialog
            this.setCurrentView(view);
            view.revert(); // this will do the SCC refresh notify
            view._observedFileChange = 0;
        }
    }
    this.setCurrentView(originalView);
}
