/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */


//--- globals

var editorExtensionUI;
var viewManager;
var outputViewManager = null; // output pane

var globalPrefService; //defined in koGlobalOverlay.js
var globalPrefs = null;

// Indicate the desired find dialog tab to show when the find dialog is
// launched.
var gFindDialogTabName = null;
var helpObserverSvc = null;

/* XXX The scintilla widget will set this in it's focus event handler
       (koWidgets.xml).
   Currently this is only used for the find dialog to fix searches in xslt
   editor/input panes Because we switch the focus to a xul textbox when we
   gotoline or find/replace, and are operating in a new window, we end up not
   being able to properly operate in the xslt input pane.  find/goto now use
   this variable rather than retrieving the current focus or using the global
   viewmanager.
*/
var gLastFocusedViewManager = null;

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
const VK_BACKSPACE = 8;
const VK_RIGHT = 39;
const VK_LEFT = 37;

//TODO: put this and stuff from projectviewer.js in an seperate source file.

//////////////////////////////////////////////////////////////////
// Linting
//////////////////////////////////////////////////////////////////

const PROXY_SYNC    = 0x0001;  // acts just like a function call.
const PROXY_ASYNC   = 0x0002;  // fire and forget.  This will return immediately 
                               // and you will lose all return information.
const PROXY_ALWAYS  = 0x0004;  // ignore check to see if the eventQ is on the 
                               // same thread as the caller, and alway return a 
                               // proxied object.

// No point looking up constants every single event!
const SC_MOD_BEFOREINSERT = Components.interfaces.ISciMoz.SC_MOD_BEFOREINSERT;
const SC_MOD_BEFOREDELETE = Components.interfaces.ISciMoz.SC_MOD_BEFOREDELETE;
const SC_FOLDLEVELHEADERFLAG = Components.interfaces.ISciMoz.SC_FOLDLEVELHEADERFLAG;
const SC_MOD_INSERTTEXT = Components.interfaces.ISciMoz.SC_MOD_INSERTTEXT;
const SC_MOD_DELETETEXT = Components.interfaces.ISciMoz.SC_MOD_DELETETEXT;                                                            
const SC_MOD_CHANGESTYLE = Components.interfaces.ISciMoz.SC_MOD_CHANGESTYLE;
const SC_MOD_CHANGEFOLD = Components.interfaces.ISciMoz.SC_MOD_CHANGEFOLD;

// These are modifications we want to receive from scintilla.  We will not
// receive any onModification call in our eventist class if the notification is
// not part of SC_MOD_WANTED.
const SC_MOD_WANTED = SC_MOD_CHANGESTYLE | 
                      SC_MOD_INSERTTEXT | SC_MOD_DELETETEXT | 
                      SC_MOD_BEFOREDELETE | SC_MOD_BEFOREINSERT;

function Notified () {};
Notified.prototype = {
    Observe: function (prefSet, prefSetID, prefName) {
        if (prefName == "autoSaveMinutes") {
            _setAutoSaveTimer();
        } else if (prefName == "cursorColor" ||
            prefName == "currentLineColor" ||
            prefName == "useCurrentLineColor" ||
            prefName == "selectBack" ||
            prefName == "edgeColor" ||
            prefName == "useSelectFore" || prefName == "selectFore" ||
            prefName == "ySlop")
        {
            var ids = ['scin', 'input-scin', 'redirect-scin',
                       'runoutput-scintilla'];
            for (var i in ids) {
                update_scintilla_global_pref(
                    document.getElementById(ids[i]).ISciMoz, prefName);
            }
        }
        //logDebug("Preference "+prefName+" has changed "+this+" by "+prefSetID);
    }
};


var lintDisplayer;
var lintManager;
var proxyManager;

// jsLintBuffer implements koILintBuffer and koILintObserver
// and provides an abstraction for
// dealing with the virtual buffer/scimoz/py_inst mess.
function jsLintBuffer(view) {
    this.view = view;
    this.prefs = view.prefs;
    this.active = false;
    this.errorString = null;

    // Register the lint buffer with the observer service to watch
    // for changes to the linting preference.
    this.observer = proxyManager.getProxyForObserver(this,
        PROXY_SYNC | PROXY_ALWAYS);
    lintManager.addObserver(this.observer, this.view.viewName);

    // create a proxy to this
    this.proxy = proxyManager.getProxyForBuffer(this,
                                                PROXY_SYNC | PROXY_ALWAYS);
    this.lintResults = null;
    this._lintTimeout = null;
    this._clearTimeout = null;

    this._observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
    globalPrefs.AddObserver(this);
    this.prefs.AddObserver(this);
}

jsLintBuffer.prototype = {

    LINT_DELAY: 1000,  // milliseconds to wait before linting

    // koILintBuffer impementation
    get text() {
        if (!this.view.manager) {
            // The view is going away and the manager has already been
            // destroyed.
            //XXX Need to fix synchronization here.
            return '';
        }
        current_view = this.view.manager.getCurrentView();
        if (current_view && current_view._scimoz) {
            //XXX This returns raw text as from a binary read. Currently this
            //    forces each of the linters to clean up the line endings as
            //    for text file read (i.e. '\n' only). Perhaps that step should
            //    be centralized here.
            //XXX Copy of entire buffer. Don't do it.
            return current_view._scimoz.text;
        } else {
            logError("Getting text attribute when there is no scimoz!"); 
            return '';
        }
    },

    // nsIObserver implementation
    Observe: function(subject, topic, data) {
        if(data == "editUseLinting") {
            if(this.prefs.getBooleanPref("editUseLinting")) {
                this.request();
            }
            else {
                this.cancel();
                this.clear();
            }
        }
    },

    // public extensions
    activate: function() {
        // dump("jsLintBuffer.activate()\n");
        this.active = true;
    },

    deactivate: function() {
        // dump("jsLintBuffer.deactivate\n");
        this.cancel();
        this.active = false;
    },

    _notifyStart: function() {
        try {
            var empty = new Object();
            this._observerSvc.Notify(empty, "current_view_check_started", "");
        } catch(e) {
            // raises exception if no observers are registered
        }
    },

    _request: function() {
        // Linters can't chdir to a remote URL, so don't pass in a working
        // directory unless it's local.  The regex is robust: when we add
        // new protocols we will automatically filter for them.

        var cwd = null;
        var path = displayPathFromURL(this.view.url);
        if (!path.match(/^\w+:\/\//)) {
            cwd = path;
        }

        if (this.syntaxChecker != null) {
            this._notifyStart();
            lintManager.addRequest(this.proxy, this.view.py_inst.encoding,
                                   lintManager.PRIORITY_HIGH,
                                   this.view.viewName, this.syntaxChecker,
                                   cwd);
        }    
        if (this.lintChecker != null) {
            this._notifyStart();
            lintManager.addRequest(this.proxy, this.view.py_inst.encoding,
                                   lintManager.PRIORITY_HIGH,
                                   this.view.viewName, this.lintChecker,
                                   cwd);
        }
        // If there is neither a linter nor a syntax checker, just verify that
        // the encoding is correct
        if (this.syntaxChecker == null && this.lintChecker == null) {
            this._notifyStart();
            lintManager.addRequest(this.proxy, this.view.py_inst.encoding,
                                   lintManager.PRIORITY_HIGH,
                                   this.view.viewName, "", cwd);
        }        
        this._lintTimeout = null;
    },

    request: function() {
        this.cancel();
        this.clear();
        // XXX is this right with multiple views?
        this._lintTimeout = window.setTimeout(
            'viewManager.getCurrentView().lintBuffer._request();',
            this.LINT_DELAY);
    },

    _clear: function() {
        this.lintResults = null;
        if (this.syntaxChecker != null) {
            lintManager.broadcastResultsCleared(this.view.viewName,
                                                this.syntaxChecker);
        }
        if (this.lintChecker != null) {
            lintManager.broadcastResultsCleared(this.view.viewName,
                                                this.lintChecker);
        }
    },

    clear: function() {
        // no need to clear if no results.
        if (this.lintResults) {
            this._clearTimeout = window.setTimeout(
                'viewManager.getCurrentView().lintBuffer._clear();', 0);   
        }
    },

    cancel: function() {
        if (this._lintTimeout) {
            window.clearTimeout(this._lintTimeout);
            this._lintTimeout = null;
        }
        if (this._clearTimeout) {
            window.clearTimeout(this._clearTimeout);
            this._clearTimeout = null;
        }
        if (typeof(lintManager) == 'undefined' || !lintManager) {
            return;
        }
        lintManager.cancelRequests(this.view.viewName);
    },

    updateDisplay: function(startPos, styleLen) {
        if (this.lintResults) {
            current_view = this.view.manager.getCurrentView();
            var language = this.view.py_inst.sourceFile.
            getLanguageBlockAtPosition(0).language;
            lintDisplayer.display(current_view._scimoz, this.lintResults,
                                  language.indicatorBits, startPos, styleLen);
        }
    },

    destructor: function() {
        //dump("jsLintBuffer.destructor()\n");
        if (typeof(lintManager) == 'undefined' || !lintManager) {
            return;
        }
        this.cancel();
        lintManager.removeObserver(this.observer);
        view = null;

        // Stop observing changes in the editUseLinting preference.
        globalPrefs.RemoveObserver(this);
        this.prefs.RemoveObserver(this);
    },

    // private accessors
    // provide access to attributes that could be in the scimoz/mydata
    // pair or in a virtual buffer

    get language() {
        return this.view.py_inst.language;
    },

    get syntaxChecker() {
        var result = null;
        var v = this.view.manager.getCurrentView();
        if (v) {
            var linterLanguageService = v.py_inst.sourceFile.
                getLanguageBlockAtPosition(0).language.
                getLanguageService(Components.interfaces.koILinterLanguageService);
            if (linterLanguageService) {
                result = linterLanguageService.syntaxChecker;
            }
        }
        return result; 
    },
  
    get lintChecker() {
        var result = null;
        var v = this.view.manager.getCurrentView();
        if (v) {
            var linterLanguageService = v.py_inst.sourceFile.
                getLanguageBlockAtPosition(0).language.
                getLanguageService(Components.interfaces.koILinterLanguageService);
            if (linterLanguageService) {
                result = linterLanguageService.lintChecker;
            }
        }
        return result; 
    },

    // Methods called by the Lint Manager when interesting things happen
    // to our document.
    notify: function(viewName, linterType, cwd, lintResults, errorString) {
        this.lintResults = lintResults;
        this.errorString = errorString;
        if (this.active) {
            var scin = this.view.manager.getCurrentView()._scimoz;
            this.updateDisplay(0, scin.length)
            try {
                var empty = new Object();
                this._observerSvc.Notify(empty, "current_view_check_finished",
                                         "");
            } catch(e) {
                // do nothing: Notify sometimes raises an exception if (???)
                // receivers are not registered?
            }
        }
    },

    notifyClear: function(viewName, linterType) {
        if (this.active) {
            var scin = this.view.manager.getCurrentView()._scimoz;
            current_view = this.view.manager.getCurrentView();
            var language = this.view.py_inst.sourceFile.
                getLanguageBlockAtPosition(0).language;
            lintDisplayer.displayClear(current_view._scimoz,
                                       language.indicatorBits);
        }
    }
}


function initialize_lint() {
    lintDisplayer = Components.classes["@activestate.com/koLintDisplayer;1"].
                    createInstance(Components.interfaces.koILintDisplayer);

    lintManager = Components.classes["@activestate.com/koLintManager;1"].
                  createInstance(Components.interfaces.koILintManager);

    //XXX we should use nsIProxyObjectManager but it is not scriptable yet
    //XXX for now we use my hack
    proxyManager = Components.classes["@activestate.com/koLintProxyWrapper;1"].
                   createInstance(Components.interfaces.koILintProxyWrapper);
}



//////////////////////////////////////////////////////////////////
// Misc. routines
//////////////////////////////////////////////////////////////////

// Used by the Python implemented editor extensions for a very basic UI
function EditorExtensionUI() {}
EditorExtensionUI.prototype = {
    askInteger: function(caption, prompt, min, max, value) {
        // XXX - todo - make a better dialog with validation built in.
        while (true) {
            var got = promptDialog(prompt, value.value, "OK", "Cancel")
            if (!got) { // cancelled.
                value.value = 0;
                return false;
            }
            var int_got = parseInt(got, 10)
            if (!isNaN(int_got) && (min == max || (min <= int_got && max >= int_got))) {
                value.value = int_got;
                return true;
            }
            alertDialog("Please enter a number between "+min+" and "+max);
        }
        // not reached.
        logError("NOT REACHED!!");
        return false;
    },

    askYesNo: function(caption, prompt) {
        return "Yes" == yesNoDialog(prompt, "Yes");
    },

    // nsISupports interface
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.koIEditorExtensionUI))
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
}



// we keep a queue of actions to be taken once 
var _updateUIActionQueue = [];
var _gUpdateUItimeout = -1;

function addUpdateUIAction(func, args) {
    _updateUIActionQueue.push({'func': func, 'args': args});
}

function executeUpdateUIActions() {
    while (_updateUIActionQueue.length) {
        var action = _updateUIActionQueue.shift();
        action.func.apply(window, action.args);
    }
    _gUpdateUItimeout = -1;
}

function deleteText(scin, position, length) {
    scin.targetStart = position;
    scin.targetEnd = position + length;
    scin.replaceTarget(0, "");
}


function _UpdateEditorWrapper(v){
    if (typeof(v.py_inst_events) != 'undefined' && v.py_inst_events)
        v.py_inst_events.onUpdateUI();
}

function eventist(viewmanager) {
    this.viewmanager = viewmanager;
    this._observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                        getService(Components.interfaces.nsIObserverService);
}
eventist.prototype = {
    destructor: function() {
        this.viewmanager = null;
    },

    onStyleNeeded: function(position) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onStyleNeeded(position);
    },

    onCharAdded: function(ch) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onCharAdded(ch);
    },

    onSavePointReached: function() {
        var v = this.viewmanager.getCurrentView();
        if(v) {
            v.py_inst_events.onSavePointReached()
            this.viewmanager.indicateIfDirty(v);
            koRequestUpdateCommands("save");
        } else {
            logError("eek -- no current view!");
        }
        return;
    },

    onSavePointLeft: function() {
        var v = this.viewmanager.getCurrentView();
        if(v) {
            v.py_inst_events.onSavePointLeft()
            this.viewmanager.indicateIfDirty(v);
            koRequestUpdateCommands("save");
        } else {
            logError("eek -- no current view!");
        }
        return;
    },

    onModifyAttemptRO: function() {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onModifyAttemptRO()
    },

    onKey: function(ch, modifiers) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onKey(ch, modifiers);
    },

    onDoubleClick: function() {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return true;
    },

    onUpdateUI: function() {
        var view = this.viewmanager.getCurrentView();
        if (typeof(view) == 'undefined' || !view) return false;
        if (_gUpdateUItimeout != -1) {
            window.clearTimeout(_gUpdateUItimeout);
        }
        _StatusBar_UpdateLineCol(view);
        addUpdateUIAction(koRequestUpdateCommands, ["select"]);
        addUpdateUIAction(_UpdateEditorWrapper, [view]); 
        _gUpdateUItimeout = window.setTimeout('executeUpdateUIActions();', 50);
        return true;
    },

    onModified: function(position, modificationType, text, length,
                       linesAdded, line, foldLevelNow, foldLevelPrev)
    {
        var changestyle = modificationType & SC_MOD_CHANGESTYLE;
        var modtext = modificationType & 
                      (SC_MOD_INSERTTEXT | SC_MOD_DELETETEXT);
        var v = this.viewmanager.getCurrentView();
        if (!v) return false;

        if (modificationType & SC_MOD_BEFOREDELETE ||
            modificationType & SC_MOD_BEFOREINSERT)
        {
            var scin = v._scimoz;
            var thisLine = scin.lineFromPosition(scin.currentPos);
            if (scin.getFoldLevel(thisLine) & SC_FOLDLEVELHEADERFLAG &&
                !scin.getFoldExpanded(thisLine))
            {
                scin.toggleFold(thisLine);
            }
        }

        if(changestyle || modtext) {
            if (v.lintBuffer != null && v.useLinting) {
                if (modtext) {
                    v.lintBuffer.request();
                } else if (changestyle) {
                    v.lintBuffer.updateDisplay(position,
                                               v._scimoz.length-position);
                }
            }
        }
        var ret = true;
        // Only call to koeditorwrapper for those notifications that we handle
        // there.
        if (modtext) {
            ret = v.py_inst_events.onModified(position, modificationType,
                                              text, length, linesAdded, line,
                                              foldLevelNow, foldLevelPrev);
        }

        return ret;
    },

    onMacroRecord: function(message, wParam, lparam) { 
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onMacroRecord(message, wParam, lparam);
    },

    onMarginClick: function(modifiers, position, margin) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }

        // Python currently handles the folding margin.
        if (margin == 2) {
            var lineClicked = v._scimoz.lineFromPosition (position);
            toggle_breakpoint_at_line(v, lineClicked); // need filename here!
        }
        return v.py_inst_events.onMarginClick(modifiers, position, margin);
    },

    onNeedShown: function(position, length) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onNeedShown(position, length);
    },

    onPosChanged: function(position) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        // The following implements the suggestion #2 that I made on the
        // scintilla mailing list in the following email.
        
        // If scintilla implements it, we should take this code out [DA]
        
        //I like the new caret policies, although that table sure is hard
        //to understand without trying out the various combinations.  [I
        //_really_ like the new perf in word wrapping, thanks Neil!]. 
        //
        //I have a frustration with the y caret policy that I'd like some
        //suggestions on. When using the keyboard (both for typing and
        //moving around), I like the behavior of CARET_SLOP | CARET_STRICT
        //| CARET_EVEN, with a slop of 10 lines or so.  It keeps the
        //edited text in a central location without causing massive
        //vertical jerks which the eye isn't used to (the eye _is_ used to
        //performing _horizontal_ saccades and reading text off-fixation,
        //so I use a more jumpy policy on X -- ah, the benefits of a Ph.D.
        //in vision research!). The problem occurs when one uses the mouse
        //to change the caret and places it in the UZ.  There are, as far
        //as I can tell, two possible choices.
        //
        //    1) apply the y caret policy on mouse up, which I don't like
        //    because after the mouse up, the caret isn't where the mouse
        //    cursor is, which is disturbing.  This is what something like
        //    'vi' does, but it's not a big deal in vi since the mouse isn't
        //    so tightly bound with caret position and most users don't use
        //    the mouse much.
        //    
        //    2) let scintilla apply the y caret policy when it feels like
        //    it, which means that after a click in the UZ, character
        //    insertions or even horizontal cursor motions will cause a
        //    vertical jump.  Also confusing.
        //    
        //I'm not happy with either.  I have _two_ alternatives: 
        //
        //    1) On mouse clicks in the UZ, perform a _smooth_ vertical
        //    scrolling, giving the eye's smooth pursuit system (what we use
        //    to track smoothly moving objects) a chance to kick in and
        //    follow.  Note that 'smooth' is relative -- doing line-by-line
        //    scrolling is ok -- in my experiments, it's plenty smooth
        //    enough (doing it by pixel-based shifts would make it look more
        //    like the behavior of Mac OS/X or DEC vt100's, depending on how
        //    old you are).
        //
        //2) On mouse clicks in the UZ, temporarily reset the slop so that
        //    the mouse click Y position defines the new UZ and that "by
        //    fiat", the current caret policy is within the newly defined UZ.
        //    Have the slop "grow" back to its "real" definition as the user
        //    moves further "in". That's nice in that it offers the least
        //    surprise to the user, I think.

        var scimoz = v._scimoz;
        var curLine = scimoz.lineFromPosition(scimoz.currentPos);
        var origySlop = globalPrefs.getLongPref('ySlop');
        var ySlop;
        var top, bottom;
        top = curLine - scimoz.firstVisibleLine;
        bottom = scimoz.firstVisibleLine + scimoz.linesOnScreen - curLine - 1;
        ySlop = Math.min(origySlop, top);
        ySlop = Math.min(ySlop, bottom);
        if (origySlop != ySlop) {
            scimoz.setYCaretPolicy(scimoz.CARET_SLOP | scimoz.CARET_STRICT | scimoz.CARET_EVEN,
                                   ySlop);
        }
        return v.py_inst_events.onPosChanged(position);
    },

    onZoom: function(position) {
        var v = this.viewmanager.getCurrentView();
        if (v == null) {
            logError("this.viewmanager.getCurrentView() == null!");
            return false;
        }
        return v.py_inst_events.onZoom(position);
    },
    
    // nsISupports interface
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.ISciMozEvents))
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
}


// Controller for "view" related edit menu items.
// XXX - should be moved to view.js.
// XXX - note - we could probably get _significant_ performance
// gains by ensuring that this controller is _only_ in place when
// a view is current.  When we can guarntee this, all "isCommandEnabled()"
// calls, which currently call viewManager.getCurrentView(), can be replaced with
// "return true;".  This may already be true, but not going to
// attempt this optimization for release.


function EditorItemsController(viewManager) {
    this._viewManager = viewManager;
    this._incrementalSearchPattern = '';
    this._lastIncrementalSearchText = '';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
EditorItemsController.prototype = new Controller();
EditorItemsController.prototype.constructor = EditorItemsController;

EditorItemsController.prototype.destructor = function() {
    this._viewManager = null;
}

// boolean prefs are marked "not supported" if there is no view
// This is to handle that "isCommandEnabled" is actually doing
// the state of the checkbox.
EditorItemsController.prototype._is_bool_pref_command_supported = function(pref_name) {
    return this._viewManager.getCurrentView() != null;
}

// This is probably bogus.  Instead of asking scintilla its current state
// we should just ask the preference service for the state.
// See _is_bool_real_pref_command_enabled
EditorItemsController.prototype._is_bool_pref_command_enabled = function(sci_prop_name) {
    var v = this._viewManager.getCurrentView();
    if (v) {
        var exist = v._scimoz[sci_prop_name];
        return exist;
    }
    return false;
}

EditorItemsController.prototype._is_bool_real_pref_command_enabled = function(pref_name) {
    var cv = this._viewManager.getCurrentView();
    if (cv) {
        return cv.prefs.getBooleanPref(pref_name)
    }
    return false;
}

EditorItemsController.prototype._do_bool_pref_command = function(pref_name, existing_val) {
    this._viewManager.getCurrentView().prefs.setBooleanPref(pref_name, !existing_val);
    // DONT update SciMoz here - let the preference notify mechanism
    // trigger the normal preference handling code.
}


//---- find stuff

EditorItemsController.prototype.is_cmd_find_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}
EditorItemsController.prototype.do_cmd_find = function() {
    doFind();
}

EditorItemsController.prototype.is_cmd_findNextResult_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}
EditorItemsController.prototype.do_cmd_findNextResult = function() {
    FindResultsTab_NextResult();
}

EditorItemsController.prototype.is_cmd_findNext_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}
EditorItemsController.prototype.do_cmd_findNext = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var pattern = MRU_get("find-patternMru");
    if (pattern) {
        var context = Components.classes[
            "@activestate.com/koFindContext;1"]
            .createInstance(Components.interfaces.koIFindContext);
        context.type = findSvc.options.preferredContextType;
        Find_FindNext(window, context, pattern);
    } else {
        doFind();
    }
}

EditorItemsController.prototype.is_cmd_findPrevious_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}
EditorItemsController.prototype.do_cmd_findPrevious = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var pattern = MRU_get("find-patternMru");
    if (pattern) {
        var context = Components.classes["@activestate.com/koFindContext;1"]
                      .createInstance(Components.interfaces.koIFindContext);
        context.type = findSvc.options.preferredContextType;
        // go in the opposite direction from the current setting
        findSvc.options.searchBackward = !findSvc.options.searchBackward;
        Find_FindNext(window, context, pattern);
        // restore the direction setting
        findSvc.options.searchBackward = !findSvc.options.searchBackward;
    } else {
        doFind();
    }
}

EditorItemsController.prototype.is_cmd_findNextSelected_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}

EditorItemsController.prototype.do_cmd_findNextSelected = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var v = this._viewManager.getCurrentView();
    var scimoz = v._scimoz
    if (!scimoz) return;
    var pattern = scimoz.selText;
    if (pattern == '') {
        scimoz.wordLeft();
        var curChar = scimoz.getWCharAt(scimoz.currentPos);
        while (curChar.search(/\W/) == -1) {
            pattern += curChar;
            scimoz.charRight();
            curChar = scimoz.getWCharAt(scimoz.currentPos);
        }
    }

    var context = Components.classes["@activestate.com/koFindContext;1"]
        .createInstance(Components.interfaces.koIFindContext);
    context.type = findSvc.options.preferredContextType;
    Find_FindNext(window, context, pattern);
}

EditorItemsController.prototype.is_cmd_replace_enabled = function() {
    return this._viewManager.getCurrentView() ? true : false;
}
EditorItemsController.prototype.do_cmd_replace = function() {
    doReplace();
}

EditorItemsController.prototype.is_cmd_fontFixed_supported = function() {
    return this._is_bool_pref_command_supported();
}
EditorItemsController.prototype.is_cmd_fontFixed_enabled = function() {
    return this._is_bool_real_pref_command_enabled("editUseFixedFont");
}
EditorItemsController.prototype.do_cmd_fontFixed = function() {
    var existing = this._viewManager.getCurrentView().prefs.getBooleanPref("editUseFixedFont")
    this._do_bool_pref_command("editUseFixedFont", existing);
}

EditorItemsController.prototype.is_cmd_viewWhitespace_supported = function() {
    return this._is_bool_pref_command_supported();
}
EditorItemsController.prototype.is_cmd_viewWhitespace_enabled = function() {
    return this._is_bool_pref_command_enabled("viewWS");
}
EditorItemsController.prototype.do_cmd_viewWhitespace = function() {
    var v = this._viewManager.getCurrentView();
    this._do_bool_pref_command("showWhitespace", v._scimoz.viewWS);
}

EditorItemsController.prototype.is_cmd_viewEOL_supported = function() {
    return this._is_bool_pref_command_supported();
}
EditorItemsController.prototype.is_cmd_viewEOL_enabled = function() {
    return this._is_bool_pref_command_enabled("viewEOL");
}
EditorItemsController.prototype.do_cmd_viewEOL = function() {
    var v = this._viewManager.getCurrentView();
    this._do_bool_pref_command("showEOL", v._scimoz.viewEOL);
}

EditorItemsController.prototype.is_cmd_viewIndentationGuides_supported = function() {
    return this._is_bool_pref_command_supported();
}
EditorItemsController.prototype.is_cmd_viewIndentationGuides_enabled = function() {
    var v = this._viewManager.getCurrentView();
    return v && v._scimoz.indentationGuides;
}
EditorItemsController.prototype.do_cmd_viewIndentationGuides = function() {
    var v = this._viewManager.getCurrentView();
    v._scimoz.indentationGuides = !v._scimoz.indentationGuides;
}

EditorItemsController.prototype.is_cmd_viewLineNumbers_supported = function() {
    return this._is_bool_pref_command_supported();
}
EditorItemsController.prototype.is_cmd_viewLineNumbers_enabled = function() {
    var v = this._viewManager.getCurrentView();
    return v && v._scimoz.getMarginWidthN(0) > 0;
}
EditorItemsController.prototype.do_cmd_viewLineNumbers = function() {
    var v = this._viewManager.getCurrentView();
    this._do_bool_pref_command("showLineNumbers", v._scimoz.getMarginWidthN(0) > 0);
}

EditorItemsController.prototype.is_cmd_bookmarkToggle_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
EditorItemsController.prototype.do_cmd_bookmarkToggle = function() {
    var v = this._viewManager.getCurrentView();
    var line_no = v._scimoz.lineFromPosition(v._scimoz.selectionStart);
    var markerState = v._scimoz.markerGet(line_no);
    if (markerState & (1 << mo.bm)) {
        v._scimoz.markerDelete(line_no, mo.bm);
    } else {
        v._scimoz.markerAdd(line_no, mo.bm);
    }
}

EditorItemsController.prototype.is_cmd_bookmarkRemoveAll_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
EditorItemsController.prototype.do_cmd_bookmarkRemoveAll = function() {
    var v = this._viewManager.getCurrentView();
    v._scimoz.markerDeleteAll(mo.bm);
}

EditorItemsController.prototype.is_cmd_bookmarkGotoNext_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
EditorItemsController.prototype.do_cmd_bookmarkGotoNext = function() {
    var v = this._viewManager.getCurrentView();
    var thisLine = v._scimoz.lineFromPosition(v._scimoz.selectionStart);
    marker_mask = 1 << mo.bm;
    var nextLine = v._scimoz.markerNext(thisLine+1, marker_mask);
    if (nextLine < 0) {
        // try for search from top of file.
        nextLine = v._scimoz.markerNext(0, marker_mask);
    }
    if (nextLine < 0 || nextLine == thisLine) {
        StatusBar_AddMessage("No next bookmark can be found.", "bookmark",
                             3000, true);
    } else {
        v._scimoz.ensureVisibleEnforcePolicy(nextLine);
        v._scimoz.gotoLine(nextLine);
    }
}

EditorItemsController.prototype.is_cmd_bookmarkGotoPrevious_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
EditorItemsController.prototype.do_cmd_bookmarkGotoPrevious = function() {
    var v = this._viewManager.getCurrentView();
    var thisLine = v._scimoz.lineFromPosition(v._scimoz.selectionStart);
    marker_mask = 1 << mo.bm;
    var prevLine = v._scimoz.markerPrevious(thisLine-1, marker_mask);
    if (prevLine < 0) {
        // try for search from bottom of file.
        prevLine = v._scimoz.markerPrevious(v._scimoz.lineCount-1, marker_mask);
    }
    if (prevLine < 0 || prevLine == thisLine) {
        StatusBar_AddMessage("No previous bookmark can be found.", "bookmark",
                             3000, true);
    } else {
        v._scimoz.ensureVisibleEnforcePolicy(prevLine);
        v._scimoz.gotoLine(prevLine);
    }
}

EditorItemsController.prototype.is_cmd_gotoLine_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
EditorItemsController.prototype.do_cmd_gotoLine = function() {
    doGotoLine();
}

EditorItemsController.prototype._is_cmd_folding_enabled = function() {
    var cv = this._viewManager.getCurrentView();
    return cv && cv.py_inst.doingFolding();
}
EditorItemsController.prototype.is_cmd_foldExpand_enabled = EditorItemsController.prototype._is_cmd_folding_enabled;
EditorItemsController.prototype.do_cmd_foldExpand = function() {
    var v = this._viewManager.getCurrentView();
    var lineno = v._scimoz.lineFromPosition(v._scimoz.currentPos);
    if ((v._scimoz.getFoldLevel(lineno) & v._scimoz.SC_FOLDLEVELHEADERFLAG) &&
        !v._scimoz.getFoldExpanded(lineno))
    {
        v._scimoz.toggleFold(lineno)
    }
}
EditorItemsController.prototype.is_cmd_foldExpandAll_enabled = EditorItemsController.prototype._is_cmd_folding_enabled;
EditorItemsController.prototype.do_cmd_foldExpandAll = function() {
    var v = this._viewManager.getCurrentView();
    for (var lineno=0;lineno < v._scimoz.lineCount;lineno++) {
        if ((v._scimoz.getFoldLevel(lineno) & v._scimoz.SC_FOLDLEVELHEADERFLAG) &&
            !v._scimoz.getFoldExpanded(lineno))
        {
                v._scimoz.toggleFold(lineno)
        }
    }
}
EditorItemsController.prototype.is_cmd_foldCollapse_enabled = EditorItemsController.prototype._is_cmd_folding_enabled;
EditorItemsController.prototype.do_cmd_foldCollapse = function() {
    var v = this._viewManager.getCurrentView();
    var lineno = v._scimoz.lineFromPosition(v._scimoz.currentPos);
    if ((v._scimoz.getFoldLevel(lineno) & v._scimoz.SC_FOLDLEVELHEADERFLAG) &&
        v._scimoz.getFoldExpanded(lineno))
    {
        v._scimoz.toggleFold(lineno)
    }
}
EditorItemsController.prototype.is_cmd_foldCollapseAll_enabled = EditorItemsController.prototype._is_cmd_folding_enabled;
EditorItemsController.prototype.do_cmd_foldCollapseAll = function() {
    var v = this._viewManager.getCurrentView();
    v._scimoz.colourise(0,v._scimoz.textLength-1);
    for (var lineno=0;lineno<v._scimoz.lineCount;lineno++) {
        if ((v._scimoz.getFoldLevel(lineno) & v._scimoz.SC_FOLDLEVELHEADERFLAG) &&
            v._scimoz.getFoldExpanded(lineno))
        {
            v._scimoz.toggleFold(lineno)
        }
    }
}

// Auxiliary function used by findNextFunction, findPreviousFunction, and
// findAllFunctions. The searchType argument has the following meaning: 0 =
// search forward, 1 = search backwards, 2 = find all
EditorItemsController.prototype._findFunction = function(searchType) { 
    try {
        var view = this._viewManager.getCurrentView();
        var language = view.py_inst.sourceFile.getLanguageBlockAtPosition(0).language;
        var re = language.namedBlockRE;
        var namedBlockDescription = language.namedBlockDescription;
        if (re == null || re == '') return;

        if (findSvc == null) {
            findSvc = Components.classes["@activestate.com/koFindService;1"].
                      getService(Components.interfaces.koIFindService);
        }
        var patternType, caseSensitivity, searchBackward, matchWord, ignoreHiddenText;
        patternType = findSvc.options.patternType;
        caseSensitivity = findSvc.options.caseSensitivity;
        searchBackward = findSvc.options.searchBackward;
        matchWord = findSvc.options.matchWord;
        ignoreHiddenText= findSvc.options.ignoreHiddenText;
        
        findSvc.options.patternType = findSvc.options.FOT_REGEX_PYTHON;
        findSvc.options.caseSensitivity = findSvc.options.FOT_SENSITIVE;
        if (searchType == 1) {
            findSvc.options.searchBackward = 1;
        } else {
            findSvc.options.searchBackward = 0;
        }
        findSvc.options.matchWord = 0;
        findSvc.options.ignoreHiddenText = 0;
 
        var context = Components.classes["@activestate.com/koFindContext;1"]
                      .createInstance(Components.interfaces.koIFindContext);
        context.type = Components.interfaces.koIFindContext.FCT_CURRENT_DOC;
        context.name = 'the current document';
        parentWindow = window;
        if (searchType == 2) {
            Find_FindAll(window, context, re, namedBlockDescription);
        } else {
            Find_FindNext(window, context, re);
        }
        findSvc.options.patternType = patternType ;
        findSvc.options.caseSensitivity = caseSensitivity ;
        findSvc.options.searchBackward = searchBackward ;
        findSvc.options.matchWord = matchWord ;
        findSvc.options.ignoreHiddenText = ignoreHiddenText ;
 
    }  catch (e) {
        logError(e);
    }
}

EditorItemsController.prototype.is_cmd_findNextFunction_enabled = function() {
    view = this._viewManager.getCurrentView()
    return view != null &&  view.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.namedBlockRE != '';
}

EditorItemsController.prototype.do_cmd_findNextFunction = function() {
    this._findFunction(0);
}

EditorItemsController.prototype.is_cmd_findPreviousFunction_enabled = function() {
    view = this._viewManager.getCurrentView()
    return view != null &&  view.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.namedBlockRE != '';
}

EditorItemsController.prototype.do_cmd_findPreviousFunction= function() {
    this._findFunction(1);
}

EditorItemsController.prototype.is_cmd_findAllFunctions_enabled = function() {
    view = this._viewManager.getCurrentView()
    return view != null &&  view.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.namedBlockRE != '';
}

EditorItemsController.prototype.do_cmd_findAllFunctions= function() {
    this._findFunction(2);
}

EditorItemsController.prototype.is_cmd_startMacroMode_enabled = function() {
    return (this._viewManager.getCurrentView() && !gMacroMgr.recording);
}

EditorItemsController.prototype.do_cmd_startMacroMode= function() {
    gMacroMgr.startRecording();
}

EditorItemsController.prototype.is_cmd_stopMacroMode_enabled = function() {
    return gMacroMgr.recording;
}

EditorItemsController.prototype.do_cmd_stopMacroMode= function() {
    gMacroMgr.stopRecording();
}

EditorItemsController.prototype.is_cmd_executeLastMacro_enabled = function() {
    return this._viewManager.getCurrentView() != null && ! gMacroMgr.recording && gMacroMgr._currentMacro != null;
}

EditorItemsController.prototype.do_cmd_executeLastMacro = function() {
    gMacroMgr.executeLastMacro()
}

EditorItemsController.prototype.is_cmd_saveMacroToToolbox_enabled = function() {
    return this._viewManager.getCurrentView() != null && ! gMacroMgr.recording && gMacroMgr._currentMacro != null;
}

EditorItemsController.prototype.do_cmd_saveMacroToToolbox = function() {
    gMacroMgr.saveToToolbox();
}

EditorItemsController.prototype.is_cmd_startIncrementalSearch_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}

EditorItemsController.prototype.do_cmd_startIncrementalSearch= function() {
    this._startIncrementalSearch(false);
}

EditorItemsController.prototype.is_cmd_startIncrementalSearchBackwards_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}

EditorItemsController.prototype.do_cmd_startIncrementalSearchBackwards= function() {
    this._startIncrementalSearch(true);
}

function key_event_handler_for_isearch(e) {
    gLastFocusedViewManager.editorItemsController.keyPressForSearch(e);
}

function mouse_event_handler_for_isearch(e) {
    // If called from <scintilla>, 'this' will be the XBL widget.  Weird JS.
    if (this.key_handler) {
        this.key_handler = null;
    }
    if (this.mouse_handler) {
        this.mouse_handler = null;
    }
    this.removeEventListener('blur', mouse_event_handler_for_isearch, false);

    gLastFocusedViewManager.editorItemsController._stopIncrementalSearch("Clicked away", false);
}

EditorItemsController.prototype._startIncrementalSearch = function(backwards) {
    var scintilla = getFocusedScintilla();
    if (!scintilla) { 
        logError("Couldn't start incremental search with no focused scintilla");
        return;
    }
    var scimoz = scintilla.ISciMoz;
    StatusBar_AddMessage("Incremental Search:", "isearch", 0, false)
    scintilla.key_handler = key_event_handler_for_isearch;
    scintilla.mouse_handler = mouse_event_handler_for_isearch;
    scintilla.addEventListener('blur', mouse_event_handler_for_isearch, false);
    this._incrementalSearchStartPos = Math.min(scimoz.currentPos, scimoz.anchor);
    var pattern = scimoz.selText;
    this._incrementalSearchContext = Components.classes["@activestate.com/koFindContext;1"]
        .createInstance(Components.interfaces.koIFindContext);
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }

    // Save original find settings 
    this._origFindOptions = new Object();
    this._origFindOptions.searchBackward = findSvc.options.searchBackward;
    this._origFindOptions.matchWord= findSvc.options.matchWord;
    this._origFindOptions.patternType = findSvc.options.patternType;
    this._origFindOptions.caseSensitivity = findSvc.options.caseSensitivity;

    findSvc.options.searchBackward = backwards;
    findSvc.options.matchWord = false;
    findSvc.options.patternType = Number(globalPrefs.getStringPref('isearchType'));
    findSvc.options.caseSensitivity = Number(globalPrefs.getStringPref('isearchCaseSensitivity'));
    this._incrementalSearchContext.type = findSvc.options.FCT_CURRENT_DOC;
    this._incrementalSearchPattern = pattern;
    if (pattern) {
        // we don't want to find the current selection, dummy.
        scimoz.currentPos = this._incrementalSearchStartPos + pattern.length;  
        this._lastIncrementalSearchText = pattern;
        StatusBar_AddMessage("Incremental Search: " + pattern, "isearch", 0, false)
        Find_FindNext(window, this._incrementalSearchContext,
                      pattern, undefined, true);
    }
}

EditorItemsController.prototype._stopIncrementalSearch = function(why, highlight) {
    StatusBar_AddMessage("Incremental Search Stopped: " + why, "isearch", 3000, highlight)
    if (this._origFindOptions) {
        // Restore original find settings 
        findSvc.options.searchBackward = this._origFindOptions.searchBackward;
        findSvc.options.matchWord = this._origFindOptions.matchWord;
        findSvc.options.patternType = this._origFindOptions.patternType;
        findSvc.options.caseSensitivity = this._origFindOptions.caseSensitivity;
        this._origFindOptions = null;
    }
    this._incrementalSearchPattern = '';
}
    
function DoTheScimozWiggle(scimoz) {
    // this is the wiggle needed to set lastX
    var c,a;
    c = scimoz.currentPos;
    a = scimoz.anchor;
    scimoz.currentPos = c;
    scimoz.anchor = c;
    if (scimoz.currentPos > 0) {
        scimoz.charLeft()
        scimoz.charRight()
    } else {
        scimoz.charRight()
        scimoz.charLeft()
    }
    scimoz.currentPos = c;
    scimoz.anchor = a;
}

EditorItemsController.prototype.is_cmd_rawKey_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}

EditorItemsController.prototype.do_cmd_rawKey= function() {
    var scintilla = gLastFocusedViewManager.getCurrentView()._sci_widget;
    scintilla.key_handler = editorItemsController.rawHandler;
    scintilla.addEventListener('blur', gCancelRawHandler, false);
    scintilla.ISciMoz.focus = true;
    StatusBar_AddMessage("Enter Control Character:", "raw_input", 0, false)
}

function gCancelRawHandler(event) {
    if (this.key_handler) {
        this.key_handler = null;
    }
    StatusBar_AddMessage(null, "raw_input", 0, false)
}

EditorItemsController.prototype.rawHandler= function(event) {
    try {
        var scintilla = gLastFocusedViewManager.getCurrentView()._sci_widget;
        scintilla.key_handler = null;
        var scimoz = scintilla.ISciMoz;
        event.cancelBubble = true;
        event.preventDefault();
        if (event.ctrlKey) {
            // Need to convert from charCode to ASCII value
            scimoz.replaceSel(String.fromCharCode(event.charCode-96));
        } else {
            if (event.charCode != 0) {  // not sure why space doesn't work.
                scimoz.replaceSel(String.fromCharCode(event.charCode));
            } else {
                switch (event.keyCode) {
                    case 27: // VK_ESCAPE
                    case 13: // VK_ENTER
                    case 9:  // VK_TAB
                    case 8:  // VK_BACKSPACE
                        scimoz.replaceSel(String.fromCharCode(event.keyCode));
                        break;
                    default:
                        // do nothing
                }
            }
        }
        StatusBar_AddMessage(null, "raw_input", 0, false)
    } catch (e) {
        logError(e);
    }
}

EditorItemsController.prototype.keyPressForSearch = function(event) {
    try {
        var v = gLastFocusedViewManager.getCurrentView();
        var scintilla = v._sci_widget
        var scimoz = v._scimoz
        var pos = scimoz.selectionStart < scimoz.selectionEnd ? scimoz.selectionStart : scimoz.selectionEnd;
        var eventkey = gKeybindingMgr.event2keylabel(event);
        var triggerkeys = gKeybindingMgr.command2keysequences('cmd_startIncrementalSearch');
        var backwardstriggerkeys = gKeybindingMgr.command2keysequences('cmd_startIncrementalSearchBackwards');
        // If the key sequence associated with incremental search is more than one key long,
        // - If the last key is a Ctrl or Alt key, use it.
        // - otherwise, disable the 'multiple hits' feature
        var triggerkey;
        if (triggerkeys) {
            triggerkey = triggerkeys[triggerkeys.length-1];
        }
        var backwardstriggerkey = '';
        if (backwardstriggerkeys) {
            backwardstriggerkey = backwardstriggerkeys[backwardstriggerkeys.length-1];
        }
        if ((triggerkeys.length == 1 ||
             triggerkey.indexOf('Ctrl') != -1 ||
             triggerkey.indexOf('Alt') != -1) &&
             (eventkey == triggerkey || eventkey == backwardstriggerkey)) {
                // we're hitting the same key as was used to start the search,
                // so we just want to redo the search
                event.cancelBubble = true;
                if (this._lastIncrementalSearchText == '') return;
                if (this._incrementalSearchPattern == "") {  // Second ctrl-I after a non-search action
                    this._incrementalSearchPattern = this._lastIncrementalSearchText;
                    gFindSession.Reset();
                }
                if (eventkey == backwardstriggerkey) {
                    findSvc.options.searchBackward = true;
                } else {
                    findSvc.options.searchBackward = false;
                }
                var findres = Find_FindNext(window, this._incrementalSearchContext,
                      this._incrementalSearchPattern, undefined, true);
                if (findres == false) {
                    StatusBar_AddMessage("Incremental Search: No more occurences of " + this._incrementalSearchPattern +' found', "isearch", 0, false)
                } else {
                    this._incrementalSearchStartPos = scimoz.currentPos;
                    StatusBar_AddMessage("Incremental Search: " + this._incrementalSearchPattern, "isearch", 0, false)
                }
                return;
            }
        var isShiftLeftKey = event.shiftKey && event.keyCode == VK_LEFT;
        var isShiftRightKey = event.shiftKey && event.keyCode == VK_RIGHT;
        var isBS = event.keyCode == VK_BACKSPACE;
        var isRegularKey = event.keyCode == 0;
        if (!event.ctrlKey && !event.altKey && ( isRegularKey || isShiftRightKey || isShiftLeftKey || isBS )) {
            var pattern = this._incrementalSearchPattern;
            event.cancelBubble = true;
            event.preventDefault();
            if (isShiftRightKey) {
                var nextChar = scimoz.getWCharAt(scimoz.currentPos);
                pattern = pattern + nextChar;
                event.stopPropagation();  // We don't want normal shift-arrow behavior to occur
            } else if (isShiftLeftKey || isBS) {
                if (pattern == '') return; // nothing left to backtrack
                pattern = pattern.substr(0, pattern.length-1);
                event.stopPropagation();  // We don't want normal shift-arrow behavior to occur
            } else {
                pattern = pattern + String.fromCharCode(event.charCode);
            }
            this._lastIncrementalSearchText = pattern;
            this._incrementalSearchPattern = pattern;
            var oldStart = scimoz.selectionStart;
            var oldEnd = scimoz.selectionEnd;
            scimoz.selectionStart = scimoz.selectionEnd = this._incrementalSearchStartPos-1;
            if (this._incrementalSearchPattern == '') {
                StatusBar_AddMessage("Interactive Search: |", "isearch", 0, false)
                return;
            }
            gMacroMgr.undo();
            findres = Find_FindNext(window, this._incrementalSearchContext, this._incrementalSearchPattern, undefined, true);
            if (! findres) {
                StatusBar_AddMessage("Interactive Search: No occurences of " + this._incrementalSearchPattern + " found.", "isearch", 0, true);
                scimoz.selectionStart = oldStart;
                scimoz.selectionEnd = oldEnd;
            } else {
                StatusBar_AddMessage("Interactive Search: " + this._incrementalSearchPattern +'|', "isearch", 0, false)
            }
            return;
        } else {
            scintilla.key_handler = null;
            scintilla.mouse_handler = null;
            scintilla.removeEventListener('blur', mouse_event_handler_for_isearch, false);
            // Anything, not just escape, cancels incremental search
            this._stopIncrementalSearch("Search canceled.");
            if (event.keyCode == 27) {  // but most things propagate -- escape doesn't.
                event.cancelBubble = true;
            }
            return;
        }
    } catch (e) {
        logError(e);
    }
}

// Controller object for Dir Pane
function FileItemsController(viewManager) {
    this._viewManager = viewManager
    this._bufferSwitchingObserver = new _BufferSwitchingObserver(viewManager,
                                                                 this);
    this._observingCtrlKeyUp = false;
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
FileItemsController.prototype = new Controller();
FileItemsController.prototype.constructor = FileItemsController;

FileItemsController.prototype.destructor = function(command) {
    this._viewManager = null;
}

FileItemsController.prototype.supportsCommand = function(command) {
    // Note that we always say we support the MRU command, even
    // if no entry - "doCommand" will fail.
    if (command.indexOf("cmd_mruFile") == 0) {
        return true;
    }

    return Controller.prototype.supportsCommand.apply(this, [command]);
}

FileItemsController.prototype.isCommandEnabled = function(command) {
    if (command.indexOf("cmd_mruFile") == 0) {
        return true;
    }

    return Controller.prototype.isCommandEnabled.apply(this, [command]);
}

FileItemsController.prototype.doCommand = function(command) {
    var groups = command.match(/cmd_mruFile([0-9]+)/);
    if (groups && groups[1]) {
        var url = MRU_get("mruFileList", Number(groups[1]));
        doFileOpen(url);
    }

    Controller.prototype.doCommand.apply(this, [command]);
}

FileItemsController.prototype.is_cmd_open_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_open = function() {
    var fp = dlg_openFile();
    if (fp == null) return;
    var file = new dlg_get_URL_andName(fp);
    doOpenPath(file.url);
}
FileItemsController.prototype.is_cmd_open_remote_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_open_remote = function() {
    doOpenRemoteFiles(); // in fileops.js
}

FileItemsController.prototype.is_cmd_open_template_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_open_template = function() {
    var fp = dlg_openTemplate(); // in fileops.js
    if (fp == null) return;
    var file = new dlg_get_URL_andName(fp);
    doOpenPath(file.url);
}

FileItemsController.prototype.is_cmd_openURL_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_openURL = function() {
    var url;
    url = doOpenURL("Enter a URL for the file you wish to open:");
    if (url) {
        doOpenPath(url);
    }
}

FileItemsController.prototype.is_cmd_print_enabled = function() {
    var view = this._viewManager.getCurrentView();
    return view == null ? false : true;
}
FileItemsController.prototype.do_cmd_print = function() {
    return doPrint();
}

FileItemsController.prototype.is_cmd_exportHTML_enabled = function() {
    var view = this._viewManager.getCurrentView();
    return view == null ? false : true;
}
FileItemsController.prototype.do_cmd_exportHTML = function() {
    return doExportHTML();
}

FileItemsController.prototype.is_cmd_revert_enabled = function() {
    // Enabled if save is enabled and the buffer is not untitled
    var v = this._viewManager.getCurrentView();
    if (v) {
        return !v.untitled && this.is_cmd_save_enabled();
    } else {
        return false;
    }
}
FileItemsController.prototype.do_cmd_revert = function() {
    var v = this._viewManager.getCurrentView();

    // assert(v && v.url && v.isDirty);
    if (!(v && v.url && v.isDirty())) {
        throw Components.results.NS_ERROR_UNEXPECTED;
    }
    
    if ("Yes" == yesNoDialog("Okay to lose changes to " +
                             v.getLeafName(), "Yes"))
    {
        v.revert();
    }
}

FileItemsController.prototype.is_cmd_save_enabled = function() {
    var view = this._viewManager.getCurrentView();
    return (view != null) && view.isDirty();
}
FileItemsController.prototype.do_cmd_save = function() {
    var view = this._viewManager.getCurrentView();
    if (view == null) {
        logError("this._viewManager.getCurrentView() == null!");
    }
    view.doSave();
}

FileItemsController.prototype.is_cmd_saveAll_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_saveAll = function() {
    var i;
    // Save all dirty views
    this._viewManager.doSaveAll();

    // Save all dirty projects
    var projects = gProjectManager.getDirtyProjectIDs();
    for (i = 0; i < projects.length; i++) {
        gProjectManager.saveProject(projects[i]);
    }

    // Save the toolbox data
    try {
        toolboxMgr.toolbox.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alertDialog('There was an error saving the toolbox: ' +
                    lastErrorSvc.getLastErrorMessage());
    }

    // Save the dialogs
    var guiBuilderSvc = Components.classes["@activestate.com/koGuiBuilderService;1"].
            getService(Components.interfaces.koIGuiBuilderService);
    var dirtyDialogs = guiBuilderSvc.getDirtyURLs(new Object())
    for (i = 0; i < dirtyDialogs.length; i++) {
        guiBuilderSvc.Save(dirtyDialogs[i]);
    }

    // Save the prefs
    globalPrefService.saveState();
}

FileItemsController.prototype.is_cmd_saveAs_enabled = function() {
    view = this._viewManager.getCurrentView();
    return view != null;
}
FileItemsController.prototype.do_cmd_saveAs = function() {
    var view = this._viewManager.getCurrentView();
    if (view == null) {
        logError("this._viewManager.getCurrentView() == null!");
    }
    return view.doSaveAs();
}
FileItemsController.prototype.is_cmd_saveAs_remote_enabled = function() {
    view = this._viewManager.getCurrentView();
    return view != null;
}
FileItemsController.prototype.do_cmd_saveAs_remote = function() {
    var view = this._viewManager.getCurrentView();
    if (view == null) {
        logError("this._viewManager.getCurrentView() == null!");
    }
    return view.doRemoteSaveAs();
}

FileItemsController.prototype.is_cmd_saveAsTemplate_enabled = function() {
    view = this._viewManager.getCurrentView();
    return view != null;
}
FileItemsController.prototype.do_cmd_saveAsTemplate = function() {
    var view = this._viewManager.getCurrentView();
    if (view == null) {
        logError("this._viewManager.getCurrentView() == null!");
    }
    return view.doSaveAsTemplate();
}

FileItemsController.prototype.is_cmd_closeKombo_enabled = function() {
    return true;
}
FileItemsController.prototype.do_cmd_closeKombo = function() {
    doFileQuit();
}

FileItemsController.prototype.is_cmd_closeAll_enabled = function() {
    return this._viewManager.getCurrentView() != null;
}
FileItemsController.prototype.do_cmd_closeAll = function(doDirect) {
    return this._viewManager.doFileCloseAll();
}

// Buffer Switching:
// - bufferNext and bufferPrevious:
//   These will switch to the next and previous tabs in the editor's tabbox
//   widget. They are used in the "Windows -> {Next|Previous} File" menu
//   entries.
// - bufferNextMostRecent and bufferNextLeastRecent:
//   These will switch to the recently visited tabs similar to Alt-Tab in
//   Windows (a stack of the visit order is maintained). They are used via the
//   Ctrl-Tab and Ctrl-Shift-Tab kye bindings. Special processing is added to
//   have the "recent views" stack only be updated after Ctrl is released. For
//   example, pressing Tab twice while holding down the Ctrl key should only
//   add the final switched-to file to the top of the recently-visited stack.
//

FileItemsController.prototype.is_cmd_bufferNext_enabled = function() {
    // only if the switch would actually change buffers.
    var v = this._viewManager.getNextView();
    if (v != null && this._viewManager.getCurrentView() != v) {
        return true;
    }
    return false;
}

FileItemsController.prototype.do_cmd_bufferNext = function() {
    var next = this._viewManager.getNextView()
    if (next==null) {
        logWarning("cmd_bufferNext is enabled, but can not find another buffer!\n");
    } else {
        next.manager.setCurrentView(next);

        // after processing this command, the tab transfers
        // focus on to the next widget.  This refocuses scin.
        setTimeout(function(mgr) {mgr.setFocus();}, 0, next.manager);
    }
}

FileItemsController.prototype.is_cmd_bufferPrevious_enabled = function() {
    // only if the switch would actually change buffers.
    var v = this._viewManager.getPreviousView();
    if (v != null && this._viewManager.getCurrentView() != v) {
        return true;
    }
    return false;
}

FileItemsController.prototype.do_cmd_bufferPrevious = function() {
    var prev = this._viewManager.getPreviousView()
    if(prev != null) {
        prev.manager.setCurrentView(prev);

        // after processing this command, the tab transfers
        // focus on to the next widget.  This refocuses scin.
        setTimeout(function(mgr) {mgr.setFocus();}, 0, prev.manager);
    }
}

function _BufferSwitchingObserver(viewMgr, fileItemsController)
{
    this._viewMgr = viewMgr;
    this._controller = fileItemsController;
}

_BufferSwitchingObserver.prototype.Observe = function(subject, topic, data)
{
    var vm = getFocusedViewManager();
    if (!vm) return;
    
    switch (topic) {
    case 'ctrl_keyup':
        var obSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
        obSvc.RemoveObserver(this._controller._bufferSwitchingObserver,
                             'ctrl_keyup');
        this._controller._observingCtrlKeyUp = false;
        viewManager.exitBufferSwitchingSession();
        viewManager.setMostRecentView(viewManager.getCurrentView());
        break;
    }
}

FileItemsController.prototype.is_cmd_bufferNextMostRecent_enabled = function() {
    return true;
}

FileItemsController.prototype.do_cmd_bufferNextMostRecent = function() {
    var v = viewManager.getNextMostRecentView();
    if (v != null) {
        viewManager.enterBufferSwitchingSession();
        var obSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
        if (! this._observingCtrlKeyUp) {
            obSvc.AddObserver(this._bufferSwitchingObserver, 'ctrl_keyup');
            this._observingCtrlKeyUp = true;
        }
        viewManager.setCurrentView(v);
        
        // after processing this command, the tab transfers
        // focus on to the next widget.  This refocuses scin.
        setTimeout(function(mgr) {mgr.setFocus();}, 0, v.manager);
    }
}


FileItemsController.prototype.is_cmd_bufferNextLeastRecent_enabled = function() {
    return true;
}

FileItemsController.prototype.do_cmd_bufferNextLeastRecent = function() {
    var v = viewManager.getNextLeastRecentView();
    if (v != null) {
        viewManager.enterBufferSwitchingSession();
        var obSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
        obSvc.AddObserver(this._bufferSwitchingObserver, 'ctrl_keyup');
        viewManager.setCurrentView(v);
        
        // after processing this command, the tab transfers
        // focus on to the next widget.  This refocuses scin.
        setTimeout(function(mgr) {mgr.setFocus();}, 0, v.manager);
    }
}


FileItemsController.prototype.is_cmd_bufferClose_enabled = function() {
    var view = this._viewManager.getCurrentView();
    return view == null ? false : true;
}
FileItemsController.prototype.do_cmd_bufferClose = function(doDirect) {
    var view = this._viewManager.getCurrentView();
    if (view == null) {
        logError("this._viewManager.getCurrentView() == null!");
    }
    var res = view.manager.closeView(view);
    return res;
}
FileItemsController.prototype.do_cmd_editPrefs = function(doDirect) {
    doGlobalPrefs(null);
}
FileItemsController.prototype.is_cmd_editPrefsCurrent_enabled = function () {
    return this._viewManager.getCurrentView() ? true : false;
}
FileItemsController.prototype.do_cmd_editPrefsCurrent = function () {
    doFilePrefs('settings');
}


////////////////////////////////////////////////////////////////

//---- UI visual state handling for debugger state changes
// Save/Update the visibility of komodo.xul widgets according to the
// editor/debugger state. The intention is to 
//  1. persist the user's lkayout for editting
//  2. always pop open the output pane when debugging starts

var _gDebuggerState = null;

function komboDbgEventist() {}
komboDbgEventist.prototype = {
    onStateChange : function(state, state_extra) {
        // If starting debugging then open the output pane.
        if (
             (
               _gDebuggerState == DBG_STATE_STOPPED ||
               _gDebuggerState == null
             )
             &&
             (
               state == DBG_STATE_BREAK ||
               state == DBG_STATE_RUNNING ||
               state == DBG_STATE_STARTING
             )
           )
        {
            _setSplitterState("redirect-splitter", 0);
        }
        _gDebuggerState = state;

//        // Ensure that a non-visible line is scrolled to on break (XXX:UI143)
//        var v = viewManager.getCurrentView();
//        if (v) {
//            // force re-draw (XXX this causes a segfault)
//            document.getElementById("scinbox")
//                    .setAttribute("style", "display: block;");  
//            v._ensureLineVisible(0); // 0 means "current"
//        }
    }
};

// _autoSaveTimer will notify the views that they need to save if the buffer is dirty
function _autoSaveTimer()
{
    var empty = new Object();
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
    getService(Components.interfaces.nsIObserverService);
    
    try {
        observerSvc.Notify(empty, "autosave", empty);
    } catch (e) {
        ;
    }
}

// _setAutoSaveTimer sets the interval at which we will autosave dirty buffers
var _autoSaveInterval = 0;
function _setAutoSaveTimer() {
    // set default timer
    var minutes = 0;
    if (globalPrefs.hasPref("autoSaveMinutes")) {
        minutes = globalPrefs.getLongPref("autoSaveMinutes");
    }
    // clear existing timer
    if (_autoSaveInterval) {
        window.clearInterval(_autoSaveInterval);
    }
    // set the new timer
    if (minutes > 0) {
        _autoSaveInterval = window.setInterval ( "_autoSaveTimer()", minutes*60000 ) 
    }
}

function _setSplitterState(id, collapsed)
{
    var splitter = document.getElementById(id);
    if (collapsed) {
        splitter.setAttribute('state', 'collapsed');
    } else {
        splitter.removeAttribute('state');
    }
}


function saveVisibilities()
{
    // Capture and persist the layout on exit of a mode.
    var splitter = document.getElementById("redirect-splitter");
    var collapsed = splitter.getAttribute("state") == "collapsed";
    globalPrefs.setBooleanPref("redirect-splitter-collapsed", collapsed);
    var outputPane = document.getElementById("output_area");
    globalPrefs.setLongPref("output-pane-height",
                            outputPane.getAttribute("height"));

    splitter = document.getElementById("projectviewer-splitter");
    collapsed = splitter.getAttribute("state") == "collapsed";
    globalPrefs.setBooleanPref("projectviewer-splitter-collapsed", collapsed);
    var projectPane = document.getElementById("projectviewbox");
    globalPrefs.setLongPref("project-pane-width",
                            projectPane.getAttribute("width"));

    splitter = document.getElementById("toolbox-splitter");
    collapsed = splitter.getAttribute("state") == "collapsed";
    globalPrefs.setBooleanPref("toolbox-splitter-collapsed", collapsed);
    var toolboxPane = document.getElementById("toolboxviewbox");
    globalPrefs.setLongPref("toolbox-pane-width",
                            toolboxPane.getAttribute("width"));
}


function restoreVisibilities()
{
    var menuitem;
    // restore GUI layout
    var outputPane = document.getElementById("output_area");
    outputPane.setAttribute("height",
        globalPrefs.getLongPref("output-pane-height"));
    var collapsed = globalPrefs.getBooleanPref("redirect-splitter-collapsed");
    _setSplitterState("redirect-splitter", collapsed);
    menuitem = document.getElementById('menu_viewOutput');
    if (collapsed) {
        menuitem.setAttribute('checked', 'false');
    } else {
        menuitem.setAttribute('checked', 'true');        
    }
    
    var projectPane = document.getElementById("projectviewbox");
    projectPane.setAttribute("width",
        globalPrefs.getLongPref("project-pane-width"));
    collapsed = globalPrefs.getBooleanPref("projectviewer-splitter-collapsed");
    _setSplitterState("projectviewer-splitter", collapsed);
    menuitem = document.getElementById('menu_viewProjects');
    if (collapsed) {
        menuitem.setAttribute('checked', 'false');
    } else {
        menuitem.setAttribute('checked', 'true');        
    }

    var toolboxPane = document.getElementById("toolboxviewbox");
    toolboxPane.setAttribute("width",
        globalPrefs.getLongPref("toolbox-pane-width"));
    collapsed = globalPrefs.getBooleanPref("toolbox-splitter-collapsed");
    _setSplitterState("toolbox-splitter", collapsed);
    menuitem = document.getElementById('menu_viewToolbox');
    if (collapsed) {
        menuitem.setAttribute('checked', 'false');
    } else {
        menuitem.setAttribute('checked', 'true');        
    }
}


var notified = new Notified();

function setupUI()  {
    setupWin();  // window co-ordinates
    // Buttons
    toggleButtons(true, true);  //koGlobalOverlay.xul
    //Toolbars
    var toggleValue;
    toggleValue = prefs.getBooleanPref('showStandardToolbar');
    toggleToolboxItem('editorToolbar', false, toggleValue);
    toggleValue = prefs.getBooleanPref('showDebugToolbar');
    toggleToolboxItem('debuggerToolbar', false, toggleValue);
    toggleValue = prefs.getBooleanPref('showSCCToolbar');
    toggleToolboxItem('sccToolbar', false, toggleValue);
}

function setupWin() {
    // First get the size of the primary screen.
    var screenmgr = Components.classes["@mozilla.org/gfx/screenmanager;1"].
                  getService(Components.interfaces.nsIScreenManager);
    var screen = screenmgr.primaryScreen;
    var screenLeft = new Object();
    var screenTop = new Object();
    var screenWidth = new Object();
    var screenHeight = new Object();
    screen.GetRect(screenLeft, screenTop, screenWidth, screenHeight);

    // Now the values from our preferences
    var komodoX = globalPrefs.getLongPref("screenX");
    var komodoY = globalPrefs.getLongPref("screenY");
    var komodoW = globalPrefs.getLongPref("outerWidth");
    var komodoH = globalPrefs.getLongPref("outerHeight");

    // Ensure everything is on screen.
    if (komodoX > screenWidth.value) {
        komodoX = 0;
    }

    if (komodoY > screenHeight.value) {
        komodoY = 0;
    }

    if ((komodoX + komodoW) > screenWidth.value) {
        komodoW = screenWidth.value - komodoX;
    }

    if ((komodoY + komodoH) > screenHeight.value) {
        komodoH = screenHeight.value - komodoY;
    }

    // Set the in-range values to the window.
    var workspaceWin = document.getElementById('komodo_main')
    workspaceWin.setAttribute('screenX', komodoX);
    workspaceWin.setAttribute('screenY', komodoY);
    workspaceWin.setAttribute('width', komodoW);
    workspaceWin.setAttribute('height', komodoH);
}

function scimoz_widget_init(sci_widget, margin)
{
    if (typeof(margin)=='undefined') margin = 0;
    sci_widget.init();
    var scimoz = sci_widget.ISciMoz;
    // Remember our Scintilla "scratch" document as a global - we
    // will re-select this when no other docs are available.
    sci_widget.scratch_document = scimoz.docPointer;
    scimoz.addRefDocument( sci_widget.scratch_document );

    // Don't let scintilla send us every single event it fires, these are the
    // only ones we care about.
    scimoz.modEventMask = SC_MOD_WANTED;

    // Set up markers here
    scimoz.markerDefine(mo.cl, scimoz.SC_MARK_SHORTARROW);
    scimoz.markerSetFore(mo.cl, RGB (0x8f, 0x00, 0x00)); 
    scimoz.markerSetBack(mo.cl, rgb.yellow); 

    scimoz.markerDefine(mo.clb, 32);
    scimoz.markerSetFore(mo.clb, RGB (0x8f, 0x00, 0x00)); // These should be prefified
    scimoz.markerSetBack(mo.clb, rgb.yellow); // These should be prefified

    // The working breakpoint
    scimoz.markerDefine(mo.wb, scimoz.SC_MARK_CIRCLE);
    scimoz.markerSetFore(mo.wb, RGB (0x8f, 0x00, 0x00));
    scimoz.markerSetBack(mo.wb, rgb.red);

    // Failed breakpoints (not at a breakable position)
    scimoz.markerDefine(mo.fb, scimoz.SC_MARK_CIRCLE);
    scimoz.markerSetFore(mo.fb, RGB (0x8f, 0x00, 0x00));
    scimoz.markerSetBack(mo.fb, rgb.green);

    // bookmarks
    scimoz.markerDefine(mo.bm, scimoz.SC_MARK_ARROWDOWN);
    scimoz.markerSetFore(mo.bm, RGB (0x00, 0x00, 0x00));
    scimoz.markerSetBack(mo.bm, RGB(0x00, 0xFF, 0xFF)); // cyan

    // And set up the 2 margins (Python still owns the folding one).
    scimoz.setMarginTypeN(0, scimoz.SC_MARGIN_NUMBER);

    // Set up global settings like cursor color and current background color.
    update_scintilla_global_pref(scimoz, 'cursorColor');
    update_scintilla_global_pref(scimoz, 'currentLineColor');
    update_scintilla_global_pref(scimoz, 'useCurrentLineColor');
    update_scintilla_global_pref(scimoz, 'selectBack');
    update_scintilla_global_pref(scimoz, 'selectFore');
    update_scintilla_global_pref(scimoz, 'useSelectFore');
    update_scintilla_global_pref(scimoz, 'edgeColor');
    update_scintilla_global_pref(scimoz, 'ySlop');
    scimoz.setXCaretPolicy(scimoz.CARET_SLOP, 75 ); // don't feel the need to make that a pref yet.

    if (!margin) return;
    scimoz.setMarginTypeN(2, scimoz.SC_MARGIN_SYMBOL);
    scimoz.setMarginMaskN(2, 0x1F);
    scimoz.setMarginSensitiveN(2, 1);
}

function hexColorToLong(hexstring) {
    // Converts a color of the form #ffaabb to a long to send to scintilla.
    try {
        var r, g, b;
        r = parseInt(hexstring.substring(1, 3), 16);
        g = parseInt(hexstring.substring(3, 5), 16);
        b = parseInt(hexstring.substring(5, 7), 16);
        return RGB(r, g, b);
    } catch (e) {
        logError(e);
        return null;
    }
}

function update_scintilla_global_pref(scimoz, prefName) {
    switch (prefName) {
    case 'currentLineColor':
        scimoz.caretLineBack= hexColorToLong(globalPrefs.getStringPref(prefName));
        break;
    case 'cursorColor':
        scimoz.caretFore = hexColorToLong(globalPrefs.getStringPref(prefName));
        break;
    case 'edgeColor':
        scimoz.edgeColour = hexColorToLong(globalPrefs.getStringPref(prefName));
        break;
    case 'selectBack':
        scimoz.setSelBack(1, hexColorToLong(globalPrefs.getStringPref('selectBack')));
        break;
    case 'useSelectFore':
    case 'selectFore':
        scimoz.setSelFore(globalPrefs.getBooleanPref('useSelectFore'),
                          hexColorToLong(globalPrefs.getStringPref('selectFore')));
        break;
    case 'useCurrentLineColor':
        scimoz.caretLineVisible = globalPrefs.getBooleanPref(prefName);
        break;
    case 'ySlop':
        var slop = globalPrefs.getLongPref(prefName)
        scimoz.setYCaretPolicy(scimoz.CARET_SLOP | scimoz.CARET_STRICT | scimoz.CARET_EVEN,slop);
        scimoz.setVisiblePolicy(scimoz.VISIBLE_SLOP | scimoz.VISIBLE_STRICT, slop);
        break;
    default:
        break;
    }
}

function SciMozUnload(id) {
    var sci_widget = document.getElementById(id);
    var scimoz = sci_widget.ISciMoz;
    try {
        scimoz.setCommandUpdateTarget(null);
        sci_widget.scimoz_eventist = null;
        if (scimoz.docPointer != sci_widget.scratch_document) {
            logError("*** EEEK - Kombo shutting down, but scratch "+
                     "pointer for [" + id +
                     "] isnt current  - if we die, you know why!!!\n");
        } else {
            scimoz.releaseDocument( sci_widget.scratch_document );
        }
    } catch (e) {
        logDebug("Scintilla ["+id+"] was gone before we unloaded!");
    }
    sci_widget.scratch_document = 0;
}


var debuggerItemsController;
var editorItemsController;
var fileItemsController;

function KomboOnLoad(pveventist) {
    // Get the preferences first, everything kind of depends on it
    globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
                        getService(Components.interfaces.koIPrefService);
    globalPrefs = globalPrefService.prefs;

    if (globalPrefs.hasPref("encodingEnvironment") &&
        globalPrefs.getBooleanPref("encodingEnvironment")) {
        var locale = _gInitSvc.getStartupLocale();
        var enc = locale.split('.')[1].toLowerCase();
        globalPrefs.setStringPref("encodingDefault",enc)
    }
    
    StatusBar_Init();

    editorExtensionUI = new EditorExtensionUI();
    initialize_lint(); // PERFX .11sec
    debugger_onLoad(); // editor-debugger.js

    globalPrefs.AddObserver(notified);

    // Grab style service to force early initialization.  This moves a big
    // hit to startup, as opposed to the load of the first file.
    var stylesvc = Components.classes["@activestate.com/koStyleManager;1"].
                   getService(Components.interfaces.koIStyleManager);

    // register all the languages we support
    // XXX we may want to move this into a preference
    // we were doing lots of xpcom calls just to get the access key
    // this change increases startup speed significantly, must see
    // about putting these into some preference/config.
    var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].
                           getService(Components.interfaces.koILanguageRegistryService)
    languageRegistry.registerLanguage("C++",'c');
    languageRegistry.registerLanguage("HTML",'m');
    languageRegistry.registerLanguage("JavaScript",'a');
    languageRegistry.registerLanguage("Perl",'p');  
    languageRegistry.registerLanguage("Python",'y');
    languageRegistry.registerLanguage("Tcl",'t');
    languageRegistry.registerLanguage("Text",'e');
    languageRegistry.registerLanguage("XML",'x');
    languageRegistry.registerLanguage("XSLT",'s');
    languageRegistry.registerLanguage("Pascal",'');
    languageRegistry.registerLanguage("PHP",'h');
    languageRegistry.registerLanguage("Java",'j');
    languageRegistry.registerLanguage("C#",'#');
    languageRegistry.registerLanguage("Lua",'u');
    languageRegistry.registerLanguage("Makefile",'k');
    languageRegistry.registerLanguage("IDL",'i');
    languageRegistry.registerLanguage("Diff",'d');
    languageRegistry.registerLanguage("Batch",'b');
    languageRegistry.registerLanguage("LaTeX",'l');
    languageRegistry.registerLanguage("Lisp",'');
    languageRegistry.registerLanguage("Ada",'D');
    languageRegistry.registerLanguage("Ruby",'');
    languageRegistry.registerLanguage("VisualBasic",'v');
    languageRegistry.registerLanguage("SQL",'q');
    languageRegistry.registerLanguage("Eiffel",'f');
    languageRegistry.registerLanguage("Baan",'');
    languageRegistry.registerLanguage("nnCrontab",'');
    languageRegistry.registerLanguage("Matlab",'');
    languageRegistry.registerLanguage("Bullant",'');

    /* build the language-dependent menus */
    var count = new Object();
    var temp = new Object();
    languageRegistry.getLanguages(temp,count);
    var languages = temp.value;

    languageRegistry.getAccessKeys(temp, count);
    var accesskeys = temp.value;

    var broadcasterFileNew = document.getElementById("komodoFileNewItems");
    var viewAsMenu = document.getElementById("popup_viewAsLanguage");
    var commandset = document.getElementById("komodoViewAsLanguageItems");
    var defaultLanguage = null;  // get from prefs
    if (globalPrefs.hasPref("fileDefaultNew")) {
        defaultLanguage = globalPrefs.getStringPref("fileDefaultNew");
    }

    for (var i = 0; i < languages.length; i++) {
        var languageName = languages[i];
        var cmd = document.createElementNS(XUL_NS, 'broadcaster');
        cmd.setAttribute("id", "cmd_new"+languageName);
        cmd.setAttribute("oncommand", "doFileNew('" + languageName + "');");
        broadcasterFileNew.appendChild(cmd);

        // now generate the View As Language menu
        cmd = document.createElementNS(XUL_NS, 'broadcaster');
        cmd.setAttribute("id", "cmd_viewAs"+languageName);
        cmd.setAttribute("oncommand", "doViewAs('" + languageName + "');");
        commandset.appendChild(cmd);
        var menuitem3 = document.createElementNS(XUL_NS, 'menuitem');
        menuitem3.setAttribute("id", "menu_viewAs" + languageName);
        menuitem3.setAttribute("accesskey", accesskeys[i]);
        menuitem3.setAttribute("label", languageName);
        menuitem3.setAttribute("observes", "cmd_viewAs"+languageName);
        viewAsMenu.appendChild(menuitem3);
    }

    document.oldpersist = document.persist; // XXX This looks like cruft.

    setupUI();

    // Most SciMoz configuration is done as we toggle visiblity - all we do
    // now is hook the events.

    var sci_widget = document.getElementById("scin");
    scimoz_widget_init(sci_widget, 1);

    /* XXX split editor support, see scintillaview.js for details
    var sci_widget_split = document.getElementById("scin-split");
    scimoz_widget_init(sci_widget_split, 1);
    viewManager = new EditorViewManager('tabbox', sci_widget, sci_widget_split);
    */
    viewManager = new EditorViewManager('tabbox', sci_widget);

    gLastFocusedViewManager = viewManager;
    sci_widget.fillInTooltip = FillInKomboTooltip;

    var input_sci_widget = document.getElementById("input-scin");
    scimoz_widget_init(input_sci_widget, 1);

    var redirect_sci_widget = document.getElementById("redirect-scin");
    scimoz_widget_init(redirect_sci_widget);
    outputViewManager = new ScintillaViewManager('Output Buffer',
                                                 redirect_sci_widget);
    SCCOnLoad(); // peSCC.js

    var sci_work = document.getElementById('scin-work');
    scimoz_widget_init(sci_work);

    // Attach our various command controllers
    debuggerItemsController = new DebuggerItemsController();
    window.controllers.insertControllerAt(0, debuggerItemsController);
    editorItemsController = new EditorItemsController(viewManager);
    viewManager.editorItemsController = editorItemsController;
    window.controllers.insertControllerAt(0, editorItemsController);
    fileItemsController = new FileItemsController(viewManager);
    window.controllers.insertControllerAt(0, fileItemsController);

    // Now all the controllers are in place we can hide scintilla, which
    // triggers command update events.
    viewManager._hideScintilla();

    // Send all our events to init everything correctly.
    // buffer-visible and buffer-focus already sent by "hideScintilla"
    // debugger commands also listen to "buffer-visible", so "debug" need not be sent.
    koRequestUpdateCommands("select");
    koRequestUpdateCommands("clipboard");
    koRequestUpdateCommands("undo");
    koRequestUpdateCommands("find");
    koRequestUpdateCommands("save");
    koRequestUpdateCommands("macro");
    
    var myDebugger = Components.classes["@activestate.com/debugger;1"].
                     getService(Components.interfaces.koIDebugger);
    var dbgEventist = new komboDbgEventist() 
    myDebugger.addListener( dbgEventist );

    inputPaneOnLoad();

    // The stack frame. 
    stackOnLoad(); // PERFX .2sec

    // set the autosave timer
    _setAutoSaveTimer();

    restoreVisibilities();

    // start the help observer
    helpObserverSvc = new HelpObserver();
}

function KomboOnUnload() {
    helpObserverSvc.destroy();
    
    saveVisibilities();

    // The stack frame.
    stackOnUnload();

    if (lintManager) {
        // free the lint manager
        lintManager.terminate();
        lintManager = null;
    }

    //RxOnUnload();
    viewManager.destructor();
    outputViewManager.destructor();

    // Unhook events to prevent a cycle.
    SciMozUnload("scin");
    SciMozUnload("redirect-scin");
    SciMozUnload("scin-work");
    SciMozUnload("input-scin");

    globalPrefs.RemoveObserver(notified);

    debugger_onUnLoad(); // editor-debugger.js
    StatusBar_Fini();
}

function KomboCanClose() {
    if (dbgd && dbgd.dbg && dbgd.dbg.state != DBG_STATE_STOPPED) {
        var prompt = "This command will stop the debugger. Do you still wish to quit?";
        if ("Yes" != yesNoDialog(prompt, "Yes")) {
            return false;
        }
        komodoQuitting = true; //XXX tell editor-debugger that we are quiting
        terminate_debugger();
    }
    var rc = CloseAllFilesProjectsDialogs();
    if (rc) {
        // We must process all pending events, else the reflow
        // caused by updating the MRU on file save (believe it or not!)
        // can cause death.
        koProcessPendingEvents();
    } else if (komodoQuitting) {
        // we were debugging and the user has cancelled the quit.  
        // Make sure the debugger gets updated appropriately.
        komodoQuitting = false;
        change_debugger_ui_state(DBG_STATE_STOPPED);
    }
    return rc;
}

function OfferToSaveFilesProjectsDialogs(urlsToConsider, title, prompt) {
    // This function is called by several others to offer to save documents
    // which are "dirty".  This includes views, dialogs, and projects.
    // This function takes the list of URLs it is input and
    //  --- if it is null, offers to save all dirty URLs
    //  --- if it is not null, filters to save only those elements of URLs which
    //      correspond to known dirty URLs.
    //
    var i,j;

    // Determine the list of dirty URLs to consider offering to save.
    var dirtyProjects = gProjectManager.getDirtyProjectURLs();
    var allDirtyURLs = dirtyProjects;
    var dirtyViews = viewManager.getDirtyURLs(new Object());
    allDirtyURLs = allDirtyURLs.concat(dirtyViews);

    var guiBuilderSvc = Components.classes["@activestate.com/koGuiBuilderService;1"].
            getService(Components.interfaces.koIGuiBuilderService);
    var dirtyDialogs = guiBuilderSvc.getDirtyURLs(new Object());
    allDirtyURLs = allDirtyURLs.concat(dirtyDialogs);

    var dirtyURLsToConsider;
    if (urlsToConsider == null) {
        dirtyURLsToConsider = allDirtyURLs;
    } else {
        // Filter the given urlsToConsider by the known list of dirty URLs.
        dirtyURLsToConsider = new Array();
        for (i = 0; i < urlsToConsider.length; i++) {
            for (j = 0; j < allDirtyURLs.length; j++) {
                if (urlsToConsider[i] == allDirtyURLs[j]) { 
                    dirtyURLsToConsider.push(urlsToConsider[i]);
                    break;
                }
            }
        }
    }
    if (dirtyURLsToConsider.length == 0) {
        // None of the URLs being sent in are dirty.
        return true;
    }
    
    // Offer to save those URLs.
    var obj = new Object();
    obj.URLs = dirtyURLsToConsider;
    obj.title = title;
    obj.prompt = prompt;
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                             "komodo_closeConfirm",
                             "chrome,close=no,modal,resizable=true",
                             obj);
    if (!obj || obj.ret == false) {
        // user pressed ESC or cancel
        return false;
    }
    var selectedURLs = obj.URLs;
    
    // Save the URLs that the user selected.
    var view, url, found;
    for (i = 0; i < selectedURLs.length; i++) {
        url = selectedURLs[i];
        found = false;
        for (j = 0; j < dirtyViews.length; j++) {
            if (dirtyViews[j] == url) {
                found = true;
                if (! viewManager.ConfirmedSave(url)) {
                    return false;  // error or something.
                }
                break;
            }
        }
        if (!found) {
            for (j=0; j < dirtyDialogs.length; j++) {
                if (dirtyDialogs[j] == url) {
                    found = true;
                    if (! guiBuilderSvc.Save(url)) {
                        return false;  // error or something.
                    }
                }
                break;
            }
        }
        if (!found) {
            var id;
            for (j=0; j < dirtyProjects.length; j++) {
                if (dirtyProjects[j] == url) {
                    found = true;
                    id = gProjectManager.getProjectIdByURL(url);
                    if (id < 0 || !gProjectManager.saveProject(id)) {
                        return false;  // error or something.
                    }
                }
                break;
            }
        }
        if (!found) {
            alertDialog("Komodo couldn't save the data corresponding to " + url);
            logError("Unknown view -- not a view, project or dialog: " + url);
            return false;
        }
    }
    return true;
}
function CloseAllFilesProjectsDialogs() {
    var msg = "Save changes to the selected documents and dialogs?";
    var ret = OfferToSaveFilesProjectsDialogs(null, "Close All", msg);
    if (ret == false) return false;

    gProjectManager.closeAllProjectsEvenIfDirty();
    var guiBuilderSvc = Components.classes["@activestate.com/koGuiBuilderService;1"].
            getService(Components.interfaces.koIGuiBuilderService);
    guiBuilderSvc.QuitAll();
    viewManager.closeAllViewsEvenIfDirty();
    return true; 
}

// These tooltip functions are pretty dumb - just simple whitespace
// sep'd words for "demo" purposes!
function IsWhitespace(string) {
    return /^\s/.test(string);
}

function FillInKomboTooltip(widget, x, y) {
    var sm = widget.ISciMoz;
    var pos = sm.positionFromPoint(x, y);
    // dump("Getting scintilla tooltip for document position " + pos + "\n");
    if (pos > 0 && pos < sm.length) {
        var lineNo = sm.lineFromPosition(pos);
        var column = sm.getColumn(pos);
        var charOffset = pos - sm.positionFromLine(lineNo) + 1;
        var text;
        view = widget.viewManager.getCurrentView();
        if (view.lintBuffer.lintResults) {
            var results = new Object();
            var numResults = new Object();
            view.lintBuffer.lintResults.getResultsInLineRange(lineNo + 1,
                                                              lineNo + 1,
                                                              results,
                                                              numResults);
            for (var i = 0; i < numResults.value; i++) {
                var r = results.value[i];
                //XXX lintResult.column* are actually misnomers. They give
                //XXX character offsets from the begining of the line, not
                //XXX column numbers
                if (r.columnStart <= charOffset && charOffset <= r.columnEnd) {
                    return r.description;
                }
            }
        }
    }
    return null;
}


////////////////////////////////////////////////////////////////////
// support for find and go to line
////////////////////////////////////////////////////////////////////

function doFind() {
    _callFind("find");
}

function doReplace() {
    _callFind("replace");
}

function _callFind(tabName) {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while find.xul is loading. The find dialog will retrieve these
    // contents when it is ready.
    InputBuffer_Start();

    gFindDialogTabName = tabName;
    openWindowUniqueInstance("chrome://komodo/content/find/find.xul",
                             "komodo_find",
                             "chrome,dialog,dependent");
}


function doGotoLine() {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while goto.xul is loading. The find dialog will retrieve these
    // contents when it is ready.

    // Get the current view before calling inputbuffer start so we have the
    // correct focus in the goto dialog.
    var v = getFocusedView();
    InputBuffer_Start();

    openWindowUniqueInstance("chrome://komodo/content/find/goto.xul",
                             "komodo_goto",
                             "chrome,dialog,modal");
}


//---- Input buffering
// When you need to capture user input while a slow XUL window is loading you
// can use the input buffer. Usage:
//  - in some JS code:
//      InputBuffer_Start()
//      // open XUL window
//
//  - in slow XUL window onload handler:
//      var contents = InputBuffer_Finish();
//      // use the contents somehow
//

var _gInputBuffer_active;

function InputBuffer_Start()
{
    _gInputBuffer_active = 1;
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    inputBufferWidget.focus();
}

function InputBuffer_OnFocus()
{
    // if it is not active the hidden input buffer should not have the focus
    if (!_gInputBuffer_active) {
        //XXX Had to put this in a setTimeout or else the scintilla controllers
        //    do not get setup properly.
        window.setTimeout('gLastFocusedViewManager.setFocus();', 1);
    }
}

function InputBuffer_Finish()
{
    // Return the contents of the input buffer and stop buffering.
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    var contents = inputBufferWidget.value;
    inputBufferWidget.value = "";

    _gInputBuffer_active = 0;
    return contents;
}



//---- Jumping to lint results

function JumpToNextLintResult()
{
    // Do nothing if there is no current view.
    var vm = getFocusedViewManager();
    if (!vm) return;
    var view = vm.getCurrentView();
    if (!view || typeof(view)=="undefined") return;

    // Do nothing if linting is turned off for the current file.
    if (! view.prefs.getBooleanPref("editUseLinting")) return;

    // If there are no results, then say so.
    //   lintResults == null        -> inprogress
    //   lintResults.length == 0    -> ok
    //   lintResults.length > 0     -> errors/warnings
    var lintResults = view.lintBuffer.lintResults;
    if (!lintResults) {
        alertDialog("Syntax checking is currently in progress.");
    } else if (! lintResults.getNumResults()) {
        alertDialog("There are no syntax errors.");
    } else {
        // Determine the current position.
        var scimoz = view._scimoz;
        var caretPos = scimoz.currentPos;
        var caretLine = scimoz.lineFromPosition(caretPos) + 1;
        var caretCol = scimoz.getColumn(caretPos) + 1;
        
        var next = lintResults.getNextResult(caretLine, caretCol);
        //dump("next lint result: line="+next.lineStart+", column="+
        //     next.columnStart+"\n");
        
        var pos = scimoz.positionFromLine(next.lineStart-1) +
                  (next.columnStart-1);  // subtract one for 1- vs. 0-based
        scimoz.ensureVisibleEnforcePolicy(next.lineStart-1);
        scimoz.gotoPos(pos);
        scimoz.selectionStart = pos;
        scimoz.selectionEnd = pos;
    }
}



// Class Browser code

function classBrowserClearTree() {
    var tree = document.getElementById('codechildren');
    while (tree.hasChildNodes()) {
        tree.removeChild(tree.firstChild);
    }
}

function classBrowserAddClass(className, lineNo) {
    var tree = document.getElementById('codechildren');
    var container = document.createElement('treeitem');
    tree.appendChild(container);
    container.setAttribute('container', 'true');
    container.setAttribute('id', className);
    container.setAttribute('open', 'false');
    var row = document.createElement('treerow');
    container.appendChild(row);
    var cell;
    cell = document.createElement('treecell');
    cell.setAttribute('class', 'treecell-indent');
    cell.setAttribute('value', className);
    cell.setAttribute('lineNo', String(lineNo));
    row.appendChild(cell);
}

function classBrowserAddMethod(className, methodName, lineNo) {
    var children = document.createElement('treechildren');
    var item = document.createElement('treeitem');
    children.appendChild(item);
    var row = document.createElement('treerow');
    item.appendChild(row);
    var cell = document.createElement('treecell');
    cell.setAttribute('class', 'treecell-indent');
    cell.setAttribute('indent', "true");
    cell.setAttribute('value', methodName);
    cell.setAttribute('lineNo', String(lineNo));
    row.appendChild(cell);
    var folder = document.getElementById(className);
    folder.appendChild(children);
}

function classbrowser_receiver () {}
classbrowser_receiver.prototype = {
    clearTree: function() {
        classBrowserClearTree();
    },

    addClass: function(className, lineNo) {
        classBrowserAddClass(className, lineNo);
    },

    addMethod: function(className, methodName, lineNo) {
        classBrowserAddMethod(className, methodName, lineNo);
    },

    // nsISupports interface
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) && 
            !iid.equals(Components.interfaces.koClassBrowserReceiver))
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
}


//////////////////////////////////////////////////////////
// Debugging utility functions
//////////////////////////////////////////////////////////

/* List attributes of a given object */
function introspect(obj) {
   for (var elt in obj) {
      if (elt == 'style') {
         continue;
      }
      try {
         dump(elt + "=" + obj[elt] + "\n");
      } catch(e) {
         dump(elt + ": unknown" + "\n");
      }
   }
}
 
function dumpAttr(dNode, attrName) {
   try {
      dump("Attr " + attrName + " of node " + dNode.id + " = ");
      dump("'" + dNode.getAttribute(attrName) + "'\n");
   } catch (e) {
      dump("undefined.\n");
   }
}

function dumpCoordinates(objname, obj) {
  dump("Coordinates for " + objname + "\n");
  dump("x = " + obj.boxObject.x +
       ", y = "  + obj.boxObject.y +
       ", width = "  + obj.boxObject.width +
       ", height = "  + obj.boxObject.height +
       "\n");
}

function dumpTabSizes() {
    dump("|tab panel| = " +
         document.getElementById('tabbox').childNodes.length +
         ", |tab box| = " +
         document.getElementById('tabpanel').childNodes.length +
         "\n");
}

function inputPaneOnLoad() {
    var input_sci_widget = document.getElementById("input-scin");
    input_sci_widget.init();
}

function getInputFileSciMoz() {
    var input_sci_widget = document.getElementById("input-scin");
    return input_sci_widget.ISciMoz;
}

function getEditorSciMoz() {
    return document.getElementById("scin").ISciMoz;
}

