
function MacroManager() {
    this.recording = false;
    this._currentMacro = new Array();
    this._currentMacroText = '';
}


MacroManager.prototype = new Object();
MacroManager.prototype.constructor = MacroManager;

MacroManager.prototype.startRecording = function() {
    //dump("Starting to record\n");
    if (this._currentMacro) {
        delete this._currentMacro;
    }
    this._currentMacro = new Array();
    this._currentMacro.push("// Macro recorded on " + Date() + '\n');
    var macroVersionNumber = "1";
    this._currentMacro.push("assertMacroVersion(" + macroVersionNumber + ');\n');
    this._currentMacro.push("gLastFocusedViewManager.setFocus();\n");
    this._currentMacro.push("editor.beginUndoAction();");
    this._currentMacroText = new Array();
    this.recording = true;
    koRequestUpdateCommands("macro");
    StatusBar_AddMessage("Recording Macro", "macro", 0, true)
}

MacroManager.prototype.stopRecording = function() {
    this.recording = false;
    this._finishTextMacro();
    this._currentMacro.push("editor.endUndoAction();");
    koRequestUpdateCommands("macro");
    StatusBar_AddMessage(null, "macro", 0, false)
    StatusBar_AddMessage("Macro Recorded", "macro", 3000, true)
}
MacroManager.prototype._finishTextMacro = function() {
    if (this._currentMacroText != '') {
        this._currentMacro.push("editor.replaceSel('" + 
                                this._currentMacroText.replace('\\', '\\\\', 'g').replace("\'", "\\'", 'g') +
                                "');\n");
        this._currentMacroText = '';
    }
}

MacroManager.prototype.appendCommand = function(command) {
    if (! this._currentMacro) {
        logWarning("Trying to append a command while not in a macro!!!");
        return;
    }
    this._finishTextMacro();
    this._currentMacro.push("koGoDoCommand('" + command +"');\n");
}

MacroManager.prototype.recordKeyPress = function(c) {
    if (! this._currentMacro) {
        logWarning("Trying to record keypresses while not in a macro!!!");
        return;
    }
    this._currentMacroText += c;
}

MacroManager.prototype.undo = function() {
    if (this._currentMacroText != '') {
        this._currentMacroText = this._currentMacroText.slice(0, this._currentMacroText.length-1);
    } else {
        this._currentMacro.pop();
    }
}

MacroManager.prototype.appendCode = function(code) {
    if (! this._currentMacro) {
        logWarning("Trying to append code while not in a macro!!!");
        return;
    }
    this._finishTextMacro();
    this._currentMacro.push(code);
}

MacroManager.prototype.executeMacro = function(macro) {
    var scin = document.getElementById('scin').ISciMoz;

    scin.beginUndoAction();
    try {
        if (macro == null) {
            macro = this._currentMacro;
        }
        if (macro == null) return;
        if (this.recording) {
            this.recording = false; // avoid recursion!
        }
        // XXX weird stuff.
        // For some reason, Scintilla always thinks the lines are folded, which results in selection of
        // folded lines, which is typically not what we want So we unfold all the lines in the buffer.
        for (var lineno=0;lineno < scin.lineCount;lineno++)
            if ((scin.getFoldLevel(lineno) & scin.SC_FOLDLEVELHEADERFLAG) &&
                !scin.getFoldExpanded(lineno))
                    scin.toggleFold(lineno)
        var code = (macro.join(''));
        MacroEvalCode(code);
    } catch (e) {
        logError(e);
    }
    scin.endUndoAction();
}

MacroManager.prototype.executeLastMacro = function(macro) {
    StatusBar_AddMessage("Executing Last Recorded Macro", "macro", 3000, false)
    this.executeMacro(this._currentMacro);
}

MacroManager.prototype.saveToToolbox = function(macro) {
    var os = Components.classes["@activestate.com/koOs;1"].
              createInstance(Components.interfaces.koIOs);
    var koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);

    var macrosDir = os.path.join(koDirs.userDataDir, 'MyMacros');
    if (! os.path.exists(macrosDir)) {
        os.mkdir(macrosDir);
    }
    var defaultname = 'MyMacro.js';
    var url;
    if (os.path.isdir(macrosDir)) {
        url = LocalFileToURL(macrosDir) + '/' + defaultname;
    } else {
        logError("Macros Directory " + macrosDir + " does not exist or is not a directory!");
        url = '';
    }
    var fname = dlg_saveLocalFileAsFromLanguage('JavaScript', defaultname, url).file.path;
    if (!fname) { return; }
    if (fname.slice(-3) != '.js') {
        fname += '.js'
    }
    
    os.writefile(fname, this._currentMacro.join(''));
    makeMacroFromFile(fname);
}

var gMacroMgr = new MacroManager();
