    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */
   

const NS_ERROR_FILE_NOT_FOUND = 2152857618;

var globalCounter = 0;

function ScintillaView(viewmanager, mysci_widget) {
    if (arguments.length == 0) return;
    // perfTimer("ScintillaView", "construct", true);
    AbstractView.apply(this, [viewmanager]);

    this.item = new partWrapper(null,0);
    
    this.url = null;
    this.leafName = null;
    this.fileTemplate = null;
    this._sci_widget = mysci_widget;
    this._scimoz = mysci_widget.ISciMoz;
    this._activated = false;
    this.card = 0;
    this._observedFileChange = 0;

    this.lintBuffer = null; // We get modified events before we have set this up.
    this.viewState = null;
    
    // We can now go and load up the code, creating the  new instance.
    this.py_inst = Components.classes['@activestate.com/koEditorWrapper;1'].createInstance(Components.interfaces.koIEditorWrapper);
    this.py_inst_events = this.py_inst.QueryInterface(Components.interfaces.ISciMozEvents);
    this.py_inst.extensionUI = editorExtensionUI;
    this.prefs = this.py_inst.viewPrefs;

    // And now, fill the thing with some content.
    globalCounter++;
    this.viewName = "view" + globalCounter;
    this.lintBuffer = new jsLintBuffer(this);

    /* XXX split editor support, see scintillaview.js::clone() for details
    this.sci_widget_split = document.getElementById("scin-split");
    this._splitViewSplitter = document.getElementById('split-edit-splitter');
    this._splitView_splitterstate = this._splitViewSplitter.getAttribute('state');
    */
    
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
          getService(Components.interfaces.nsIObserverService);    
     // perfTimer("ScintillaView", "construct done");
};



// The following two lines ensure proper inheritance (see Flanagan, p. 144).
ScintillaView.prototype = new AbstractView();
ScintillaView.prototype.constructor = ScintillaView;

/* XXX The way that we are doing observers here isn't very smart. We are using
   the observer mechanism to make method calls that have to be manually dispatched!
   What we should do is have the JavaScript views implement an XPCOM interface
   and give koEditorWrapper a pointer to these views. Look at the implementation for
   ScintillaView.prototype.Observe. And this will only get worse unless we do
   something about it now (but I don't have time :-)! */
   
ScintillaView.prototype.addObservers = function() {
    this.observerSvc.AddObserver(this, 'autosave');    
    this.observerSvc.AddObserver(this, 'language_changed_'+this.viewName);
}

ScintillaView.prototype.removeObservers = function() {
    this.observerSvc.RemoveObserver(this, 'autosave');    
    this.observerSvc.RemoveObserver(this, 'language_changed_'+this.viewName);
}

/* clone is used for supporting a split view system.  This clones the
    settings and document pointer of the view provided
*/
/* XXX split editor support, see scintillaview.js::clone() for details
    XXX this is especially SLOW

The split editor window uses modifications in
    kombo.js, TabViewManager.js, EditorViewManager.js, komodo.xul

A second scintilla widget is set up in komodo.xul.  The EditorViewManager/TabViewManager
uses ScintillaViewManager to manage the second scintilla.  This is available in all tabs.
The following clone function is called from TabViewManager to clone view preferences and
the doc pointer.
                           
major issues:
        ui prefs
        turning on/off split view per file
        performance (may be better to use a second TabViewManager in place of ScintillaViewManager)
        clone is VERY slow and done on every tab switch

ScintillaView.prototype.clone = function(view) {
    if (typeof(view)=='undefined' || !view) return;
    if (!view.py_inst || !view.py_inst.language) return;

    // destruct any stuff that needs to be
    if (this.lintBuffer) {
        this.lintBuffer.deactivate();
        this.lintBuffer.destructor();
    }
    if (this._activated) {
        this.removeControllers();
        this.py_inst.deactivate();
        this.py_inst.destroy();
        this._activated = false;
    }
    // clone needed attributes
    this.viewState = view.viewState;
    this.viewName = view.viewName;
    this.initWithLanguage(view.py_inst.language);
    this._scimoz.docPointer = view._scimoz.docPointer;
    
    // construct needed items
    this.lintBuffer = new jsLintBuffer(this);


    // restore settings
    update_breakpoints(this);
    reset_stack_position(this);

    // If we stashed away the state when we lost focus or opened, restore it now.
    if (this.viewState != null) {
        this._restoreViewState();
    }

    this.installControllers();
    this.lintBuffer.activate();
    this.py_inst.activate();
    this._scimoz.docPointer = view._scimoz.docPointer;
    this._activated = true;
}
*/

ScintillaView.prototype.initWithURL = function(url, leafName) {
    // load view state from MRU, or set to NULL if we don't exist.
    var viewStateMRU = globalPrefService.getPrefs("viewStateMRU");
    if (viewStateMRU.hasPref(url)) {
        var viewState = viewStateMRU.getPref(url);
        if (!viewState.hasPref("viewState")) {// This IF condition is purely for Komodo 1.2 developers.
                                             // No version of Komodo was released where this will be true
                                             // Delete very very soon
            this.viewState = viewState;
        } else { // Delete to here
            this.viewState = viewState.getPref("viewState");
            this.prefs.update(viewState.getPref("viewPrefs"));
        }
    }
    else {
        this.viewState = null;
    }

    this.item.url = this.url = url;
    this.leafName = leafName;
    if (!this.leafName) {
        this.leafName = this.url;
    }

    this.py_inst.init(this._scimoz, this.viewName, this._sci_widget.scratch_document, null,this.viewState);    
    this.untitled = false;

    this.addObservers();
}

ScintillaView.prototype.initWithLanguage = function(language) {
    var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"].getService(Components.interfaces.koILanguageRegistryService);
    if (typeof(language) == 'undefined' || !language) language = 'Text';
    var koLanguage = languageRegistry.getLanguage(language);
    var ext;
    
    try {
        ext = koLanguage.defaultExtension;
    } catch (e) {
        alertDialog("Internal Error - Language " + language + " did not define a default Extension - using .txt");
        ext = ".txt";
    }
    this.url = getNextUntitledName(language, ext);
    this.item.url = this.leafName = this.url;
    this.py_inst.init(this._scimoz, this.viewName, this._sci_widget.scratch_document, language,null);
    this.untitled = true;
    
    this.addObservers();    
}

ScintillaView.prototype.initWithFileTemplate = function(fileTemplate) {
    var ospath = Components.classes["@activestate.com/koOsPath;1"].getService(Components.interfaces.koIOsPath);
    var ext = ospath.getExtension(fileTemplate.fileURI)
    this.url = getNextUntitledName(ospath.withoutExtension(makeLeafName(fileTemplate.fileURI)), ext);
    this.item.url = this.leafName = this.url;
    this.fileTemplate = fileTemplate
    this.py_inst.init(this._scimoz, this.viewName, this._sci_widget.scratch_document, null,null);
    this.untitled = true;
    
    this.addObservers();
}

ScintillaView.prototype.destructor = function() {
    AbstractView.prototype.destructor.apply(this);
    
    this.removeObservers();
    
    if (this.lintBuffer) {
      this.lintBuffer.destructor();
    }
    this.lintBuffer = null;
    this.manager = null;
    this.py_inst.destroy();
    this.py_inst = null;
    this.py_inst_events = null;
}

ScintillaView.prototype.isDirty = function() {
    return this.py_inst.isDirty   
}

ScintillaView.prototype.isDiskFileDirty = function() {
    return this.py_inst.isDiskFileDirty();   
}

ScintillaView.prototype.revert = function() {
    if (this.manager.getCurrentView()!=this) {
        logError("ScintillaView::revert ["+this.url+"] not current view!");
        return;
    }
    var errmsg = new Object;
    // for some reason, the controllers get messed up by reloading the file
    // removing them, and resinstalling after the load cleans it up.  but we
    // cannot do a full deactivate/activate,since deactivate switches scins docpointer
    // to a scratch buffer, so the load doesn't work.
    window.setCursor("wait");
    this.removeControllers();
    // Save the view state before reloading, so we can attempt a restore.
    this._saveViewState();
    // reload the contents from disk.
    var errno = this.py_inst.load(this.url, errmsg);
    // Due to the funky koEditorWrapper layout, we need to re-activate
    this.py_inst.activate()
    this._restoreViewState();
    this.installControllers();
    window.setCursor("auto");
    if (errno != 0) {
        alertDialog("Unable to load " + displayNameFromURL(this.url) + ". " + errmsg.value, true, true);
        return;
    }
    if (this.lintBuffer && this.useLinting) this.lintBuffer.request();
    // remove autosave file if they reverted
    this.py_inst.removeAutoSaveFile();
    // If it's an SCC file, do a refresh
    if (this.url) {
        this.observerSvc.Notify(this,'file_status_now',this.url);
    }
}

ScintillaView.prototype.loadURI = function(language) {
    // perfTimer("ScintillaView.loadURI", "start", true);
    if (!AbstractView.prototype.loadURI.apply(this, [language])) {
        return 0;
    }
    
    var errmsg = new Object;
    var errno = 0;
    var loadsaved = 0;
    var bHaveAutoSave = this.py_inst.haveAutoSave(this.url);
    if (bHaveAutoSave &&
        yesNoDialog("It appears the file '"+this.leafName+"' was not properly saved, would you like to restore the backup?", "Yes") == "Yes") {
        errno = this.py_inst.restoreAutoSave(this.url, errmsg);
        loadsaved = 1;
    } else {
        if (bHaveAutoSave)
            this.py_inst.removeAutoSaveFile();
            
        if (this.fileTemplate) {
            errno = this.py_inst.loadFileTemplate(this.fileTemplate, errmsg);
        }
        else {
            try {
                errno = this.py_inst.load(this.url, errmsg);
            } catch(e) {
                logError("Unable to load file "+this.url+", unknown excpetion: "+e+"\n");
                errno = -1;
                errmsg.value = "File may have corrupt or unsupported Unicode formatting, see log files for details.\n";
            }
        }
    }

    if (errno != 0) {
    //alertDialog("Unable to load " + this.url + ". " + errmsg.value);
        this.errmsg = errmsg.value;

        return 0;
    } else if (loadsaved) {
        koRequestUpdateCommands("save");
        this.manager.indicateIfDirty(this);
    }
        // py_inst.load tries to determine the language based on filename, and always sets
        // py_inst.language, overwriting what we did before the load.  If the file does not 
        // have a recognized extension, it sets to text.
        // if we have predefined the language, then reset the language now!
    
    if (language && language != this.py_inst.language) this.py_inst.language = language;
    // perfTimer("ScintillaView.loadURI", "done");
    return 1;
}

ScintillaView.prototype.installControllers = function () {
    this._sci_widget.controllers.insertControllerAt(0, this.py_inst.controller)
}

ScintillaView.prototype.__defineGetter__("useLinting",
function() 
{
  return this.prefs.getBooleanPref("editUseLinting");
});

ScintillaView.prototype.removeControllers = function () {
    // Remove the controller for this view
    if (this.py_inst) {
        try {
            //dump("removing py_inst.controller from controllers\n");
            this._sci_widget.controllers.removeController(this.py_inst.controller);
        }
        catch(e) {
            logError("Failed to remove py_inst.controller from controllers for ["+this.url+"].\n");
            logError("Here's why: " + e);
        }
    }
    else {
        logError("ScintillaView::removeControllers: View lacked a py_inst");
    }
}

ScintillaView.prototype.activate = function () {
    if (this._activated) {
        logError("ScintillaView::activate ["+this.url+"] already activated!");
    }

    this.py_inst.activate();

    this.manager.selectView(this); // must select view after activation, else repaints may happen before we have setup the lexer
    
    this.updateUI();
    
    // XXX this should be factored out when debug stuff is refactord
    // fix breakpoint status indication bug #7006
    update_breakpoints(this);
    reset_stack_position(this);

    // If we stashed away the state when we lost focus or opened, restore it now.
    if (this.viewState != null) {
        this._restoreViewState();
    }

    this.installControllers();
    this.lintBuffer.activate();

    //XXX This was apparently necessary at some point but no longer seems to
    //    be. In fact this causes the Find dialog woes when it tries to switch
    //    buffer while searching across files.
    //this.manager.setFocus();

    // check to see if we need to revert the file
    var checkDisk = (this.prefs.hasBooleanPref("checkDiskFile") &&
        this.prefs.getBooleanPref("checkDiskFile"));
    if (checkDisk && !this._observedFileChange) {
      var err = this.py_inst.isDiskFileDirty();
      if (err==-1 && !this.untitled) {
          this.untitled = true;
          if (yesNoDialog("The file '"+this.leafName+"' is no longer on disk, would you like to save the file now?", "Yes") == "Yes") {
              this.doSave();
          } else {
              this.py_inst.diskFileDirty=1; // mark dirty if different than disk!
              this.manager.indicateIfDirty(this);
          }
      } else if (err==1) {
          if (yesNoDialog("The file '"+this.leafName+"' on disk has changed, would you like to reload the file?", "No") == "Yes") {
              this.revert();
          } else {
              this.py_inst.diskFileDirty=1; // mark dirty if different than disk!
              this.manager.indicateIfDirty(this);
          }
      }
    }
    
    // Ensure any buffer sensitive commands are updated.
    koRequestUpdateCommands("buffer-focus");
    koRequestUpdateCommands("debug");
     // The selection may have changed.
    koRequestUpdateCommands("select");
    // The save and undo state of the file will be different.
    koRequestUpdateCommands("save");
    koRequestUpdateCommands("undo");
    koRequestUpdateCommands("macro");
    
    this._activated = true;

    // XXX Really this notification should be made in the base AbstractView
    // but the base activate() would most likely be called before this
    // activate() and this notification, to be useful, must be called
    // after activation.
    try {
        var empty = new Object();
        this.observerSvc.Notify(empty, "post_activate_view", "");
    } catch(e) {
        // do nothing: Notify sometimes raises an exception if (???) receivers
        // are not registered?
    }
};

ScintillaView.prototype.deactivate = function () {
    if (!this._activated) {
        logError("ScintillaView::deactivate ["+this.url+"] already deactivated!");
    }
    
    // XXX Really this notification should be made in the base AbstractView
    // but the base deactivate() would most likely be called after this
    // deactivate() and this notification, to be useful, must be called
    // before deactivation.
    try {
        var empty = new Object();
        this.observerSvc.Notify(empty, "pre_deactivate_view", "");
    } catch(e) {
        // do nothing: Notify sometimes raises an exception if (???) receivers
        // are not registered?
    }
    
    this._activated = false;
    //dump("ScintillaView.prototype.deactivate: "+this.url+"\n");

    this._saveViewState();
    this.lintBuffer.deactivate();
    this.removeControllers();
      
    this.py_inst.deactivate();
}

ScintillaView.prototype.canClose = function() {
    if (this.isDirty()) {
	this.manager.setCurrentView(this);
        if (!this.HandleFileToSave()) {
            //they cancelled out, or a save failed
            return false;
        }
    }
    // always attempt to remove any auto save file
    this.py_inst.removeAutoSaveFile();
    return true;
}

ScintillaView.prototype.close = function() {
    if (this.viewState) {
        var viewStateMRU = globalPrefService.getPrefs("viewStateMRU");
        var viewStateSave = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        viewStateSave.setPref("viewState", this.viewState)
        viewStateSave.setPref("viewPrefs", this.prefs)

        viewStateSave.id = this.url
        viewStateMRU.setPref(viewStateSave);
    }
}

ScintillaView.prototype.gotoPosition = function(pos) {
    // We need an integer.
    pos = Math.floor(pos);
    if (!pos) {
        alertDialog("The specified position is invalid");
        return false;
    }
    // Make sure we are activated first.
    this.manager.setCurrentView(this);
    // And just jump to the position.
    // scintilla is zero based.
    this._scimoz.gotoLine(pos-1);
    this._scimoz.ensureVisibleEnforcePolicy(pos-1);
    return true;
}

ScintillaView.prototype.viewAs = function(language) {
    this.removeControllers(); // controllers may be language specific - must re-install.
    this.py_inst.language = language;
    this.installControllers();
}

ScintillaView.prototype.PrintView = function () {
    var styledHTML = this._getHTML();
    // hand it over to Mozilla and print it
    with (window.frames[0].document) {
        open();
        write(styledHTML);
        close();
    }
    try {
        _content.print();
    } catch (e) { }

    window.setCursor("auto");
    return;
}

ScintillaView.prototype.ExportHTML = function () {
    var styledHTML = this._getHTML();
    var fp = dlg_fileSelectViaDialog(Components.interfaces.nsIFilePicker.modeSave,
                                "Choose Output HTML File", '',
                                this.leafName.replace('.', '_', 'g')+'.html', "HTML Files", "*.html");
    if (!fp) {
        return;
    }
    try {
        var file = new dlg_get_URL_andName(fp);
        filewinfuncs_forceExtension(file, '.html');
        os.writefile(file.fullName, styledHTML);
    } catch (e) {
        logError(e);
    }
    return;
}


ScintillaView.prototype._getHTML = function () {
    //This is capable of printing selected text
    window.setCursor("wait");
    this._scimoz.colourise(0, -1); // force colouring of the entire buffer before printing
    // Send it to the pretty printer
    var lang = this.py_inst.sourceFile.getLanguageBlockAtPosition(0).language;
    var langName = lang.name;
    var simplePrintLanguageService = lang.
        getLanguageService(Components.interfaces.koIPrettyPrinterLanguageService);
    var styledHTML = simplePrintLanguageService.getHTML(this._scimoz, title, langName, lang.styleBits);
    window.setCursor("auto");
    return styledHTML;
}

ScintillaView.prototype.saveContents = function(url /*= this.view*/) {
  if (!this.py_inst) return false;
  window.setCursor("wait");
  if (url == undefined || url == null) url = this.url;
  var errmsg = new Object;
  var errno = this.py_inst.save(url, errmsg);
  if (errno != 0) {
    alertDialog("Unable to save " + url + ". " + errmsg.value);
    window.setCursor("auto");
    return false;
  }
  // Update the MRU list.
  MRU_addURL("mruFileList", url);
  window.setCursor("auto");
  
  // if the file was a SCC file, but is no longer one, we
  // have to clear here since we will not receive file notifications
  // to clear SCC information.  Otherwise, status data does get
  // reset.
  this.item = new partWrapper(null,0); 
  
  this.observerSvc.Notify(this,'file_changed', this.url);

  return true;
}
    
ScintillaView.prototype.preSave = function() {
  // if the file is untitled, either it has never been saved, or no file exists with this name
  // (ie. file on disk was renamed or deleted)
  if (!this.untitled && this.py_inst.isDiskFileDirty()==1) {
      if (yesNoDialog("The file on disk has changed, are you sure you want to overwrite those changes?", "No") == "No") {
          return false;
      }
  }
  if (!this.py_inst.isEncodable) {
    if (alertDialog("Some of the characters in the file '" + this.leafName + "' cannot be saved using the current encoding. " +
                       "You can either remove those characters or choose a different encoding in the 'Current File Settings' dialog.",
                       false, false, "Remove All", "Cancel") == "Remove All") {
        if (yesNoDialog("You will lose data if you proceed, are you sure that you want to continue?", "No") == "Yes") {
          this.py_inst.removeUnencodeable();
          return true;
        }            
    }                           
    return false;
  }
  return true;
}

ScintillaView.prototype.doSave = function() {
  // XXX hack attack
  // get the current language, and reset to that language
  // after a save if the save changes the language. 
  // This allows viewas to stay the same
  // when you save a file.  This should be reworked in a
  // cleaner way, but will require several changes in
  // editorwrapper and scintillaview to have it be nice
  var lang = this.py_inst.language;
  var newfile = this.untitled;
  if (this.preSave() && this.realSave()) {
    if (!newfile && this.py_inst.language != lang) {
        // XXX editorwrapper just removed the controller on us
        // this has lots of heavy consequences, and causes a funky jump if word-wrapping is on.
        this.py_inst.language = lang; 
        this.installControllers();
    }
    this.py_inst.removeAutoSaveFile();
    return true;
  }
  return false;
}

/* XXX this should be changed to work like autoSave(), but will
require changes to koEditorWrapper to make that work.  Until then,
performance is poor for this operation.  When that change is made,
also change views.js where this is called from. */
ScintillaView.prototype.dirtySave = function() {
    var cv = this.manager.getCurrentView();
    if (cv == this) {
        this.doSave();
    } else {
        this.manager.setCurrentView(this);
        this.doSave();
        koRequestUpdateCommands("save");
        this.manager.indicateIfDirty(this);
        this.manager.setCurrentView(cv);
    }
}

ScintillaView.prototype.realSave = function() {
  if (this.untitled) {
    return this.realSaveAs();
  } else if (!this.py_inst.isWritable) {
        if (typeof(this.item.properties['scc']) != 'undefined' &&
            this.item.properties['scc']) {
            if (alertDialog("The file '" + this.leafName + "' is a read only file under source code control. " 
                                              + "To save it Komodo must first check it out.",true,false) == "OK") {
                // check file out then save
                if (editSCCFile(this.item))
                    return this.saveContents();
            }
            return true;
        }
    	if (yesNoDialog("The file '" + this.leafName + "' is read only. " 
					  + "Do you want to save your changes under a different name?", 
					  "Yes") == "Yes") {
	  return this.realSaveAs();
	} else {
	  return true;
	}
  } else {
    return this.saveContents();
  }
}

ScintillaView.prototype.doSaveAs = function() {
  if (this.preSave()) {
    return this.realSaveAs();
  }
  return false;
}

ScintillaView.prototype.realSaveAs = function() {
    var fp = dlg_saveLocalFileAs(this);
    if (!fp) {
        return false;
    }
    var file = new dlg_get_URL_andName(fp);
    return this._finishSaveAs(file.url);
}

ScintillaView.prototype.doRemoteSaveAs = function() {
  if (this.preSave()) {
    return this.realRemoteSaveAs();
  }
  return false;
}

ScintillaView.prototype.realRemoteSaveAs = function() {
    var url = doSaveAsRemoteFiles(this.leafname);
    if (!url) return false;
    return this._finishSaveAs(url);
}
    
ScintillaView.prototype._finishSaveAs = function(url) {
    //  introspect(file);
    if (this.manager.getViewForFile(url) != null) {
        alertDialog("Sorry, but you can't save a file with the same name as another loaded file.");
        return false;
    }
    
    var ln = makeLeafName(url);
    this.manager.setCurrentView(this);

    if (this.saveContents(url)) {
        // update the view with the new settings.
        this.manager.renameView(this, url, ln);
        this.manager.indicateIfDirty(this);
        updateTitlebar(this);
        // XXX controller was removed on us by editorwrapper, reinstall
        this.installControllers();
    }

    return true;
}

ScintillaView.prototype.preSaveTemplate = function() {
  // if the file is untitled, either it has never been saved, or no file exists with this name
  // (ie. file on disk was renamed or deleted)
  if (!this.untitled && this.py_inst.isDiskFileDirty()==1) {
      if (yesNoDialog("The file on disk has changed, are you sure you want to ignore those changes?", "No") == "No") {
          return false;
      }
  }
  if (!this.py_inst.isEncodable) {
    if (alertDialog("Some of the characters in the file '" + this.leafName + "' cannot be saved using the current encoding. " +
                       "You can either remove those characters or choose a different encoding in the 'Current File Settings' dialog.",
                       false, false, "Remove All", "Cancel") == "Remove All") {
        if (yesNoDialog("You will lose data if you proceed, are you sure that you want to continue?", "No") == "Yes") {
          this.py_inst.removeUnencodeable();
          return true;
        }            
    }                           
    return false;
  }
  return true;
}

ScintillaView.prototype.doSaveAsTemplate = function() {
  if (this.preSaveTemplate()) {
    return this.realSaveAsTemplate();
  }
  return false;
}

ScintillaView.prototype.realSaveAsTemplate = function() {
    var fp = dlg_saveFileAsTemplate(this);
    if (!fp) {
        return false;
    }
    var file = new dlg_get_URL_andName(fp);
    //  introspect(file);
    // allow if we're the save view, this allows diskfilegone situations to saveas correctly
    fview = this.manager.getViewForFile(file.url);
    if (fview != null && fview != this) {
        alertDialog("Sorry, but you can't save a file with the same name as another loaded file.");
        return false;
    }
    this.manager.setCurrentView(this);
    this.saveContents(file.url);
    return true;
}


//
// Saving and Restoring scintilla state.
//
ScintillaView.prototype._restoreFoldPoints = function() {
    var foldPoints = this.viewState.getPref("foldPoints");
    for (var i=0;i<foldPoints.length;i++) {
        this._scimoz.toggleFold(foldPoints.getLongPref(i));
    }
}

ScintillaView.prototype._getFoldPoints = function() {
    var linesToToggle = this.viewState.getPref("foldPoints");
    linesToToggle.reset();
    var lineLength = this._scimoz.lineCount;
    var i = 0;
    var foldParent;
    var haveThem = new Array();
    while (i < lineLength) {
        if (!this._scimoz.getLineVisible(i)) {
            foldParent = this._scimoz.getFoldParent(i);
            if (!this._scimoz.getFoldExpanded(foldParent)
                && !(foldParent in haveThem)) {
                linesToToggle.appendLongPref(foldParent);
                haveThem[foldParent] = 1;
            }
        }
        i++;
    }
}

ScintillaView.prototype._saveViewState = function() {
    // Save the view state away, ready to be restored when this view again has focus.
    // This view state is also stored in the MRU, so is even reused when the file
    // is closed and reopened.
    var bookmarks;
    if (this.viewState == null) {
        this.viewState = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        var foldPoints = Components.classes['@activestate.com/koOrderedPreference;1'].createInstance();
        foldPoints.id = "foldPoints";
        this.viewState.setPref("foldPoints", foldPoints);

        bookmarks = Components.classes['@activestate.com/koOrderedPreference;1'].createInstance();
        bookmarks.id = "bookmarks";
        this.viewState.setPref("bookmarks", bookmarks);
    }
    var result  = this.viewState;
    result.setLongPref("selectionStart", this._scimoz.selectionStart);
    result.setLongPref("selectionEnd", this._scimoz.selectionEnd);
    result.setLongPref("anchor", this._scimoz.anchor);

    result.setLongPref("xOffset", this._scimoz.xOffset);
    result.setLongPref("firstVisibleLine", this._scimoz.firstVisibleLine);
    // XXX if this language would be what is normally used for this file,
    // do not save it into view prefs
    result.setStringPref("language", this.py_inst.language);
    // XXX if this encoding would be what is normally used for this file,
    // do not save it into view prefs
    result.setStringPref("encoding", this.py_inst.encoding.python_encoding_name);

    // XXX - todo - readonly should probably not be maintained here, but from
    // other heuristics.
    result.setBooleanPref("readOnly", this._scimoz.readOnly);
    
    /* XXX split editor support, see scintillaview.js::clone() for details
    result.setBooleanPref("splitterState", this._splitView_splitterstate == 'open');
    */
    
    this._getFoldPoints();
    // Get the bookmarks.
    bookmarks = result.getPref("bookmarks");
    bookmarks.reset();
    var marker_mask = 1 << mo.bm;
    var lineNo = -1;
    while (true) {
        lineNo = this._scimoz.markerNext(lineNo+1, marker_mask);
        if (lineNo < 0)
            break;
        bookmarks.appendLongPref(lineNo);
    }
}

ScintillaView.prototype._restoreViewState = function() {
    // Save the view state away, ready to be restored when this view again has focus.
    var vs = this.viewState;
    this._scimoz.selectionStart = vs.getLongPref("selectionStart");
    this._scimoz.selectionEnd = vs.getLongPref("selectionEnd");
    this._scimoz.anchor = vs.getLongPref("anchor");
    if (vs.hasPref("language")) {
        this.py_inst.language = vs.getStringPref("language");
        // XXX STUPID.  editorwrapper removes the controller, so
        // we must reinstall it.
        this.installControllers();
    }
    if (vs.hasPref("encoding")) {
        var encoding_name = vs.getStringPref("encoding");
        this.py_inst.setEncodingFromEncodingName(encoding_name);
    }    
    // TODO: fix horizontal scrolling
    var fvl = vs.getLongPref("firstVisibleLine");

    // lineScroll() is relative but for some reason 
    // this._scimoz.firstVisibileLine is always 0 when we get here
    // TODO: find out who's clearing firstVisibleLine to 0
    if (this._scimoz.firstVisibleLine != 0) {
        // XXX nasty assert here breaks stuff all over
	//throw "ASSERT: this._scimoz.firstVisibleLine == 0";
    }
    this._scimoz.lineScroll(0, fvl);
    // xOffset added later - old prefsets may not use it!
    if (this._scimoz.wrapMode != this._scimoz.SC_WRAP_WORD &&
        vs.hasLongPref("xOffset"))
        this._scimoz.xOffset = vs.getLongPref("xOffset");
    else
        this._scimoz.xOffset = 0;

    // XXX - todo - readonly should probably not be maintained here, but from
    // other heuristics.
    this._scimoz.readOnly = vs.getBooleanPref("readOnly");
    
    /* XXX split editor support, see scintillaview.js::clone() for details
    this._splitView_splitterstate = 'collapsed';
    if (vs.hasBooleanPref("splitterState") && vs.getBooleanPref("splitterState")) {
        this._splitView_splitterstate = 'open';
    }
    this._splitViewSplitter.setAttribute('state', this._splitView_splitterstate);
    */
    
    
    this._restoreFoldPoints();
    // restore the bookmarks.
    // Perf optimization - if the buffer has any bookmarks, assume
    // it is already correct (ie, scintilla itself has tracked the state.)
    var marker_mask = 1 << mo.bm;
    if (this._scimoz.markerNext(0, marker_mask) < 0) {
        var bookmarks = vs.getPref("bookmarks");
        var num_bookmarks = bookmarks.length;
        for (var i=0;i<num_bookmarks;i++) {
            this._scimoz.markerAdd(bookmarks.getLongPref(i), mo.bm);
        }
    }
    return true;
}

ScintillaView.prototype.updateUI = function() {
    // Setup the margins
    var marginWidth = 16;
    // margin #0: line numbers - handled in command updater.
    // margin #1: folding points
    this._scimoz.setMarginWidthN (1, this.py_inst.doingFolding() ? marginWidth : 0);
    // margin #2: breakpoints and bookmarks - always enabled.
    this._scimoz.setMarginWidthN(2, marginWidth);  
  

}

ScintillaView.prototype.AutoSave = function() {
    if (this.manager.getCurrentView() == this) {
        //dump("file is current, using current scimoz\n");
        this.py_inst.doAutoSave(this._scimoz);
    } else {
        // handle saving behind the scenes
        //dump("file is NOT current, using work scimoz\n");
        var sci_work = document.getElementById('scin-work').ISciMoz;
        this.py_inst.doAutoSave(sci_work);
    }
}

ScintillaView.prototype.Observe = function (subject ,topic, fname) {
    if (topic == 'autosave' && !this.untitled && this.isDirty()) {
        this.AutoSave();
    } else if (topic == 'language_changed_'+this.viewName) {
        var language = fname;
        
        if (this.lintBuffer && this.useLinting) this.lintBuffer.request();
        this.updateUI();
        koRequestUpdateCommands('debug');

        if (this == this.manager.getCurrentView()) {
            var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                              getService(Components.interfaces.nsIObserverService);
            try {
                observerSvc.Notify(this, "current_view_language_changed", language);
            } catch(e) {
                // do nothing: Notify sometimes raises an exception if (???)
                // receivers are not registered?
            }
        }
    }        
}
