/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */
   
// Globals
var templateMgr = Components.classes["@activestate.com/koTemplateManager;1"].
            getService(Components.interfaces.koITemplateManager);
var globalPrefs = Components.classes["@activestate.com/koPrefService;1"]
            .getService(Components.interfaces.koIPrefService).prefs;
var dialog;
var templates;

function openTemplatesFolder()  {
    var selectedFolder = templateMgr.getUserTemplateDirectory();
    var os = Components.classes["@activestate.com/koOs;1"].getService(Components.interfaces.koIOs);
    var infoService = Components.classes['@activestate.com/koInfoService;1'].
        getService(Components.interfaces.koIInfoService);
  
    if (infoService.isWindows) {
        os.startfile(selectedFolder);
    } else {
        curDir = os.getcwd();
        os.chdir(selectedFolder);
        os.system('xterm &');
        os.chdir(curDir);
    }
}

function OnLoad()  {
    // set up UI elements
    dialog = {};
    dialog.directoryTree = document.getElementById('directoryTree');
    dialog.templateTree = document.getElementById('templateTree');
    
    koDoSetOKCancel(doOk, doCancel);    
    var hierarchy = templateMgr.getTemplateHierarchy();
    var directories = buildDirectories(hierarchy, true);
    dialog.directoryTree.appendChild(directories);
    
    // Folder selection
    var selectedFolder = globalPrefs.getStringPref('defaultTemplatesFolder');
    var folder = document.getElementById('dir'+selectedFolder);
    if (folder != null)  {
        
        dialog.directoryTree.selectItem(folder);
    }
    else  {
        // select the first by default
        dialog.directoryTree.selectItem(dialog.directoryTree.lastChild.childNodes[0]);
    }

    // File selection
    var selectedFile = globalPrefs.getStringPref('defaultTemplatesFile');
    var template = templates.getElementsByAttribute('id', 'file'+selectedFile)[0];
    if (template != null)
        dialog.templateTree.selectItem(template);
    else
        dialog.templateTree.selectItem(templates.firstChild);
    dialog.templateTree.focus();
}

function doOk()  {
    if (!openTemplate())
        throw("Could not open the selected template");
    persistSelected();    
    return true;    
}

function doCancel()  {
    window.arguments[0].template = null;
    return true;
}

function persistSelected()  {
    // persist the selected templates folder and file
    var selectedItem;
    var currentSelection;
    selectedItem = dialog.directoryTree.selectedItems[0];
    currentSelection = selectedItem.getAttribute("id");
    var folder = currentSelection.substring(3, currentSelection.length);
    globalPrefs.setStringPref('defaultTemplatesFolder', folder);
    selectedItem = dialog.templateTree.selectedItems[0];
    currentSelection = selectedItem.getAttribute("id");
    var file = currentSelection.substring(4, currentSelection.length);
    globalPrefs.setStringPref('defaultTemplatesFile', file);
    return;
}

function buildDirectories(hierarchy, toplevel)  {
    var treechildren, treeitem, treerow, treecell;
    var newchildren, newitem, newrow, newcell;
    var children = new Object();
    var count = new Object();
    var i, child;
    
    if (toplevel)  {
        hierarchy.getChildren(children, count);
        children = children.value;
        
        newchildren = document.createElementNS(XUL_NS, 'treechildren');
        newchildren.setAttribute('flex', '1');
        for (i=0;i<children.length;i++)  {
            child = buildDirectories(children[i], false);
            if (child!=null)
                newchildren.appendChild(child);
        }
        return newchildren;
    }    

    if (hierarchy.container == true)  {
        // build tree
        var iscontainer = false;
        treeitem = document.createElementNS(XUL_NS, 'treeitem');
        treeitem.setAttribute('id', 'dir'+hierarchy.name);
        treeitem.hierarchy = hierarchy;
        treerow = document.createElementNS(XUL_NS, 'treerow');
        treecell = document.createElementNS(XUL_NS, 'treecell');
        treecell.setAttribute('id', 'folder-cell');
        treecell.setAttribute('class', 'folder-icon treecell-indent');

        hierarchy.getChildren(children, count);
        children = children.value;
        
        treecell.setAttribute('label', hierarchy.name);
        treerow.appendChild(treecell);
        treeitem.appendChild(treerow);
        
        newchildren = document.createElementNS(XUL_NS, 'treechildren');
        newchildren.setAttribute('flex', '1');
        for (i=0;i<children.length;i++)  {
            child = buildDirectories(children[i], false);
            if (child!=null)
                newchildren.appendChild(child);
            if (children[i].container == true)
                iscontainer = true;
        }
        if (iscontainer)  {
            treeitem.setAttribute('container', 'true');
            treeitem.setAttribute('persist', 'open');
        }
        else  {
            treeitem.removeAttribute('container');
        }
        
        treeitem.appendChild(newchildren);
        
        return treeitem;
    }
    else  {
        return null;
    }
}

function buildTemplates(hierarchy, toplevel)  {
    var treechildren, treeitem, treerow, treecell;
    var newchildren, newitem, newrow, newcell;
    var children = new Object();
    var count = new Object();
    
    hierarchy.getChildren(children, count);
    children = children.value;

    treechildren = document.createElementNS(XUL_NS, 'treechildren');
    treechildren.setAttribute('flex', '1');
    
    for (var i=0;i<children.length;i++)  {
        if (children[i].container == false)  {
            treeitem = document.createElementNS(XUL_NS, 'treeitem');
            treeitem.setAttribute('id', 'file'+children[i].name);
            treeitem.setAttribute('container', 'true');
            treeitem.hierarchy = children[i];
            treerow = document.createElementNS(XUL_NS, 'treerow');
            treecell = document.createElementNS(XUL_NS, 'treecell');
            treecell.setAttribute('id', 'templateCell');
            treecell.setAttribute('label', children[i].name);
            treecell.setAttribute("onclick", "selectTemplate(event)");
            treerow.appendChild(treecell);
            treeitem.appendChild(treerow);
            treechildren.appendChild(treeitem);
        }
    }
    
    return treechildren;
}

function showTemplates(treeitem)  {
    var item;
    if (treeitem)
        item = treeitem;
    else
        item = dialog.directoryTree.selectedItems[0];
    var hierarchy = item.hierarchy;
    templates = buildTemplates(hierarchy, true);
    if (dialog.templateTree.lastChild.nodeName == 'treechildren')
        dialog.templateTree.removeChild(dialog.templateTree.lastChild)
    dialog.templateTree.appendChild(templates);
    dialog.templateTree.selectItem(templates.firstChild);
}

function selectTemplate(event)  {
    if(event.detail == 2)  { // double click
        if (openTemplate())  {
            persistSelected();
            window.close();
        }
        else  {
            throw("Could not open the selected template");
        }
    }
}

function openTemplate()  {
    // A template should always be selected, even if user does not click on one
    var item = dialog.templateTree.selectedItems[0];
    if (item)  {
        var hierarchy = item.hierarchy;
        window.arguments[0].template = hierarchy;
        var addToToolbox = document.getElementById('addToToolbox');
        if (addToToolbox.getAttribute('checked')) {
            window.arguments[0].addToToolbox = true;
        } else {
            window.arguments[0].addToToolbox = false;
        }
        return true;
    }
    else  {
        return false;  
    }
}
