    /* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */


/*
 ****************************************************************
 * View functions
 * 
 * This code hides all concepts of "tabs", and presents a simple "view" interface.
 */

// These globals should only be used by the view functions.
// None of the view user's (eg, the file operations stuff)
// should need to use these.

// The untitled document counter.
var gUntitledCount = 0;

function getNextUntitledName( base /*= "Untitled" */, extension /* = '' */) {
    if (!base) {
        base = "Untitled";
    }

    var rc = base + (++gUntitledCount);
    if (extension) {
        rc += extension;
    }
    return rc;
}

var globalCounter = 1;  // used to generate IDs for viewsin scintillaview.js

// called from menu's built in kombo.js
function doViewAs(language) {
    var v = getFocusedView();
    if (v) v.viewAs(language);
}

// this might return NULL
function getFocusedView() {
    var vm = getFocusedViewManager();
    if (vm) return vm.getCurrentView();
    return viewManager.getCurrentView();
}

function editorHasFocus() {
    // returns true if the focus is in either the main scintilla (whether in the
    // html:embed or the <scintilla> element or in the XSLT input pane scintilla
    var focusedElement = top.document.commandDispatcher.focusedElement;
    var scin = top.document.getElementById('scin');
    var input_scin = top.document.getElementById('input-scin');
    return (childOf(focusedElement, scin) || childOf(focusedElement, input_scin));
}

function getFocusedScintilla() {
    // return the <scintilla> which has focus, or null if neither 
    // the main nor the XSLT input pane scintilla has focus.
    var focusedElement = top.document.commandDispatcher.focusedElement
    var scin = top.document.getElementById('scin');
    if (childOf(focusedElement, scin)) {
        return scin;
    }
    var input_scin = top.document.getElementById('input-scin');
    if (childOf(focusedElement, input_scin)) {
        return input_scin;
    }
    return null;
}

function projectHasFocus() {
    return childOf(top.document.commandDispatcher.focusedElement,
                   top.document.getElementById('projectviewbox'));
}

function toolboxHasFocus() {
    return childOf(top.document.commandDispatcher.focusedElement,
                   top.document.getElementById('toolboxviewbox'));
}

function childOf(child, target) {
    // Returns true if 'child' is a descendant of 'target'
    if (!child || !target) return false;
    while (child) {
        if (child == target) {
            return true;
        }
        child = child.parentNode;
    }
    return false;
}

// this ALWAYS returns a view manager
function getFocusedViewManager() {
    var focusedElement = top.document.commandDispatcher.focusedElement;

    // XXX hack alert
    // seems the latest mozilla is returning the html element inside scintilla, rather than scintilla itself.
    if (focusedElement) {
    
        var vm = null;
        if (focusedElement.tagName=="scintilla" &&
            typeof(focusedElement.viewManager) != "undefined") {
            vm = focusedElement.viewManager;
        } else if ("tagName" in focusedElement.parentNode && 
		   focusedElement.parentNode.tagName=="scintilla" &&
                   typeof(focusedElement.parentNode.viewManager) != "undefined") {
            vm = focusedElement.parentNode.viewManager;
        }

        if (vm) {
            return vm;
        }
        
    }
    // if no focused view, always return the current view for the editor.
    return viewManager;
}


// Utility for hiding a (context) menu item
// Parameters - 1. menuitem id 2. Boolean to show/hide
function HideMenuItem(id, showItem)
{
  var item = document.getElementById(id);
  if (item) {
    item.setAttribute("hidden", showItem);
  }
  else
    logError("HideMenuItem::item id="+id+" not found\n");
}

/* File pane context */
function buildFilesContextMenu(event, contextMenuNode)
{
  if ( event.target != contextMenuNode )
    return;

  var treeNode = document.getElementById('fileTree');
  var select_list = treeNode.selectedItems;
  if (treeNode.selectedItems.length == 1) {
	HideMenuItem('menu_filesTofront_cm', 'false');
	HideMenuItem('menu_fileClose_cm', 'false');
	HideMenuItem('menu_filesClose_cm', 'true');
  }
  else if (select_list.length > 1) {
	HideMenuItem('menu_filesClose_cm', 'false');
	HideMenuItem('menu_filesTofront_cm', 'true');
	HideMenuItem('menu_fileClose_cm', 'true');
  }
  else {
	logError("views.js::buildFilesContextMenu() - No files selected\n");
  }
}

function filesPaneClose() {
  var treeNode = document.getElementById('fileTree');
  var select_list = treeNode.selectedItems;
  var list_size = select_list.length;
  var fileViews = new Array();

  for (k=0; k<list_size; k++)
  {
  	fileViews[k] = viewManager.getViewForFile(select_list[k].id);
  }
  // See doCloseAll(); in fileops.js
  for(viewURL in fileViews) {
      var view = fileViews[viewURL];
      if (!view.manager.closeView(view)) {
          logError("views.js::filesPaneClose() - Could not close file: " + select_list[k].id + "\n");
      }
  }
}

function filesPaneBringToFront(treeitem) {
  var treeNode = document.getElementById('fileTree');
  var select_list = treeNode.selectedItems;
  var url = select_list[0].firstChild.firstChild.getAttribute('url')
  var fileView = viewManager.getViewForFile(url);
  if (fileView) {
    fileView.manager.setCurrentView(fileView);
  }
  else {
    logError("views.js::filesPaneBringToFront() - Could not bring to front file: " + treeitem.id + "\n");
  }
}


function filesPaneCloseFile(treeitem) {
  var treeNode = document.getElementById('fileTree');
  var select_list = treeNode.selectedItems;
  var url = select_list[0].firstChild.firstChild.getAttribute('url')
  var fileView = viewManager.getViewForFile(url);
  if (fileView) {
      if (!fileView.manager.closeView(fileView)) {
          logError("views.js::filesPaneClose() - Could not close file: " + url + "\n");
      }
  }
  else {
    logError("views.js::filesPaneClose() - Could not bring to front file: " + url + "\n");
  }
}


// checkDiskFiles is only called from the window's focus handler,
// located in komodo.xul
var gInCheckDiskFiles = false;
function checkDiskFiles() {
    if (gInCheckDiskFiles) return true;

    var globalPrefService = Components.classes["@activestate.com/koPrefService;1"].
            getService(Components.interfaces.koIPrefService);
    var checkDisk = (globalPrefService.prefs.hasBooleanPref("checkDiskFile") &&
        globalPrefService.prefs.getBooleanPref("checkDiskFile"));

    if (!checkDisk) return true;
    gInCheckDiskFiles = true;
        
    // checks open files and projects for dirtiness
    try {
        var currentvm = getFocusedViewManager();
        var currentview = currentvm.getCurrentView();

        var changedViews = {};
        var removedViews = {};
        var changed = [];
        var gone = [];
        var view, url, i, j, done, change;
        
        // get all opened files that have changed
        for (var vm in gViewManagerList) {
            var views = gViewManagerList[vm].getViews();
            for (url in views) {
                change = views[url].isDiskFileDirty();
                if (change == 1) {
                    changedViews[url] = views[url];
                    changed.push(url);
                } else if (change == -1 && !views[url].isDirty()) {
                    removedViews[url] = views[url];
                    gone.push(url);
                }
            }
        }
        
        // handle files that have changed on disk
        var selected;
        if (changed.length > 0) {
            selected = multiFileDialog('Files Change',
                        'Some files that are open have changed on '+
                        'disk, do you want to reload these files?  '+
                        'Any changes will be lost.', changed);
            for (j in changed) {
                url = changed[j];
                done = false;
                view = changedViews[url];
                view._observedFileChange = 1;
                for (i in selected) {
                    if (selected[i] == url) {
                        done = true;
                        view.manager.setCurrentView(view);
                        view.revert();
                    }
                }
                if (!done) {
                    view.py_inst.diskFileDirty=1;
                    koRequestUpdateCommands("save");
                    view.manager.indicateIfDirty(view);
                }
                view._observedFileChange = 0;
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // handle files that have been removed from disk
        if (gone.length > 0) {
            selected = multiFileDialog('Files Removed',
                        'Some files that are open have been removed '+
                        'from disk, do you want to re-save these '+
                        'files now?', gone);
            for (j in gone) {
                url = gone[j];
                done = false;
                view = removedViews[url];
                view._observedFileChange = 1;
                for (i in selected) {
                    if (selected[i] == url) {
                        done = true;
                        view.dirtySave();
                    }
                }
                if (!done) {
                    view.untitled = true; // forces a saveas
                    view.py_inst.diskFileDirty=1;
                    koRequestUpdateCommands("save");
                    view.manager.indicateIfDirty(view);
                }
                view._observedFileChange = 0;
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // get all opened projects that have changed
        var projects = gProjectManager._projects;
        var project;
        changed = [];
        gone = [];
        changedViews = [];
        removedViews = [];
        for (i = 0; i < projects.length; i++) {
            url = projects[i].url
            change = gFileStatusSvc.hasFileChanged(url);
            if (change == 1) {
                changedViews.push(i);
                changed.push(url);
            } else if (change == -1 && !projects[i].part.isDirty) {
                removedViews.push(i);
                gone.push(url);
            }
        }

        // handle files that have changed on disk
        if (changed.length > 0) {
            selected = multiFileDialog('Projects Changed',
                        'Some projects that are open have changed on disk, '+
                        'do you want to reload these projects?  '+
                        'Any changes will be lost.', changed);
            for (j in changedViews) {
                url = projects[j].url;
                done = false;
                for (i in selected) {
                    if (selected[i] == url) {
                        gProjectManager.closeProjectEvenIfDirty(j);
                        gProjectManager.loadProject(url)
                        done = true;
                    }
                }
                if (!done) {
                    projects[j].part.isDirty = true;
                }
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // handle files that have been removed from disk
        if (gone.length > 0) {
            selected = multiFileDialog('Projects Removed',
                        'Some projects that are open have been removed '+
                        'from disk, do you want to re-save these '+
                        'projects now?', gone);
            for (j in removedViews) {
                url = projects[j].url;
                done = false;
                for (i in selected) {
                    if (selected[i] == url) {
                        gProjectManager.saveProject(j);
                        done = true;
                    }
                }
                if (!done) {
                    projects[j].part.isDirty = true;
                }
                gObserverSvc.Notify(null,'file_changed',url)
            }
        }
        
        // reset the the last used manager and view!
        if (currentvm && currentvm != getFocusedViewManager()) {
            currentvm.setFocus();
        }
        if (currentview && currentview != currentvm.getCurrentView()) {
            currentvm.setCurrentView(currentview);
        }
    } catch(e) {
        logError(e);
    }
    gInCheckDiskFiles = false;
    return true;
}


/**
 * take the text from the debugger output scintilla, put it into the iframe, and switch
 */
var outputTmpFile=null;
              
function renderOutputAsHTML() {
    var swstack = document.getElementById("redirect-stack");
    var outputIndex = parseInt(swstack.getAttribute("index"));
    var scout = document.getElementById("redirect-scin").ISciMoz;
    var browser = document.getElementById("redirect-blank");
    var flags = nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
    if (outputTmpFile) {
        delete outputTmpFile;
        outputTmpFile = null;
    }
    if (outputIndex == 0) {
        outputIndex = 1;
        OutputWindow_Init()
        swstack.setAttribute("index", outputIndex);
        // force reload of blank
        //browser.webNavigation.loadURI('about:blank', flags);
    } else {
        var filename = dbgd.debugFileName;
        var basedir = null;
        if (filename) basedir = os.path.dirname(filename);
        
        // get the text from scintilla, stuff it into the iframe, and switch the deck
        var text = scout.text;
        if (text) {
            var tmpFileSvc = Components.classes["@activestate.com/koTempFileFactory;1"].
                getService(Components.interfaces.koITempFileFactory);

            if (basedir) {
                outputTmpFile = tmpFileSvc.MakeTempFileInDir(basedir,".html","w");
            } else {
                outputTmpFile = tmpFileSvc.MakeTempFile(".html","w");
            }

            outputTmpFile.puts(text);
            outputTmpFile.flush();
            outputTmpFile.close();
            var outputURL = outputTmpFile.file.URL;
            browser.webNavigation.loadURI(outputURL, flags);
            outputIndex = 0;
            swstack.setAttribute("index", outputIndex);
        }
    }
}


/**
 * view the text scintilla editor as html
 */
 var gViewHTMLTab = false;
function doViewAsHTMLTabSwitch(tab) {
    if (tab.id == 'view_html_tab') {
        if (gViewHTMLTab) {
            document.getElementById('html-view-editor').focus();
            return;
        }
        gViewHTMLTab = true;
        viewAsHTML();
        document.getElementById('html-view-editor').focus();
    } else if (gViewHTMLTab) {
        gViewHTMLTab = false;
        viewManager.setFocus();
        document.getElementById("html-view-editor").removeAttribute('src');
    } else {
        viewManager.setFocus();
    }
}

function viewAsHTML() {
    var view = viewManager.getCurrentView();
    if (!view) return;
    
    var browser = document.getElementById("html-view-editor");
    var flags = nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
    if (outputTmpFile) {
        delete outputTmpFile;
        outputTmpFile = null;
    }
    
    // get the text from scintilla, stuff it into the iframe, and switch the deck
    var text = view._scimoz.text;
    var filename = null;
    if (view.url) {
        try {
            filename = URLToLocalFile(view.url).path;
        } catch (e) {
            // This fails if the file doesn't exist!
            ;
        }
    } 
    
    // put the temp file in the basedir for the url
    var basedir = null;
    // os defined in koGlobalOverlay.js
    if (filename) basedir = os.path.dirname(filename);
    
    if (text) {
        var tmpFileSvc = Components.classes["@activestate.com/koTempFileFactory;1"].
            getService(Components.interfaces.koITempFileFactory);
        if (basedir) {
            outputTmpFile = tmpFileSvc.MakeTempFileInDir(basedir,".html","w");
        } else {
            outputTmpFile = tmpFileSvc.MakeTempFile(".html","w");
        }
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        var outputURL = outputTmpFile.file.URL;
        browser.webNavigation.loadURI(outputURL, flags);
    }
}



