/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Find and Replace results tab control (a tab in the output pane)
 *
 * TODO:
 *  - key bindings to jump to next find result in set (VS.NET does not have
 *    this either)
 *  - try to refind results that have moved from buffer editting since search
 */


//---- internal utility routines

function _OptionDesc(options, pattern)
{
    var optionDesc = "";
    if (options.matchWord) {
        optionDesc += " Match whole word.";
    }
    if (options.caseSensitivity == options.FOC_SENSITIVE
        ||
         (
           options.caseSensitivity == options.FOC_SMART
           &&
           pattern.toLowerCase() != pattern
         )
       )
    {
        optionDesc += " Match case.";
    }
    if (options.ignoreHiddenText) {
        optionDesc += " Ignore hidden text.";
    }
    return optionDesc;
}



//---- interface routines

function FindResultsTab_OnLoad()
{
    // Should be called in the onload handler for the window containing the
    // find results tab XUL.
    gFindResultsView = Components.classes[
        '@activestate.com/koFindResultsView;1']
        .createInstance(Components.interfaces.koIFindResultsView);
    var outlinerWidget = document.getElementById("findresults");
    outlinerWidget.boxObject
                  .QueryInterface(Components.interfaces.nsIOutlinerBoxObject)
                  .view = gFindResultsView;
    FindResultsTab_Clear(window);
}


function FindResultsTab_Clear(mainWindow)
{
    // Clear the list of find results.
    mainWindow.gFindResultsView.Clear(0);
    _UpdateSortIndicators(mainWindow, null);
    var descWidget = mainWindow.document.getElementById("findresults-desc");
    descWidget.setAttribute("value", "No current find/replace results.");
    descWidget.removeAttribute("baseDesc");
    mainWindow.document.getElementById('findresults-findnext-button').setAttribute('disabled', 'true')
}


function FindResultsTab_DescribeFind(mainWindow, pattern, context, options, patternAlias)
{
    // Set the tab name appropriately.
    var tabWidget = mainWindow.document.getElementById("findresults_tab");
    tabWidget.setAttribute("label", "Find Results");

    // Add a description of what was found.
    var descWidget = mainWindow.document.getElementById("findresults-desc");
    if (!patternAlias) {
        patternAlias = "'" +pattern + "'";
    }
    var baseDesc = "Found all " + patternAlias + " in " + context.name + "."
                   + _OptionDesc(options, pattern);

    descWidget.setAttribute("baseDesc", baseDesc);
    descWidget.setAttribute("value", baseDesc);

    var filenameColWidget = mainWindow.document
                            .getElementById("findresults-filename");
    if (context.type == Components.interfaces.koIFindContext.FCT_CURRENT_DOC
        || context.type == Components.interfaces.koIFindContext.FCT_SELECTION)
    {
        filenameColWidget.setAttribute("hidden", "true");
    } else {
        filenameColWidget.setAttribute("hidden", "false");
    }
    mainWindow.document.getElementById('findresults-findnext-button').removeAttribute('disabled')
}


function FindResultsTab_DescribeReplace(mainWindow, pattern, replacement,
                                        context, options)
{
    // Set the tab name appropriately.
    var tabWidget = mainWindow.document.getElementById("findresults_tab");
    tabWidget.setAttribute("label", "Replace Results");

    // Add a description of what was replaced.
    var descWidget = mainWindow.document.getElementById("findresults-desc");
    var baseDesc = "Replaced all '" + pattern + "' with '" + replacement
                   + "' in " + context.name + "."
                   + _OptionDesc(options, pattern);

    descWidget.setAttribute("baseDesc", baseDesc);
    descWidget.setAttribute("value", baseDesc);

    var filenameColWidget = mainWindow.document
                            .getElementById("findresults-filename");
    if (context.type == Components.interfaces.koIFindContext.FCT_CURRENT_DOC
        || context.type == Components.interfaces.koIFindContext.FCT_SELECTION)
    {
        filenameColWidget.setAttribute("hidden", "true");
    } else {
        filenameColWidget.setAttribute("hidden", "false");
    }
    mainWindow.document.getElementById('findresults-findnext-button').removeAttribute('disabled')
}


function FindResultsTab_AddFindResult(mainWindow, url, startIndex, endIndex,
                                      value, fileName, lineNum, columnNum,
                                      context)
{
    //XXX should raise exception if not in find all "mode"
    mainWindow.gFindResultsView.AddFindResult(url, startIndex, endIndex, value,
                                              fileName, lineNum, columnNum,
                                              context);
    var descWidget = mainWindow.document.getElementById("findresults-desc");
    var desc = descWidget.getAttribute("baseDesc");
    var numHits = mainWindow.gFindResultsView.rowCount;
    if (numHits == 1) {
        desc += " (1 result found";
    } else {
        desc += " (" + numHits + " results found";
    }
    var numUrls = mainWindow.gFindResultsView.GetNumUrls();
    if (numUrls == 1) {
        desc += " in 1 file)";
    } else {
        desc += " in " + numUrls + " files)";
    }
    descWidget.setAttribute("value", desc);
}


function FindResultsTab_AddReplaceResult(mainWindow, url, startIndex, endIndex,
                                         value, replacement, fileName, lineNum,
                                         columnNum, context)
{
    //XXX should raise exception if not in replace all "mode"
    mainWindow.gFindResultsView.AddReplaceResult(url, startIndex, endIndex,
                                                 value, replacement,
                                                 fileName, lineNum, columnNum,
                                                 context);
    var descWidget = mainWindow.document.getElementById("findresults-desc");
    var desc = descWidget.getAttribute("baseDesc");
    var numHits = mainWindow.gFindResultsView.rowCount;
    if (numHits == 1) {
        desc += " (1 replacement made";
    } else {
        desc += " (" + numHits + " replacements made";
    }
    var numUrls = mainWindow.gFindResultsView.GetNumUrls();
    if (numUrls == 1) {
        desc += " in 1 file)";
    } else {
        desc += " in " + numUrls + " files)";
    }
    descWidget.setAttribute("value", desc);
}


function FindResultsTab_Show(mainWindow)
{
    // open output pane in komodo.xul
    var splitterWidget = mainWindow.document
                         .getElementById("redirect-splitter");
    splitterWidget.removeAttribute('state');
    var viewMenuItem = mainWindow.document.getElementById("menu_viewOutput");
    viewMenuItem.setAttribute("checked", "true");        
    
    // switch to "Find Results" tab
    var tabsWidget = mainWindow.document.getElementById("oTabbox");
    var tabWidget = mainWindow.document.getElementById("findresults_tab");
    tabsWidget.selectedTab = tabWidget; 

    // (controversial?) give the find results tab the focus
    //XXX does not work because the input buffer is passing focus back to the
    //editor
    var findResultsWidget = mainWindow.document
                            .getElementById("findresults-body");
    findResultsWidget.focus()
}


function FindResultsTab_OnClick(event)
{
    // c.f. mozilla/mailnews/base/resources/content/threadPane.js
    var t = event.originalTarget;

    // single-click on a column
    if (t.localName == "outlinercol") {
        _FindResultsTab_ColumnClick(t.id);
    }

    // double-click in the outliner body
    else if (event.detail == 2 && t.localName == "outlinerbody") {
        _FindResultsTab_DoubleClick();
    }
}


function _FindResultsTab_ColumnClick(columnID)
{
    gFindResultsView.Sort(columnID);
    _UpdateSortIndicators(window, columnID);
}


function FindResultsTab_NextResult() {
    var outlinerWidget = document.getElementById("findresults");
    var i = outlinerWidget.currentIndex;
    if (i == -1 || i >= outlinerWidget.boxObject.view.rowCount-1) {
        i = 0;
    } else {
        i += 1;
    }
    outlinerWidget.boxObject.view.selection.currentIndex = i;
    outlinerWidget.boxObject.view.selection.select(i);
    outlinerWidget.boxObject.ensureRowIsVisible(i);
    _FindResultsTab_DoubleClick();
    
}

function _FindResultsTab_DoubleClick()
{
    // Jump to the find/replace result.

    // Open and/or switch to the appropriate file.
    var outlinerWidget = document.getElementById("findresults");
    var i = outlinerWidget.currentIndex;
    doFileOpen(gFindResultsView.GetUrl(i));
    var view = window.getFocusedView();
    var scimoz = view._scimoz;

    // Try to find the match or replacement result. If it cannot be
    // found then just go to the start of the line where it used to be.
    // XXX Could try and be more sophisticated in the search.
    var start = gFindResultsView.GetStartIndex(i);
    var end = gFindResultsView.GetEndIndex(i);
    var value = gFindResultsView.GetValue(i);
    var replacement = gFindResultsView.GetReplacement(i);
    // - first try the original indeces (if the file has not changed the result
    //   should still be there)
    //   XXX This *can* fluke to the wrong result if the buffer has changed.
    //       The *correct* answer would be to only attempt this if the buffer
    //       is identical to when the find-/replace-all was done.
    if (scimoz.getTextRange(start, end) == value) {
        //dump("Jump To Find Result: found at the original indices\n");
        scimoz.selectionStart = start;
        scimoz.selectionEnd = end;
    }
    // - next, try using the line and column number to find it again (this will
    //   survive basic changes in the document that do not add or remove lines)
    else {
        var lineNum = gFindResultsView.GetLineNum(i);
        var columnIndex = gFindResultsView.GetColumnNum(i);
        var lineStartIndex = scimoz.positionFromLine(lineNum-1);
        var possibleValue = scimoz.getTextRange(lineStartIndex+columnIndex,
                                                lineStartIndex+columnIndex
                                                +value.length);
        var possibleReplacement = scimoz.getTextRange(lineStartIndex+columnIndex,
                                                      lineStartIndex+columnIndex
                                                      +replacement.length);
        if (possibleValue == value) {
            //dump("Jump To Find Result: found value by line:col\n");
            scimoz.selectionStart = lineStartIndex+columnIndex;
            scimoz.selectionEnd = lineStartIndex+columnIndex+value.length;
        } else if (replacement != "" && possibleReplacement == replacement) {
            //dump("Jump To Find Result: found replacement, '"+replacement+"', by line:col\n");
            scimoz.selectionStart = lineStartIndex+columnIndex;
            scimoz.selectionEnd = lineStartIndex+columnIndex+replacement.length;
        }

        // - next, try searching within the current line
        //   XXX Note, this bails if there is more than one possible match on
        //       the current line. This could attempt to be more intelligent,
        //       like perhaps selecting the match that is closest to the
        //       original column number.
        else {
            var lineText = scimoz.getTextRange(lineStartIndex,
                            scimoz.getLineEndPosition(lineNum-1));
            var valueIndex = lineText.indexOf(value);
            var replacementIndex = lineText.indexOf(replacement);
            if (valueIndex != -1
                && lineText.lastIndexOf(value) == valueIndex)
            {
                //dump("Jump To Find Result: found value by searching in the current line\n");
                scimoz.selectionStart = lineStartIndex+valueIndex;
                scimoz.selectionEnd = lineStartIndex+valueIndex+value.length;
            } else if (replacement != "" && replacementIndex != -1
                       && lineText.lastIndexOf(replacement) == replacementIndex)
            {
                //dump("Jump To Find Result: found replacement by searching in the current line\n");
                scimoz.selectionStart = lineStartIndex+replacementIndex;
                scimoz.selectionEnd = lineStartIndex+replacementIndex+replacement.length;
            }

            // - XXX next, could try to search a couple lines above and below
            //   but for now will just give up
            else {
                //dump("Jump To Find Result: did not find it, just using current line\n");
                scimoz.selectionStart = lineStartIndex;
                scimoz.selectionEnd = lineStartIndex;

                // Let the user know about the problem on the status bar.
                StatusBar_AddMessage(
                    "The specified text has been moved or deleted.", "find",
                    3000, true);
            }
        }
    }

    // be sure we store the selection
    start = scimoz.selectionStart;
    end = scimoz.selectionEnd;
    
    // restore the selection
    scimoz.selectionStart = start;
    scimoz.selectionEnd = end;
    
    // If the invoked find result is the last visible one and there are more,
    // then scroll the list of find results by one.
    // XXX Could also scroll up if double-clicking on top visible item.
    var box = outlinerWidget.boxObject;
    if (box.getLastVisibleRow()-1 == i && gFindResultsView.rowCount-1 > i) {
        box.scrollByLines(1);
    }

    DoTheScimozWiggle(scimoz);

    // transfer focus to the editor
    view.manager.setFocus();
}


function FindResultsTab_OnKeyPress(event)
{
    if (event.keyCode == 13) {
        _FindResultsTab_DoubleClick();
        return false;
    }
    return true;
}


function _UpdateSortIndicators(mainWindow, sortId)
{
    var sortedColumn = null;

    // set the sort indicator on the column we are sorted by
    if (sortId) {
        sortedColumn = mainWindow.document.getElementById(sortId);
        if (sortedColumn) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // remove the sort indicator from all the columns
    // except the one we are sorted by
    var currCol = mainWindow.document.getElementById("findresults").firstChild;
    while (currCol) {
        while (currCol && currCol.localName != "outlinercol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


