/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

const _DEBUG = false;

function initPanel ( aPrefTag )
{
    hPrefWindow.onpageload( aPrefTag )
}

function _getPrefAttributeForElement(elt) {
    var prefattr = elt.getAttribute("prefattribute");
    if (prefattr) // believe it :)
        return prefattr;
    // use a default
    switch (elt.tagName) {
        case "menulist":
        case "textbox":
            return "value";
        case "radio":
        case "radiogroup":
        case "checkbox":
            return "checked";
        default:
            logError("element '" + elt.getAttribute("id") + "' is of unknown type '" + elt.tagName + "' and has no 'prefattribute' tag");
    }
    return "value"; // I guess!
}

window.doneLoading = false; 
 
function koPrefWindow( frame_id, prefset /* = null */ )
{
    this.contentFrame   = frame_id
    this.prefset           = null;
    this.prefservice      = null;
    this.cancelHandlers = [];
    this.okHandlers     = [];
    this.customPageData = [];
    this.currentPageTag = null;
  
    this.onload();
    this.init(prefset);
}

koPrefWindow.prototype =
{
    onload:	function () {
        // we really need a prefset per panel, but will always need a "global".
        try {
            this.prefservice = Components.classes["@activestate.com/koPrefService;1"].
                getService(Components.interfaces.koIPrefService);
        } catch(e) {
            dump("*** Failed to create preference service object\n");
            return;
        }
    },
    
    init: function (prefset /* = null */) {
        // setup the prefset.  If not specified, we use the global prefs.
        this.save_prefs_service = false;
        if (!prefset) {
            prefset = this.prefservice.prefs;
            this.save_prefs_service = true;
        }
        // Clone the prefset, so we can update that on the fly.
        // When OK is pressed, we update the original with our
        // modified preferences.
        this.prefset = prefset.clone();
        this.orig_prefset = prefset;

        // debug
        //cnt=new Object(); ids=new Object();this.prefset.getPrefIds(ids, cnt);
        //dump("Pref IDs using are " + ids.value + "\n");
    },

    _doOKHandlers: function() {
        for( var i = 0; i < this.okHandlers.length; i++ ) {
            var prefset = this._getCurrentPrefSet();
            var customData = this.customPageData[this.okHandlers[i].tag];
            if (!this.okHandlers[i].func(prefset, customData)) {
                logDebug("OK handler for " + this.okHandlers[i].tag + " returned false - not closing dialog");
                return false;
            }
        }
        return true;
    },
    
    _doCancelHandlers: function() {
        for( var i = 0; i < this.cancelHandlers.length; i++ ) {
            var prefset = this._getCurrentPrefSet();
            var customData = this.customPageData[this.cancelHandlers[i].tag];
            if (!this.cancelHandlers[i].func(prefset, customData)) {
                logDebug("Cancel handler for " + this.cancelHandlers[i].tag + " returned false - not closing dialog");
                return false;
            }
        }
        return true;
    },
    
    onOK: function () {
        // Save the prefs to our temp prefset, so the OK handlers see the new values.
        if (!this.savesinglepageprefs())
            return false;
        
        if (!this._doOKHandlers())
            return false;

        var prefPanelTree = document.getElementById( "prefsTree" );
        var selectedItem = prefPanelTree.selectedItems[0];
        var currentSelection = selectedItem.getAttribute("id");
        this.setPref('defaultSelectedPref', currentSelection);

        this.savePrefs();
        this.setResult('ok');
        close();
        return true;
    },

    onCancel: function () {
        if (!this._doCancelHandlers())
            return false;
        
        this.setResult('cancel');
        close();
        return true;
    },

    setResult: function (res) {
        if (window.arguments && window.arguments[1])
            window.arguments[1].res=res;
        return true;
    },	

    registerOKCallbackFunc: function ( aFunctionReference, aPrefTag ) {
        // check not already there
        if (aPrefTag) {
            for (var i=0;i<this.okHandlers.length;i++)
                if (this.okHandlers[i].tag == aPrefTag)
                    return;
        }
        this.okHandlers[this.okHandlers.length] = {
            func: aFunctionReference,
            tag: aPrefTag
        };
    },

    registerCancelCallbackFunc: function ( aFunctionReference, aPrefTag ) {
        // check not already there
        if (aPrefTag) {
            for (var i=0;i<this.cancelHandlers.length;i++)
                if (this.cancelHandlers[i].tag == aPrefTag)
                    return;
        }
        this.cancelHandlers[this.cancelHandlers.length] = {
            func: aFunctionReference,
            tag: aPrefTag
        };
    },

    // might end up with per-page prefsets.
    _getCurrentPrefSet: function() {
        return this.prefset;
    },

    // Get the DOM element that is the parent of the current pref widgets
    // (ie, the object we can enumerate to find the fields.
    _getPrefElementsParent: function() {
        var element_parent = document;
        if (this.contentFrame) {
            /***
            dump("content frame is '" + this.contentFrame + "'\n");
            dump("Frame[0] is " + window.frames[0] + "\n");
            dump("Frame[0].doc is " + window.frames[0].document + "\n");
            koDumpDOM(window.frames[0]);
            element_parent = window.frames[this.contentFrame].document;
            dump("got the element parent as " + element_parent + "\n");
            koDumpDOM(element_parent);
            ***/
            element_parent = window.frames[0].document; // XXXX - WTF is the right thing here!?!?!?!
        }
        return element_parent;
    },

    _setElementFromPrefValue: function(elt) {
        // Set a UI element from its preference value.
        // Returns the preference value (which may not be the label on the widget,
        // especially for menu lists, radios, etc
        var prefstring    = elt.hasAttribute("prefstring") ? elt.getAttribute( "prefstring" ) : elt.id;
        var prefvalue = this.getPref(prefstring);
        var prefattribute;
        var items, item, i;
        switch (elt.tagName) {
        case "menulist":
            prefattribute = _getPrefAttributeForElement(elt);
            var popup = elt.getElementsByTagName("menupopup")[0];

            items = elt.getElementsByTagName("menuitem");
            //dump("menulist has " + items.length + " items, prefvalue is " + prefvalue + "\n");
            for (i=0;i<items.length;i++) {
                item = items[i];
                //dump("Item value is " + item.getAttribute("value") + ", label is " + item.getAttribute("label") + "\n");
                if (item.getAttribute(prefattribute) == prefvalue) {
                    //dump("set menu popup to " + item.getAttribute("label") + "\n");
                    elt.selectedItem = item;
                    elt.setAttribute("value", prefvalue);
                    return prefvalue;
                }
            }
            break;

        case "radiogroup":
            prefattribute = _getPrefAttributeForElement(elt);
            items = elt.getElementsByTagName("radio");
            for (i=0;i<items.length;i++) {
                item = items[i];
                if (item.getAttribute(prefattribute) == prefvalue) {
                    //dump("preval is "+prefvalue+" Item value is " + item.getAttribute(prefattribute) +
                    //            ", label is " + item.getAttribute("label") + "\n");
                    item.checked = true;
                    elt.selectedItem = item;
                    return prefvalue;
                }
            }

        default:
            if (elt.getAttribute("prefwidget") == "true") {
                elt.value = prefvalue;
                return prefvalue;
            } else {
                prefattribute = _getPrefAttributeForElement(elt);
                // XX - is this needed?
                // seems a shame that textboxes are different?
                //dump("Set pref '" + prefstring + "' to value '" + prefvalue + "'\n");
                if (elt.tagName == "textbox" && prefattribute == "value") {
                    elt.value = prefvalue;
                } else {
                    elt.setAttribute(prefattribute, prefvalue);
                }
                return prefvalue;
            }
        }
        logError("Could not set widget '" + elt.id + "' (pref id '" + prefstring + "') to value '" + prefvalue + "'");
        throw("Invalid preference");
        return null; // shut-up strict mode.
    },

    _getPrefValueFromElement: function(elt) {
        // Given a UI widget, get the value for the preference.
        //dump("_getPrefValue for ID " + elt.tagName + "\n");
        var prefattribute, items, i;
        switch (elt.tagName) {
        case "menulist":
            prefattribute = _getPrefAttributeForElement(elt);
            var popup = elt.getElementsByTagName("menupopup")[0];
            var popup_value = elt.getAttribute("label");
            
            items = elt.getElementsByTagName("menuitem");
            for (i=0;i<items.length;i++) {
                var item = items[i];
                if (item.getAttribute("label") == popup_value) {
                    //dump("preference value for popup is " + item.getAttribute("value") + "\n");
                    return item.getAttribute(prefattribute);
                }
            }
            break;

        case "radiogroup":
            prefattribute = _getPrefAttributeForElement(elt);
            items = elt.getElementsByTagName("radio");
            if (elt.selectedItem) {
                //dump("radiogroup has " + items.length + " items, selected item is "+elt.selectedItem.getAttribute(prefattribute)+"\n");
                return elt.selectedItem.getAttribute(prefattribute)
            }
            break;

        default:
            if (elt.getAttribute( "prefwidget" )=="true") {
                return elt.value;
            } else {
                var prefattr = _getPrefAttributeForElement(elt);
                var value;

                //XXX is the special handling of textboxes really needed?
                if (elt.tagName == "textbox" && prefattr == "value")
                    value = elt.value;
                else
                    value = elt.getAttribute( prefattr );
                //dump("widget '" + elt.id + "set preference val to '" + value + "'\n");
                return value;
            }
        }
        logError("Could not get preference value from element '" + elt.id + "'");
        throw("Invalid preference");
        return null; // shut-up strict mode.
    },

    getPref: function (aPrefString) {
        var prefs = this._getCurrentPrefSet();		
        var aPrefType = prefs.getPrefType(aPrefString);
        if (aPrefType == null) {
            var msg = "pref.id is UNDEFINED, aPrefString = " + aPrefString;
            logError(msg);
            throw msg;
        }
        switch ( aPrefType ) {
        case "boolean":
            return prefs.getBooleanPref(aPrefString);
            break;
        case "long":
            return prefs.getLongPref(aPrefString);
            break;
        case "double":
            return prefs.getDoublePref(aPrefString);
            break;
        case "string":
            return prefs.getStringPref(aPrefString);
            break;
        default:
            return prefs.getPref(aPrefString);
            break;
        }
        return null;  // shutup strict mode
    },
    
    setPref: function ( prefId, value ) {
        var prefs = this._getCurrentPrefSet();
        var prefType = prefs.getPrefType(prefId);
        if (prefType == "boolean") {
            if (typeof(value)=="string")
                value = value == "true" ? true : false;
            prefs.setBooleanPref(prefId, value);
        } else if (prefType == "long") {
            prefs.setLongPref(prefId, value);
        } else if (prefType == "double") {
            prefs.setDoublePref(prefId, value);
        } else if (prefType == "string") {
            prefs.setStringPref(prefId, value);
        } else {
            throw "Unknown preference type ("+prefType+")";
        }
    },

    savePrefs: function () {
        this.orig_prefset.update(this.prefset);
        if (this.save_prefs_service) {
            this.prefservice.saveState();
        }
    },

    _doPageUnload: function() {
        if (!this.savesinglepageprefs())
            return false;
        var prefset = this._getCurrentPrefSet();
        var pageTag = this.currentPageTag;

        // Now the preference set is up to date, call any custom page unloaders to update custom data.
        if( pageTag && this.contentFrame && ('OnPreferencePageUnloading' in window.frames[ this.contentFrame ])) { // is there a start function.
            var custom_data = this.customPageData[pageTag];
            window.frames[ this.contentFrame ].OnPreferencePageUnloading(prefset, custom_data);
        }
        return true;
    },

    switchPage: function () {
        var prefPanelTree = document.getElementById( "prefsTree" );
        var selectedItem = prefPanelTree.selectedItems[0];

        var oldURL = document.getElementById( this.contentFrame ).getAttribute("src");
        var newURL = selectedItem.firstChild.firstChild.getAttribute("url");
        if (newURL == oldURL)
            return;

        if (oldURL)
            this._doPageUnload();
        document.getElementById(this.contentFrame).setAttribute("src", newURL);
        // Page OnLoad scripts fire, which end up calling initPage()
        // (which should die - may as well just auto-trigger.)
    },

    // return the widget on the current page with the pref ID, or null.
    findElementWithPreferenceId: function(prefId) {
        var elements = this._getPrefElementsParent().getElementsByAttribute("pref", "true");
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            var widgetPrefId = element.hasAttribute("prefstring") ? element.getAttribute("prefstring") : element.id;
            if (prefId == widgetPrefId)
                return element;
        }
        return null;
    },

    onSinglePageOK: function () {
        if (!this._doPageUnload())
            return false;
        if (!this._doOKHandlers())
            return false;
        this.savePrefs();
        return true;
    },

    savesinglepageprefs: function () {
        var elements = this._getPrefElementsParent().getElementsByAttribute( "pref", "true" );
        var rc = true;
        for( var i = 0; i < elements.length; i++ ) {
            var element = elements[i];
            var prefId = element.hasAttribute("prefstring") ? element.getAttribute("prefstring") : element.id;

            try {
                var value = this._getPrefValueFromElement(element);
                var old_value = this.getPref(prefId);
                if (old_value != value) {
                    //dump("savesinglepageprefs: setting preference ID '" + prefId + "' to value '" + value +"'\n");
                    this.setPref(prefId, value);
                }
            } catch (e) {
                logError("Could not save pref '" + prefId + "' to value '" + value + "': " + e + "\n");
                rc = false; // but keep updating what prefs we can.
            }
        }
        // look for pref-widgets and save the prefs
        var prefWidgets = this._getPrefElementsParent().getElementsByAttribute( "prefwidget", "true" );
        for( i = 0; i < prefWidgets.length; i++ ) {
            var widget = prefWidgets[i];
            //dump("calling saveprefs for "+widget.getAttribute("prefstring")+"\n");
            widget.saveprefs(this);
        }
        return rc;
    },

    onsinglepageload: function () {
        this.onpageload(null);
    },

    onpageload: function ( aPageTag ) {
        this.currentPageTag = aPageTag;
        var prefset = this._getCurrentPrefSet();
        // First see if we need the once-only "OnPreferencePageInitalize" function.
        if ( aPageTag && !(aPageTag in this.customPageData) ) { // no custom page data - first time we have seen it.
            var new_custom_data = null;
            if( this.contentFrame && ('OnPreferencePageInitalize' in window.frames[ this.contentFrame ])) { // is there a start function.
                try {
                    new_custom_data = window.frames[ this.contentFrame ].OnPreferencePageInitalize(prefset);
                } catch (e) {
                    logError("Calling 'OnPreferencePageInitalize' function for page '" + aPageTag + "' + failed: " + e);
                }
            }
            this.customPageData[aPageTag] = new_custom_data;
        }
        
        // look for pref-widgets and initialize them
        var prefWidgets = this._getPrefElementsParent().getElementsByAttribute( "prefwidget", "true" );
        for( var i = 0; i < prefWidgets.length; i++ ) {
            var widget = prefWidgets[i];
            //dump("calling onpageload for "+widget.getAttribute("prefstring")+"\n");
            widget.onpageload(this);
        }

        // Setup all our widget values.
        var prefElements = this._getPrefElementsParent().getElementsByAttribute( "pref", "true" );
        //dump("page load loading " + prefElements.length + " items\n");
        for( i = 0; i < prefElements.length; i++ ) {
            var element = prefElements[i];
            this._setElementFromPrefValue(element);
        }
        // Call the page loading function.
        if( aPageTag && this.contentFrame && ('OnPreferencePageLoading' in window.frames[ this.contentFrame ])) { // is there a start function.
            var custom_data = this.customPageData[aPageTag];
            window.frames[ this.contentFrame ].OnPreferencePageLoading(prefset, custom_data);
        }
    },

    closeBranches: function ( aComponentName ) {
        var panelChildren = document.getElementById( "panelChildren" );
        var panelTree = document.getElementById( "prefsTree" );
        for( var i = 0; i < panelChildren.childNodes.length; i++ ) {
            var currentItem = panelChildren.childNodes[i];
            if( currentItem.id != aComponentName && currentItem.id != "appearance" )
                currentItem.removeAttribute( "open" );
        }
        var openItem = document.getElementById( aComponentName );
        panelTree.selectItem( openItem );
    }
};
