/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Constants and global variables
const tag = 'chrome://komodo/content/pref/pref-association.xul';
var associations; // list of association objects 
var cellparent, cellparent_value;
const VK_ENTER = 13;  // the keycode for the "Enter" key

var dialog  = {}; //XXX used both for easy access to named XUL elements and to keep
// a handle on functions that we need in PrefAssociation_OkCallback(). Every
// reference to one of these functions that might be called, directly or
// indirectly, from PrefAssociation_OkCallback() must be accessed through
// the dialog object, or this will break.  For consistency, we always call
// these functions through the dialog object.

function Association(pattern, language, invocationName) {
  this.pattern = pattern;
  this.language = language;
  this.invocationName = invocationName;
  this.id = Association._generateId();
}

// Each Association object has a unique identifier.  This is used to
// establish a relationship between the association object and the
// items in the tree that represent it.

Association._index = 0;  // used for generating ids
Association._generateId = function() {
  return "assoc" + Association._index++;
}

// Handlers

function PrefAssociation_OnLoad() {
    dialog.addPatternTextfield = document.getElementById('addPatternTextfield');
    dialog.addLanguageList = document.getElementById('addLanguageList');
    dialog.addButton = document.getElementById('addButton');
    dialog.patternsTree = document.getElementById('patternsTree');
    dialog.associationList = document.getElementById('associationList');
    dialog.languageList = document.getElementById('languageList');

    parent.hPrefWindow.onpageload(tag);
    parent.hPrefWindow.registerOKCallbackFunc(PrefAssociation_OkCallback, tag);
}

function PrefAssociation_OkCallback(prefset, customData) {
    var oldAssociations = customData.original;
    var newAssociations = customData.associations;

    // only write out the prefs if they have actually changed
    if (!customData.areAssociationListsEquivalent(oldAssociations, newAssociations)) {
      var collectionPref = 
           Components.classes["@activestate.com/koOrderedPreference;1"]
              .createInstance(Components.interfaces.koIOrderedPreference);
      for (var i = 0; i < newAssociations.length; i++) {
          var singlePref = 
              Components.classes["@activestate.com/koOrderedPreference;1"]
                 .createInstance(Components.interfaces.koIOrderedPreference);
          singlePref.appendStringPref(newAssociations[i].pattern);
          singlePref.appendStringPref(newAssociations[i].language);
          singlePref.appendStringPref(newAssociations[i].invocationName);
          collectionPref.appendPref(singlePref);
      }
      prefset.setPref("fileAssociations", collectionPref);
    }
    return true;
}

// Return an array of Association objects, built from the fileAssociations
// preference.
function getAssociationListFromPreference(prefset) {
  var list = new Array();  
  if (prefset.hasPref("fileAssociations")) {
    var fileAssociationsPref = prefset.getPref("fileAssociations");
    for (var i = 0; i < fileAssociationsPref.length; i++) {
      var tuple = fileAssociationsPref.getPref(i).QueryInterface(Components.interfaces.koIOrderedPreference);
      // invocations were added later
      var invocationName = tuple.length > 2 ? tuple.getStringPref(2) : "";
      list.push(new Association(tuple.getStringPref(0), 
              tuple.getStringPref(1), invocationName));
    }
  }
  return list;
}

// Return true if the arrays of Association objects, a and b, correspond to 
// the same underlying representation as a preference.
function areAssociationListsEquivalent(a, b) {
  if (a.length != b.length) return false;

  for (var i = 0; i < a.length; i++) {
    // we don't care about the IDs
    if (a[i].pattern != b[i].pattern || a[i].language != b[i].language || a[i].invocationName != b[i].invocationName) {
      return false;
    }
  }

  return true;
}

function OnPreferencePageInitalize(prefset) {
    var data = new Object();
    data.associations = getAssociationListFromPreference(prefset);
    data.original = getAssociationListFromPreference(prefset);
    data.areAssociationListsEquivalent = areAssociationListsEquivalent; // needs to be called when this page not loaded.
    return data;
}

function OnPreferencePageLoading(prefset, data) {
  associations = data.associations;
  dialog.languageList.selection = document.getAnonymousNodes(dialog.languageList)[0].firstChild.firstChild.getAttribute('label');

  dialog.addLanguageList.selection = document.getAnonymousNodes(dialog.addLanguageList)[0].firstChild.firstChild.getAttribute('label');

  // fill out the associations tree - should cache this entire tree.
  // adding items while collapsed is:
  // a) much faster, and
  // b) does not crash on Linux (who knows why, but this solves it :)
  dialog.associationList.setAttribute("collapsed", true);
  var treeitem;
  for (var i = 0; i < associations.length; i++) {
    treeitem = createAssociationTreeitem(associations[i]);
    dialog.associationList.appendChild(treeitem);
  }
  dialog.associationList.setAttribute("collapsed", false);
  if (typeof(data.selectedId) != 'undefined' && data.selectedId)  {
      var selItem = document.getElementById(data.selectedId);
      dialog.patternsTree.selectItem(selItem);
      dialog.patternsTree.ensureElementIsVisible(selItem);
  }
  else  {  // startup
      dialog.patternsTree.selectItem(dialog.associationList.firstChild);
  }
}

function OnPreferencePageUnloading(prefset, data) {
    data.associations = associations;
    data.selectedId = dialog.patternsTree.selectedItems[0].id;
}

function OnPreferencePageSaving(prefset, data) {
}

function onAddAssociation() {
  var pattern = dialog.addPatternTextfield.value;
  var language = dialog.addLanguageList.selection;
  var invocation = "";
  var association = new Association(pattern, language, invocation);

  associations.push(association);

  var treechildren = document.getElementById('associationList');
  var newitem = createAssociationTreeitem(association);
  treechildren.appendChild(newitem);

  dialog.addPatternTextfield.setAttribute('value', '');
  
  dialog.patternsTree.ensureElementIsVisible ( newitem );
  dialog.patternsTree.selectItem ( newitem );
  dialog.addPatternTextfield.focus();
  updateButtonStates();
}

function onRemoveAssociation() {
  var kids = dialog.associationList;
  var selItem = dialog.patternsTree.selectedItems[0];  // treeitem
  var assocId = trimString(selItem.id);

  // remove the item from the association array
  var index = getIdIndex(assocId);
  associations.splice(index, 1);

  // Remove the item from the tree
  var nextUpId;
  if (selItem.nextSibling)
      nextUpId = selItem.nextSibling.id;
  else if (selItem.previousSibling)
      nextUpId = selItem.previousSibling.id;
  kids.removeChild(selItem);
  var nextUp = document.getElementById(nextUpId);
  dialog.patternsTree.selectItem(nextUp);
  cellparent = nextUp;
}

function onEditPattern(event) {
  textfield = event.target;
  cellparent = event.target.parentNode;
  var text_inputed = textfield.value;
  cellparent.removeChild(textfield);
  text_inputed = formatChanged(text_inputed);

  var index = getIdIndex(cellparent.getAttribute('assocId'));
  associations[index].pattern = text_inputed;

  // need to re-generate the treeitem that was changed
  // It would be nice to do the following 5 lines, but replaceChild doesn't work
  /*treeitem = createAssociationTreeitem(associations[index]);
  dialog.associationList.replaceChild(treeitem, cellparent);
  treeitem.setAttribute('label', text_inputed)
  dialog.patternsTree.ensureElementIsVisible (treeitem);
  dialog.patternsTree.selectItem(treeitem);*/
  var cell = document.createElement('treecell');
  cell.setAttribute('label', associations[index].pattern);
  cell.setAttribute('assocId', associations[index].id);
  cell.setAttribute('flex', '1');

  var row = document.createElement('treerow');
  row.appendChild(cell);
  cellparent.setAttribute('data', associations[index].language);
  cellparent.appendChild(row);
}

function selectAssociation(event, treeitem)  {
    var item, lang;
    if (treeitem)
        item = treeitem;
    else
        item = dialog.patternsTree.selectedItems[0];

    lang = item.getAttribute('data');
    dialog.languageList.selection = lang;
    
    if (event)  {
        cellparent = event.target;
        // clicking in the scroll bar will also trigger this - and in some cases may actually *crash* mozilla.
        // By only handling clicks directly in the treerow (ie, the tree itself) we avoid this problem.
        if (cellparent.tagName == "treerow" && event.detail == 2)  {  // double click or ENTER to edit
            dialog.languageList.selection = lang;
            setEditable(item)
            return;
        }
    }
}

function updateAssociation()  {
      if (dialog.patternsTree.selectedItems[0])  {
          var item = dialog.patternsTree.selectedItems[0];
          var cell = dialog.patternsTree.selectedItems[0].firstChild.firstChild;
          var index = getIdIndex(cell.getAttribute('assocId'));
          associations[index].language = dialog.languageList.selection;
          item.setAttribute('data', associations[index].language);
      }
}

function setEditable(item) {
    textfield = document.createElement("textbox");
    cellparent_value = cellparent.getAttribute('label');
    textfield.setAttribute('value', cellparent_value);
    cellparent.setAttribute('label', "");
    textfield.setAttribute("flex", "1");
    textfield.setAttribute("class", "plain");
    cellparent.appendChild(textfield);
    textfield.focus();
    if(cellparent_value.length > 0) 
        textfield.select();
    textfield.setAttribute("onblur", "doEntryBlur(event);"); 
    textfield.setAttribute("onkeypress", "doEntryChange(event);"); 
}

function doEntryBlur(event) {
    cellparent.removeChild(textfield);
    cellparent.setAttribute('label', cellparent_value);
}

function doEntryChange(event) {
    // do it when return key hit
    if(event.keyCode == VK_ENTER) {
        _doEntryChange(event);
    }
}

function _doEntryChange(event) {
    event.cancelBubble = true;
    onEditPattern(event);
}

// This function is for special handling of values changed
// It can be extended if need arises for other special characters
function formatChanged (text)  {
    text = text.replace('\\', '\\\\', 'g'); // format '\' characters or corruption will occur
    return text;
}

function updateButtonStates() {
  var pattern = trimString(dialog.addPatternTextfield.value);
  if (!pattern || getPatternIndex(pattern) != null) {
    dialog.addButton.setAttribute('disabled', 'true');
  } else {
    dialog.addButton.removeAttribute('disabled'); 
  }
}

// DOM construction helpers

function createAssociationTreeitem(association) {
  var cell = document.createElement('treecell');
  cell.setAttribute('label', association.pattern);
  cell.setAttribute('assocId', association.id);

  var row = document.createElement('treerow');
  row.appendChild(cell);
  
  var item = document.createElement('treeitem');
  item.setAttribute('id', association.id);
  item.setAttribute('data', association.language);
  item.appendChild(row);

  return item;
}

function getPatternIndex(pattern) {
  for (var i = 0; i < associations.length; i++) {
    if (pattern == associations[i].pattern) return i;
  }
  return null;
}

function getIdIndex(id) {
  for (var i = 0; i < associations.length; i++) {
    if (id == associations[i].id) return i;
  }
  return null;
}

// Remove whitespace from both ends of a string
function trimString(string)
{
  if (!string) return "";
  return string.replace(/(^\s+)|(\s+$)/g, '')
}
