/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* 
 * pref-customize.js -- Buffer colour / font customisation routines
 */

try  {
    var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"].createInstance();
    if( enumerator )
      enumerator = enumerator.QueryInterface(Components.interfaces.nsIFontEnumerator);
}
catch(e)  {
}

var Linux = false;
var infoService = Components.classes['@activestate.com/koInfoService;1'].
        getService(Components.interfaces.koIInfoService);
if(infoService.platform.match(/^linux/)) {
  Linux = true;
}

var koEncodingServices = Components.classes["@activestate.com/koEncodingServices;1"]
        .getService(Components.interfaces.koIEncodingServices);

// Globals
const tag = 'chrome://komodo/content/pref/pref-fonts.xul';
var dialog;
var gFontTypes   = ["serif","sans-serif", /*"cursive", "fantasy",*/"monospace"];
var gFontLanguages = ['x-western','x-central-euro','ja','zh-TW',
                        'zh-CN','ko','x-cyrillic','x-baltic','el',
                        'tr','x-unicode','th','he','ar'];
var gFontsProp, gFontsFixed, gFontNames, gFontFamily;

var mainWindow = parent.opener;
var gView = mainWindow.getFocusedView();
var gSciMoz;

var gLanguageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"]
            .getService(Components.interfaces.koILanguageRegistryService);
var gStyleManager = Components.classes["@activestate.com/koStyleManager;1"]
            .getService(Components.interfaces.koIStyleManager);

var gCursorColor, gCurrentLineColor, gSelectBackColor, gSelectForeColor, gEdgeColor;
var gUseCurrentLineColor, gUseSelectFore;
var gCurrentLanguageGroup, gDefaultLanguageGroup;
var gLanguageGroup;
var fixedBold, fixedItalic, propBold, propItalic;
var fixedForeColour, fixedBackColour, propForeColour, propBackColour;
var gPrefsService;

// we have to build our language grouping first
var gEncodings = new Object();

function generateFontList()
{
    // get all the fonts on the system
    var strFontSpecs;
    var j;
    gFontsProp = [];
    gFontsFixed = [];
    gFontNames = {};
    for (var i=0;i<gFontLanguages.length;i++) {
        if (Linux) {
            //strFontFaces =
            var re = /^([^-]+)-([^-]+)/;
            strFontSpecs = enumerator.EnumerateFonts(gFontLanguages[i], 
                                                     'sans',
                                                     new Object());
            for (j=0; j < strFontSpecs.length; j++) {
                if (typeof(gFontNames[strFontSpecs[j]])=='undefined' ||
                    !gFontNames[strFontSpecs[j]]) {
                    var n = re.exec(strFontSpecs[j]);
                    var pn = n[2]+' ('+n[1]+')';
                    gFontNames[strFontSpecs[j]] = pn;
                    gFontsProp[gFontsProp.length] = strFontSpecs[j];
                    gFontsFixed[gFontsFixed.length] = strFontSpecs[j];
                }
            }
        } else {

            for( var t = 0; t < gFontTypes.length; t++ )
            {
                // build and populate the font list for the newly chosen font type
                strFontSpecs = enumerator.EnumerateFonts(gFontLanguages[i], 
                                                         gFontTypes[t], 
                                                         new Object());
                for (j=0; j < strFontSpecs.length; j++) {
                    if (typeof(gFontNames[strFontSpecs[j]])=='undefined' ||
                        !gFontNames[strFontSpecs[j]]) {
                        gFontNames[strFontSpecs[j]]=strFontSpecs[j];
                        if (gFontTypes[t]=='monospace') {
                            gFontsFixed[gFontsFixed.length] = strFontSpecs[j];
                        } else {
                            gFontsProp[gFontsProp.length] = strFontSpecs[j];
                        }
                    }
                }
            }
        }
    }
}


function PrefFonts_OnLoad()  {
    generateFontList();
    updateEncodingPopup();
    parent.hPrefWindow.onpageload(tag);
    parent.hPrefWindow.registerOKCallbackFunc(PrefFonts_OkCallback, tag);
    parent.hPrefWindow.registerCancelCallbackFunc(PrefFonts_CancelCallback, tag);
    gPrefsService = Components.classes['@activestate.com/koPrefService;1'].
        getService(Components.interfaces.koIPrefService);
    
    init();
}

function OnPreferencePageInitalize(prefset) {
    // read the prefs
    var data = new Object();
    data.gLanguageGroup = {};
    data.gStyleManager = gStyleManager;
    var gPrefsService = Components.classes['@activestate.com/koPrefService;1'].
        getService(Components.interfaces.koIPrefService);
    data.gCurrentLineColor = gPrefsService.prefs.getStringPref("currentLineColor");
    data.gCursorColor = gPrefsService.prefs.getStringPref("cursorColor");
    data.gEdgeColor = gPrefsService.prefs.getStringPref("edgeColor");
    data.gSelectBackColor = gPrefsService.prefs.getStringPref("selectBack");
    data.gSelectForeColor = gPrefsService.prefs.getStringPref("selectFore");
    data.gUseCurrentLineColor = gPrefsService.prefs.getBooleanPref('useCurrentLineColor');
    data.gUseSelectFore = gPrefsService.prefs.getBooleanPref('useSelectFore');

    var encname = gPrefsService.prefs.getStringPref('encodingDefault');
    if (mainWindow.getFocusedView()) {
        // XXX scintilla specific, replace with a function
        encname = mainWindow.getFocusedView().py_inst.encoding.python_encoding_name;
    }
    data.gDefaultLanguageGroup = koEncodingServices.get_encoding_info(encname).fontspec;
    
    data.gLanguageGroup = {};
    var popup = document.getElementById('selectLangsPopup');
    for (var i=0; i<popup.childNodes.length; i++) {
        var child = popup.childNodes[i];
        var value = child.getAttribute('value');
        data.gLanguageGroup[value] = {};
        data.gLanguageGroup[value].gFixedStyle = 
            gStyleManager.getDefaultEncodingStyle("editor", value, 1);
        data.gLanguageGroup[value].gPropStyle = 
            gStyleManager.getDefaultEncodingStyle("editor", value, 0);
    }
    
    updateColorPickerVisibility(document.getElementById('currentLineColorPicker'), data.gUseCurrentLineColor);
    updateColorPickerVisibility(document.getElementById('selectForeColorPicker'), data.gUseSelectFore);
    updateColorPickerVisibility(document.getElementById('edgeColorPicker'), 1);
    return data;
}

function OnPreferencePageLoading(prefset, data) {
    gDefaultLanguageGroup = data.gDefaultLanguageGroup;
    gCurrentLanguageGroup = gDefaultLanguageGroup;
    gLanguageGroup = data.gLanguageGroup;
    
    var gPrefsService = Components.classes['@activestate.com/koPrefService;1'].
        getService(Components.interfaces.koIPrefService);
    gCurrentLineColor = gPrefsService.prefs.getStringPref('currentLineColor');
    gCursorColor =  gPrefsService.prefs.getStringPref('cursorColor');
    gSelectBackColor =  gPrefsService.prefs.getStringPref('selectBack');
    gSelectForeColor =  gPrefsService.prefs.getStringPref('selectFore');
    gEdgeColor =  gPrefsService.prefs.getStringPref('edgeColor');
    gUseSelectFore = data.gUseSelectFore;
    gUseCurrentLineColor= data.gUseCurrentLineColor;
    updateColorPickerVisibility(document.getElementById('currentLineColorPicker'), gUseCurrentLineColor);
    updateColorPickerVisibility(document.getElementById('selectForeColorPicker'), gUseSelectFore);
    updateColorPickerVisibility(document.getElementById('edgeColorPicker'), gEdgeColor);
}

function updateEncodingPopup()
{
    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,new Object());
    gEncodings = temp.value;
    
    var popup = document.getElementById('selectLangsPopup');
    for (var i=0; i< gEncodings.length; i++) {
        var item = document.createElement('menuitem');
        item.setAttribute('value',gEncodings[i].fontspec);
        item.setAttribute('label',gEncodings[i].friendly_encoding_name);
        popup.appendChild(item);
    }
}

function updateColorPickerVisibility(colorpicker, show) {
    //dump("colorpicker = "+ colorpicker+ " and show = " + show +'\n');
    if (show) colorpicker.removeAttribute('collapsed');
    else colorpicker.setAttribute('collapsed', 'true');
}

function OnPreferencePageUnloading(prefset, data) {
  data.gDefaultLanguageGroup = gDefaultLanguageGroup;
  data.gLanguageGroup = gLanguageGroup;
  data.gCursorColor = gCursorColor;
  data.gSelectBackColor = gSelectBackColor;
  data.gSelectForeColor = gSelectForeColor;
  data.gCurrentLineColor = gCurrentLineColor;
  data.gUseSelectFore = gUseSelectFore;
  data.gUseCurrentLineColor = gUseCurrentLineColor;
  data.gEdgeColor = gEdgeColor;
}

function init()  {
    // Get a handle on the UI widgets
    dialog = {};
    dialog.fixedList = document.getElementById('fixed');
    dialog.propList = document.getElementById('proportional');
    dialog.fixedSize = document.getElementById('fixedSize');
    dialog.propSize = document.getElementById('propSize');
    dialog.fixedboldButton = document.getElementById('fixedboldButton');
    dialog.fixeditalicButton = document.getElementById('fixeditalicButton');
    dialog.propboldButton = document.getElementById('propboldButton');
    dialog.propitalicButton = document.getElementById('propitalicButton');
    dialog.fixedColorPicker = document.getElementById('fixedColorPicker');
    dialog.propColorPicker = document.getElementById('propColorPicker');
    dialog.currentLineColorPicker = document.getElementById('currentLineColorPicker');
    dialog.cursorColorPicker = document.getElementById('cursorColorPicker');
    dialog.selectBackColorPicker = document.getElementById('selectBackColorPicker');
    dialog.selectForeColorPicker = document.getElementById('selectForeColorPicker');
    dialog.edgeColorPicker = document.getElementById('edgeColorPicker');
    dialog.useCurrentLineColor = document.getElementById('useCurrentLineColor');
    dialog.useSelectFore = document.getElementById('useSelectFore');
    dialog.edgeColor = document.getElementById('edgeColor');
    
    dialog.defaultLanguageGroup = document.getElementById('selectLangs');
    // the dialog initialy shows the fonts for the default editor encoding
    //var encname = gPrefsService.prefs.getStringPref('encodingDefault');
    //dialog.defaultLanguageGroup.value = koEncodingServices.get_encoding_info(encname).fontspec;
    dialog.defaultLanguageGroup.value = gCurrentLanguageGroup;
    // Font Lists
    initFonts(dialog.defaultLanguageGroup.value);
    setupFonts(); // set up UI for from fixed and proportional prefs

    // Fixed and Proportional colours
    startupColours();
}

function PrefFonts_OkCallback(prefset, customData)  {
    customData.gStyleManager.saveStyles();
    return true;
}

function PrefFonts_CancelCallback(prefset, customData)  {
    customData.gStyleManager.revertStyles();
    return true;
}

function onResetDefaults(event) {
    gStyleManager.resetDefaults();

    var popup = document.getElementById('selectLangsPopup');
    for (var i=0; i<popup.childNodes.length; i++) {
        var child = popup.childNodes[i];
        var value = child.getAttribute('value');
        gLanguageGroup[value] = {};
        gLanguageGroup[value].gFixedStyle = 
            gStyleManager.getDefaultEncodingStyle("editor", value, 1);
        gLanguageGroup[value].gPropStyle = 
            gStyleManager.getDefaultEncodingStyle("editor", value, 0);
    }

    setupFonts();
    startupColours();    
}

function onClickFixedFont() {
    var newFont = dialog.fixedList.value;
    gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.font = newFont;
}

function onClickProportionalFont() {
    var newFont = dialog.propList.value;
    gLanguageGroup[gCurrentLanguageGroup].gPropStyle.font = newFont;
}

function onClickFixedSize() {
    var newSize = dialog.fixedSize.label;
    gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.size = newSize;
}

function onClickProportionalSize() {
    var newSize = dialog.propSize.label;
    gLanguageGroup[gCurrentLanguageGroup].gPropStyle.size = newSize;
}

function onClickUseSelectFore(){
    gUseSelectFore = dialog.useSelectFore.getAttribute('checked');
    updateColorPickerVisibility(dialog.selectForeColorPicker,
                        gUseSelectFore);
}

function onClickUseCurrentLineColor(){
    gUseCurrentLineColor= dialog.useCurrentLineColor.getAttribute('checked');
    updateColorPickerVisibility(dialog.currentLineColorPicker,
                        gUseCurrentLineColor);
}

function onClickBold(button)  {
    var changeBold;
    switch(button.id) {
      case "fixedboldButton":
        changeBold = !fixedBold;
        gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.bold = changeBold;
        fixedBold = changeBold;
        break;
     case "propboldButton":
        changeBold = !propBold;
        gLanguageGroup[gCurrentLanguageGroup].gPropStyle.bold = changeBold;
        propBold = changeBold;
        break;
    }
    toggleButton(button.id, changeBold);
}

function onClickItalic(button)  {
    var changeItalic;
    switch(button.id) {
      case "fixeditalicButton":
        changeItalic = !fixedItalic;
        gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.italic = changeItalic;
        fixedItalic = changeItalic;
        break;
     case "propitalicButton":
        changeItalic = !propItalic;
        gLanguageGroup[gCurrentLanguageGroup].gPropStyle.italic = changeItalic;
        propItalic = changeItalic;
        break;
    }
    toggleButton(button.id, changeItalic);
}

function setColour(colorpicker) {
    switch(colorpicker.id) {
        case "fixedColorPicker":
            gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.fore = colorpicker.fore;
            gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.back = colorpicker.back;
            break;
        case "propColorPicker":
            gLanguageGroup[gCurrentLanguageGroup].gPropStyle.fore = colorpicker.fore;
            gLanguageGroup[gCurrentLanguageGroup].gPropStyle.back = colorpicker.back;
            break;
        case "cursorColorPicker":
            gCursorColor = colorpicker.color;
            break;
        case "edgeColorPicker":
            gEdgeColor = colorpicker.color;
            break;
        case "selectBackColorPicker":
            gSelectBackColor = colorpicker.color;
            break;
        case "selectForeColorPicker":
            gSelectForeColor = colorpicker.color;
            break;
        case "currentLineColorPicker":
            gCurrentLineColor = colorpicker.color;
            break;
    }
}

function listElement( aListID )
  {
    this.listElement = document.getElementById( aListID );
  }

// XXX - this sucks - all this code is also cloned in pref-fontslang.js :(
// XXX - if you fix this, also fix that!
listElement.prototype =
  {
    clearList:
      function ()
        {
          // removing the menupopup node child of the menulist seems to die!
          // so just remove all *its* child nodes.
          var popup = this.listElement.firstChild;
          while (popup.childNodes.length)
              popup.removeChild( popup.firstChild );
        },

    appendString:
      function ( aString )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              popupNode = document.createElement( "menupopup" );
          }

          var itemNode = document.createElement( "menuitem" );
          itemNode.setAttribute( "value", aString );
          itemNode.setAttribute( "label", aString );
          this.listElement.removeAttribute( "disabled" );
          popupNode.appendChild( itemNode );
          this.listElement.appendChild( popupNode );
        },

    appendStrings:
      function ( faces, specs )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              logError("This should never be called - we no longer ever delete the menupopup!!!\n");
              popupNode = document.createElement( "menupopup" );
              this.listElement.appendChild( popupNode );
          }

          faces.sort();
          for( var i = 0; i < faces.length; i++ )
            {
              if( faces[i] == "" )
                {
                  this.listElement.setAttribute( "value", "" );
                  this.listElement.setAttribute( "label", "No fonts available for this language" );
                  this.listElement.setAttribute( "disabled", "true" );
                }
              else
                {
                  var itemNode = document.createElement( "menuitem" );
                  if (specs) {
                    itemNode.setAttribute( "value", specs[faces[i]] );
                  } else {
                    itemNode.setAttribute( "value", faces[i] );
                  }
                  itemNode.setAttribute( "label", faces[i] );
                  // for debug purposes, you may want the fontspec
                  //itemNode.setAttribute( "label", specs[faces[i]] );
                  this.listElement.removeAttribute( "disabled" );
                  popupNode.appendChild( itemNode );
                }
            }
        }
  };

function selectLanguage() {
    gCurrentLanguageGroup = document.getElementById('selectLangs').value;
    initFonts(gCurrentLanguageGroup);
    setupFonts();
    startupColours();
}

function initFonts(language)  {
    if (typeof(language)=='undefined') language = gDefaultLanguageGroup;
    // Do all fonts list first
    var fixedElement = new listElement( 'fixed' );
    var propElement = new listElement( 'proportional' );
    var strFontFaces = [];
    var strFontSpecs = {};
    // Fixed and Proportional
    fixedElement.clearList();
    propElement.clearList();

    if (Linux) {
        //strFontFaces =
        var re = /^([^-]+)-([^-]+)-(.*)/;
        for (var j=0; j < gFontsProp.length; j++) {
            var n = re.exec(gFontsProp[j]);
            if (language == n[3]) {
                strFontFaces[strFontFaces.length] = gFontNames[gFontsProp[j]];
                strFontSpecs[gFontNames[gFontsProp[j]]] = gFontsProp[j];
            }
        }
        propElement.appendStrings(strFontFaces,strFontSpecs);
        fixedElement.appendStrings(strFontFaces,strFontSpecs);
        return;
    } else {
        propElement.appendStrings(gFontsProp,null);
        fixedElement.appendStrings(gFontsFixed,null);
    }
}

function _findFontName(name) {
    if (typeof(gFontNames[name]) == 'undefined') {
        return name;
    }
    return gFontNames[name];
}

function setupFonts()  {
    // order is font, size, back, fore, italic, bold
    
    var fLabel = gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.font;
    var pLabel = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.font;
    var fSize= gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.size;
    var pSize = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.size;
    var m;
    // set the menulists
    dialog.fixedList.setAttribute('label', _findFontName(fLabel));
    dialog.propList.setAttribute('label', _findFontName(pLabel));

    dialog.fixedSize.setAttribute('label', fSize);
    m = document.getElementById('fixedSize_'+fSize);
    dialog.fixedSize.selectedItem = m;

    dialog.propSize.setAttribute('label', pSize);
    m = document.getElementById('propSize_'+pSize);
    dialog.propSize.selectedItem = m;
    
    // bold/italic
    propBold = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.bold;
    toggleButton(dialog.propboldButton.id, propBold);
    fixedBold = gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.bold;
    toggleButton(dialog.fixedboldButton.id, fixedBold);
    propItalic = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.italic;
    toggleButton(dialog.propitalicButton.id, propItalic);
    fixedItalic = gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.italic;
    toggleButton(dialog.fixeditalicButton.id, fixedItalic);
    
}

function startupColours()  {
    fixedForeColour = gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.fore;
    fixedBackColour = gLanguageGroup[gCurrentLanguageGroup].gFixedStyle.back;
    dialog.fixedColorPicker.fore = fixedForeColour;
    dialog.fixedColorPicker.back = fixedBackColour;
    
    propForeColour = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.fore;
    propBackColour = gLanguageGroup[gCurrentLanguageGroup].gPropStyle.back;
    dialog.propColorPicker.fore = propForeColour;
    dialog.propColorPicker.back = propBackColour;    
    dialog.edgeColorPicker.color = gPrefsService.prefs.getStringPref("edgeColor");
    dialog.cursorColorPicker.color = gPrefsService.prefs.getStringPref("cursorColor");
    dialog.currentLineColorPicker.color = gPrefsService.prefs.getStringPref("currentLineColor");
    dialog.selectBackColorPicker.color = gPrefsService.prefs.getStringPref("selectBack");
    dialog.selectForeColorPicker.color = gPrefsService.prefs.getStringPref("selectFore");
}

function toggleButton(id, value)  {
    var button = document.getElementById(id);
    button.setAttribute('toggled', value);
}    
