/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* 
 * pref-fontslang.js -- Buffer colour / font customisation routines
 */

try  {
    var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"].createInstance();
    if( enumerator )
      enumerator = enumerator.QueryInterface(Components.interfaces.nsIFontEnumerator);
    var fontCount = { value: 0 }
    var fonts = enumerator.EnumerateAllFonts(fontCount);
}
catch(e)  {
}

var Linux = false;
var infoService = Components.classes['@activestate.com/koInfoService;1'].
getService(Components.interfaces.koIInfoService);
if(infoService.platform.match(/^linux/)) {
  Linux = true;
}
   
// Globals
const tag = 'chrome://komodo/content/pref/pref-fontslang.xul';
var gStyleNamesCount = 0;
var gStyleNames = new Array();
var gFontTypes   = ["serif","sans-serif", /*"cursive", "fantasy",*/"monospace"];
var mainWindow = parent.opener;
var gView = mainWindow.getFocusedView();
var gSciMoz;
var gLang;
var gCurrentLanguage;
var uienabled = false;
var gStyleManager = Components.classes["@activestate.com/koStyleManager;1"]
            .getService(Components.interfaces.koIStyleManager);
var gLanguageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"]
            .getService(Components.interfaces.koILanguageRegistryService);
var gGlobalPrefs = Components.classes["@activestate.com/koPrefService;1"]
            .getService(Components.interfaces.koIPrefService).prefs;
var gCurrentStyle = null, oldFontType;
var dialog;

function PrefFontslang_OnLoad()  {
  parent.hPrefWindow.onpageload(tag);
  parent.hPrefWindow.registerOKCallbackFunc(PrefFontslang_OkCallback, tag);
  parent.hPrefWindow.registerCancelCallbackFunc(PrefFontslang_CancelCallback, tag);
  init();
}  

function init()  {
    // Get a handle on the UI widgets
    dialog = {};
    
    dialog.languageList = document.getElementById('languageList');
    dialog.currentFont = document.getElementById('currentFont');
    dialog.currentSize = document.getElementById('currentSize');
    dialog.boldSelectButton = document.getElementById('boldSelectButton');
    dialog.italicSelectButton = document.getElementById('italicSelectButton');
    dialog.currentColorPicker = document.getElementById('currentColorPicker');
    
    dialog.previewBox = document.getElementById('previewBox');
    dialog.previewText = document.getElementById('previewText');    
    
    setOutlinerView();
    
    // Get the language of the buffer
    if (gView) {
        gSciMoz = gView._scimoz;
        gLang = gView.py_inst.sourceFile.getLanguageBlockAtPosition(0).language;
        gCurrentLanguage = gLang.name;
    }
    else {  // default to first
        gCurrentLanguage = document.getAnonymousNodes(dialog.languageList)[0].firstChild.firstChild.getAttribute('label');
        gLang = gLanguageRegistry.getLanguage(gCurrentLanguage);
    }
    
    dialog.languageList.selection = gCurrentLanguage;
    
    initFonts();
    changeLanguage(gCurrentLanguage);
    disableUI(); // disable language options until selection is made
}

function OnPreferencePageInitalize(prefset) {
     // read the prefs
    var data = new Object();
    data.gStyleManager = gStyleManager;
    return data;
}

function OnPreferencePageLoading(prefset, data) {
}

function OnPreferencePageUnloading(prefset, data) {
}

function PrefFontslang_OkCallback(prefset, customData)  {
    //TODO - only save if they have changed
    customData.gStyleManager.saveStyles();    
    return true;
}

function PrefFontslang_CancelCallback(prefset, customData)  {
    //TODO - only cancel if they have changed
    customData.gStyleManager.revertStyles();
    return true;
}

var gStyleNamesView = {
    rowCount : 0,
    outliner : null,
    isSeparator : function(index) {return false;},
    isContainer : function(index) {return false;},
    setOutliner : function(out) { this.outliner = out; },
    getCellText : function(row,column){
      if (column=="elColumn")  {
        // XXX - this is accessed every time we leave the panel,
        // so make sure gSyleNames is in scope, or is will throw an error
        if ("gStyleNames" in window)
            return gStyleNames.value[row];
        else
            return "";
      }
      else return "";
    },
    getRowProperties : function(row,prop){},
    getColumnProperties : function(column,prop){},
    getCellProperties : function(row,prop){},
    isContainer : function(row) {return false;},

    // Private
    matchData : null,
    setRowCount : function(rowCount) {
        this.rowCount = rowCount;
        this.outliner.rowCountChanged(0, this.rowCount);
        this.outliner.invalidate();
    },
    selectionChanged : function() {
        var styleName = gStyleNames.value[this.selection.currentIndex];
        var useFixedFont = gGlobalPrefs.getBooleanPref("editUseFixedFont")
        gCurrentStyle = gStyleManager.getStyle("editor", gCurrentLanguage, styleName, useFixedFont);
        updateStyleUI();
        updatePreview();
    },
    performAction : function(action) { },
    
    /* our own defined methods */
    sort: function() {
        gStyleNames.value.sort();
    }    
};

function setOutlinerView() {
     document.getElementById('stylenames-outliner').outlinerBoxObject.view=gStyleNamesView;
}

function onReset(event) {
    gStyleManager.resetLanguage(gCurrentLanguage);
    changeLanguage();
}

function onClickCurrentFont(event) {
    var currentFont = event.target.value;
    gCurrentStyle.font = currentFont;
    updatePreview();
}

function onClickCurrentSize(event) {
    var currentSize = event.target.value;
    gCurrentStyle.size = currentSize;
    updatePreview();
}

function onClickCurrentBold(button)  {
    var currentBold = gCurrentStyle.bold;
    toggleButton(button.id, !currentBold);
    gCurrentStyle.bold = !currentBold;
    updatePreview();
}

function onClickCurrentItalic(button)  {
    var currentItalic = gCurrentStyle.italic;    
    toggleButton(button.id, !currentItalic);    
    gCurrentStyle.italic = !currentItalic;
    updatePreview();
}

function setCurrentColour(colorpicker) {
    var backcolor = colorpicker.back;
    var forecolor = colorpicker.fore;

    gCurrentStyle.cssFore = forecolor;
    gCurrentStyle.cssBack = backcolor;
    
    updatePreview();
}

function updateStyleUI() {
    // re-enable UI
    if (!uienabled)
        enableUI();
    // Set up the font, size, and colour elements.
    var currentFont = gCurrentStyle.font;
    var currentSize = gCurrentStyle.size;
    var currentBold = gCurrentStyle.bold;
    var currentItalic = gCurrentStyle.italic;
    var currentFore = gCurrentStyle.cssFore;
    var currentBack = gCurrentStyle.cssBack;    

    dialog.currentFont.selectedItem = document.getElementsByAttribute("label", currentFont)[0];   // Font
    dialog.currentSize.selectedItem = document.getElementsByAttribute("label", currentSize)[0];  // Size
    
    toggleButton(dialog.boldSelectButton.id, currentBold);  // bold
    toggleButton(dialog.italicSelectButton.id, currentItalic);  // italic

    // colours
    dialog.currentColorPicker.fore = currentFore;
    dialog.currentColorPicker.back = currentBack;
}

function updatePreview()  {
    //dialog.previewBox
    var currentFont = gCurrentStyle.font;
    var currentSize = gCurrentStyle.size;
    var currentBold = gCurrentStyle.bold;
    var currentItalic = gCurrentStyle.italic;
    var currentFore = gCurrentStyle.cssFore;
    var currentBack = gCurrentStyle.cssBack;    
    
    mainWindow.addStyle(dialog.previewText, 'font-family', currentFont);
    mainWindow.addStyle(dialog.previewText, 'font-size', currentSize+'pt');
    if (currentBold)
        mainWindow.addStyle(dialog.previewText, 'font-weight', 'bold');
    else
        mainWindow.addStyle(dialog.previewText, 'font-weight', 'normal');
    if (currentItalic)
        mainWindow.addStyle(dialog.previewText, 'font-style', 'italic');
    else
        mainWindow.addStyle(dialog.previewText, 'font-style', 'normal');
    mainWindow.addStyle(dialog.previewText, 'color', currentFore);
    mainWindow.addStyle(dialog.previewText, 'background-color', currentBack);
}

function enableUI()  {
    dialog.currentFont.setAttribute('disabled', false);
    dialog.currentSize.setAttribute('disabled', false);
    dialog.boldSelectButton.setAttribute('disabled', false);
    dialog.italicSelectButton.setAttribute('disabled', false);
    dialog.currentColorPicker.setAttribute('disabled', false);
    uienabled = true;
}

function disableUI()  {
    dialog.currentFont.setAttribute('disabled', true);
    dialog.currentSize.setAttribute('disabled', true);
    dialog.boldSelectButton.setAttribute('disabled', true);
    dialog.italicSelectButton.setAttribute('disabled', true);
    dialog.currentColorPicker.setAttribute('disabled', true);
    uienabled = false;
}

function changeLanguage(startlang)  {
    // Do the Elements first
    if (!startlang)
        gCurrentLanguage = dialog.languageList.selection;
        
    // Grab a list of available styles.
    gStyleManager.getStyleNames(gCurrentLanguage, gStyleNames, new Object());

    gStyleNamesCount = gStyleNames.value.length;

    gStyleNamesView.setRowCount(gStyleNamesCount); // set length of outliner list
    gStyleNamesView.sort();

    // And the Rest ...
    document.getElementById("set-language-defaults").setAttribute("label", "Reset " + gCurrentLanguage + " settings");
    
    if (uienabled)
        setTimeout("disableUI()", 5);    
}

// XXX - this sucks - all this code is also cloned in pref-fonts.js :(
// XXX - if you fix this, also fix that!
function listElement( aListID )
  {
    this.listElement = document.getElementById( aListID );
  }

listElement.prototype =
  {
    clearList:
      function ()
        {
          // removing the menupopup node child of the menulist seems to die!
          // so just remove all *its* child nodes.
          var popup = this.listElement.firstChild;
          while (popup.childNodes.length)
              popup.removeChild( popup.firstChild );
        },

    appendString:
      function ( aString )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              popupNode = document.createElement( "menupopup" );
              this.listElement.appendChild( popupNode );
          }

          var itemNode = document.createElement( "menuitem" );
          itemNode.setAttribute( "value", aString );
          itemNode.setAttribute( "label", aString );
          this.listElement.removeAttribute( "disabled" );
          popupNode.appendChild( itemNode );
        },

    appendStrings:
      function ( aDataObject )
        {
          var popupNode;
          popupNode = this.listElement.firstChild;
          if (!popupNode)  {  //clearList may have been called
              logError("This should never be called - we no longer ever delete the menupopup!!!\n");
              popupNode = document.createElement( "menupopup" );
              this.listElement.appendChild( popupNode );
          }
          var faces = aDataObject.toString().split(",");
          faces.sort();
          for( var i = 0; i < faces.length; i++ )
            {
              if( faces[i] == "" )
                {
                  this.listElement.setAttribute( "value", faces[i] );
                  this.listElement.setAttribute( "label", "No fonts available for this language" );
                  this.listElement.setAttribute( "disabled", "true" );
                }
              else
                {
                  var itemNode = document.createElement( "menuitem" );
                  itemNode.setAttribute( "value", faces[i] );
                  itemNode.setAttribute( "label", faces[i] );
                  this.listElement.removeAttribute( "disabled" );
                  popupNode.appendChild( itemNode );
                }
            }
        }
  };

function initFonts()  {
    // build and populate the font list
    var currentFont = new listElement( 'currentFont' );
    var strFontFaces;
    // Fixed and Proportional
    var language = 'x-western';
    currentFont.clearList();

    // XXX:HACK on linux, EnumerateFonts always lists every font,
    // rather than just the matching ones.  Also, it returns them in
    // foundry-face-encoding format when we just want "face".  So we
    // strip foundry and encoding from the font names, and use the one
    // set of fonts for both fixedElement and propElement font lists.
    if (Linux) {
        var strFontSpecs = enumerator.EnumerateFonts('x-western', 'sans',
                                                     fontCount);
        strFontFaces = []
        // get the face name ...
        var re = /^([^-]+)-([^-]+)-(.*)/;
        for (var j = 0; j < strFontSpecs.length; j++) {
            var face = re.exec(strFontSpecs[j])[2];
            var haveface = false;
            for (var f = 0; f < strFontFaces.length; f++) {
                if (strFontFaces[f] == face) {
                    haveface = true;
                    break;
                }
            }
            if (!haveface) strFontFaces[strFontFaces.length] = face;
        }
        currentFont.appendStrings(strFontFaces);
        return;
    }

    for( var i = 0; i < gFontTypes.length; i++ )
      {
        strFontFaces = enumerator.EnumerateFonts(language, gFontTypes[i], fontCount);
        currentFont.appendStrings(strFontFaces);
      }
      // sorting ?
}

function toggleButton(id, value)  {
    var button = document.getElementById(id);
    button.setAttribute('toggled', value);
}    
