/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */
   
// Constants and global variables
const tag = 'chrome://komodo/content/pref/pref-intl.xul';
var koEncodingServices = Components.classes["@activestate.com/koEncodingServices;1"]
            .getService(Components.interfaces.koIEncodingServices);
var globalPrefService = Components.classes["@activestate.com/koPrefService;1"]
            .getService(Components.interfaces.koIPrefService);
var langPrefs;
var currentLangPrefs;
var newHasChanged = false;
var startEncoding;
var latestLanguage;
var langToUse;
var dialog = null;

function PrefIntl_OnLoad() {
    // set up UI elements
    dialog = {};
    dialog.newlangList = document.getElementById('newlangList');
    dialog.newencodingList = document.getElementById('newencodingList');
    dialog.bomBox = document.getElementById('bomBox');
    dialog.encodingDefault = document.getElementById('encodingDefault');
    dialog.encodingEnvironment = document.getElementById('encodingEnvironment');
    dialog.encodingAutoDetect = document.getElementById('encodingAutoDetect');
    dialog.encodingXMLDec = document.getElementById('encodingXMLDec');
    
    // order is important here
    // 1- set up oepn encoding list
    // 2 - load the page with preferences, which will set the 'pref' values
    // 3 - do new excoding, which is the manual getting of prefs for per language encoding
    doDefaultEncoding();    
    // register with preferences
    parent.hPrefWindow.onpageload(tag);
    parent.hPrefWindow.registerOKCallbackFunc(PrefIntl_OkCallback, tag);
    doNewEncoding();

    if (dialog.encodingEnvironment.checked)
        changeDefaultEncoding();
}

function OnPreferencePageInitalize(prefset) {
    var data = new Object();
    // set up the prefs
    data.langPrefs = prefset.getPref('languages');
    data.langPrefs.parent = prefset;    
    data.view = parent.opener.getFocusedView();
    data.mainWin = parent.opener;
    data.setEncoding = setEncoding; // needs to be called when this page not loaded.
    data.offPanel = false;
    if (data.view)
        data.langToUse = data.view.py_inst.language;
    else   // default
        data.langToUse = "Text";
        
    return data;
}

function PrefIntl_OkCallback(prefset, customData)  {
    //if (customData.newHasChanged)  {
        if (!customData.setEncoding(customData))  {  // check if encoding is appropriate
            return false;
        }
        else  {
            prefset.setPref("languages", customData.langPrefs);
            return true;
        }
    //}
    return true;
}

// Start up & returning to this panel in a session
function OnPreferencePageLoading(prefset, data) {
    data.newHasChanged = false;
    data.offPanel = false;
    langPrefs = data.langPrefs;
    langPrefs.parent = data.langPrefs.parent;
    langToUse = data.langToUse;
}    

// Switches to another prefs panel
function OnPreferencePageUnloading(prefset, data) {
    data.newHasChanged = newHasChanged;
    data.langPrefs = langPrefs;
    data.langPrefs.parent = langPrefs.parent;
    data.langToUse = dialog.newlangList.selection;
    if (!data.setEncoding(data))  {  // check if encoding is appropriate
        // stay on this panel
        //var prefPanelTree = parent.document.getElementById( "prefsTree" );
        //prefPanelTree.selectedItem = parent.document.getElementById('fileSettingsItem');
        //parent.hPrefWindow.switchPage();
        return false;
    }
    else  {
        data.offPanel = true;
        return true;
    }
}

function doNewEncoding()  {
    var count = new Object();
    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,count);
    var encodings = temp.value;

    var init_hierarchy = koEncodingServices.encoding_hierarchy;
    var menupopup = parent.opener.buildEncodingHierarchy(init_hierarchy, true, 'changeNewEncoding(this)');

    // Add default option to the top of the list -> Auto-Detect
    var menuitem = document.createElementNS(XUL_NS, 'menuitem');
    menuitem.setAttribute('label', 'Default Encoding');
    menuitem.setAttribute('data', 'Default Encoding');
    menuitem.setAttribute('oncommand', 'changeNewEncoding(this)');
    menupopup.insertBefore(menuitem, menupopup.childNodes[0]);

    dialog.newencodingList.removeChild(dialog.newencodingList.firstChild);
    dialog.newencodingList.appendChild(menupopup);
    dialog.newencodingList.selectedItem = dialog.newencodingList.firstChild.firstChild;
    changeNewLanguage(null, langToUse);
}

function doBOM(encodingInfo)  {
    var bom = encodingInfo.supports_byte_order_marker;
    if (bom)  {
        dialog.bomBox.setAttribute('disabled', 'false');
        return true;
    }
    else  {
        dialog.bomBox.setAttribute('disabled', 'true');
        return false;  
    }
}

function _setSelectedEncoding(elt,encoding) {
    var items = elt.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        var item = items[i];
        //dump("Item value is " + item.getAttribute("value") + ", label is " + item.getAttribute("label") + "\n");
        if (item.getAttribute('data') == encoding) {
            //dump("set menu popup to " + item.getAttribute("label") + "\n");
            elt.setAttribute("value", encoding);
            elt.selectedItem = item;
            return;
        }
    }
}

function changeNewLanguage(item, name)  {
    var encoding;
    if (item == null)  {  // startup - first time in
        latestLanguage = name;
        currentLangPrefs = langPrefs.getPref('languages/'+latestLanguage);  // make sure to qualify the pref set
        currentLangPrefs.parent = langPrefs;
        dialog.newlangList.selection = name;
    }
    else  {
        var value = dialog.newlangList.selection;
        latestLanguage = value;
        currentLangPrefs = langPrefs.getPref('languages/'+latestLanguage);  // make sure to qualify the pref set
    }

    if (currentLangPrefs.hasPref(latestLanguage+'/newEncoding'))  {
        encoding = currentLangPrefs.getStringPref(latestLanguage+'/newEncoding');
    }        
    else  {
        logError("Could not retrieve the language encoding from preferences.");
    }

    _setSelectedEncoding(dialog.newencodingList,encoding);
    
    if (encoding == 'Default Encoding') {
        dialog.bomBox.setAttribute('checked', false);
        dialog.bomBox.setAttribute('disabled', 'true');
    } else {
        var encodingInfo = koEncodingServices.get_encoding_info(encoding);
        if (doBOM(encodingInfo))  {    
            var bomPref = currentLangPrefs.getBooleanPref(latestLanguage+'/newBOM');
            dialog.bomBox.setAttribute('checked', bomPref);
        }    
    }
}

function changeNewEncoding(item)  {
    var newEncoding = item.getAttribute('data');
    currentLangPrefs.setStringPref(latestLanguage+'/newEncoding', newEncoding);
    if (newEncoding == 'Default Encoding') {
        dialog.bomBox.setAttribute('checked', false);
        dialog.bomBox.setAttribute('disabled', 'true');
    } else 
    if (doBOM(koEncodingServices.get_encoding_info(newEncoding)))  {
        var bomPref = currentLangPrefs.getBooleanPref(latestLanguage+'/newBOM');
        dialog.bomBox.setAttribute('checked', bomPref);        
    }
    newHasChanged = true;
}

function changeNewBOM(box)  {
    currentLangPrefs.setBooleanPref(latestLanguage+'/newBOM', box.checked);
    newHasChanged = true;
}

// Checks to see if encoding selected is appropriate for the file type
function setEncoding(customData)  {
    if (!customData.offPanel)  {
        var koEncoding = Components.classes["@activestate.com/koEncoding;1"]
                    .createInstance(Components.interfaces.koIEncoding);    
        koEncoding.python_encoding_name = dialog.newencodingList.selectedItem.getAttribute('data');
        koEncoding.byte_order_marker = (dialog.bomBox.getAttribute('disabled') == 'false'
                                        && dialog.bomBox.getAttribute('checked') == 'true')

        var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"]
                .getService(Components.interfaces.koILanguageRegistryService);
        var language = languageRegistry.getLanguage(dialog.newlangList.selection);
        
        var warning = language.getEncodingWarning(koEncoding);
        if (warning == "" || 
            customData.mainWin.alertDialog(warning +
                                           ' Are you sure that you want to change the encoding?', false, false, "Ok", "Cancel") == "Ok")  {
            return true;
        }
        else  {
            // reset the pref ??
            return false;
        }
    }
    else  {
        return true;  
    }
}

function doDefaultEncoding()  {
    var count = new Object();
    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,count);
    var encodings = temp.value;

    var init_hierarchy = koEncodingServices.encoding_hierarchy;
    var menupopup = parent.opener.buildEncodingHierarchy(init_hierarchy, true, 'changeOpenEncoding(this)');
    dialog.encodingDefault.removeChild(dialog.encodingDefault.firstChild);
    dialog.encodingDefault.appendChild(menupopup);
    dialog.encodingDefault.selectedItem = dialog.encodingDefault.firstChild.firstChild;
}

function _getSelectedEncoding(elt,encoding) {
    var items = elt.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        var item = items[i];
        //dump("Item value is " + item.getAttribute("value") + ", label is " + item.getAttribute("label") + "\n");
        if (item.getAttribute('data') == encoding) {
            //dump("set menu popup to " + item.getAttribute("label") + "\n");
            elt.setAttribute("value", encoding);
            return item;
        }
    }
    return null;
}

function changeDefaultEncoding() {
    var locale = parent.opener._gInitSvc.getStartupLocale();
    var encoding = locale.split('.')[1].toLowerCase();
    if (dialog.encodingEnvironment.checked) {
        // disable the menu
        var selectedItem = _getSelectedEncoding(dialog.encodingDefault,encoding);
        dialog.encodingDefault.removeChild(dialog.encodingDefault.firstChild);
        var menupopup = document.createElementNS(XUL_NS, 'menupopup');
        menupopup.appendChild(selectedItem);
        dialog.encodingDefault.appendChild(menupopup);
    } else {
        // enable the menu
        doDefaultEncoding();
    }
    _setSelectedEncoding(dialog.encodingDefault,encoding);
}

function changeOpenEncoding(item)  {
}
