/* Copyright (c) 2000-2002 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Constants and global variables
const tag = 'chrome://komodo/content/pref/pref-keys.xul';
var dialog  = {}; 
var keySession = null;

// Handlers

function PrefKeys_OnLoad() {
    try {
        dialog.children = document.getElementById('children');
        dialog.tree = document.getElementById('tree');
        dialog.configurations= document.getElementById('configurations');
        dialog.configpopup= document.getElementById('configpopup');
        dialog.gKeybindingMgr = parent.opener.gKeybindingMgr;
        dialog.editkeybinding = document.getElementById('keybinding');
        dialog.editkeybinding.gKeybindingMgr = dialog.gKeybindingMgr;
        dialog.editkeybinding.mainwindow = parent.opener;
        dialog.gKeybindingMgr.parseGlobalData();
        dialog.deleteButton = document.getElementById('deleteConfiguration');
        updateUI();
        updateConfigMenu();
        dialog.keyname = '';
        updateCommandList()
        parent.hPrefWindow.onpageload(tag);
        parent.hPrefWindow.registerOKCallbackFunc(PrefKeys_OkCallback, tag);
    } catch (e) {
        logError(e)
    }
}

function OnPreferencePageInitalize(prefset) 
{
    var data = new Object();
    dialog.prefset = prefset;
    data.keybindingMgr = dialog.gKeybindingMgr;
    return data;
}

// Start up & returning to this panel in a session
function OnPreferencePageLoading(prefset, data) 
{
    
}

// Switches to another prefs panel
function OnPreferencePageUnloading(prefset, data) 
{
}

function PrefKeys_OkCallback(prefset, data) {
    try {
        if (typeof(dialog) != 'undefined' && dialog && dialog.gKeybindingMgr) {
            dialog.gKeybindingMgr.saveAndApply(dialog.prefset);
            // XXX need to re-apply the parts in the gPartManagers
            dialog.gKeybindingMgr.applyPartKeybindings();
        }
    } catch (e) {
        logError("pref-keys.js: Error in OnPreferencePageUnloading:" + e);
    }
    return true;
}

// Callbacks and global functions having to do with configuration handling

function updateConfigMenu() {
    // Update the configuration menu based on the list of known configurations
    // XXX: Note that there is a very weird bug with this code which makes it unreliable if it's called multiple
    // times -- after the second call, key events seem to go to never never land for a while or a few events
    // and the keyboard handling is messed up -- which is why we use addConfigurationToMenu() and
    // removeConfigurationFromMenu() instead of calling this code again.
    
    var configs = dialog.gKeybindingMgr.getConfigurations();
    var currentConfig = dialog.gKeybindingMgr.currentConfiguration;
    var menuitem;
    while (dialog.configpopup.hasChildNodes()) {
        dialog.configpopup.removeChild(dialog.configpopup.lastChild);
    }
    var c;
    var selectedmenuitem;
    for (i in configs) {
        menuitem = document.createElement('menuitem');
        c = configs[i];
        menuitem.setAttribute('label', c.slice(c.indexOf(': ')+2, c.length));
        menuitem.setAttribute('data', c);
        menuitem.setAttribute('oncommand', "return switchConfig(\""+configs[i]+"\");");
        dialog.configpopup.appendChild(menuitem);
        if (c == currentConfig) {
            selectedmenuitem = menuitem;
        }
    }
    menuitem = document.createElement('menuitem');
    menuitem.setAttribute('data', "");
    menuitem.setAttribute('label', "Save As New Configuration...");
    menuitem.setAttribute('oncommand', "newConfig();");
    dialog.configpopup.appendChild(menuitem);
    //dialog.configpopup.selectedItem = selectedmenuitem;
    dialog.configurations.selectedItem = selectedmenuitem;
    dialog.configurations.setAttribute("label", dialog.gKeybindingMgr.currentConfiguration.split(': ')[1]);
}

function updateSelectedMenuItem() {
    // Make the right menuitem be the 'selected' menuitem in the menulist of
    // known configurations.
    var configs = dialog.gKeybindingMgr.getConfigurations();
    var currentConfig = dialog.gKeybindingMgr.currentConfiguration;
    var child;
    var i;
    for (i = 0; i < dialog.configpopup.childNodes.length; i++) {
        child = dialog.configpopup.childNodes[i];
        if (child.getAttribute('data') == currentConfig) {
            dialog.configurations.selectedItem = child;
            break;
        }
    }    
}

function addConfigurationToMenu(c) {
    // Add a configuration to the menulist of known configurations.
    var newchild = document.createElement('menuitem');
    newchild.setAttribute('label', c.slice(c.indexOf(': ')+2, c.length));
    newchild.setAttribute('data', c);
    newchild.setAttribute('oncommand', "return switchConfig(\""+c+"\");");
    dialog.configpopup.insertBefore(newchild, dialog.configpopup.lastChild);
    dialog.configurations.selectedItem = newchild;
}

function removeConfigurationFromMenu(configname) {
    // Remove the menuitem corresponding to 'configname' from the menulist of known configurations.
    var child;
    var i;
    //dump("removeConfigurationFromMenu called with " + configname + "\n");
    for (i = 0; i < dialog.configpopup.childNodes.length; i++) {
        child = dialog.configpopup.childNodes[i];
        //dump("Got " + child.getAttribute('data') + " and looking for " + configname +'\n');
        if (child.getAttribute('data') == configname) {
            //dump("GOT IT\n");
            dialog.configurations.selectedItem = child.parentNode.firstChild;
            child.parentNode.removeChild(child)
            break;
        }
    }    
}

function newConfig() {
    // called by the last menuitem in the configuration menulist
    var newconfig = promptDialog("Name of new keyboard configuration:","Custom Configuration");
    if (newconfig) {
        newconfig = dialog.gKeybindingMgr.saveConfigurationAs(newconfig, dialog.prefset);
        dialog.gKeybindingMgr.switchConfiguration(newconfig, dialog.prefset);
        addConfigurationToMenu(newconfig);
        updateUI();
    } else {
        switchConfig(dialog.gKeybindingMgr.currentConfiguration);
    }
}

function switchConfig(configname) {
    // called when users switch to another configuration from the menulist
    dialog.gKeybindingMgr.switchConfiguration(configname, dialog.prefset);
    updateUI();
}

function updateUI() {
    var configname = dialog.gKeybindingMgr.currentConfiguration;
    updateSelectedMenuItem();
    if (dialog.gKeybindingMgr.configurationWriteable(configname)) {
        dialog.deleteButton.removeAttribute('disabled');
    } else {
        dialog.deleteButton.setAttribute('disabled', 'true');
    }
    dialog.editkeybinding.updateCurrentKey();
}

function deleteConfiguration()  {
    // called by the Delete button next to the configuration menu list.
    //koDumpDOM(dialog.configurations);
    var configname = dialog.configurations.selectedItem.getAttribute('data');
    if (typeof(configname) == 'undefined') {
        logError("SelectedItem doesn't have a data attribute");
        alertDialog("There was a problem delete the configuration.");
        return;
    }
    dialog.gKeybindingMgr.deleteConfiguration(configname, dialog.prefset);
    removeConfigurationFromMenu(configname);
    updateUI();
}

function selectKey(event, treeitem)  {
    // Called by the handler in the command list tree.
    var item;
    if (treeitem)
        item = treeitem;
    else {
        item = dialog.tree.selectedItems[0];
    }
    if (item) {
        dialog.editkeybinding.commandId = item.getAttribute('name');
        dialog.editkeybinding.updateCurrentKey();
    }
}

function updateCommandList() {
    // Add treeitems to the tree based on the information from the keybinding manager.
    // This is slow, which is why it's inlined.
    var i, cell, row, item, command;
    var commanditems = dialog.gKeybindingMgr.commanditems
    for (i in commanditems) {
        command = commanditems[i];
        cell = document.createElement('treecell');
        cell.setAttribute('label', command.desc);
        row = document.createElement('treerow');
        row.appendChild(cell);
        item = document.createElement('treeitem');
        item.setAttribute('name', command.name);
        item.setAttribute('data', command.desc);
        item.appendChild(row);
        dialog.children.appendChild(item);
    }
    dialog.children.setAttribute("collapsed", false);
}

