/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// Constants and global variables
const nsIFilePicker       = Components.interfaces.nsIFilePicker;

const tag = 'chrome://komodo/content/pref/pref-servers.xul';
var cellparent, cellparent_value;
const VK_ENTER = 13;  // the keycode for the "Enter" key
var servers;
var dialog = {}; //XXX used both for easy access to named XUL elements and to keep
// a handle on functions that we need in PrefAssociation_OkCallback(). Every
// reference to one of these functions that might be called, directly or
// indirectly, from PrefAssociation_OkCallback() must be accessed through
// the dialog object, or this will break.  For consistency, we always call
// these functions through the dialog object.

// Classes

function Server(type, alias, hostname, port, path, username, password) {
  this.type = type;
  this.alias = alias;
  this.hostname = hostname;
  this.port = port;
  this.path = path;
  this.username = username;
  this.password = password;
  this.id = Server._generateId();
}

// Each Association object has a unique identifier.  This is used to
// establish a relationship between the association object and the
// items in the tree that represent it.

Server._index = 0;  // used for generating ids
Server._generateId = function() {
  return "server" + Server._index++;
}


// Handlers

function PrefServers_OnLoad() {
    dialog = {};
    dialog.server_types = document.getElementById("server_types");
    dialog.serversList = document.getElementById("serversList");
    dialog.serversPopup = document.getElementById("serversPopup");
    dialog.alias = document.getElementById("alias");
    dialog.hostname = document.getElementById("hostname");
    dialog.port = document.getElementById("port");
    dialog.path = document.getElementById("path");
    dialog.username = document.getElementById("username");
    dialog.password = document.getElementById("password");
    dialog.alias = document.getElementById("alias");
    dialog.alias = document.getElementById("alias");
    dialog.buttonDelete = document.getElementById("buttonDelete");
    dialog.buttonAdd = document.getElementById("buttonAdd");
    dialog.anonymousCheckbox = document.getElementById("anonymousCheckbox");
    dialog.buttonDelete.setAttribute('disabled', 'true');
    dialog.buttonAdd.setAttribute('disabled', 'true');

    parent.hPrefWindow.onpageload(tag);
    parent.hPrefWindow.registerOKCallbackFunc(PrefServers_OkCallback, tag);
}

function PrefServers_OkCallback(prefset, customData) {
    var oldServers = customData.original;
    var newServers = customData.servers;

    // only write out the prefs if they have actually changed
    if (!customData.areServerListsEquivalent(oldServers, newServers)) {
      var collectionPref = 
           Components.classes["@activestate.com/koOrderedPreference;1"]
              .createInstance(Components.interfaces.koIOrderedPreference);
      for (var i = 0; i < newServers.length; i++) {
          var singlePref = 
              Components.classes["@activestate.com/koOrderedPreference;1"]
                 .createInstance(Components.interfaces.koIOrderedPreference);
          singlePref.appendStringPref(newServers[i].alias);
          var info = newServers[i].type+":"+newServers[i].hostname+":"+newServers[i].port+":"+
                          newServers[i].path+":"+newServers[i].username+":"+newServers[i].password;
          singlePref.appendStringPref(info);
          collectionPref.appendPref(singlePref);
      }
      prefset.setPref("deploymentServerList", collectionPref);
    }
    return true;
}

function _strcmp(a,b) {
    if (a==b) return 0;
    if (a<b) return -1;
    return 1;
}
// Return an array of Association objects, built from the fileAssociations
// preference.
function getServerListFromPreference(prefset) {
  var list = new Array();  
  if (prefset.hasPref("deploymentServerList")) {
    var deploymentServerList = prefset.getPref("deploymentServerList");
    for (var i = 0; i < deploymentServerList.length; i++) {
      var tuple = deploymentServerList.getPref(i).QueryInterface(Components.interfaces.koIOrderedPreference);
      // invocations were added later
      var info = new String(tuple.getStringPref(1));
      var server_info = info.split(":")
      list.push(new Server(server_info[0], tuple.getStringPref(0), 
              server_info[1], server_info[2], server_info[3],
              server_info[4], server_info[5]));
    }
  }
  list.sort(function (a,b) { return _strcmp(a.alias,b.alias); });
  return list;
}

// Return true if the arrays of Association objects, a and b, correspond to 
// the same underlying representation as a preference.
function areServerListsEquivalent(a, b) {
  if (a.length != b.length) return false;

  for (var i = 0; i < a.length; i++) {
    // we don't care about the IDs
    if (a[i].type != b[i].type ||
        a[i].alias != b[i].alias ||
        a[i].hostname != b[i].hostname ||
        a[i].port != b[i].port ||
        a[i].path != b[i].path ||
        a[i].username != b[i].username ||
        a[i].password != b[i].password) {
      return false;
    }
  }

  return true;
}

function OnPreferencePageInitalize(prefset) {
    var data = new Object();
    data.servers = getServerListFromPreference(prefset);
    data.original = getServerListFromPreference(prefset);
    data.areServerListsEquivalent = areServerListsEquivalent; // needs to be called when this page not loaded.
    return data;
}

function OnPreferencePageLoading(prefset, data) {
    document.getElementById("buttonDelete").setAttribute('disabled', 'true');
    document.getElementById("buttonAdd").setAttribute('disabled', 'true');
    servers = data.servers;
    // fill out the associations tree - should cache this entire tree.
    var selectedServer = null;
    if (typeof(data.selectedServer) != 'undefined')
        selectedServer = data.selectedServer;
    _setMenuList(selectedServer);
}

function _setMenuList(selectedServer) {
    var menulist = document.getElementById("serversList");
    var menupopup = document.getElementById("serversPopup");
    var menuitem;
    var indexSelect = -1;
    var children = menupopup.childNodes;
    var i;
    for (i = 0; i < children.length; i++) {
        menupopup.removeChild(children[i]);
    }
    for (var i = 0; i < servers.length; i++) {
        menuitem = createServersMenuItem(servers[i]);
        menupopup.appendChild(menuitem);
        if (servers[i].alias == selectedServer)
            indexSelect = i;
    }
    if (servers.length > 0)  {
        if (indexSelect == -1)
            indexSelect = 0;
        menulist.selectedIndex = indexSelect;
        onEditServerEntry(servers[indexSelect].id, null, menulist.selectedItem);
    }
}

function OnPreferencePageUnloading(prefset, data) {
    data.servers = servers;
    if (dialog.serversList.selectedItem)
        data.selectedServer = dialog.serversList.selectedItem.getAttribute("label");
}

function OnPreferencePageSaving(prefset, data) {
}

function hidePassword(password) {
    // XXX 2way encrypt the password somehow, to prevent plaintext in the prefs files
    var crypto = 
       Components.classes["@activestate.com/koPasswordCrypto;1"]
          .getService(Components.interfaces.koIPasswordCrypto);
    
    return crypto.encodePassword(password);    
}

/* XXX duplicated in filebrowser.js */
function decodePassword(password) {
    if (typeof(password) == 'undefined' || !password) return '';
    // XXX 2way encrypt the password somehow, to prevent plaintext in the prefs files
    var crypto = 
       Components.classes["@activestate.com/koPasswordCrypto;1"]
          .getService(Components.interfaces.koIPasswordCrypto);
    var text = password;
    try {
        text = crypto.decodePassword(password);    
    } catch(e) { 
        /* password is empty, that's ok, we'll ask for it later */ 
        logWarning("Dropping password that could not be decoded. See bug 18752.");
    }
    return text
}

function checkAddButtonStatus() {
    var alias = dialog.alias.value;
    var hostname = dialog.hostname.value;
    if (alias && hostname) {
        // enable add button
        document.getElementById("buttonAdd").removeAttribute('disabled');
    } else {
        // dissable add button
        document.getElementById("buttonAdd").setAttribute('disabled', 'true');
    }
}

var current_server_idx = -1;
var current_server_cell = null;
function onAddServerEntry() {
    var type = dialog.server_types.selectedItem.getAttribute("label");
    var alias = dialog.alias.value;
    var hostname = dialog.hostname.value;
    var port = dialog.port.value;
    var path = dialog.path.value;
    var username = dialog.username.value;
    var password = dialog.password.value;
    // prevent adding empty entries
    if (!alias && !hostname) {
        logWarning("Emtpy alias and hostname in server entry!");
        return;
    }
    // Ensure that no ":" chars are used in any of the fields. ":" is used
    // uncarefully as a delimiter in the server saving code.
    var fieldWithColon = null;
    if (alias.indexOf(":") != -1) {
        fieldWithColon = "alias";
    } else if (hostname.indexOf(":") != -1) {
        fieldWithColon = "hostname";
    } else if (port.indexOf(":") != -1) {
        fieldWithColon = "port";
    } else if (path.indexOf(":") != -1) {
        fieldWithColon = "path";
    } else if (username.indexOf(":") != -1) {
        fieldWithColon = "username";
    }
    if (fieldWithColon) {
        parent.opener.alertDialog("The " + fieldWithColon
                                  + " field cannot contain a ':' character");
        var textbox = document.getElementById(fieldWithColon);
        textbox.focus();
        textbox.setSelectionRange(0, textbox.value.length);
        return;
    }
    var server;
    
    if (current_server_idx > -1) {
        server = servers[current_server_idx];
        server.type = type;
        server.alias = alias;
        server.hostname = hostname;
        server.port = port;
        server.path = path
        server.username = username;
        server.password = "";
        if (password != "" && password != null) 
            server.password = hidePassword(password);
        servers[current_server_idx] = server;
        current_server_cell.setAttribute('label', server.alias);
        document.getElementById("serversList").setAttribute('label', server.alias);
    } else {
        if (getAliasIndex(alias) > -1) {
            alertDialog("The entry name is not unique, please correct this.\n");
            return;
        }
        if (password) password = hidePassword(password);
        server = new Server(type, alias, hostname, port, path, username, password);
        servers.push(server);
        servers.sort(function (a,b) { return _strcmp(a.alias,b.alias); } );
        _setMenuList(server.alias);
    }
    setMenuSelection("serversList",server.alias);
    dialog.buttonAdd.setAttribute('disabled', 'true');
}

function setMenuSelection(id, label) {
    var menu = document.getElementById(id);
    var items = menu.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        if (items[i].getAttribute("label") == label) {
            menu.selectedIndex = i
            return;
        }
    }
}

function onEditServerEntry(id, item, event) {
    var server_idx = getIdIndex(id);
    if (server_idx < 0) return;
    var server = servers[server_idx];
    setMenuSelection("server_types",server.type);
    dialog.alias.value = server.alias;
    dialog.hostname.value = server.hostname;
    dialog.port.value = server.port;
    dialog.path.value = server.path;
    dialog.username.value = server.username;

    dialog.password.value = decodePassword(server.password);
        
    current_server_idx = server_idx;
    if (item != null)
        current_server_cell = event.target;
    else
        current_server_cell = event;
    dialog.buttonDelete.removeAttribute('disabled');
    dialog.buttonAdd.setAttribute('disabled','true');
    dialog.buttonAdd.setAttribute('label', 'Update');
    if (dialog.username.value == 'anonymous')
        dialog.anonymousCheckbox.checked = true;
    else
        dialog.anonymousCheckbox.checked = false;
}

function onDeleteServerEntry() {
    if (current_server_idx > -1) {
        servers.splice(current_server_idx, 1);
        dialog.serversPopup.removeChild(current_server_cell);
        onClearServerEntry();
        if (servers.length > 0) {
            var menulist = document.getElementById("serversList");
            menulist.selectedIndex = 0;
            onEditServerEntry(servers[0].id, null, menulist.selectedItem);
        }
    }
}

function onClearServerEntry() {
    dialog.server_types.selectedIndex = 0;
    dialog.serversList.selectedIndex = -1;
    dialog.alias.value = "";
    dialog.hostname.value = "";
    dialog.port.value = "";
    dialog.path.value = "/";
    dialog.username.value = "";
    dialog.password.value = "";
    current_server_idx = -1;
    current_server_cell = null;
    dialog.buttonDelete.setAttribute('disabled', 'true');
    dialog.buttonAdd.setAttribute('disabled', 'true');
    dialog.buttonAdd.setAttribute('label', 'Add');
    dialog.anonymousCheckbox.checked = false;
}

// DOM construction helpers

function createServersMenuItem(server) {
    var cell = document.createElementNS(XUL_NS, 'menuitem');
    cell.setAttribute('id', server.id);
    cell.setAttribute('label', server.alias);
    cell.setAttribute('assocId', server.id);
    cell.setAttribute("onclick", "onEditServerEntry('"+server.id+"',this, event)");
    return cell;
}

function getIdIndex(id) {
    for (var i = 0; i < servers.length; i++) {
        if (id == servers[i].id)
            return i;
    }
    return -1;
}

function getAliasIndex(alias) {
    for (var i = 0; i < servers.length; i++) {
        if (alias == servers[i].alias)
            return i;
    }
    return -1;
}

// Remove whitespace from both ends of a string
function trimString(string)
{
    if (!string)
        return "";
    return string.replace(/(^\s+)|(\s+$)/g, '');
}


function onBrowseFTP() {
    // get all the info we want to use
    var type = dialog.server_types.selectedItem.getAttribute("label");
    var alias = dialog.alias.value;
    var hostname = dialog.hostname.value;
    var port = dialog.port.value;
    var path = dialog.path.value;
    var username = dialog.username.value;
    var password = dialog.password.value;
    if (password != null && password != "") {
        // this was already saved, get the password
        var idx = getAliasIndex(alias);
        var server = servers[idx];
        password = server.password;
    }
    // build an explicit path
    var url = "ftp://";
    if (username) {
        url += username
        if (password) url += ":"+password;
        url += "@";
    }
    url+=hostname
    if (port) url+= ":"+port;
    url+=path;
    
    var fileBrowser = doFilebrowser("Pick Directory", url, nsIFilePicker.modeGetFolder);
    if (fileBrowser) {
        //dump("selected "+fileBrowser.directory+"\n");
        dialog.path.value = fileBrowser.directory;
    }
}

function fillAnonymous()  {
    var loginUserBox = document.getElementById('username');
    var loginPwdBox = document.getElementById('password');
    loginUserBox.value = document.getElementById('bundle_anonymous').getString("anonymousUser");
    loginPwdBox.value = document.getElementById('bundle_anonymous').getString("anonymousPwd");
}
