/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

//---- globals
var _findingInterps = false;
var dialog;

// preferences used in the pref window
var _elementIDs = [
//XXX need infrastructure for these in the editor
//    "tclUseLinting",
//    "tclUseTooltips"
    "tclshDefaultInterpreter",
    "wishDefaultInterpreter",
];


//---- functions

function PrefTcl_OnLoad()
{
    dialog = {};
    dialog.tclshFoundText = document.getElementById('tclshFoundText');
    dialog.wishFoundText = document.getElementById('wishFoundText');
    
    var tag = 'chrome://komodo/content/pref/pref-tcl.xul';
    parent.hPrefWindow.onpageload(tag);
    parent.hPrefWindow.registerOKCallbackFunc(PrefTcl_OkCallback, tag); 

    _findingInterps = true;
    PrefTcl_InsertFindingMessage(document.getElementById("avail-tclsh-interp-list"));
    PrefTcl_InsertFindingMessage(document.getElementById("avail-wish-interp-list"));
    PrefTcl_PopulateTclInterps()

//  There are redraw issues with doing this. If the collapsed license section is shown
//  then there are a lot of wierd artifacts.
//  setTimeout("PrefTcl_PopulateTclInterps()", 0);
}


function PrefTcl_OkCallback(prefset)
{
    var ok = true;
    
    // ensure that the default tcl interpreters are valid
    var defaultTclshInterp = prefset.getStringPref("tclshDefaultInterpreter");
    var defaultWishInterp  = prefset.getStringPref("wishDefaultInterpreter");
    var koSysUtils = Components.classes["@activestate.com/koSysUtils;1"].
        getService(Components.interfaces.koISysUtils);

    if (defaultTclshInterp != "") {
        if (! koSysUtils.IsFile(defaultTclshInterp)) {
            parent.opener.alertDialog("No Tcl interpreter could be found at '" +
                defaultTclshInterp + "'. You must make another selection for " +
                "the default Tcl interpreter.\n");
            ok = false;
            document.getElementById("tclshDefaultInterpreter").focus();
        }
    }

    if (defaultWishInterp != "") {
        if (! koSysUtils.IsFile(defaultWishInterp)) {
            parent.opener.alertDialog("No Tcl Wish interpreter could be found at '" +
                defaultWishInterp + "'. You must make another selection for " +
                "the default Tcl Wish interpreter.\n");
            ok = false;
            document.getElementById("wishDefaultInterpreter").focus();
        }
    }

    return ok;
}


function PrefTcl_UnSelectInterp()
{
    var tree = document.getElementById("tree-avail-interp-list");
    tree.clearItemSelection();
}

function PrefTcl_UnSelectWishInterp()
{
    var tree = document.getElementById("tree-avail-wish-interp-list");
    tree.clearItemSelection();
}

function PrefTcl_SelectInterp()
{
    if (! _findingInterps) {
        // Load the default interpeter textfield with the selection
        var tree = document.getElementById("tree-avail-interp-list");
        var defaultInterp = document.getElementById("tclDefaultInterpreter");
        if (tree.selectedItems.length > 0) {
            var selection = tree.selectedItems[0];
            //          treeitem . treerow  . treecell 
            var value = selection.firstChild.firstChild.getAttribute("label");
            defaultInterp.value = value;
        }
    }
}

function PrefTcl_SelectWishInterp()
{
    if (! _findingInterps) {
        // Load the default interpeter textfield with the selection
        var tree = document.getElementById("tree-avail-wish-interp-list");
        var defaultInterp = document.getElementById("tclWishInterpreter");
        if (tree.selectedItems.length > 0) {
            var selection = tree.selectedItems[0];
            //          treeitem . treerow  . treecell 
            var value = selection.firstChild.firstChild.getAttribute("label");
            defaultInterp.value = value;
        }
    }
}

function _CreateTclInterpItem(fname)
{
    var treecell = document.createElementNS(XUL_NS, 'treecell');
    treecell.setAttribute('label', fname);
    var treerow = document.createElementNS(XUL_NS, 'treerow');
    treerow.appendChild(treecell);
    var treeitem = document.createElementNS(XUL_NS, 'treeitem');
    treeitem.appendChild(treerow);
    return treeitem;
}

function isTclPro(interp)
{
    var os = Components.classes["@activestate.com/koOs;1"].
                     getService(Components.interfaces.koIOs);
    var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                     getService(Components.interfaces.koIInfoService);
 
    if (infoSvc.isWindows) 
        return os.path.exists(os.path.join(os.path.dirname(interp), 'prowrap.exe'));
    else
        return os.path.exists(os.path.join(os.path.dirname(interp), 'prowrap'));
}

function filterInterpreters(availInterps)
{
    for (var i = 0; i < availInterps.length;){
        if (!isTclPro(availInterps[i])) {
            availInterps.splice(i, 1);
        }
        else {
            ++i;
        }
    }
    return availInterps
}

function PrefTcl_PopulateTclInterps()
{
    // Populate the (tree) list of available Tcl interpreters on the current
    // system.
    
    var tclInfoEx = Components.classes["@activestate.com/koTclProInfoEx;1"]
                     .createInstance(Components.interfaces.koITclInfoEx);

    var os = Components.classes["@activestate.com/koOs;1"]
                     .createInstance(Components.interfaces.koIOs);
    
    var numFound = new Object();
    var availInterps = tclInfoEx.FindInstallationPaths(numFound);
    var licensedTclsh = Array();
    var licensedWish = Array();
    var unlicensedInstalls = Array();
    
    for (var i = 0; i < availInterps.length; ++i){
        tclInfoEx.installationPath = availInterps[i]
        
        if (tclInfoEx.haveLicense) {
            if (os.path.exists(tclInfoEx.tclsh_path)) {
                licensedTclsh.push(tclInfoEx.tclsh_path)
            }
            if (os.path.exists(tclInfoEx.wish_path)) {
                licensedWish.push(tclInfoEx.wish_path)
            }
        } else {
             unlicensedInstalls.push(tclInfoEx.installationPath)           
        }
    }
    
    PrefTcl_PopulateTclshInterps(licensedTclsh);
    PrefTcl_PopulateWishInterps(licensedWish);
    PrefTcl_PopulateUnlicensedInstalls(unlicensedInstalls);

    _findingInterps = false;
}

function PrefTcl_InsertFindingMessage(availInterpList)
{
    // remove any existing items and add a "finding..." one
    while (availInterpList.hasChildNodes()) {
        availInterpList.removeChild(availInterpList.childNodes[0]);
    }
    availInterpList.appendChild(_CreateTclInterpItem(
        "Finding available Tcl interpreters..."));    
}

function PrefTcl_PopulateTclshInterps(availInterps)
{
    var availInterpList = document.getElementById("avail-tclsh-interp-list");
    var defaultInterp = document.getElementById("tclshDefaultInterpreter");
    
    // populate the tree listing them
    if (availInterps.length == 0) {
        // tell the user no interpreter was found and direct them to
        // ActiveState to get one
        document.getElementById("avail-interp-box").
            setAttribute("collapsed", "true");
        document.getElementById("no-avail-tclsh-interps-message").
            setAttribute("collapsed", "false");
        dialog.tclshFoundText.setAttribute("collapsed", "true");
    } else {
        availInterpList.removeChild(availInterpList.childNodes[0]); // "Finding..."
        for (var i = 0; i < availInterps.length; i++) {
            availInterpList.appendChild(_CreateTclInterpItem(
                availInterps[i]));
        }
    }
}

// Populate the (tree) list of available Tcl wish interpreters on the current
// system.
function PrefTcl_PopulateWishInterps(availInterps)
{
    var availInterpList = document.getElementById("avail-wish-interp-list");
    var defaultInterp = document.getElementById("wishDefaultInterpreter");
    
    // populate the tree listing them
    if (availInterps.length == 0) {
        // tell the user no interpreter was found and direct them to
        // ActiveState to get one
        document.getElementById("avail-wishinterp-box").
            setAttribute("collapsed", "true");
        document.getElementById("no-avail-wish-interps-message").
            setAttribute("collapsed", "false");
        dialog.wishFoundText.setAttribute("collapsed", "true");
    } else {
        availInterpList.removeChild(availInterpList.childNodes[0]); // "Finding..."
        for (var i = 0; i < availInterps.length; i++) {
            availInterpList.appendChild(_CreateTclInterpItem(
                availInterps[i]));
        }
    }
}

function PrefTcl_PopulateUnlicensedInstalls(unlicensedInstalls)
{
    if (unlicensedInstalls.length == 0) {
        return;
    }
    
    document.getElementById("license-groupbox").
        setAttribute("collapsed", "false");
    
    var unlicensedInstallsList = document.getElementById("unlicensed-tclpro-installations-list");
    
    for (var i = 0; i < unlicensedInstalls.length; i++) {
        unlicensedInstallsList.appendChild(_CreateTclInterpItem(
            unlicensedInstalls[i]));
    }
}
