/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// The Part wrapper class.  This wrapps koIParts into JS objects w/ a part attribute.
// This is done for two reasons: 1) performance -- accessing JS objects should be much faster
// than accessing XPCOM objects.  2) isolating run-time properties as opposed to serialized properties.
// Plugins will typically _read_ properties of the .part, but store runtime data in the wrapped object.

// gObserverSvc is located in statusbar.js :/

function partWrapper(part, depth) {
    var i;
    this.properties = {}; // filled in by extensions for view datapoints
    this.level = depth;
    this.part = part;
    this.type = 'file';
    this.styles = {}; // outline properties
    this.iconstyle = {};
    this.iconopenstyle = {};
    this._name = null
    this.renameable = false;
    this.url = null;
    this.isContainerOpen = false;
    this.isContainer = false;
    this.children = null;
    
    // XXX getting the name should be in the plugins I guess
    if (part) {
        this.type = part.type;  // Performance optimization
        if (part.hasAttribute('name')) {
            this.properties['Name'] = this._name = part.getStringAttribute('name');
        }
        // This should be done by the file plugin.
        if (part.hasAttribute('url')) {
            this.url = part.getStringAttribute('url');
            if (!this._name) {
                // os defined in koGlobalOverlay.js
                this.properties['Name'] = this._name = os.path.basename(this.url);
            }
        }
        this.applyPartKeybindings(false); // do not recurse at this point;
        try {
            part = part.QueryInterface(Components.interfaces.koIContainerBase);
            // This should be moved out someday.
            this.isContainer = true;
            this.isContainerOpen = false;
        } catch (e) {/* no error condition here*/}
    }
    if (!this.name) this.name = this.type;

    /* pass the new part through the extension manager */
    extensionManager.callHandler(this, 'onUpdatePart', null);
    
    if (this.isContainer) {
        this.children = [];
        var children = new Array();
        part.getChildren(children, new Object());
        if (children.value.length) {
            for (i=0; i< children.value.length; i++) {
                this.children[i] = new partWrapper(children.value[i], depth+1);
            }
        }
    }
    
    // let the status service know we're ready for it
    if (this.url) {
        try {
            gObserverSvc.Notify(self, 'file_added', this.url);
        } catch(e) { /* exception if no listeners */ }
    }
    
    return this;    
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
partWrapper.prototype.constructor = partWrapper;

partWrapper.prototype.__defineGetter__("value",
function() 
{
    if (!this.part) return null;
    return this.part.value;
});

partWrapper.prototype.__defineSetter__("value",
function(value) 
{
    if (this.part) this.part.value = value;
});

partWrapper.prototype.__defineGetter__("name",
function() 
{
  return this._name;
});

partWrapper.prototype.__defineSetter__("name",
function(value) 
{
  this.properties['Name'] = this._name = value;
  if (this.part)
    this.part.setStringAttribute('name',this._name);
});

partWrapper.prototype.applyPartKeybindings = function(recurse) {
    if (!this.part) return;
    var i;
    var part = this.part;
    
    if (gKeybindingMgr && part.hasAttribute('keyboard_shortcut')) {
        var keylabels = part.getStringAttribute('keyboard_shortcut').split('###');
        var keylabel;
        var keysequence;
        for (i in keylabels) {
            keylabel = keylabels[i];
            if (keylabel != '') {
                keysequence = keylabel.split(', ')
                if (typeof(part.shortcut_command_attribute) != 'undefined') {
                    var parameter = part.getStringAttribute(part.shortcut_command_attribute);
                    if (parameter) {
                        gKeybindingMgr.assignKey(part.shortcut_command, keysequence, parameter);
                        gKeybindingMgr.makeKeyActive(part.shortcut_command, keysequence);
                    }
                } else {
                    logWarning("Error setting keyboard shortcut for " + part);
                }
            }
        }
    }
    if (recurse) {
        if (this.children) {
            // Do the same thing to the children.
            for (i = 0; i < this.children.length; i++) {
                this.children[i].applyPartKeybindings(recurse);
            }
        }
    }
}

/* this gets unique paths within a tree of parts.  If the part
   does not have children, its full path is returned.  In the case
   of a part having children, the directory for each child is added
   (there will be no dupes because a hash is used).
   This function is written primarily for use by scc. */
partWrapper.prototype.updateUniquePaths = function(urls,recursive /*=false*/) {
    if (typeof(recursive)=='undefined') recursive = false;
    if (recursive && this.children && this.children.length > 0) {
        if (this.url) {
            if (os.path.isdir(this.url)) {
                urls[this.url] = 1;
            } else {
                urls[os.path.dirname(this.url)] = 1;
            }
        }
        for (var child in this.children) {
            this.children[child].updateUniquePaths(urls,recursive);
        }
    } else if (this.url) {
        var dir = os.path.dirname(this.url);
        if (typeof(urls[dir]) == 'undefined')
            urls[this.url] = 1;
    }
}

partWrapper.prototype.dump = function() {
    dump('--------------------------\n');
    dump('URL: '+this.url+'\n');
    dump('Properties:\n');
    for (i in this.properties) {
        dump('    '+i+': '+this.properties[i]+'\n');
    }
    dump('IconStyle:\n');
    for (i in this.iconstyle) {
        dump('    '+i+': '+this.iconstyle[i]+'\n');
    }
    dump('IconOpenStyle:\n');
    for (i in this.iconopenstyle) {
        dump('    '+i+': '+this.iconopenstyle[i]+'\n');
    }
}

partWrapper.prototype.hasChild = function(child,recurse) {
    if (typeof(recurse)=='undefined') recurse = false;
    for (var i = 0; i < this.children.length; i++) {
        if (this.children[i] == child) 
            return true;
        if (recurse && this.children[i].hasChild(child,recurse))
            return true;
    }
    return false;
}

partWrapper.prototype.removeChild = function(child) {
    for (var i = 0; i < this.children.length; i++) {
        if (this.children[i] == child) {
            this.children.splice(i,1);
            if (typeof(child.part.shortcut_command) != 'undefined' && child.part.shortcut_command) {
                try {
                    gObserverSvc.Notify(child.part, 'kb-unload', child.part.shortcut_command);
                } catch(e) { /* exception if no listeners */ }
            }
            this.part.removeChild(child.part);
            // let the status service know we're removing this
            if (child.url) {
                try {
                    gObserverSvc.Notify(self, 'file_removed', child.url);
                } catch(e) { /* exception if no listeners */ }
            }
            return;
        }
    }
}

partWrapper.prototype.getDragData = function() {
    if (!this.part) return null;
    // if the part is not dragable, don't drag it
    if (this.part.hasAttribute('dragable') && !this.part.getBooleanAttribute('dragable')) return null;
    
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].
                                createInstance(Components.interfaces.nsITransferable);
    var flavors = new Object();
    this.part.getDragFlavors(flavors, new Object());
    for (var i = 0; i < flavors.value.length; i++) {
        trans.addDataFlavor(flavors.value[i]);
    }

    var genTextData = Components.classes["@mozilla.org/supports-wstring;1"].
        createInstance(Components.interfaces.nsISupportsWString);
    //originalTarget is outlinerbody
    genTextData.data = this.part.getDragData();
    trans.setTransferData(flavors.value[0], genTextData, genTextData.data.length * 2);
    return trans;
}

partWrapper.prototype.updateWrappedPart = function() {
    if (!this.isContainer || !this.part) {
        return null;
    }
    // The 'part', wrapped in 'wrapped', has children or children of ... of children which may not be wrapped.
    var gotit;
    var child;
    var wrappedChild;
    /* get the part-type children */
    var children = new Array();
    this.part.getChildren(children, new Object());

    var unwrappedchildren = new Array();

    for (var i=0; i< children.value.length; i++) {
        child = children.value[i];
        /* we have a part, we don't know if it's wrapped or not */
        gotit = 0;
        /* iterate over the children of the wrapped part */
        for (var wrappedChildIndex in this.children) {
            wrappedChild = this.children[wrappedChildIndex];
            if (wrappedChild.part == child) {
                gotit = 1;
                /* check the grandchildren */
                try {
                    child = child.QueryInterface(Components.interfaces.koIContainerBase);
                } catch (e) {
                    child = null;
                }
                if (child) {
                    wrappedChild.updateWrappedPart();
                }
                /* pass the new part through the extension manager */
                extensionManager.callHandler(wrappedChild, 'onUpdatePart', null);                
                break;
            }
        }
        if (! gotit) {
            /* brand new part, needs wrapping */
            unwrappedchildren.push(child);
        }
    }

    /* firstchild is returned so that the outliner view can scroll to an apropriate position */
    var firstchild = null;
    for (childIndex in unwrappedchildren ) {
        child = unwrappedchildren[childIndex];
        wrappedChild = new partWrapper(child, this.level+1);
        if (!firstchild) firstchild = wrappedChild;
        this.children.push(wrappedChild);
    }
    if (firstchild) this.sortChildren();
    return firstchild;
}

partWrapper.prototype.sortChildren = function() {
    if (!this.children || this.children.length < 1) return;
    if (this.children.length > 1) {
        this.children.sort(compareWrappers);
    }
    for (var child in this.children) {
        this.children[child].sortChildren();
    }
}

var gWrapperComparison = 'Name';
var gWrapperSortDir = 1;
function compareWrappers(a,b) {
    var au = typeof(a.properties[gWrapperComparison]) == 'undefined';
    var bu = typeof(b.properties[gWrapperComparison]) == 'undefined';
    // deal with non-existent columns
    if (au && bu) return 0;
    else if (au) return -1 * gWrapperSortDir;
    else if (bu) return 1 * gWrapperSortDir;
    var a1 = a.properties[gWrapperComparison].toLowerCase();
    var b1 = b.properties[gWrapperComparison].toLowerCase();
    if (Date.parse(a1)) {
        //dump('converting to date '+a1+' and '+b1+'\n');
        a1 = Date.parse(a1);
        b1 = Date.parse(b1);
    } else if (parseFloat(a1)) {
        //dump('converting to number '+a1+' and '+b1+'\n');
        a1 = parseFloat(a1);
        b1 = parseFloat(b1);
    }
    //dump('comparing '+gWrapperComparison+' '+a1+' and '+b1+'\n');
    // ok, they both have the property, compare
    if (a1 < b1)
        return -1 * gWrapperSortDir;
    else if (a1 > b1)
        return 1 * gWrapperSortDir;
    return 0;
}