/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// The "command" project extension.
//
// This extension manages "run command" project entries.
//

//--- Register the extension.

registeredExtensions[registeredExtensions.length] = new peCommand();


//---- The extension implementation.

function peCommand() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peCommand.prototype.constructor = peCommand;

peCommand.prototype.init = function() {
}

peCommand.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_addCommand_Project', this);
    extensionManager.registerCommand('cmd_addCommand', this);
    extensionManager.registerCommand('cmd_commandProperties', this);
    extensionManager.registerCommand('cmd_runCommand', this);
}

peCommand.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_command,
                                     'ondblclick', this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_command,
                                     'onUpdatePart', this);
}

peCommand.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIContainer,
                                    'Add New Command...','cmd_addCommand');
    extensionManager.createMenuItem(Components.interfaces.koIPart_command,
                                    'Run', 'cmd_runCommand');
    extensionManager.createMenuItem(Components.interfaces.koIPart_command,
                                    'Properties', 'cmd_commandProperties');
}

peCommand.prototype.ondblclick = function(part,event) {
    if (part.type != 'command') return;
    this._runCommand(part)
}

peCommand.prototype.supportsCommand = function(command, part) {
    var view = gFocusedProjectView;
    if (!view) return false;
    //dump("supportsCommand for " + command +'\n');
    switch (command) {
    case 'cmd_addCommand_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_addCommand':
        return true;
    case 'cmd_commandProperties':
    case 'cmd_runCommand':
        var items = gFocusedProjectView.getSelectedItems();
        if (items.length == 1 && items[0].part
            && items[0].part.type == 'command') {
            return true;
        } else {
            return false;
        }
    default:
        break;
    }
    return false;
}

peCommand.prototype.isCommandEnabled = peCommand.prototype.supportsCommand;

peCommand.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_addCommand_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
    case 'cmd_addCommand':
        this._addCommand(item);
        break;
    case 'cmd_commandProperties':
        item = gFocusedProjectView.getSelectedPart();
        this._commandProperties(item);
        break;
    case 'cmd_runCommand':
        item = gFocusedProjectView.getSelectedPart();
        this._runCommand(item);
        break;
    default:
        break;
    }
}


peCommand.prototype._addCommand = function(item)
{
    var part = Components.classes['@activestate.com/koPart?type=command;1']
               .createInstance(Components.interfaces.koIPart_command);
    part.type = 'command';
    part.setStringAttribute('name', "New Command");
    var obj = new Object();
    obj.part = part;
    openWindowUniqueInstance(
        "chrome://komodo/content/run/commandproperties.xul", 
        "komodo_commandproperties",
        "chrome,close=yes,modal=yes", obj);
    if (obj.retval == "OK") {
        var index = -1;
        if (typeof(item)=='undefined' || !item) 
            item = gFocusedProjectView.getSelectedPart();
        if (item) index = gFocusedProjectView.getIndexByPart(item);    
        gFocusedProjectView.manager.addItem(part,index);
    }
}


peCommand.prototype._commandProperties = function(item)
{
    var obj = new Object();
    obj.part = item.part;
    obj.item = item;
    openWindowMultipleInstance(
        "chrome://komodo/content/run/commandproperties.xul", 
        "komodo_commandproperties",
        "chrome,close=yes", obj);
}


function Run_NamedCommand(name) {
    var item = toolboxMgr.findItemByAttributeValue('name', name);
    if (!item) {
        item = gProjectManager.findItemByAttributeValue('name', name);
    }
    if (item) {
        Run_RunCommand(
            window,
            item.part.value,
            item.part.getStringAttribute("cwd"),
            unescapeWhitespace(item.part.getStringAttribute("env"), '\n'),
            item.part.getBooleanAttribute("insertOutput"),
            item.part.getBooleanAttribute("operateOnSelection"),
            item.part.getBooleanAttribute("doNotOpenOutputWindow"),
            item.part.getStringAttribute("runIn")
        );
    }
}

peCommand.prototype._runCommand = function(item)
{
    Run_RunCommand(
        window,
        item.part.value,
        item.part.getStringAttribute("cwd"),
        unescapeWhitespace(item.part.getStringAttribute("env"), '\n'),
        item.part.getBooleanAttribute("insertOutput"),
        item.part.getBooleanAttribute("operateOnSelection"),
        item.part.getBooleanAttribute("doNotOpenOutputWindow"),
        item.part.getStringAttribute("runIn")
    );
}

peCommand.prototype.onUpdatePart = function(part,event)
{
    if (part.type == 'command') {
        part.renameable = true;
        part.iconstyle['command'] = 1;
    }
}


function AddCommandToToolbox(command, cwd, env, insertOutput,
                             operateOnSelection, doNotOpenOutputWindow, runIn)
{
    var part = Components.classes['@activestate.com/koPart?type=command;1']
                .createInstance(Components.interfaces.koIPart_command);
    part.type = 'command';
    part.value = command;
    part.setStringAttribute('name', command);
    part.setStringAttribute('cwd', cwd);
    part.setStringAttribute('env', escapeWhitespace(env));
    part.setBooleanAttribute('insertOutput', insertOutput);
    part.setBooleanAttribute('operateOnSelection', operateOnSelection);
    part.setBooleanAttribute('doNotOpenOutputWindow', doNotOpenOutputWindow);
    part.setStringAttribute('runIn', runIn);
    toolboxMgr.addItem(part);
}

