/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFile();

function peFile() {
    this.sysutils = Components.classes["@activestate.com/koSysUtils;1"].
              getService(Components.interfaces.koISysUtils);
    this.koLanguageRegistryService =
                Components.classes["@activestate.com/koLanguageRegistryService;1"].
                            getService(Components.interfaces.koILanguageRegistryService);
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                       getService(Components.interfaces.nsIObserverService);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFile.prototype.constructor = peFile;

peFile.prototype.init = function() {
    // register our command handlers
    this.observerSvc.AddObserver(this, 'file_status');
    this.observerSvc.AddObserver(this, 'file_status_updated');

    extensionManager.setDatapoint('Date','mtime');
    extensionManager.setDatapoint('Size','size');
    extensionManager.setDatapoint('Status','status');
    extensionManager.setDatapoint('Rev','myRevision');
    extensionManager.setDatapoint('Depot Rev','depotRevision');
    extensionManager.setDatapoint('Action','action');
}

peFile.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_dumpPartProperties', this);
    extensionManager.registerCommand('cmd_editFilePart', this);
    extensionManager.registerCommand('cmd_convertToMacro', this);
    extensionManager.registerCommand('cmd_refreshStatus', this);
    extensionManager.registerCommand('cmd_editFileProperties', this);
}

peFile.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_file,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_ProjectRef,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_file,'onUpdatePart',this);
    extensionManager.addEventHandler(Components.interfaces.koIProject,'onUpdatePart',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_ProjectRef,'onUpdatePart',this);
}

peFile.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Open File','cmd_editFilePart');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Refresh Status','cmd_refreshStatus');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Save As Macro in Toolbox','cmd_convertToMacro');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file, 
                                    'Properties', 'cmd_editFileProperties');
    extensionManager.createMenuItem(Components.interfaces.koIPart_file,
                                    'Rename','cmd_renamePart');

    // XXX debug info
    extensionManager.createMenuItem(Components.interfaces.koIPart,'Dump',
                                    'cmd_dumpPartProperties');
}

peFile.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_dumpPartProperties':
        // Only return true in developer builds
        return infoSvc.installerType == '';
    case 'cmd_editFilePart':
        return true;
    case 'cmd_convertToMacro':
        var items = gFocusedProjectView.getSelectedItems()
        if (items.length != 1 || items[0].type != 'file') return false;
        var url = items[0].url;
        // gLanguageRegistry is initialized in peMacro.js
        var language = gLanguageRegistry.suggestLanguageForFile(url)
        return (language == 'JavaScript' || language == 'Perl' || language == 'Python'); 
    case 'cmd_refreshStatus':
        if (editorHasFocus()) {
            //dump("Scintilla has focus -- actions should be based on the current file\n");
            var view = viewManager.getCurrentView();
            item = view.item;
            if (! view.url) {
                // This is an unsaved file -- command doesn't apply.
                return false;
            }
            return true;
        } else {
            if (! projectHasFocus() && !toolboxHasFocus()) {
                //dump("Scintilla or project or toolbox -- none have focus -- returning early \n");
                return true;
            } else {
            // otherwise, look at the focused project view's selected parts.
                item = gFocusedProjectView.getSelectedPart();
                // item must be a file
                if (!item || !item.url) return false;
            }
            return true;
        }
        break;
    case 'cmd_editFileProperties':
        return true;
    default:
        break;
    }
    return false;
}

peFile.prototype.isCommandEnabled = peFile.prototype.supportsCommand;

peFile.prototype.doCommand = function(command) {
    var item, items = null;
    var i;
    switch (command) {
    case 'cmd_editFilePart':
        // get the current selection, then open the file
        items = gFocusedProjectView.getSelectedItems();
        for (i = 0; i < items.length; i++) {
            if (items[i].type == 'file') viewManager.doFileOpen(items[i].url);
        }
        break;
    case 'cmd_refreshStatus':
        doRefreshStatus();
        break;
    case 'cmd_dumpPartProperties':
        item = gFocusedProjectView.getSelectedPart();
        if (item) item.dump();
        break;
    case 'cmd_convertToMacro':
        item = gFocusedProjectView.getSelectedPart();
        if (item) {
            makeMacroFromFileItem(item);
        }
        break;
    case 'cmd_editFileProperties':
        item = gFocusedProjectView.getSelectedPart();
        this._fileProperties(item);
        break;
    default:
        break;
    }
}

peFile.prototype.ondblclick = function(item,event) {
    doOpenPath(item.url);
}

peFile.prototype.onUpdatePart = function(item,event)
{
    var useprojecticon = false;
    try {
        useprojecticon = item.part.QueryInterface(Components.interfaces.koIProject) != null;
    } catch (e) {}    
    if (!useprojecticon) {
        try {
            useprojecticon = item.part.QueryInterface(Components.interfaces.koIPart_ProjectRef) != null;
        } catch (e) {}    
    }
    // event is null for onUpdatePart
    // get the file properties!
    if (useprojecticon) {
        item.iconstyle['project'] = 1;
        item.iconopenstyle['project'] = 1;
    } else {
        item.iconstyle['projectFile'] = 1;
    }
    item.properties['tooltiptext'] = displayNameFromURL(item.url);
    if (typeof(item.properties['filesize']) == 'undefined') {
        this.getDefaultFileProperties(item);
    }
    if (item.url) {
        var pKeys = {};
        var pValues = {};
        gFileStatusSvc.getFileStatus(item.url, pKeys, new Object(), pValues, new Object());
        for (var index = 0; index < pKeys.value.length; index++) {
            updateStatusItem(item, pKeys.value[index], pValues.value[index]);
        }
    }
}

peFile.prototype.getDefaultFileProperties = function(item)
{
    item.properties['size'] = 0;
    item.properties['read'] = true;
    item.properties['write'] = true;
    item.properties['ctime']  = "Not Available";
    item.properties['mtime']  = "Not Available";
    item.properties['atime']  = "Not Available";
    item.properties['scc'] = '';
    item.iconopenstyle['readwrite']=item.iconstyle['readwrite']=0;
    item.iconopenstyle['readonly']=item.iconstyle['readonly']=0;
}

// XXX duplicated from currentViewPrefs.js
function getDateTimeString(datetime) {
    // XXX not localization friendly
    var now = new Date();
    var months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
    var mins = datetime.getMinutes();
    if (mins.toString().length == 1)  {
        mins = '0' + mins;
    }
    var hours = datetime.getHours();
    var fulltime;
    if (hours > 12)  {
        hours = hours-12;
        fulltime = hours.toString() + ':' + mins + " PM";
    }
    else  {
        fulltime = hours.toString() + ':' + mins + " AM";
    }
    var dtstring;
    if (datetime.getFullYear() == now.getFullYear() &&
        datetime.getMonth() == datetime.getMonth() &&
        now.getDate() == datetime.getDate()) {
            dtstring = fulltime;
    } else if (datetime.getFullYear() == now.getFullYear()) {
        dtstring = datetime.getMonth()+'/'+datetime.getDate()+'/'+datetime.getFullYear();
    } else {
        dtstring = datetime.getMonth()+'/'+datetime.getFullYear();
    }
    return dtstring;
}

function updateStatusItem (item, key, data) {
    //item.properties = {};
    if (typeof(key) == 'undefined') return;
    if (typeof(data) == 'undefined') data = '0';
    //dump('key: '+key+' data: '+data+' url: '+item.url+'\n');
    switch(key) {
        case 'missing':
        case 'readonly':
        case 'readwrite':
        case 'sync':
        case 'ok':
        case 'conflict':
            item.properties[key] = parseInt(data);
            item.iconopenstyle[key] = item.iconstyle[key] = parseInt(data);
            break;
        case 'action':
            //dump(item.url+" action "+data+"\n");
            item.iconopenstyle['edit'] = item.iconstyle['edit'] = 0;
            item.iconopenstyle['add'] = item.iconstyle['add'] = 0;
            item.iconopenstyle['delete'] = item.iconstyle['delete'] = 0;
            if (data) item.iconopenstyle[data] = item.iconstyle[data] = 1;
            item.properties[key] = data;
            break;
        default:
            item.properties[key] = data;
            break;
    }
    //dump('key: '+key+' data: ['+item.properties[key]+']\n');
}

function updateStatus (url, keys,values) {
    var item;
    var index;
    var i;
    // get the item from the url
    var items = findItemsByURL(url);
    //dump(items.length+' matching parts!\n');
    for (i=0; i < items.length; i++) {
        item = items[i];
        //this.getDefaultFileProperties(item);
        for (index = 0; index < keys.length; index++) {
            updateStatusItem(item, keys[index], values[index]);
        }
    }
}

function doRefreshStatus () {
    var urls = {};
    var url;
    var item, items;
    if (editorHasFocus()) {
        //dump("Scintilla has focus -- actions should be based on the current file\n");
        var view = viewManager.getCurrentView();
        item = view.item;
        if (! view.url) {
            // This is an unsaved file -- no SCC commands can apply.
            return;
        }
        item.url = view.url;
        urls[item.url]=1;
    } else {
        if (! projectHasFocus() && !toolboxHasFocus()) {
            //dump("Scintilla or project or toolbox -- none have focus -- returning early \n");
            return;
        } else {
            items = gFocusedProjectView.getSelectedItems();
            // otherwise, look at the focused project view's selected parts.
            for (i=0; i<items.length; i++) {
                items[i].updateUniquePaths(urls,true);
            }
        }
    }
    try {
        for (url in urls) {
            gObserverSvc.Notify(this,'file_status_now',url);
        }
    } catch(e) { /* noone listening, ignore error */ }
}

peFile.prototype._fileProperties = function(item)
{
    if (typeof(item.properties['missing']) != 'undefined' && item.properties['missing']) {
        alertDialog("The file " + item.name + " does not exist on disk at " + displayNameFromURL(item.url));
    } else {
        var obj = new Object();
        obj.item = item;
        obj.task = 'edit';
        openWindowMultipleInstance(
            "chrome://komodo/content/project/fileProperties.xul", 
            "komodo_fileproperties",
            "chrome,close=yes,dialog=no", obj);
    }
}

peFile.prototype.Observe = function (object, topic, url)
{
    switch (topic) {
        case "file_status":
            try {
                var pKeys = {};
                var pValues = {};
                gFileStatusSvc.getFileStatus(url, pKeys, new Object(), pValues, new Object());
                //dump('updating status for '+url+' with '+pKeys.value.length+' items\n');
                if (pKeys.value.length) {
                    updateStatus(url, pKeys.value, pValues.value);
                }
            } catch(e) {
                dump(e);
            }
            break;
        case "file_status_updated":
            for (var i=0; i < gPartManagers.length; i++) {
                gPartManagers[i].refreshView();
            }
            break;
        default:
            // ignore
            break;
    }
}

