/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peGUI();

function peGUI() {
    try {
        this.GuiBuilder = Components.classes["@activestate.com/koGuiBuilderService;1"].
                  getService(Components.interfaces.koIGuiBuilderService);
    } catch (e) {
        logError(e);
        this.GuiBuilder = null;
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peGUI.prototype.constructor = peGUI;

peGUI.prototype.init = function() {
}

peGUI.prototype.registerCommands = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.registerCommand('cmd_editGUIDialog', this);
        extensionManager.registerCommand('cmd_testGUIDialog', this);
        extensionManager.registerCommand('cmd_addGUIDialog', this);
        extensionManager.registerCommand('cmd_addGUIDialog_Project', this);
    } catch (e) {
        logError(e);
    }
}
peGUI.prototype.registerEventHandlers = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.addEventHandler(Components.interfaces.koIPart_dialog,'onUpdatePart',this);
        extensionManager.addEventHandler(Components.interfaces.koIPart_dialog,'ondblclick',this);
    } catch (e) {
        logError(e);
    }
}
peGUI.prototype.registerMenus = function() {
    if (this.GuiBuilder == null) return;
    try {
        extensionManager.createMenuItem(Components.interfaces.koIContainer,'Add New Dialog...','cmd_addGUIDialog');
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,'Edit Dialog...','cmd_editGUIDialog');
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,'Test Dialog...','cmd_testGUIDialog');
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog,'Refresh Status','cmd_refreshStatus');
        extensionManager.createMenuItem(Components.interfaces.koIPart_dialog, 'Properties', 'cmd_editFileProperties');
    } catch (e) {
        logError(e);
    }
}

peGUI.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_editGUIDialog':
        if (!item) {
            item = gFocusedProjectView.getSelectedPart();
        }
        return item != null;
    case 'cmd_testGUIDialog':
        if (!item) {
            item = gFocusedProjectView.getSelectedPart();
        }
        return item && item.part.hasAttribute('startup');
    case 'cmd_addGUIDialog_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_addGUIDialog':
        return true;
    default:
        break;
    }
    return false;
}

peGUI.prototype.isCommandEnabled = peGUI.prototype.supportsCommand;

peGUI.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_editGUIDialog':
        item = gFocusedProjectView.getSelectedPart();
        this.GuiBuilder.Edit(item.url);
        break;
    case 'cmd_testGUIDialog':
        var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                          getService(Components.interfaces.koISysUtils);
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                      getService(Components.interfaces.koIInfoService);
        item = gFocusedProjectView.getSelectedPart();
        var startupfile = URLToLocalFile(item.part.getStringAttribute('startup')).path;
        // XXX Move this logic to language services -- that way it will work with
        // other languages when GUIBuilder adds support for new languages
        var language = gLanguageRegistry.suggestLanguageForFile(startupfile)
        var cmd = '';
        var interp = '';
        switch (language) {
            case 'Perl':
                interp = globalPrefs.getStringPref("perlDefaultInterpreter");
                if (interp == '') {
                    if (infoSvc.isWindows) {
                        interp = 'wperl';
                    } else {
                        interp = 'perl';
                    }
                    interp = sysUtilsSvc.Which(interp);
                    if (!interp) {
                        alertDialog("To test this Perl dialog "+
                                    "you must have Perl on your PATH or "+
                                    "the Perl default interpreter "+
                                    "preference must be correctly set.");
                        return;
                    }
                }
                // XXX somewhat of a hack -- should not have to do this each time.
                var perlInfoEx = Components.classes["@activestate.com/koPerlInfoEx;1"].createInstance(Components.interfaces.koIPerlInfoEx);
                perlInfoEx.installationPath = perlInfoEx.getInstallationPathFromBinary(interp);
                var modules = new Array();
                modules.push('Tk')
                if (! perlInfoEx.haveModules(modules.length, modules)) {
                    alertDialog("Perl Dialogs need the Tk module to be tested."+
                                " The Perl installation at " + perlInfoEx.installationPath +
                                " does not have Tk installed.");
                    return;
                }
                cmd = '"' + interp + '" "' + startupfile + '"';
                break;
            case 'Tcl':
                interp = globalPrefs.getStringPref("wishDefaultInterpreter");
                if (interp == '') {
                    interp = 'wish';
                    if (sysUtilsSvc.Which(interp) == '') {
                        alertDialog("Testing this dialog with " + language + " requires that '" + interp + "' be on your PATH or that the " + language + " default interpreter preference be correctly set.");
                        return;
                    }
                }
                cmd = interp + ' "' + startupfile +'"';
                break;
            case 'Python':
                interp = globalPrefs.getStringPref("pythonDefaultInterpreter");
                if (interp == '') {
                    if (infoSvc.isWindows) {
                        interp = 'pythonw';
                    } else {
                        interp = 'python';
                    }
                    if (sysUtilsSvc.Which(interp) == '') {
                        alertDialog("Testing this dialog with " + language + " requires that '" + interp + "' be on your PATH or that the " + language + " default interpreter preference be correctly set.");
                        return;
                    }
                }
                cmd = interp + ' "' + startupfile +'"';
                break;
            default:
                alertDialog("This version of Komodo does not know how to test Dialogs with " + language);
                return;
        }
        if (cmd) {
            var userEnvSvc = Components.classes["@activestate.com/koUserEnviron;1"].
                  getService(Components.interfaces.koIUserEnviron);
            var env = userEnvSvc.GetEncodedEnvironment();
            Run_RunCommand(window,
                           cmd,
                           null, // cwd
                           env, // env 
                           false, // insertOutput
                           false, // operateOnSelection
                           true, // doNotOpenOutputWindow
                           'no-console');
        }
        break;
    case 'cmd_addGUIDialog_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addGUIDialog':
        // get the current selection, then createthe file
        var index = -1;
        var basedir = '';
        if (!item) item = gFocusedProjectView.getSelectedPart();
        if (item) {
            // Finding the basedir is somewhat broken if an item is selected in the toolbox
            // but it's not a folder -- its base directory will be used.  Harumph.
            var project = item.part.project;
            index = gFocusedProjectView.getIndexByPart(item);
            if (item.url) {
                var url = item.url;
                if (item.type != 'folder') {
                    // Projects and files have URLs which are not the directory but the URL of the files!
                    url = url.substr(0, url.lastIndexOf('/'))
                }
                basedir = URLToLocalFile(url).path;
            }
        }
        var validName = false;
        var file;
        var validNameRe = /^[a-zA-Z_]\w*\.ui$/;
        while (validName == false) {
            var fp = dlg_fileSelectViaDialog(Components.interfaces.nsIFilePicker.modeSave,
                                        "Choose Dialog File", basedir,
                                        "Dialog.ui", "GUI Builder UI File", "*.ui");
            if (!fp) {
                return;
            }
            file = new dlg_get_URL_andName(fp);
            filewinfuncs_forceExtension(file, '.ui');
            if (file.leafName.match(validNameRe) != null) {
                validName = true;
            }
            if (!validName) {
                alertDialog("Invalid name - dialog names must start with a letter and contain only letters, numbers and the underscore character (_).")
            }
        }

        var ask = true;
        if (prefs.hasPref('defaultGUILanguage')) {
            var pref = prefs.getStringPref('defaultGUILanguage');
            if (pref != '') {
                language = pref;
                ask = false;
            }
        }
        if (ask) {
            var obj = new Object();
            window.setCursor("wait");
            var retval = openWindowUniqueInstance("chrome://komodo/content/pref/choose_gui_language.xul",
                                        "komodo_pick_gui_language",
                                        "chrome,dialog,modal,close=no",
                                        obj);
            if (!retval || !obj || obj.ret == false) {
                // User pressed cancel or escape.
                return;
            }
            language = obj.language;
            var savePref = obj.dontAskAgain;
            if (savePref) {
                prefs.setStringPref('defaultGUILanguage', language);
            }
        }

        var part = Components.classes["@activestate.com/koPart?type=dialog;1"]
                                        .createInstance(Components.interfaces.koIPart_dialog);
        part.setStringAttribute('url', file.url);
        part.setStringAttribute('language', language); // used where? XXX
        gFocusedProjectView.manager.addItem(part,index);
        this.GuiBuilder.Create(part.getStringAttribute('url'), language);
        break;
    default:
        break;
    }
}

peGUI.prototype.ondblclick = function(item,event) {
    this.GuiBuilder.Edit(item.url);
}

peGUI.prototype.onUpdatePart = function(part,event)
{
    part.iconopenstyle['guifile'] = 1
    part.iconstyle['guifile'] = 1
}
