/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// A project extension to manage source code control operations in Komodo.

// start the extension
registeredExtensions[registeredExtensions.length] = new peSCC();


// generic globals
var _prefSvc = Components.classes['@activestate.com/koPrefService;1'].
               getService(Components.interfaces.koIPrefService);
var _cvsSvc = Components.classes["@activestate.com/koCVS;1"].
              getService(Components.interfaces.koICVS);
var _p4Svc = Components.classes["@activestate.com/koP4;1"].
             getService(Components.interfaces.koIP4);
var _obSvc = Components.classes["@mozilla.org/observer-service;1"].
             getService(Components.interfaces.nsIObserverService);


//---- Registry of active SCC services
// 'sccModules' is a dictionary of active SCC services conforming to the
// virtual (so virtual we have defined a common base class) koISCC interface.
// Generally there is one such service for each SCC system (e.g. CVS) that
// Komodo supports.
// 
// "Active" means that the SCC system is (1) functional (cannot support CVS
// if CVS is not installed on the user's machine) and (2) enabled (user
// preference). Monitoring (2) can be done via Komodo pref observer system.
// Monitoring (1) is done by observing the 'subsystem_changed'
// notification.
//
var sccModules = {};
var _sccObserver = new _SCCObserver();
var _sccPrefObserver = new _SCCPrefObserver();

function _activateCVSService() {
    // Activate of deactivate the CVS service depending on current state.
    if (_cvsSvc.isFunctional && _prefSvc.prefs.getBooleanPref("cvsEnabled")) {
        //dump("peSCC: CVS service enabled\n");
        sccModules['cvs'] = _cvsSvc;
    } else {
        //dump("peSCC: CVS service disabled\n");
        sccModules['cvs'] = null;
    }
}

function _activateP4Service() {
    // Activate of deactivate the P4 service depending on current state.
    if (_p4Svc.isFunctional && _prefSvc.prefs.getBooleanPref("p4Enabled")) {
        //dump("peSCC: P4 service enabled\n");
        sccModules['p4'] = _p4Svc;
    } else {
        //dump("peSCC: P4 service disabled\n");
        sccModules['p4'] = null;
    }
}

function _SCCObserver() {}
_SCCObserver.prototype.Observe = function(subject, topic, data)
{
    // The "subsystem_changed" notification indicates that a Komodo
    // subsystem has changed. "data" indentifies the subsystem. If data is
    // 'p4' or 'cvs' then we might need to update the list of active SCC
    // services.
    switch (topic) {
    case 'scc_status_message':
        SCCScinAppendText(data);
        break;
    case 'subsystem_changed':
        switch (data) {
        case 'cvs':
            _activateCVSService();
            break;
        case 'p4':
            _activateP4Service();
            break;
        }
        break;
    }    
}
_SCCObserver.prototype.init = function() {
    _obSvc.AddObserver(this, 'subsystem_changed');
    _obSvc.AddObserver(this, 'scc_status_message');
}
_SCCObserver.prototype.destructor = function() {
    _obSvc.RemoveObserver(this, 'subsystem_changed');
    _obSvc.RemoveObserver(this, 'scc_status_message');
}

function _SCCPrefObserver() {};
_SCCPrefObserver.prototype.Observe = function(prefSet, prefSetID, prefName)
{
    //dump("peSCC: observed pref notification: prefName='"+prefName+"'\n");
    switch (prefName) {
    case "cvsEnabled":
        _activateCVSService();
        break;
    case "p4Enabled":
        _activateP4Service();
        break;
    }
};
_SCCPrefObserver.prototype.init = function() {
    _prefSvc.prefs.AddObserver(this);
}
_SCCPrefObserver.prototype.destructor = function() {
    if (_prefSvc && _prefSvc.prefs) {
        _prefSvc.prefs.RemoveObserver(this);
    }
}


//---- SCC project extension implementation.

function peSCC() {
    this.sysutils = Components.classes["@activestate.com/koSysUtils;1"].
              getService(Components.interfaces.koISysUtils);
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                       getService(Components.interfaces.nsIObserverService);
    this._lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"]
               .getService(Components.interfaces.koILastErrorService);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peSCC.prototype.constructor = peSCC;

peSCC.prototype.init = function() {
    // register our command handlers
    this.registerCommands();
    
    // Setup dynamic registration of active SCC services.
    _activateCVSService();
    _activateP4Service();
    _sccPrefObserver.init();
    _sccObserver.init();
}

peSCC.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_SCCedit', this);
    extensionManager.registerCommand('cmd_SCCadd', this);
    extensionManager.registerCommand('cmd_SCCremove', this);
    extensionManager.registerCommand('cmd_SCCupdate', this);
    extensionManager.registerCommand('cmd_SCCcommit', this);
    extensionManager.registerCommand('cmd_SCCdiff', this);
    extensionManager.registerCommand('cmd_SCCrevert', this);

    extensionManager.registerCommand('cmd_SCCadd_folder', this);
    extensionManager.registerCommand('cmd_SCCupdate_folder', this);
    extensionManager.registerCommand('cmd_SCCcommit_folder', this);
    extensionManager.registerCommand('cmd_SCCdiff_folder', this);
    extensionManager.registerCommand('cmd_SCCrevert_folder', this);
}

peSCC.prototype.registerEventHandlers = function() {
}

peSCC.prototype.registerMenus = function() {
    var menupopup = document.getElementById('menu_SCCmenu');
    var folderpopup = document.getElementById('menu_SCCmenu_folder');
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_file, document.createElement('menuseparator'));
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_file, menupopup);
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_collection, document.createElement('menuseparator'));
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_collection, folderpopup);
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_directory, document.createElement('menuseparator'));
    extensionManager.addMenuItem(Components.interfaces.koIPart_SCC_directory, folderpopup);
}

peSCC.prototype._getSCCInterface = function(item)
{
    // is this a folder operation?
    var ok = false;
    var itemInteface = null;
    if (!item.part) {
        return Components.interfaces.koIPart_SCC_file;
    }
    try {
        ok = item.part.QueryInterface(Components.interfaces.koIPart_SCC_file) != null;
        if (ok) itemInteface = Components.interfaces.koIPart_SCC_file;
    } catch(e) {}
    try {
        ok = item.part.QueryInterface(Components.interfaces.koIPart_SCC_collection) != null;
        if (ok) itemInteface |= Components.interfaces.koIPart_SCC_collection;
    } catch(e) {}
    try {
        ok = item.part.QueryInterface(Components.interfaces.koIPart_SCC_directory) != null;
        if (ok) itemInteface |= Components.interfaces.koIPart_SCC_directory;
    } catch(e) {}
    return itemInteface;
}

peSCC.prototype._isSCCFile = function(item_interface)
{
    return item_interface & Components.interfaces.koIPart_SCC_file;
}

peSCC.prototype._isSCCCollection = function(item_interface)
{
    return item_interface & Components.interfaces.koIPart_SCC_collection;
}

peSCC.prototype._isSCCDirectory = function(item_interface)
{
    return item_interface & Components.interfaces.koIPart_SCC_directory;
}

/* This takes an item, and determines what type of scc
   flags we are dealing with. returns true if we can do anything, or
   false if we cannot do anything */
peSCC.prototype.determineItemSCCSupport = function(item, sccObj) {
    // prevent non-cvs directories from having to go through this torture
    if (typeof(item.properties['scc_exclude']) != 'undefined'
        && item.properties['scc_exclude']) return false;
    sccObj.sccInterface = this._getSCCInterface(item);
    if (!sccObj.sccInterface) return false;
    
    sccObj.isscc_directory = this._isSCCDirectory(sccObj.sccInterface);
    sccObj.isscc_collection = this._isSCCCollection(sccObj.sccInterface);
    sccObj.isscc_container = sccObj.isscc_directory || sccObj.isscc_collection;
    sccObj.isscc_file = this._isSCCFile(sccObj.sccInterface);
    sccObj.has_scc = typeof(item.properties['scc']) != 'undefined' && item.properties['scc'] != '';
    sccObj.is_scc = sccObj.scc_file || sccObj.isscc_container;
    if (!sccObj.is_scc) return false;

    sccObj.can_scc = sccObj.has_scc || 
        (typeof(item.properties['scc_can']) != 'undefined' && item.properties['scc_can']);
    sccObj.missing = (typeof(item.properties['missing']) != 'undefined'
                    && item.properties['missing']);
    
    var in_repository = null;
    if (!sccObj.can_scc && !sccObj.missing) {
        item.properties['scc_can'] = '';
        in_repository = gFileStatusSvc.isRepository(item.url);
        if (in_repository) {
            sccObj.can_scc = item.properties['scc_can'] = in_repository;
        }
        item.properties['scc_exclude'] = !sccObj.has_scc && !sccObj.can_scc;
    }

    //dump('item '+item.url+' is '+item.properties['scc']+ ' action '+item.properties['action']+'\n');
    if (!sccObj.has_scc && !sccObj.can_scc) return false;
    return true;
}

peSCC.prototype.supportsCommand = function(command, item) {
    // do we have either cvs or scc?
    if (!globalPrefs.getBooleanPref('cvsEnabled') && !globalPrefs.getBooleanPref('p4Enabled')) return false;
    
    // If scintilla has focus, should be talking about the file in scintilla
    if (!item) {
        if (editorHasFocus()) {
            //dump("Scintilla has focus -- actions should be based on the current file\n");
            var view = gLastFocusedViewManager.getCurrentView();
            if (!view || !view.url) {
                // This is an unsaved file -- no SCC commands can apply.
                return 0;
            }
            item = view.item;
            item.url = view.url;
        } else {
            if (!projectHasFocus() && !toolboxHasFocus()) {
                //dump("Scintilla or project or toolbox -- none have focus -- returning early \n");
                return 0;
            } else {
                // otherwise, look at the focused project view's selected parts.
                item = gFocusedProjectView.getSelectedPart();
                // item must be a file
                if (!item) return false;
            }
        }
    }
 
    // XXX look into cacheing sccObj into the item
    var sccObj = new Object();
    if (!this.determineItemSCCSupport(item,sccObj)) return false;
    var isfoldercmd = command.indexOf('_folder')>1;
    if (isfoldercmd && !sccObj.isscc_container)
        return false;
    
    /* folder handling is different between p4 and cvs */
    var folders_in_repository = item.properties['scc'] == 'cvs' || item.properties['scc_can'] == 'cvs';
    var folders_revertable = item.properties['scc'] == 'p4' || item.properties['scc_can'] == 'p4';
    
    /* if cvs, and this is a directory, then the directory MUST be in cvs */
    var foldercmd_ok = (sccObj.isscc_container && isfoldercmd) && 
            ((folders_in_repository && sccObj.isscc_directory && sccObj.has_scc) || 
            (!folders_in_repository && sccObj.can_scc));
    
    switch (command) {
    case 'cmd_SCCedit':
        return sccObj.has_scc && item.properties['scc'] != 'cvs' &&
            (typeof(item.properties['action']) == 'undefined' || item.properties['action'] == '');
        break;
    case 'cmd_SCCadd_folder':
        return sccObj.can_scc && !sccObj.has_scc && isfoldercmd 
                && sccObj.isscc_directory && folders_in_repository;
        break;
    case 'cmd_SCCadd':
        return sccObj.can_scc && !sccObj.has_scc;
        break;
    case 'cmd_SCCupdate_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCCupdate':
        return sccObj.has_scc && item.properties['sync'];
        break;
    case 'cmd_SCCremove':
        return sccObj.has_scc && item.properties['ok'];
        break;
    case 'cmd_SCCdiff_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCCdiff':
        return sccObj.has_scc &&(item.properties['sync'] ||
                item.properties['action'] == 'edit');
        break;
    case 'cmd_SCCrevert_folder':
        return foldercmd_ok && folders_revertable;
        break;
    case 'cmd_SCCrevert':
        return sccObj.has_scc && (item.properties['sync'] ||
                item.properties['action'] != '');
        break;
    case 'cmd_SCCcommit_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCCcommit':
        return sccObj.has_scc && item.properties['action'] != ''
            && (!item.properties['sync'] || 
                item.properties['action'] == 'delete');
        break;
    default:
        break;
    }
    return false;
}

peSCC.prototype.isCommandEnabled = peSCC.prototype.supportsCommand;

/* recursivly build a list of items that are being used in
   this scc operation */
peSCC.prototype._addVirtualFolderChildren = function(item)
{
    var items = [];
    if (typeof(item.children) == 'undefined' || !item.children) 
        return items;
        
    for (var i = 0; i < item.children.length; i++) {
        if (item.children[i].isContainer && 
            !item.children[i].url && 
            item.children[i].children.length > 0) {
            
            var c = this._addVirtualFolderChildren(item.children[i]);
            if (c.length > 0) {
                items = items.concat(c);
            }
        } else if (item.children[i].url) {
            items.push(item.children[i]);
        }
    }
    return items;
}

peSCC.prototype.doCommand = function(command) {
    var cmd = null;
    var error = null;
    var urls=[];
    var _sccUrls = [];
    _sccUrls['p4'] = []
    _sccUrls['cvs'] = []
    var item = null;
    var items = [];
    var i;
    if (editorHasFocus()) {
        //dump("Scintilla has focus -- actions should be based on the current file\n");
        var view = gLastFocusedViewManager.getCurrentView();
        if (view && view.url) {
            item = view.item;
            item.url = view.url;
            items.push(item);
        }
    } else if (projectHasFocus() || toolboxHasFocus()) {
        items = gFocusedProjectView.getSelectedItems();
    }
    
    if (items.length < 1) return;

    var isfoldercmd = command.indexOf('_folder')>1;
    for (i=0; i<items.length; i++) {
        // XXX look into cacheing sccObj into the item
        var sccObj = new Object();
        if (!this.determineItemSCCSupport(items[i],sccObj)) return;

        var type = items[i].properties['scc'];
        if (!type) type = items[i].properties['scc_can'];
        if (!type) continue;
        
        if (isfoldercmd && sccObj.isscc_directory && items[i].type == 'project') {
            _sccUrls[type].push(os.path.dirname(items[i].url));
        } else if (isfoldercmd && sccObj.isscc_collection) {
            if (typeof(items[i].url) != 'undefined' && items[i].url)
                _sccUrls[type].push(items[i].url);
            // add the dialogs children
            items = items.concat(this._addVirtualFolderChildren(items[i]));
        } else {
            _sccUrls[type].push(items[i].url);
        }
        
        // XXX implement virtual folder support
        // check for virtual folders, add the children of the virtual
        // folders to our items array.
        //if (sccObj.isscc_collection && items.children.length > 0) {
        //    var c = this._addVirtualFolderChildren(items[i]);
        //    if (c.length > 0) {
        //        items = items.concat(c);
        //    }
        //}
    }
    var output = '';
    //dump('selected for scc operation '+urls +'\n');
    for (var scchandler in sccModules) {
        if (!sccModules[scchandler]) continue; // skip unregistered SCC services
        if (_sccUrls[scchandler].length < 1) continue;
        urls = _sccUrls[scchandler];
        //try {
            switch (command) {
            case 'cmd_SCCedit':
                window.setCursor("wait");
                output = sccModules[scchandler].edit(urls.length, urls);
                output = this.generateCommandInfo(scchandler+' edit',urls,output);
                SCCScinAppendText(output);
                break;
            case 'cmd_SCCadd_folder':
                if (!isfoldercmd) return;
            case 'cmd_SCCadd':
                this.doAdd(urls, sccModules, scchandler);
                break;
            case 'cmd_SCCremove':
                this.doRemove(urls, sccModules, scchandler);
                break;
            case 'cmd_SCCrevert_folder':
                if (!isfoldercmd) return;
            case 'cmd_SCCrevert':
                this.doRevert(urls, sccModules, scchandler);
                break;
            case 'cmd_SCCupdate_folder':
                if (!isfoldercmd) return;
            case 'cmd_SCCupdate':
                this.doUpdate(urls, sccModules, scchandler);
                break;
            case 'cmd_SCCdiff_folder':
                if (!isfoldercmd) return;
            case 'cmd_SCCdiff':
                window.setCursor("wait");
                var options = '';
                if (scchandler == 'p4') {
                    options = globalPrefs.getStringPref('p4DiffOptions');
                } else if (scchandler == 'cvs') {
                    options = globalPrefs.getStringPref('cvsDiffOptions');
                }
                var diff = sccModules[scchandler].diff(urls.length, urls, options);
                error = this._lastErrorSvc.getLastErrorMessage();
                //output = this.generateCommandInfo(scchandler+' diff',urls,diff);
                //SCCScinAppendText(output);
                if (!error) {
                    if (diff) {
                        var diffDisplayStyle = globalPrefs.getStringPref('diffDisplayStyle');
                        if (diffDisplayStyle == 'tab') {
                            openDiffBuffer(diff);
                        } else if (diffDisplayStyle == 'diffwin') {
                            // Put together a title for the diff output win.
                            var title = "Diff:";
                            for (i=0; i < urls.length; i++) {
                                if (i > 0) title += ",";
                                title += " "+makeLeafName(urls[i]);
                            }
                            openDiffWindow(diff, title);
                        }
                    } else {
                        alertDialog('Files are in sync with repository.');
                    }
                }
                break;
            case 'cmd_SCCcommit_folder':
                if (!isfoldercmd) return;
            case 'cmd_SCCcommit':
                this.doCommit(urls, sccModules, scchandler);
                break;
            default:
                break;
            }
        //} catch (e) {
        //    alertDialog("An exception occured during SCC operation. (" + e + ')');
        //    logError(e);
        //}
    }
    window.setCursor("auto");
    koRequestUpdateCommands('SCC'); // resets toolbar for us
    error = this._lastErrorSvc.getLastErrorMessage();
    if (error)
        alertDialog(error);
}

peSCC.prototype.generateCommandInfo = function(command,urls,text) {
    var out = command+'\n';
    for (var url in urls) {
        out += '    '+urls[url]+'\n'
    }
    out += text;
    return out;
}

peSCC.prototype.doAdd = function(urls, sccModules, scchandler) {
    var obj = new Object();
    obj.URLs = urls;
    obj.title = "Add Files";
    obj.okcancel = true;
    obj.prompt = "Request addition of the selected files to the repository?";
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=true",
                                obj);
    if (!obj || obj.ret == false) {
        // user pressed ESC or cancel
        return false;
    }
    var selectedURLs = obj.URLs;
    window.setCursor("wait");
    output = sccModules[scchandler].add(urls.length, urls,'','');
    window.setCursor("auto");
    output = this.generateCommandInfo(scchandler+' add',urls,output);
    SCCScinAppendText(output);
    return true;
}

peSCC.prototype.doRemove = function(urls, sccModules, scchandler) {
    var obj = new Object();
    obj.URLs = urls;
    obj.title = "Remove Files";
    obj.okcancel = true;
    if (scchandler == 'p4') {
        obj.prompt = "Request removal of the selected files from the repository? (The selected files will be removed from your disk)";
    } else { 
        obj.prompt = "Request removal of the selected files from the repository?";
    }
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=true",
                                obj);
    if (!obj || obj.ret == false) {
        // user pressed ESC or cancel
        return false;
    }
    var selectedURLs = obj.URLs;
    window.setCursor("wait");
    var force = true;
    var recursive = false;
    var output = sccModules[scchandler].remove(urls.length, urls, force, recursive);
    viewManager.revertViewsByURLs(selectedURLs); // XXX? What should we do here?
    window.setCursor("auto");
    output = this.generateCommandInfo(scchandler+' remove',urls,output);
    SCCScinAppendText(output);
    return true;
}

peSCC.prototype.doRevert = function(urls, sccModules, scchandler) {
  try {
    var obj = new Object();
    obj.URLs = urls;
    obj.title = "Revert Changes";
    obj.okcancel = true;
    obj.prompt = "Revert selected files to repository version? (Changes in opened documents will be lost).";
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=true",
                                obj);
    if (!obj || obj.ret == false) {
        // user pressed ESC or cancel
        return false;
    }
    var selectedURLs = obj.URLs;
    window.setCursor("wait");
    try {
        var output = sccModules[scchandler].revert(selectedURLs.length, selectedURLs,'');
        output = this.generateCommandInfo(scchandler+' revert',selectedURLs,output);
        SCCScinAppendText(output);
    } catch (e) {
        logError(e);
    }
    viewManager.revertViewsByURLs(selectedURLs);
    // handle reverting projects properly
    for (var i in selectedURLs) {
        isProjectFile = selectedURLs[i].slice(-4).toLowerCase() == '.kpf';
        if (isProjectFile) {
            gProjectManager.revertProjectByURL(selectedURLs[i]);
        }
    }
    window.setCursor("auto");
  } catch (e) {
    logError(e);
  }
    return true;
}

peSCC.prototype.doCommit= function(urls, sccModules, scchandler) {
    OfferToSaveFilesProjectsDialogs(urls, "Save Modified Files", "Save selected files before performing SCC operation?");
    var url;
    var obj = new Object();
    obj.URLs = urls;
    obj.title = "Commit Changes";
    obj.okcancel = true;
    obj.prompt = "Commit changes to the selected files?";
    obj.secondaryprompt = "Change Description:"
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=true",
                                obj);
    var selectedURLs = obj.URLs;
    if (!obj || obj.ret == false || selectedURLs.length == 0) {
        // user pressed ESC or cancel
        return false;
    }
    window.setCursor("wait");
    var output = sccModules[scchandler].commit(selectedURLs.length, selectedURLs, obj.entry, '');
    viewManager.revertViewsByURLs(selectedURLs);
    window.setCursor("auto");
    output = this.generateCommandInfo(scchandler+' commit',urls,output);
    SCCScinAppendText(output);
    return true;
}

peSCC.prototype.doUpdate = function(urls, sccModules, scchandler) {
    OfferToSaveFilesProjectsDialogs(urls, "Save Modified Files", "Save selected files before performing SCC operation?");
    var url;
    var obj = new Object();
    obj.URLs = urls;
    obj.okcancel = true;
    obj.title = "Update From Repository";
    obj.prompt = "Update the selected files?";
    openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                "komodo_closeConfirm",
                                "chrome,close=no,modal,resizable=true",
                                obj);
    var selectedURLs = obj.URLs;
    if (!obj || obj.ret == false || selectedURLs.length == 0) {
        // user pressed ESC or cancel
        return false;
    }
    window.setCursor("wait");
    var changes = sccModules[scchandler].update(selectedURLs.length, selectedURLs,'');
    viewManager.revertViewsByURLs(selectedURLs);
    window.setCursor("auto");
    if (changes) {
        this.handleChanges(changes);
    }
    output = this.generateCommandInfo(scchandler+' update',urls,output);
    SCCScinAppendText(changes);
    return true;
}

peSCC.prototype.handleChanges = function(changes) {
    // re ^(?P<status>[UPARMC\?]) (?P<file>.*)$
    RegExp.multiline = true;
    var pattern = new RegExp("^([UCDP\?]) (.*?) (.*?)$","g");
    var result;
    var conflicts = [];
    var haveConflict = false;
    var removed = [];
    var haveRemoved = false;
    var added = [];
    var haveNew = false;
    var file;
    var i,j;
    var alreadyHaveIt;
    while ((result = pattern.exec(changes)) != null) {
        //dump('found: '+result[1]+' '+result[2]+' '+result[3]+'\n');
        switch (result[1]) {
        case 'C':
            // conflict
            haveConflict = true;
            conflicts.push(result[3]);
            break;
        case 'D':
            // removed
            haveRemoved = true;
            removed.push(result[3]);
            break;
        case 'P':
            break; // Patched file can't be a new file, even though it may not be in our project.
        case 'U':
            // either updated or new file -- 
            // XXX need to check to make sure that it really is new -- we add all files that aren't in the project
            alreadyHaveIt = false;
            for (j=0; j < gPartManagers.length; j++) {
                list = gPartManagers[j].getItemsByURL(result[3]);
                if (list.length > 0) {
                    alreadyHaveIt = true;
                    break;
                }
            }
            if (! alreadyHaveIt) {
                haveNew = true;
                added.push(result[3]);
            }
            break;
        }
    }
    // look to see if files were added, ask if they should be
    // added to the current project.
    var filepart, selectedURLs, obj;
    if (haveNew && gProjectManager.getCurrentProject()) {
        obj = new Object();
        obj.URLs = added;
        obj.okcancel = true;
        obj.title = "Add New Files";
        obj.prompt = "Some updated files are not in your project.  Add them to the current project?";
        openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                    "komodo_closeConfirm",
                                    "chrome,close=no,modal,resizable=true",
                                    obj);
        selectedURLs = obj.URLs;
        if (!obj || obj.ret == false || selectedURLs.length == 0) {
            // user pressed ESC or cancel
            return false;
        }
        for (i = 0; i < selectedURLs.length; i++) {
            filepart = Components.classes["@activestate.com/koPart?type=file;1"]
                                    .createInstance(Components.interfaces.koIPart_file);
            filepart.setStringAttribute('url', selectedURLs[i]);
            gProjectManager.addItem(filepart);
        }
    }
    // look to see if files were removed, notify of removal,
    // ask to remove from projects.
    if (haveRemoved) {
        var aroundToRemove = [];
        for (i = 0; i < selectedURLs.length; i++) {
            for (j=0; j < gPartManagers.length; i++) {
                list = gPartManagers[j].getItemsByURL(selectedURLs[i]);
                if (list.length > 0) {
                    aroundToRemove.push(selectedURLs[i]);
                }
            }
        }

        obj = new Object();
        obj.URLs = aroundToRemove;
        obj.okcancel = true;
        obj.title = "Remove Old Files";
        obj.prompt = "Files have been removed from source code control, remove them from your projects and toolbox?";
        openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                    "komodo_closeConfirm",
                                    "chrome,close=no,modal,resizable=true",
                                    obj);
        selectedURLs = obj.URLs;
        if (!obj || obj.ret == false || selectedURLs.length == 0) {
            // user pressed ESC or cancel
            return false;
        }
        var items, list;
        for (i = 0; i < selectedURLs.length; i++) {
            for (j=0; j < gPartManagers.length; j++) {
                list = gPartManagers[j].getItemsByURL(selectedURLs[i]);
                for (j = 0; j < list.length; j++) {
                    gPartManagers[i].removeItem(list[j]);
                }
            }
        }
    }
    
    // look to see if there are conflicts, notify of conflicts,
    // and ask to open the files in komodo.  also mark any
    // items with conflict status
    if (haveConflict) {
        obj = new Object();
        obj.URLs = conflicts;
        obj.okcancel = true;
        obj.title = "Merge Conflict";
        obj.prompt = "There are conflicts in some updated file(s), would you like to open those file(s)?";
        openWindowUniqueInstance("chrome://komodo/content/editor/confirmClose.xul", 
                                    "komodo_closeConfirm",
                                    "chrome,close=no,modal,resizable=true",
                                    obj);
        selectedURLs = obj.URLs;
        if (!obj || obj.ret == false || selectedURLs.length == 0) {
            // user pressed ESC or cancel
            return false;
        }
        for (i = 0; i < selectedURLs.length; i++) {
            doOpenPath(selectedURLs[i]);
        }
    }
    
    // XXX reset the static var, is this necessary?
    RegExp.multiline = false;
    return true;
}

function openDiffBuffer(text)
{
    // is the file already open?
    view = doFileNew('Diff');
    view._scimoz.text = text;
    view.py_inst.isDirty = 1;
    viewManager.indicateIfDirty(view);
}

function openDiffWindow(text, title /* ="Diff Output" */)
{
    if (typeof(title) == "undefined") {
        title = "Diff Output";
    }
    var obj = new Object();
    obj.title = title;
    obj.text = text;
    obj.lexer = 'SCLEX_DIFF';
    obj.lexerName = 'Diff';
    obj.dimensionPrefPrefix = "sccDiffWin-";
    openWindowUniqueInstance("chrome://komodo/content/editor/simplewindow.xul", 
                             "komodo_run", 
                             "chrome,dialog=no,resizable,scrollbars,close=yes",
                             obj);
}

function editSCCFile(item)
{
    var urls = [];
    urls.push(item.url);
    window.setCursor("wait");
    try {
        sccModules[item.properties['scc']].edit(urls.length, urls);
    } catch (e) {
        alertDialog("An exception occured during SCC operation.");
    }
    window.setCursor("auto");
    // XXX gLastErrorSvc defined in find_functions.js
    error = gLastErrorSvc.getLastErrorMessage();
    if (error) {
        alertDialog(error);
        return false;
    }
    return true;
}


/* SCC Scintilla control */

function SCCScinOnLoad() {
    var scin = document.getElementById('scin-sccoutput').ISciMoz;
    scin.lexer = scin.SCLEX_NULL;
    scin.text = '';
    // Setup the regex entry boxes with appropriate styling.
    var styleApplier = Components.classes["@activestate.com/koScintillaStyleApplier;1"].createInstance();
    var fakeViewPrefs = Components.classes['@activestate.com/koPreferenceSet;1']
        .createInstance(Components.interfaces.koIPreferenceSet);
    fakeViewPrefs.setBooleanPref('editUseFixedFont', true);
    // Set up global settings like cursor color and current background color.
    update_scintilla_global_pref(scin, 'cursorColor');
    update_scintilla_global_pref(scin, 'currentLineColor');
    update_scintilla_global_pref(scin, 'useCurrentLineColor');
    update_scintilla_global_pref(scin, 'selectBack');
    update_scintilla_global_pref(scin, 'selectFore');
    update_scintilla_global_pref(scin, 'useSelectFore');
    update_scintilla_global_pref(scin, 'edgeColor');
    styleApplier.init(scin, 'Text', fakeViewPrefs);
    styleApplier.applyStyles();
    scin.readOnly = true;
    scimoz_widget_init(document.getElementById('scin-sccoutput'), 1);
}

function SCCScinAppendText(text) {
    if (typeof(text)=='undefined' || !text) return;
    var scin = document.getElementById('scin-sccoutput').ISciMoz;
    scin.readOnly = false;
    scin.currentPos = scin.textLength-1;
    // move cursor to end of current text
    text = "-------------------------------------------------------------------\n" +
        text+'\n';
    scin.addText(text.length, text);
    scin.gotoLine(scin.lineFromPosition(scin.textLength));
    scin.readOnly = true;
}

function SCCScinClearText() {
    var scin = document.getElementById('scin-sccoutput').ISciMoz;
    scin.readOnly = false;
    scin.text = '';
    scin.readOnly = true;
}

function SCCCheckForSSH() {
    var cvsrsh = os.getenv('CVS_RSH');
    var checkSSH = cvsrsh != '';
    if (checkSSH && globalPrefs.hasPref('cvsSSHWarn'))
        checkSSH = globalPrefs.getBooleanPref('cvsSSHWarn')
    checkSSH = checkSSH && globalPrefs.getBooleanPref('cvsEnabled')
        && globalPrefs.getBooleanPref('cvsBackgroundCheck');
    if (!checkSSH) return;
    var ret = alertDialog("CVS Warning: The CVS_RSH environment variable is set to '"+cvsrsh+"', "+
                          "and you have CVS Background Status Checking turned on.  "+
                          "Read the Komodo documentation regarding the "+
                          "use of external protocols, such as SSH, with Komodo.  Failure to properly "+
                          "configure CVS may result in process lockups within Komodo.",
                        false, false, "OK", "Disable CVS Now", null, "Don't tell me again");
    if (ret.checked) {
        globalPrefs.setBooleanPref('cvsSSHWarn',false);
    }
    if (ret.res != 'OK') {
        globalPrefs.setBooleanPref('cvsEnabled',false);
    }
}

function SCCOnLoad() {
    SCCScinOnLoad();
    window.setTimeout('SCCCheckForSSH();', 1);
}
