/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peSnippet();

function peSnippet() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peSnippet.prototype.constructor = peSnippet;

peSnippet.prototype.init = function() {
}

peSnippet.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_insertSnippet', this);
    extensionManager.registerCommand('cmd_editSnippetProperties', this);
    extensionManager.registerCommand('cmd_addSnippet', this);
    extensionManager.registerCommand('cmd_addSnippet_Project', this);
}

peSnippet.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_snippet,'ondblclick',this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_snippet, 'onUpdatePart',this);
}

peSnippet.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_snippet,
                                    'Insert Snippet','cmd_insertSnippet');
    extensionManager.createMenuItem(Components.interfaces.koIPart_snippet, 
                                    'Properties', 'cmd_editSnippetProperties');
    extensionManager.createMenuItem(Components.interfaces.koIContainer,
                                    'Add New Snippet...', 'cmd_addSnippet');
}

peSnippet.prototype.ondblclick = function(part,event) {
    if (part.type != 'snippet') return;
    _insertSnippet(part)
}

peSnippet.prototype.supportsCommand = function(command, part) {
    var items = gFocusedProjectView.getSelectedItems();
    if (items.length > 1) return false;
    
    var view = gFocusedProjectView;
    if (!view) return false;
    //dump("supportsCommand for " + command +'\n');
    switch (command) {
    case 'cmd_insertSnippet':
        // XXX should just return true here, but the Extension manager isn't calling
        // isCommandEnabled, so we'll just hide the menu item for now. 
        if (!part)
            part = view.getSelectedPart();
        var vm = getFocusedViewManager();
        if (!vm) return false;
        //dump("vm.getCurrentView() = " + vm.getCurrentView() + '\n');
        if (!vm.getCurrentView()) return false;
        return true;
    case 'cmd_addSnippet_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_editSnippetProperties':
    case 'cmd_addSnippet':
        return true;
    default:
        break;
    }
    return false;
}


peSnippet.prototype.isCommandEnabled = peSnippet.prototype.supportsCommand;

peSnippet.prototype.doCommand = function(command) {
    var item = item = gFocusedProjectView.getSelectedPart();
    switch (command) {
    case 'cmd_insertSnippet':
        _insertSnippet(item);
        break;
    case 'cmd_paste':
        this.pasteSnippet();
        break;
    case 'cmd_addSnippet_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addSnippet':
        this._addSnippet(item);
        break;
    case 'cmd_editSnippetProperties':
        this._snippetProperties(item);
        break;
    default:
        break;
    }    
}

peSnippet.prototype._snippetProperties = function(item)
{
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    openWindowMultipleInstance(
        "chrome://komodo/content/project/snippetProperties.xul", 
        "komodo_snippetproperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

peSnippet.prototype._addSnippet = function(item)
{
    var snippet = Components.classes['@activestate.com/koPart?type=snippet;1']
               .createInstance(Components.interfaces.koIPart_snippet);
    snippet.setStringAttribute('name', 'New Snippet');
    snippet.setStringAttribute('set_selection', 'false');
    snippet.setStringAttribute('indent_relative', 'false');
    snippet.setStringAttribute('anchor_line', '0');
    snippet.setStringAttribute('currentPos_line', '0');
    snippet.setStringAttribute('currentPos_col', '0');
    snippet.setStringAttribute('anchor_col', '0');
    snippet.setStringAttribute('currentPos_col', '0');
    snippet.value = '';
    var obj = new Object();
    obj.item = new partWrapper(snippet, 0);
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://komodo/content/project/snippetProperties.xul", 
        "komodo_snippetProperties",
        "chrome,close=yes,modal=yes", obj);
    if (obj.res) {
        var index = -1;
        if (typeof(item)=='undefined' || !item)
            item = gFocusedProjectView.getSelectedPart();
        if (item) index = gFocusedProjectView.getIndexByPart(item);    
        gFocusedProjectView.manager.addItem(snippet,index);
    }
}


peSnippet.prototype.pasteSnippet = function()
{
    // 1. get the clipboard service
    var clipboard = Components.classes["@mozilla.org/widget/clipboard;1"].getService();
    if ( clipboard ) clipboard = clipboard.QueryInterface(Components.interfaces.nsIClipboard);
    // 2. create the transferable
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance();
    if ( trans ) 
        trans = trans.QueryInterface(Components.interfaces.nsITransferable);
    if ( trans && clipboard ) {
        // 3. register the data flavors you want, highest fidelity first!
        trans.addDataFlavor("text/unicode");
        // 4. get transferable from clipboard
        clipboard.getData ( trans , 1);  // Use global clipboard
        // 5. ask transferable for the best flavor. Need to create new JS
        //    objects for the out params. 
        var dataObj = new Object();
        var bestFlavor = new Object();
        var len = new Object();
        trans.getAnyTransferData ( bestFlavor, dataObj, len );
        if (bestFlavor.value == "text/unicode"  && dataObj) {
            dataObj = dataObj.value.QueryInterface(Components.interfaces.nsISupportsWString);
            if ( dataObj ) {  
                // ...add snippet. remember len is in bytes, not chars
                var text = dataObj.data.substring(0, len.value / 2);
                AddSnippetFromText(text); 
            }
        }
    }
}

function _insertSnippet (snippet) {   
    try {
        var vm = gLastFocusedViewManager;
        if (!vm) return;
        var view = vm.getCurrentView();
        if (!view) return;
        var scimoz = view._scimoz;
        var text, indent = '';
        scimoz.beginUndoAction();
        vm.setFocus();
        var eol = _getCurrentEOL();
        text = unescapeWhitespace(snippet.value, eol);
        var lines = text.split(eol);
        var setSelection = snippet.part.hasAttribute('set_selection') && snippet.part.getStringAttribute('set_selection') == 'true';
        var relativeIndent = snippet.part.hasAttribute('indent_relative') && snippet.part.getStringAttribute('indent_relative') == 'true';
        var numChars = 0;
        var currentSelectionLength = scimoz.selText.length;
        scimoz.replaceSel('');
        var startOfLine = scimoz.positionFromLine(scimoz.lineFromPosition(scimoz.currentPos));
        var curLine = scimoz.getTextRange(startOfLine, scimoz.currentPos);
        var _in_anchorLine = Number(snippet.part.getStringAttribute('anchor_line'));
        var _in_anchorCol = Number(snippet.part.getStringAttribute('anchor_col'));
        var _in_currentPosLine = Number(snippet.part.getStringAttribute('currentPos_line'));
        var _in_currentPosCol = Number(snippet.part.getStringAttribute('currentPos_col'));
        var _anchorLine = scimoz.lineFromPosition(scimoz.selectionStart);
        var _anchorCol = scimoz.selectionStart - scimoz.positionFromLine(_anchorLine);
        if (curLine) {
            if (relativeIndent) {
                // Get the leading whitespace
                var leadingWhitespace = /^([ \t]*)/;
                leadingWhitespace.exec(curLine);
                indent = RegExp.$1;
                numChars = scimoz.currentPos - startOfLine - indent.length;
                for (var numLeadingChars = 0; numLeadingChars < numChars; numLeadingChars ++) {
                    indent += ' ';
                }
            }
        }
        var selStart, selEnd, selStartLine, selEndLine, selStartCol, selEndCol;
        
        if (_in_anchorLine < _in_currentPosLine ||
            (_in_anchorLine == _in_currentPosLine && _in_anchorCol < _in_currentPosCol)) {
            firstIsAnchor = true;
            selStart = scimoz.positionFromLine(_anchorLine + _in_anchorLine) + _in_anchorCol;
            selEnd = scimoz.positionFromLine(_anchorLine + _in_currentPosLine) + _in_currentPosCol;
            selStartLine = _in_anchorLine;
            selEndLine = _in_currentPosLine;
            selStartCol = _in_anchorCol;
            selEndCol = _in_currentPosCol;
        } else {
            firstIsAnchor = false;
            selStart = scimoz.positionFromLine(_anchorLine + _in_currentPosLine) + _in_currentPosCol;
            selEnd = scimoz.positionFromLine(_anchorLine + _in_anchorLine) + _in_anchorCol;
            selStartLine = _in_currentPosLine;
            selEndLine = _in_anchorLine;
            selStartCol = _in_currentPosCol;
            selEndCol = _in_anchorCol;
        }
        var wouldEndAtEndOfLine = false;
        var selectionStartsAtCol0, nextChar, firstIsAnchor, line;

        // The anchor and currentPos need to be shifted by the indent being applied
        // _EXCEPT_ if the selection in the snippet corresponds to integral line selections
        // AND, in the case of the "first" marker (min(anchor,currentPos)), if the
        // insertion point of the corresponding line is at the beginning of the line.
        // The selection need to be adjusted depending on whether it's a "line selection" or not.
        // A line selection is assumed when all of the following are true
        // 1) The selection starts in the 0th column
        // 2a) either the selection ends on a different line, or
        // 2b) the selection ends at the end of the current line

        var fullLineSelection = false; // Assume it's false
        if (selStartCol == 0) {
            // if the selection starts at column 0, assume it is a 'full-line' selection
            fullLineSelection = true;
        }

        for (var i = 0; i < lines.length; i++) {
            line = lines[i];
            if (i != 0) {  // Indent all lines but the first
                line = indent + line;
            }
            if (i != lines.length-1) { // Add a newline to all lines but the last.
                line = line + eol;
                //dump("trailing with an eol!\n");
            }
            if (fullLineSelection) { // Do we still think that this may be a full line selection?
                if (i == selStartLine) { // Some tests can be done at selection start...
                    startOfLine = scimoz.positionFromLine(scimoz.lineFromPosition(scimoz.currentPos));
                    var textToLeft = scimoz.getTextRange(startOfLine, scimoz.currentPos);
                    var onlywhitespaceToLeft = (textToLeft.replace(' ', '', 'g').replace('\t', '', 'g')) == '';
                    if (!onlywhitespaceToLeft) {
                        fullLineSelection = false;
                    }
                }
            }
            scimoz.replaceSel(line);
            if (fullLineSelection) {
                if (i == selEndLine && selEndCol != 0) { // and some need to happen at selection end.
                    var realLineNo = _anchorLine + i;
                    var lineLength = scimoz.getLineEndPosition(realLineNo) - scimoz.positionFromLine(realLineNo);
                    
                    if (selEndCol + indent.length != lineLength) {
                        fullLineSelection = false;
                    }
                }
            }
        }
        // if fullLineSelection is true, that means that we shouldn't indent the selectionStart.
        
        // For the end of the selection, we adjust the indent 
        //if (!fullLineSelection || !wouldEndAtEndOfLine) {
            //dump("Adjusting selection End\n");
            //selEnd += indent.length;
        //}
        //dump("endSelCol = " + selEndCol+ '\n');
        //dump('indent.length = ' + indent.length + '\n');
        //dump("anchorLine = " + _anchorLine + '\n');
        //dump("selStartLine = " + selStartLine + '\n');
        //dump("selStartCol = " + selStartCol + '\n');
        selStart = scimoz.positionFromLine(_anchorLine + selStartLine) + selStartCol;
        //dump("selStart = " + selStart + '\n');
        if (! fullLineSelection) {
            if (relativeIndent) {
                selStart += indent.length;
                //dump("adding " + indent.length + '\n');
            } else if (selStartLine == 0) {
                selStart += numChars;  // indent by the initial offset
                //dump("adding " + numChars + '\n');
            }
        }
        //dump("selStart = " + selStart + '\n');
        selEnd = scimoz.positionFromLine(_anchorLine + selEndLine) + selEndCol;
        // We need to adjust the end if we're doing full se
        if (fullLineSelection) {
            if (selEndCol != 0) {
                selEnd += indent.length;
            }
        } else {
            if (selEndCol != 0) {
                //dump('adjusting end\n');
                selEnd += indent.length;
            }
        }
        //dump("selStart = " + selStart + '(' + selStartLine + ', ' + selStartCol + ')\n');
        //dump("selEnd = " + selEnd + '(' + selEndLine + ', ' + selEndCol + ')\n');
        
        if (firstIsAnchor) {
            scimoz.anchor = selStart;
            scimoz.currentPos = selEnd;
        } else {
            scimoz.anchor = selEnd;
            scimoz.currentPos = selStart;
        }
    } catch (e) {
        logError(e);
    }
    scimoz.endUndoAction();
}

peSnippet.prototype.onUpdatePart = function(part,event)
{
    if (part.type == 'snippet') {
        part.renameable = true;
        part.iconstyle['snippet'] = 1;
    }
}

function _AddSnippetFromText(snippettext) {
    var escapedtext = escapeWhitespace(snippettext);
    var snippet = Components.classes['@activestate.com/koPart?type=snippet;1'].createInstance(Components.interfaces.koIPart_snippet);
    snippet.type = 'snippet';
    snippet.setStringAttribute('name', snippetMakeDisplayName(snippettext));
    snippet.setStringAttribute('set_selection', 'false');
    snippet.setStringAttribute('indent_relative', 'false');
    snippet.setStringAttribute('anchor_line', '0');
    // For the column at the end, we need to find the last line
    var lastEOL = snippettext.lastIndexOf('\n');
    if (lastEOL == -1) {
        snippet.setStringAttribute('currentPos_line', '0');
        snippet.setStringAttribute('currentPos_col', String(snippettext.length));
    } else {
        snippet.setStringAttribute('currentPos_line', String(snippettext.match(/\n/g).length));
        snippet.setStringAttribute('currentPos_col', String(snippettext.slice(lastEOL + 2)).length);
    }
    snippet.setStringAttribute('anchor_col', '0');
    snippet.setStringAttribute('set_selection', 'true');
    snippet.value = escapedtext;
    return snippet;
}

function AddSnippetFromTextToToolbox(snippettext) {
    var snippet = _AddSnippetFromText(snippettext);
    toolboxMgr.addItem(snippet);
}

function AddSnippetFromText(snippettext) {
    var snippet = _AddSnippetFromText(snippettext);
    gFocusedProjectView.manager.addItem(snippet);
}

/* Utility functions */

function _getCurrentEOL() {
    var vm = getFocusedViewManager()
    var view = vm.getCurrentView();
    if (!view) {
        // we're not looking at anything -- return '\n' just to return something
        logDebug("toolboxManager: Asked for EOL when no view present...");
        return '\n'
    }
    var eol;
    if (view.py_inst.new_line_endings == view.py_inst.EOL_LF) {
        eol = '\n';
    } else if (view.py_inst.new_line_endings == view.py_inst.EOL_CR) {
        eol = '\r';
    } else if (view.py_inst.new_line_endings == view.py_inst.EOL_CRLF) {
        eol = '\r\n';
    } else {
        eol = '\n';
        logError("toolboxManager: Unknown End of line")
    }
    return eol;
}

function snippetMakeDisplayName(text) {
    // Strip leading whitespace.
    text = text.replace( /^\s+/, "" );
    // Strip trailing whitespace.
    text = text.replace( /\s+$/, "" );
    if (text.length > 30) {
        text = text.substr(0, 20) + ' ... ' + text.substr(text.length - 10, text.length)
    }
    // Compress remaining whitespace.
    text = text.replace( /\s+/g, " " );
    return text
}

function Insert_NamedSnippet(name) {
    // 
    var item = toolboxMgr.findItemByAttributeValue('name', name);
    if (!item) {
        item = gProjectManager.findItemByAttributeValue('name', name);
    }
    if (item) {
        _insertSnippet(item);
    }
}



