/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
var peWSMgr = new peWebService();
registeredExtensions[registeredExtensions.length] = peWSMgr;

function peWebService() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peWebService.prototype.constructor = peWebService;

peWebService.prototype.init = function() {
    this.registerCommands();
    this.bmMgr = Components.classes["@activestate.com/koBookmarkManager;1"].getService(Components.interfaces.koIBookmarkManager);
    this.bmextension = this.bmMgr.extension;
    this.bmrootdir = this.bmMgr.rootDirectory;
}

peWebService.prototype.registerCommands = function() {
    try {
        extensionManager.registerCommand('cmd_addWebService_Project', this);
        extensionManager.registerCommand('cmd_addWebService', this);
        extensionManager.registerCommand('cmd_viewWebService', this);
        extensionManager.registerCommand('cmd_editWebService', this);
        extensionManager.registerCommand('cmd_insertWebService', this);
        extensionManager.registerCommand('cmd_generateProxyWebService', this);
        extensionManager.registerCommand('cmd_helpWebService', this);
    } catch (e) {
        logError(e);
    }
}
peWebService.prototype.registerEventHandlers = function() {
    try {
        extensionManager.addEventHandler(Components.interfaces.koIPart_webservice,'onUpdatePart',this);
        extensionManager.addEventHandler(Components.interfaces.koIPart_webservice,'ondblclick',this);
    } catch (e) {
        logError(e);
    }
}
peWebService.prototype.registerMenus = function() {
    try {
        extensionManager.createMenuItem(Components.interfaces.koIContainer,
                                        'Add New WebService...','cmd_addWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'View WSDL...','cmd_viewWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Edit Service...','cmd_editWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Insert Service Call...','cmd_insertWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Generate Proxy...','cmd_generateProxyWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Service Help...','cmd_helpWebService');
    } catch (e) {
        logError(e);
    }
}

peWebService.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_insertWebService':
        var view = gLastFocusedViewManager.getCurrentView();
        if (!view) return false;
        if (!WSSetup_SupportedLanguage(view.py_inst.language)) return false;
    case 'cmd_helpWebService':
    case 'cmd_editWebService':
    case 'cmd_generateProxyWebService':
    case 'cmd_viewWebService':
        if (!item) {
            item = gFocusedProjectView.getSelectedPart();
        }
        return item && item.part.hasAttribute('wsdl');
    case 'cmd_addWebService_Project':
        return gProjectManager.getCurrentProject() != null;
    case 'cmd_addWebService':
        return true;
    default:
        break;
    }
    return false;
}

peWebService.prototype.isCommandEnabled = peWebService.prototype.supportsCommand;

peWebService.prototype.doCommand = function(command) {
    var item;
    switch (command) {
    case 'cmd_helpWebService':
        item = gFocusedProjectView.getSelectedPart();
        if (item.part.hasAttribute('wsdl')) {
            this.doHelp(item);
        }
        break;
    case 'cmd_editWebService':
        //dump("Called the editGUIDialog command\n");
        // get the current selection, then open the file
        item = gFocusedProjectView.getSelectedPart();
        if (item.part.hasAttribute('wsdl')) {
            this.editService(item);
        }
        break;
    case 'cmd_insertWebService':
        item = gFocusedProjectView.getSelectedPart();
        if (item.part.hasAttribute('wsdl')) {
            insertSimpleWebService(item);
        }
        break;
    case 'cmd_addWebService_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addWebService':
        // if called by a menu, popup a window and ask for the
        // url, or offer to browse.  Then call AddWebService with url
        this.addService(item);
        break;
    case 'cmd_generateProxyWebService':
        item = gFocusedProjectView.getSelectedPart();
        if (item.part.hasAttribute('wsdl')) {
            generateWebServiceProxy(item.part.getStringAttribute('name'),item.part.getStringAttribute('wsdl'));
        }
        break;
    case 'cmd_viewWebService':
        item = gFocusedProjectView.getSelectedPart();
        if (item.part.hasAttribute('wsdl')) {
            doFileOpen(item.part.getStringAttribute('wsdl'));
        }
        break;
    default:
        break;
    }
}

peWebService.prototype.ondblclick = function(item,event) {
    if (item.part.hasAttribute('wsdl')) {
        insertSimpleWebService(item);
    }
}

peWebService.prototype.onUpdatePart = function(item,event)
{
    item.iconopenstyle['WebService'] = 1
    item.iconstyle['WebService'] = 1
    item.renameable = 0;
}

peWebService.prototype.doHelp = function(item)
{
    var path = os.path.join(this.bmrootdir,
                    item.name + ".html");
    doHelp(path);
}


peWebService.prototype.addService = function(item)  {
    var obj = new Object();
    obj.type = 'add';
    obj.dir = this.bmrootdir;
    obj.url = null;
    obj.retval = false;
    // fix case where toc is deleted, but folders exist
    this.bmMgr.createFolder(this.bmrootdir);
    window.openDialog("chrome://komodo/content/webservices/bmedit.xul", 
               "komodo_wsbmedit",
               "chrome,close=yes,modal", obj);
    
    // now get the info from obj, and add the webservice
    if (obj.retval && obj.bmark) {
        insertWebServicePart(obj.bmark, obj.url, item);
    }
}

peWebService.prototype.editService = function(item)  {
    var wsdl = item.part.getStringAttribute('wsdl');
    var obj = new Object();
    obj.type = 'edit';
    obj.name = item.name;
    obj.url = wsdl;
    obj.description = item.value;
    obj.shortname = item.part.getStringAttribute('object');
    obj.retval = false;
    window.openDialog("chrome://komodo/content/webservices/bmedit.xul", 
               "komodo_wsbmedit",
               "chrome,close=yes,modal", obj);
    
    if (obj.retval)  { // ok was clicked
        item.name = obj.name;
        item.part.setStringAttribute('wsdl',obj.url);
        item.part.setStringAttribute('object', obj.shortname);
        item.part.value = obj.description;
    }
}

function _generateWebServiceDetails(url) {
    // get details from the wsdl if possible
    window.setCursor("spinning");
    var bmark = Components.classes["@activestate.com/koBookmark;1"].createInstance(Components.interfaces.koIBookmark);
    bmark.url = url;
    var errmsg = new Object();
    var success = bmark.generateFromWSDL(errmsg);
    if (!success) {
        window.setCursor("auto");
        alertDialog("Error retrieving " + bmark.url + 
                    ".  "+errmsg.value);
        return null;
    }

    var path = os.path.join(peWSMgr.bmrootdir,
                    bmark.name + peWSMgr.bmextension);
    var nbmark = peWSMgr.bmMgr.open(path);
    nbmark.url = bmark.url;
    nbmark.generateFromWSDL(errmsg);
    if (!success) {
        window.setCursor("auto");
        alertDialog("Error retrieving " + bmark.url + 
                    ".  "+errmsg.value);
        return null;
    }
    errmsg = new Object();
    success = nbmark.save(bmark.name, false, errmsg);
    if (errmsg.value) {
        window.setCursor("auto");
        alertDialog(errmsg.value);
        return null;
    }
    window.setCursor("auto");
    return bmark;
}

function insertWebServicePart(bmark, url, item)
{
    var index = -1;
    window.setCursor("spinning");
    var folderpart = Components.classes["@activestate.com/koPart?type=webservice;1"]
                                    .createInstance(Components.interfaces.koIPart_webservice);
    folderpart.setStringAttribute('wsdl', url);
    folderpart.setStringAttribute('name', bmark.name);
    folderpart.setStringAttribute('object', bmark.shortname);
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedPart();
    if (item) index = gFocusedProjectView.getIndexByPart(item);    
    gFocusedProjectView.manager.addItem(folderpart,index);
    window.setCursor("auto");
}

function AddWebService(url)
{
    // XXX for now use the koIBookmark to parse the wsdl
    var bmark = _generateWebServiceDetails(url);
    if (!bmark) return;
    insertWebServicePart(bmark, url);
    //dump("leaving AddURL\n");
}

function addWSBookmarkToToolbox(bmark, url)
{
    gFocusedProjectView = toolboxMgr.viewMgr.view;
    gFocusedProjectView.manager.viewMgr.focus();
    insertWebServicePart(bmark, url);
}

function generateWebServiceProxy(name, wsdl, language) {
    // XXX deal with language selection
    if (typeof(language) == 'undefined') {
        // XXX this is where we ask what language to generate the proxy for
        language = 'Perl';
    }
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"]
               .getService(Components.interfaces.koILastErrorService)

    var useCustomDir = prefs.hasLongPref('wsPreferCustomDirectory')?prefs.getLongPref('wsPreferCustomDirectory'):0;
    var addtoProj = prefs.hasBooleanPref('wsAddToCurrentProject')?prefs.getBooleanPref('wsAddToCurrentProject'):true;
    var directory = prefs.hasStringPref(language+'WebServiceDirectory')?prefs.getStringPref(language+'WebServiceDirectory'):null;
    var project = gProjectManager.getCurrentProject();
    if (!useCustomDir && project) {
        directory = os.path.dirname(project.url);
    }
    var leafname = name+'.pm';
    var url = leafname;
    var localPath = null;
    if (directory) {
        var local = false;
        url=directory+'/'+leafname;
        try {
            var localfile = URLToLocalFile(url);
            if (localfile) localPath = localfile.path;
        } catch(e) {
            // not a file url
            local = true;
        }
        if (local) {
            try {
                localPath = os.path.join(directory,leafname);
                url = LocalFileToURL(localPath);
            } catch(e) {
                // file does not exist
            }
        }
    }
    
    var item = null;
    if (project) item = gProjectManager.findItemByURLInProject(project, 'file', url);

    // see if the current project already has the proxy, if so, ask if they want to
    // replace the file in the current project.
    var generate = true;
    if (project && addtoProj && item) {
        if (!localPath) localPath = url;
        if (yesNoDialog("The proxy ["+localPath+"] already exists in the current project, would you like to refresh the proxy?", "Yes") != "Yes")
            generate = false;
    } else if (localPath && os.path.exists(localPath)) {
        // see if the url already exists, if so, what do we do?
        if (yesNoDialog("The proxy ["+localPath+"] already exists, would you like to refresh the proxy?", "Yes") != "Yes")
            generate = false;
    }
    
    window.setCursor("spinning");
    if (generate) {
        // generate the proxy now
        var proxygen = Components.classes["@activestate.com/KoWSProxyGenerator;1"].
                    getService(Components.interfaces.koIWSProxyGenerator);
        var text;
        try {
            text = proxygen.generate(language, wsdl);
        } catch(e) {
            window.setCursor("auto");
            var error = lastErrorSvc.getLastErrorMessage();
            if (error)
                alertDialog(error);
            else
                alertDialog('Unknown error parsing WSDL document');
            return;
        }
        
        // is the file already open?
        var view = viewManager.getViewForFile(url);
        if (view) {
            //select the view
            viewManager.setCurrentView(view);
        } else {
            view = doFileNew(language);
        }
        view._scimoz.text = text;
        viewManager.renameView(view,url,leafname);
        view.py_inst.isDirty = 1;
        viewManager.indicateIfDirty(view);
        
        if (directory && (project || !addtoProj)) {
            if (!view.saveContents()) {
                window.setCursor("auto");
                return;
            }
        } else {
            if (!view.doSaveAs()) {
                window.setCursor("auto");
                return;
            }
        }
    } else { // not generating
        doFileOpen(url);
    }
    
    // add it to the project
    if (addtoProj && project && !item) {
        var filepart = Components.classes["@activestate.com/koPart?type=file;1"]
                                        .createInstance(Components.interfaces.koIPart_file);
        filepart.setStringAttribute('url', url);
        gProjectManager.addItem(filepart);
    }
    window.setCursor("auto");
}

/******************************************************/
// old web services js functions


// XXX insertWebService() and getWSInserter() are near-duplicates of the
// functions of the same name in bookmarks.js.  These should be factored out
// into a common file somewhere.
function insertSimpleWebService(item) {
    if (typeof(item) == 'undefined' || !item) {
        item = gFocusedProjectView.getSelectedPart();
        if (!item || !item.part.hasAttribute('wsdl')) return false;
    }
        
    // First, run the WS setup wizard for this language if it has not been
    // run before.
    var view = gLastFocusedViewManager.getCurrentView();
    if (!view) return 0;
    var language = view.py_inst.language;
    try {
        WSSetup_SetupIfHaveNotAlready(language)
    } catch(e) {
        // an exception is thrown if it is an unsupported language, ignore and escape
        return false;
    }

    var shortname = item.part.getStringAttribute('object');
    var url = item.part.getStringAttribute('wsdl');
    // Do the WS insertion magic.
    var wsinserter = view.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.
        getLanguageService(Components.interfaces.koIWSInsertLanguageService);
    if (wsinserter) {
	wsinserter.insert(view._scimoz, 
			  view._scimoz.currentPos, 
			  shortname, 
			  url);
        gLastFocusedViewManager.setFocus();
    }
    return true;
}

function getWSInserter() {
    var v = gLastFocusedViewManager.getCurrentView();
    scin = v._scimoz;
    var wsinserter = v.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.
        getLanguageService(Components.interfaces.koIWSInsertLanguageService);
    return wsinserter;
}    

