/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * projectDragDrop.js
 *
 * Handle drag-drop from files panel of projectviewer onto trees
 *
 * To be loaded in the same .xul file as projectviewer.js
 */

function beginDragFileName(event) {
    //introspect(event);
    // don't propagate the event
    event.cancelBubble = true;

    // get the drag service
    var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);

    // create a transferable
    var trans = 
        Components.classes["@mozilla.org/widget/transferable;1"].
        createInstance(Components.interfaces.nsITransferable);

    trans.addDataFlavor("text/x-moz-url");
    var genTextData = 
        Components.classes["@mozilla.org/supports-wstring;1"].
        createInstance(Components.interfaces.nsISupportsWString);
    // put the full filename here
    var node1 = event.target;
    var myData;
    if (node1.nodeName == "treecell") {
        myData = node1.getAttribute("url");
    } else {
        logError("Can't get a URL for the file!??!\n");
        return false;
    }

   genTextData.data = myData;
   // add data to the transferable
   trans.setTransferData ("text/x-moz-url",
                              genTextData,
                              genTextData.data.length * 2);

   // create an array for our drag items,
   // though we only have one this time
   var transArray = 
     Components.classes["@mozilla.org/supports-array;1"]
       .createInstance(Components.interfaces.nsISupportsArray);
   // put it into the list as an |nsISupports|
   var genTrans =
     trans.QueryInterface(Components.interfaces.nsISupports);
   transArray.AppendElement(genTrans);
   // Actually kick off the drag
   var nsIDragService = Components.interfaces.nsIDragService;
   try {
       dragService.invokeDragSession(event.target,
                                 transArray,
                                 null,
                                 (nsIDragService.DRAGDROP_ACTION_COPY + 
                                  nsIDragService.DRAGDROP_ACTION_MOVE));
   } catch(e) {
       // user pressed esc or some other canelation
       // such as droping someplace not allowed
       return false;
   }
   return true;
}

// Given a node somewhere in the tree, determine what
// project the drag operation applies to.  Returns null
// if no project is found, or its node
function _getProjectDragTarget(node)
{
    if (node.tagName == "treechildren") {
        // In the main project area, below the last node.
        var rc;
        if (node.childNodes.length == 0) {
//            logDebug("d&d over empty project pane");
            rc = null;
        } else
            rc = node.childNodes[node.childNodes.length-1];
        return rc;
    } 
    if (node.tagName == "treecell") {
        node = node.parentNode;
    }
    if (node.tagName == "treerow") {
        node = node.parentNode;
    }
    if (node.tagName == "treeitem") {
        if (node.getAttribute("url"))
            return node;
        // A file node.
        return node.parentNode.parentNode
    }
    return null;
}
function DragOverProjectTree(event) {
  //   dump ("DragOverProjectTree...\n");
  // introspect(event);
  event.cancelBubble = true;
  var isOkToDrop = false;

  var dragService = 
    Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);

  var dragSession = dragService.getCurrentSession();
  if ( !dragSession ) {
    logError ("No dragSession in place.\n");
    return false;
  }
  var projNode = _getProjectDragTarget(event.target);

  if ( projNode &&
       (dragSession.isDataFlavorSupported("text/x-moz-url") ||
       dragSession.isDataFlavorSupported("application/x-moz-file"))) 
  {
        //dump("Drag over project " + projNode.getAttribute("id") + "\n");
        var trans = 
          Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
        trans.addDataFlavor("text/x-moz-url");
        trans.addDataFlavor("application/x-moz-file");

        for ( var i = 0; i < dragSession.numDropItems; ++i ) {

          dragSession.getData ( trans, i );
          var dataObj = new Object();
          var bestFlavor = new Object();
          var len = new Object();

          try {
            trans.getAnyTransferData ( bestFlavor, dataObj, len );
          } catch (e) {
            continue; // no data
          }

          var fname = null;
          switch (bestFlavor.value) {
            case "text/x-moz-url":
              // we assume non-local URLs are always valid
            var url = dataObj.value.QueryInterface(Components.interfaces.nsISupportsWString).data;
              try {
                fname = Components.classes["@mozilla.org/network/standard-url;1"].
                    createInstance(Components.interfaces.nsIFileURL);
                fname.spec = url
                isOkToDrop = os.path.isfile(fname.file.path);
                fname = null;
              } catch (e) {
                isOkToDrop = os.path.isfile(url);
              } 
              break;
            case "application/x-moz-file":
              try {
                  fname = dataObj.value.QueryInterface(Components.interfaces.nsIFile).target;
                  isOkToDrop = os.path.isfile(fname);
              } catch(e) {
                  isOkToDrop = false;
              }
              break;
            default:
              url = null;
              break;
          }
          if (!isOkToDrop) break;
        } // foreach drag item
    }
  if (projNode && isOkToDrop) {
    //XXX this is really slow and likes to refresh N times per second.
    projNode.setAttribute ( "dd-triggerrepaint", 0 );
  }
  dragSession.canDrop = isOkToDrop;
  return isOkToDrop;
}

function DropOnProjectTree(event) {

    event.cancelBubble = true;
  var dragService = 
    Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
  var dragSession = dragService.getCurrentSession();
  if ( dragSession ) {
    var trans = 
      Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    dump("got trans" + dragSession.numDropItems + "\n");
    trans.addDataFlavor("text/x-moz-url");
    trans.addDataFlavor("application/x-moz-file");
    for ( var i = 0; i < dragSession.numDropItems; ++i ) {
      dragSession.getData ( trans, i );
      var dataObj = new Object();
      var bestFlavor = new Object();
      var len = new Object();
      try {
        trans.getAnyTransferData ( bestFlavor, dataObj, len );
      } catch (e) {
        continue; // no data
      }
      var url;
      switch (bestFlavor.value) {
        case "text/x-moz-url":
          url = dataObj.value.QueryInterface(Components.interfaces.nsISupportsWString).data;
          break;
        case "application/x-moz-file":
          try {
              var fname = dataObj.value.QueryInterface(Components.interfaces.nsIFile).target;
              url = LocalFileToURL(fname).path;
          } catch(e) {
              url = null;
          }
          break;
        default:
          url = null;
          break;
      }
//      dump("Have URI of " + url + "\n");
      if ( url ) {
        var projURL = _getProjectDragTarget(event.target).getAttribute("url");
        dump("For project " + projURL + "\n");
        if (!AddItemToProject(projURL, url)) {
            logError("Unable to add file '" + url + "' to project '" + projURL + "\n");
        }
      }
    } // foreach drag item
  }
} // DropOnProjectTree

