/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * projectmanager.js
 * Handle Komodo Project Files -- main part
 *
 * To be loaded in the same .xul file as projectviewer.js
 */

// Globals
var gProjectManager = null;


//---- internal support routines

function _setProjectSplitterState(collapsed)
{
    var splitter = document.getElementById("projectviewer-splitter");
    var menuitem = document.getElementById('menu_viewProjects');
    if (collapsed) {
        splitter.setAttribute('state', 'collapsed');
        menuitem.setAttribute('checked', 'false');
    } else {
        splitter.removeAttribute('state');
        menuitem.setAttribute('checked', 'true');        
    }
}



//----- The projectManager class manages the set of open projects and
// which project is 'current'.

function projectManager() {
    baseManager.apply(this, ["projectsChanged"]);
    this._projects = [];
    this._currentProject = -1;
    this.viewMgr = document.getElementById('projectview');
    this.viewMgr.onLoad(this, null);
    gPartManagers.push(this);
    // initialize our global view
    gFocusedProjectView = this.viewMgr.view;
    // register our command handlers
    this.registerCommands();
    // add our default datapoint
    this.viewMgr.addColumns(extensionManager.datapoints);
    extensionManager.datapoints['Name']='Name';
    this.observer = new ProjectEventObserver();
    this.lastsorttype = null;
    this.sorttype = 'Name';
    this.sortdir = 1;
    this.firstsort = true;
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
projectManager.prototype = new baseManager();
projectManager.prototype.constructor = projectManager;

projectManager.prototype.getProjectsMenu = function(popup)
{
    // used by top level project->set active menu
    var curProject = this.getCurrentProject();
    var children = popup.childNodes;
    var i,mi;
    for (i = 0; i < children.length; i++) {
        popup.removeChild(children[i]);
    }
    if (this._projects.length > 0) {
        for (var project in this._projects) {
            mi = document.createElementNS(XUL_NS, 'menuitem');
            mi.setAttribute('label',this._projects[project].name);
            mi.setAttribute('oncommand','gProjectManager.setCurrentProject('+project+');');
            mi.setAttribute('type','checkbox');
            if (curProject == this._projects[project]) {
                mi.setAttribute('checked','true');
            }
            popup.appendChild(mi);
        }
    } else {
        // add an 'empty' menu item
        mi = document.createElementNS(XUL_NS, 'menuitem');
        mi.setAttribute('label','No Open Projects');
        mi.setAttribute('disabled','true');
        popup.appendChild(mi);
    }
    return true;
}

projectManager.prototype.closeProjectByURL = function(url) {
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.url == url) {
            return this.closeProject(i);
        }
    }
    return false;
}
projectManager.prototype.closeProjectEvenIfDirty = function(id) {
    var project = this._projects[id];
    this.viewMgr.view.closeToplevelPart(id);
    if (this.viewMgr.view.entriesList.length > 0) {
        this.setCurrentProject(0);
        this.viewMgr.view.mSelection.currentIndex = 0;
    } else {
        this.setCurrentProject(-1);
    }
    // remove the project from our list
    var fa = this._projects.slice(0,id);
    var fe = this._projects.slice(id+1);
    this._projects = fa;    
    this._projects = this._projects.concat(fe);
    project.part.close();
    
    MRU_addURL("mruProjectList", project.part.url);
    koRequestUpdateCommands(this._command_name);
    return true;
}

projectManager.prototype.closeProject = function(id) {
    var project = this._projects[id];
    if (project.part.isDirty) {
        var res = yesNoCancel("Save changes to project " + project.name + "?", "Yes");
        res = res.toLowerCase();
        if (res == "cancel") {
            return false;
        } else if (res == "yes") {
            try {
                project.part.save();
            } catch(ex) {
                var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                    getService(Components.interfaces.koILastErrorService);
                alertDialog('There was an error saving project, "' +
                            project.name + '": ' +
                            lastErrorSvc.getLastErrorMessage());
                return false;
            }
        }
    }
    return this.closeProjectEvenIfDirty(id);
}

projectManager.prototype.getDirtyProjectURLs= function() {
    var dirtyURLs = new Array();
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.isDirty) {
            dirtyURLs.push(this._projects[i].part.url);
        }
    }
    return dirtyURLs;    
}

projectManager.prototype.getDirtyProjectIDs= function() {
    var dirtyIDs = new Array();
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.isDirty) {
            dirtyIDs.push(i);
        }
    }
    return dirtyIDs;    
}

projectManager.prototype.closeAllProjects = function() {
    for (var i = 0; i < this._projects.length; i++) {
        if (!this.closeProject(i)) return false;
    }
    return true;
}

projectManager.prototype.closeAllProjectsEvenIfDirty = function() {
    for (var i = 0; i < this._projects.length; i++) {
        if (!this.closeProjectEvenIfDirty(i)) return false;
    }
    return true;
}

projectManager.prototype.saveProject = function(id) {
    // Returns true on success, false on failure.
    if (! id in this._projects) {
        logError("can't save unknown project with id " + id);
        return false;
    }
    var lastErrorSvc;
    var project = this._projects[id];
    if (project.properties['readonly']) {
        if (typeof(project.properties['scc']) != 'undefined' &&
            project.properties['scc']) {
            if (alertDialog("The project '" + project.name + "' is a read only file under source code control. " 
                                                + "To save it Komodo must first check it out.",true,false) == "OK") {
                // check file out then save
                if (editSCCFile(project)) {
                    try {
                        project.part.save();
                    } catch(ex) {
                        lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                            getService(Components.interfaces.koILastErrorService);
                        alertDialog('There was an error saving project "' +
                                    project.name + '": ' +
                                    lastErrorSvc.getLastErrorMessage());
                        return false;
                    }
                    gObserverSvc.Notify(this, 'file_changed', project.url);
                }
            }
            return true;
        } else
        {
            alertDialog("The project '" + project.name + "' is a read only file and cannot be saved.");
            return false;
        }
    } else {
        try {
            project.part.save();
        } catch(ex) {
            lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            alertDialog('There was an error saving project "' +
                        project.name + '": ' +
                        lastErrorSvc.getLastErrorMessage());
            return false;
        }
        gObserverSvc.Notify(this, 'file_changed', project.url);
    }
    return true;
}

projectManager.prototype.newProject = function(url) {
    var project = Components.classes["@activestate.com/koProject;1"]
                                        .createInstance(Components.interfaces.koIProject);
    project.create();
    project.url = url;
    try {
        project.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alertDialog('There was an error saving project "' +
                    project.name + '": ' +
                    lastErrorSvc.getLastErrorMessage());
        return false;
    }
    this.loadProject(url);
    try {
        gObserverSvc.Notify(self, 'file_project', url);
    } catch(e) { /* exception if no listeners */ }
    return true;
}

projectManager.prototype.revertProjectByURL = function(url) {
    this._revertProject(this.getProjectIdByURL(url),url);
}

projectManager.prototype.revertProjectById = function(id) {
    this._revertProject(id,this.getProjectById(id).url);
}

projectManager.prototype._revertProject = function(id,url) {
    this.closeProjectEvenIfDirty(id);
    this.loadProject(url);
}

projectManager.prototype.loadProject = function(url) {
    var project = Components.classes["@activestate.com/koProject;1"]
                        .createInstance(Components.interfaces.koIProject);
    
    // is the project already loaded?
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].url == url) {
            // Re-open the project pane if it is closed.
            _setProjectSplitterState(0);
            return null;
        }
    }
    window.setCursor("wait");
    try {
        project.load(url);
    } catch(e) {
        window.setCursor("auto");
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        var projectname;
        try {
            projectname = URLToLocalFile(url).path;
        } catch (e) {
            projectname = url;
        }
        if (!projectname) { 
            projectname = url;
        }
        alertDialog("Unable to load project \"" + projectname + "\": " +
                    lastErrorSvc.getLastErrorMessage());
        return null;
    }
    // if the project pane is collapsed, expand it
    var splitter = document.getElementById('projectviewer-splitter');
    if (splitter.getAttribute("state") == "collapsed") {
        expandCollapseSplitter('projectviewer-splitter', 'menu_viewProjects');
        // XXX should ensure a certain minimal width XXX
    }

    var wrapped = new partWrapper(project, 0);
    wrapped.url = url;
    this._projects[this._projects.length] = wrapped;
    this.setCurrentProject(this._projects.length-1);
    
    // add project to project outliner
    this.firstsort = true;
    this.sort();
    // Open the project by default -- this feels like it belongs elsewhere... XXX
    this.viewMgr.view.toggleOpenState(this.viewMgr.view.getIndexByPart(wrapped));

    // let the status service know it has work to do
    try {
        gObserverSvc.Notify(self, 'file_update_now', null);
    } catch(e) { /* exception if no listeners */ }
    
    MRU_addURL("mruProjectList", url);
    koRequestUpdateCommands(this._command_name);
    window.setCursor("auto");
    this.viewMgr.focus(); // always set focus here when loading projects
    return project;
}

projectManager.prototype.sort = function(sorttype) {
    if (this._projects.length < 1) return;
    var currentProjectName = null;
    if (this._projects.length > 1) {
        // ALWAYS sort the projects by name
        currentProjectName = this._projects[this._currentProject].name;
        var olddir = gWrapperSortDir;
        gWrapperSortDir = 1;
        var oldtype = gWrapperComparison;
        gWrapperComparison = 'Name';
        this._projects.sort(compareWrappers);
        gWrapperComparison = oldtype;
        gWrapperSortDir = olddir;
    }
    if (typeof(sorttype) != 'undefined') {
        this.sorttype = sorttype;
    }
    if (!this.firstsort) {
        if (this.sorttype == this.lastsorttype) {
            this.sortdir = this.sortdir * -1;
        } else {
            // reset the direction since we're sorting
            // on a different column
            this.sortdir = 1;
        }
        this.viewMgr.view.updateSortIndicators(this.sorttype);
    }
    this.firstsort = false;
    gWrapperSortDir = this.sortdir;
    gWrapperComparison = this.lastsorttype = this.sorttype;
    
    // clear the list and reinsert the projects
    this.viewMgr.view.clearList();
    for (var i in this._projects) {
        this._projects[i].sortChildren();
        if (currentProjectName &&
            this._projects[i].name == currentProjectName)
            this._currentProject = i;
        this.viewMgr.view.addToplevelPart(this._projects[i]);
    }
}

projectManager.prototype.getProjectByItem = function(item) {
    for (var i in this._projects) {
        var project = this.isChildOf(this._projects[i], item);
        if (project != null) return project;
    }
    return null;
}

projectManager.prototype.getItemByAttributeValue = function(attribute, value) {
    for (var i in this._projects) {
        var item = this.findChildByAttributeValue(this._projects[i], attribute, value);
        if (item != null) return item;
    }
    return null;
}

projectManager.prototype.getProjectByURL = function(url) {
    for (project in this._projects) {
        if (project.url == url) {
            return project;
        }
    }
    return null;
}

projectManager.prototype.getProjectIdByURL = function(url) {
    var id, project;
    for (id in this._projects) {
        project = this._projects[id];
        if (project.url == url) {
            return id;
        }
    }
    return -1;
}
projectManager.prototype.getProjectById = function(projectId) {
    return this._projects[projectId];
}

projectManager.prototype.getCurrentProjectId = function() {
    return this._currentProject;
}

projectManager.prototype.getCurrentProject = function() {
    if (this._currentProject >=0 ) return this._projects[this._currentProject];
    return null;
}

projectManager.prototype.getSelectedProjectId = function() {
    var projectId = this.viewMgr.view.getSelectedToplevelPartId();
    if (projectId < 0) projectId = this.getCurrentProjectId();
    return projectId;
}

projectManager.prototype.setCurrentProject = function(projectIndex) {
    if (this._currentProject >= 0)
        this._projects[this._currentProject].styles['projectActive']=0;
    this._currentProject = projectIndex;
    if (this._currentProject >= 0)
        this._projects[this._currentProject].styles['projectActive']=1;
    this.viewMgr.view.mOutliner.invalidate();
}

projectManager.prototype.registerCommands = function() {
    extensionManager.registerCommand("cmd_newProject",this);
    extensionManager.registerCommand("cmd_openProject",this);
    extensionManager.registerCommand("cmd_openProjectFromURL",this);
    extensionManager.registerCommand("cmd_setActiveProject",this);
    extensionManager.registerCommand("cmd_closeProject",this);
    extensionManager.registerCommand("cmd_saveProject",this);
    extensionManager.registerCommand("cmd_saveProjectAs",this);
    extensionManager.registerCommandMatch("cmd_mruProject",/cmd_mruProject([0-9]+)/,this)

    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Make Active Project','cmd_setActiveProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Close Project','cmd_closeProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Save Project','cmd_saveProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Save Project As...','cmd_saveProjectAs');
}

projectManager.prototype.supportsCommand = function(command, item) {
    var groups = command.match(/cmd_mruProject([0-9]+)/);
    if(groups && groups[1]) return true;

    switch(command) {
    case "cmd_setActiveProject":
    case "cmd_newProject":
    case "cmd_openProject":
    case "cmd_openProjectFromURL":
    case "cmd_closeProject":
    case "cmd_saveProjectAs":
        return true;
    case "cmd_saveProject":
        var nitem = gFocusedProjectView.getSelectedPart();
        if (!nitem || nitem.type != 'project') {
            nitem = this.getCurrentProject();
        }
        return (nitem && nitem.part && nitem.part.isDirty);
    default:
        return false;
    }
}

projectManager.prototype.isCommandEnabled = function(command) {
    var groups = command.match(/cmd_mruProject([0-9]+)/);
    if(groups && groups[1]) return true;
    
    switch(command) {
    case "cmd_setActiveProject":
        return this.getCurrentProjectId() != this.viewMgr.view.getSelectedToplevelPartId();
        break;
    case "cmd_newProject":
    case "cmd_openProject":
        return true;
    case "cmd_closeProject":
        return this.getCurrentProjectId() >= 0;
    case "cmd_saveProject":
        var item = gFocusedProjectView.getSelectedPart();
        if (!item || item.type != 'project') {
            item = this.getCurrentProject();
        }
        return (item && item.part && item.part.isDirty);
    case "cmd_saveProjectAs":
        var projectId = this.viewMgr.view.getSelectedToplevelPartId();
        if (projectId < 0) projectId = this._currentProject;
        return projectId >= 0;
    }
    return false; // shutup strict js
}

projectManager.prototype.doCommand = function(command) {
    var groups = command.match(/cmd_mruProject([0-9]+)/);
    if (groups && groups[1]) {
        var url = MRU_get("mruProjectList", Number(groups[1]));
        doOpenProject(url);
        return;
    }
    
    var projectId = this.getSelectedProjectId();
    
    switch(command) {
    case "cmd_setActiveProject":  this.setCurrentProject(projectId);  break;
    case "cmd_newProject": doNewProject(); break;
    case "cmd_openProject": doOpenProject(null); break;
    case "cmd_openProjectFromURL": doOpenProjectFromURL(null); break;
    case "cmd_closeProject": this.closeProject(projectId); break;
    case "cmd_saveProject": this.saveProject(projectId); break;
    case "cmd_saveProjectAs": ProjectSaveAs(projectId); break;
    }
    return;
}

projectManager.prototype.addItem = function(/* koIPart */ part, index) {
    if (typeof(index)=='undefined') index = -1;
    var isprojectref = false;
    if (index < 0) {
        try {
            /* if the target is not a project, then open it */
            isprojectref = part.QueryInterface(Components.interfaces.koIPart_ProjectRef) != null;
        } catch(e) {}
        if (isprojectref) {
            doOpenPath(part.getStringAttribute('url'));
            return false; // return false so drag-move doesn't remove the original part
        }
    }
    var isproject = false;
    try {
        /* we don't allow projects to be added to
           projects */
        isproject = part.QueryInterface(Components.interfaces.koIProject) != null;
        if (isproject) return false;
    } catch(e) {}
    
    try {
        var parent = null;
        if (index >= 0 && (this.viewMgr.view.entriesList[index].level > 0 || this.viewMgr.view.entriesList[index].isContainer)) {
            if (!this.viewMgr.view.entriesList[index].isContainer) {
                index = this.viewMgr.view.getParentIndex(index);
            }
            parent = this.viewMgr.view.entriesList[index];
        } else {
            parent = this.getCurrentProject();
        }
        if (part.hasAttribute('url')) {
            var url = part.getStringAttribute('url');
            if (parent.part.getChildWithTypeAndStringAttribute(part.type,'url',url)) 
                return false;
        }
        parent.part.addChild(part);
        var firstchild = parent.updateWrappedPart();
        this.viewMgr.view.refresh(parent);
        this.viewMgr.view.selectPart(firstchild);
        return true;
    } catch(e) { }
    return false;
}


projectManager.prototype.getItemsByURL = function(url, type) {
    items = [];
    var item;
    for (var i in this._projects) {
        item = this.findItemByURLInProject(this._projects[i], type, url);
        if (item != null) items.push(item);
    }
    return items;
}

/* We may need to optimize this if these functions end up being called a lot
  Currently they're only called when the GUI builder creates new files */

projectManager.prototype.findItemByURL = function(url) {
    for (var i in this._projects) {
        var item = this.findItemByURLInProject(this._projects[i], null, url);
        if (item != null) return item;
    }
    return null;
}

projectManager.prototype.findItemByURLInProject = function (project, type, url) {
    var item;
    var child;
    var i;
    var j;
    for (i in project.children) {
        item = project.children[i];
        if (!type || item.type == type) {
            if (item.url == url || (item.part.hasAttribute('url') && (item.part.getStringAttribute('url') == url))) {
                return item;
            }
        }
        if (item.children) {
            for (j in item.children) {
                child = this.findItemByURLInProject(item.children[j], type, url);
                if (child != null) {
                    return child;
                }
            }
        }
    }
    if (project.url == url || (project.part.hasAttribute('url')
            && project.part.getStringAttribute('url') == url)) {
        return project;
    }
    return null;
}

projectManager.prototype.findItemByAttributeValue= function (attribute, value) {
    var found;
    for (var i in this._projects) {
        found = this.findChildByAttributeValue(this._projects[i], attribute, value);
        if (found) return found
    }
    return null;
}

//-------------------------------------------------------------------------
// command implementations
//

function doOpenProject(url) {
  var fp = null;

  if (!url) {
    fp = dlg_openPRJFile();
    
    if(fp == null) {
      return;
    }
    else {
      var resourceInfo = new dlg_get_URL_andName(fp);
      url = resourceInfo.url;
    }
  }
  
  doOpenProjectFromURL(url);
}


function doOpenProjectFromURL(url) {
    if (!url) {
    url = doOpenURL("Enter the URL of the project file you wish to open:");
    }
    if (url) {
        gProjectManager.loadProject(url)
    }
}

function doNewProject(url) {
    var fp = null;
    
    if (!url) {
        fp = dlg_fileSelectViaDialog(
            Components.interfaces.nsIFilePicker.modeSave, "New Project",
            null /* default directory */, null /* default file */,
            "Komodo Project File", "*.kpf");
        if (!fp) {
            return false;
        }
        var file = new dlg_get_URL_andName(fp);
        if (!file) return false;
        filewinfuncs_forceExtension(file, '.kpf');
        url = file.url;
    }
    return gProjectManager.newProject(url) != null;
}

function ProjectSaveAs(projectId)
{
    if (projectId < 0) return false;

    var project = gProjectManager.getProjectById(projectId);

    var fileObj = URLToLocalFile(project.url);
    var leafName = fileObj.leafName;
    var fullName = fileObj.path;

    //Don't include the trailing slash.
    var dirName = fullName.substring(0, fullName.length - leafName.length - 1);

    var fp = dlg_fileSelectViaDialog(Components.interfaces.nsIFilePicker.modeSave,
                             "Save Project As", dirName,
                             leafName, "Komodo Project File", "*.kpf");
    if (!fp) {
        return false;
    }
    var file = new dlg_get_URL_andName(fp);
    filewinfuncs_forceExtension(file, '.kpf');

    if (gProjectManager.getProjectByURL(file.url) != null) {
        alertDialog("Sorry, but project " + file.url + " is already loaded.");
        return false;
    }

    project.url = project.part.url = file.url;
    project.name = project.part.name = file.leafName;
    try {
        project.part.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alertDialog('There was an error saving project "' +
                    project.name + '": ' +
                    lastErrorSvc.getLastErrorMessage());
        return false;
    }
  
    // Update the MRU projects list.
    MRU_addURL("mruProjectList", file.url);
    koRequestUpdateCommands('projectsChanged');
    try {
        gObserverSvc.Notify(this,'file_changed', project.url);
    } catch(e) { /* exception if no listeners */ }
    return true;
}

var gFileStatusSvc = null;
function fileStatusServiceOnLoad()
{
    // start the file status service
    gFileStatusSvc = Components.classes["@activestate.com/koFileStatusService;1"].getService(Components.interfaces.koIFileStatusService);
    gFileStatusSvc.init();
}

function projectManagerOnLoad()
{
    fileStatusServiceOnLoad();
    extensionManager = new projectExtensionManager();
    extensionManager.init();
    gProjectManager = new projectManager();
}

function projectManagerCanUnload()
{
    return true;
}

function projectManagerOnUnload()
{
    projectManagerCanUnload();
    gFileStatusSvc.unload();
}


function ProjectEventObserver() {
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
          getService(Components.interfaces.nsIObserverService);
    observerSvc.AddObserver(this, 'guiBuilder-save');
    observerSvc.AddObserver(this, 'guiBuilder-startup');
    observerSvc.AddObserver(this, 'guiBuilder-edit');
    observerSvc.AddObserver(this, 'guiBuilder-help');
};

ProjectEventObserver.prototype = {
    Observe: function (view, topic, message) {
        var projecturl;
        var folderurl;
        var newfileurl;
        var uifiles;
        var parts = message.split('\t');
        var i, j, list;
        if (parts.length != 2) {
            return;
        }
        switch (topic) {
            case 'guiBuilder-save':
                uifile = parts[0];
                newfileurl = parts[1];
                if (uifile != newfileurl) {
                    if (findItemsByURL(newfileurl, 'dialog').length == 0) {
                        // We haven't already added this file somewhere.
                        var dialog, filepart;
                        uifiles = findItemsByURL(uifile);
                        for (i = 0; i < uifiles.length; i++) {
                            dialog = uifiles[i];
                            filepart = Components.classes["@activestate.com/koPart?type=file;1"]
                                                            .createInstance(Components.interfaces.koIPart_file);
                            filepart.setStringAttribute('url', newfileurl);
                            dialog.part.addChild(filepart);
                            dialog.updateWrappedPart();
                        }
                    }
                }
                var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                      getService(Components.interfaces.nsIObserverService);
                observerSvc.Notify(this,'file_changed', newfileurl);
                break;
            case 'guiBuilder-startup':
                uifile = parts[0];
                newfileurl = parts[1];
                uifiles = findItemsByURL(uifile);
                for (i = 0; i < uifiles.length; i++) {
                    uifiles[i].part.setStringAttribute('startup', newfileurl);
                }
                break;
            case 'guiBuilder-edit':
                uifile = parts[0];
                newfileurl = parts[1];
                doFileOpen(newfileurl)
                break;
            case 'guiBuilder-help':
                url = parts[1];
                OpenUrlInDefaultBrowser(url);
                break;
        }
    },

    QueryInterface: function (iid) {
        if (!iid.equals(nsIObserver))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
};


function projectCheckFocus(menuEl)
{
    //gFocusedProjectView = gProjectManager.viewMgr.view;
   // return true;

    var child = null;
    if (gFocusedProjectView != gProjectManager.viewMgr.view) {
        // disable children
        child = menuEl.firstChild;
        while (child) {
            child.setAttribute('disabled',true);
            child = child.nextSibling;
        }
    } else {
        // enable children
        child = menuEl.firstChild;
        while (child) {
            child.removeAttribute('disabled');
            child = child.nextSibling;
        }
    }
    return true;
}

function projectAddUpdateMainMenu(menuEl)
{
    extensionManager.updateMenuForInterface(Components.interfaces.koIContainer, menuEl);
    var children = menuEl.childNodes;
    for (i = 0; i < children.length; i++) {
        var cmd = children[i].getAttribute('command');
        children[i].setAttribute('command',cmd+'_Project');
    }
    return true;
}

function haveFocusedProjectManager() {
    var focusedElement = top.document.commandDispatcher.focusedElement;
    if (focusedElement) {
        return focusedElement.tagName=="ProjectViewer";
    }
    return false;
}



