/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * A Command Part properties dialog
 *
 * Used for showing/editing properties of an existing command part or for
 * creating a new one.
 */

//---- globals

var gDlg = null;    // Object to hold all XUL element references.
// "add" | "properties", indicates whether this is dialog is being used to add
// a new command or edit the properties for an existing one.
var gMode = null;
var gPart = null;
// An "item" (a.k.a. a wrapper around the part) *may* be passed in. In which
// case any part name changes should be written through to the item.
var gItem = null;
var gNameTracksCommand = false;
var gCache = {};    // Cache UI values.
var gPrefSvc = null;


//---- interface routines for commandproperties.xul

function OnLoad()
{
    // A command part is expected as the first window argument.
    gPart = window.arguments[0].part;
    if ("item" in window.arguments[0]) {
        gItem = window.arguments[0].item;
    }

    gPrefSvc = Components.classes["@activestate.com/koPrefService;1"].
               getService(Components.interfaces.koIPrefService);

    gDlg = {};
    gDlg.window = document.getElementById("command-properties-window");
    gDlg.tabs = document.getElementById("tabs");
    gDlg.shortcut_tab = document.getElementById("shortcut_tab");
    gDlg.command_tab = document.getElementById("command_tab");
    gDlg.commandTextbox = document.getElementById("command-textbox");
    gDlg.nameTextbox = document.getElementById("name-textbox");
    gDlg.nameLabel  = document.getElementById("name-label");
    gDlg.opOnSelWidget = document.getElementById("operate-on-selection-checkbox");
    gDlg.insertOutputWidget = document.getElementById("insert-output-checkbox");
    gDlg.doNotOpenOutWinWidget = document.getElementById("do-not-open-output-window-checkbox");
    gDlg.runInMenulist = document.getElementById("run-in-menulist");
    gDlg.cwdTextbox = document.getElementById("cwd-textbox");
    gDlg.okButton = document.getElementById('ok');
    gDlg.cancelButton = document.getElementById('cancel');
    gDlg.applyButton = document.getElementById('apply-button');
    gDlg.keybinding = document.getElementById('keybindings');
    gDlg.keybinding.gKeybindingMgr = opener.gKeybindingMgr;
    gDlg.keybinding.part = gPart;
    gDlg.tabs.selectedTab = gDlg.command_tab;  // in some circumstances we may want to change this.
    
    koDoSetOKCancel(OK, Cancel);
    Env_OnLoad();

    // Load the UI fields.
    if (gPart.value) {
        gMode = "properties";
        gDlg.commandTextbox.setAttribute("value", gPart.value);
        LoadOptionsFromPart(gPart);
    } else {
        gMode = "add";
        LoadOptionsFromPrefs();
    }
    UpdateField("command", true);
    if (gPart.hasAttribute("name")) {
        var name = gPart.getStringAttribute("name");
        gDlg.nameTextbox.setAttribute("value", name);
        gDlg.nameLabel.setAttribute("value", name);
        if (gMode == "properties") {
            window.title = "'"+name+"' Properties";
            gDlg.keybinding.commandParam = name;
            if (name == gPart.value) {
                gNameTracksCommand = true;
            }
        } else {
            gNameTracksCommand = true;
            window.title = "Add Command";
        }
        UpdateField("name", true);
    } else {
        gNameTracksCommand = true;
    }
 
    gDlg.keybinding.init();
    gDlg.keybinding.updateCurrentKey();

    if (! gPart.value) { // then we are "adding" rather than "editting"
        gDlg.applyButton.setAttribute("collapsed", "true");
    }
    gDlg.commandTextbox.focus();
    SetWindowLocation();
}


function OnUnload()
{
    SaveWindowLocation();
    opener.focus();
}


function LoadOptionsFromPrefs()
{
    gDlg.cwdTextbox.value = gPrefSvc.prefs.getStringPref("runCwd");
    UpdateField("cwd", true);

    var runIn = gPrefSvc.prefs.getStringPref("runRunIn");
    var menuitems = gDlg.runInMenulist.firstChild.childNodes;
    for (var i=0; i < menuitems.length; i++) {
        var menuitem = menuitems[i];
        if (menuitem.getAttribute("value") == runIn) {
            gDlg.runInMenulist.selectedIndex = i;
        }
    }
    gDlg.doNotOpenOutWinWidget.checked =
        gPrefSvc.prefs.getBooleanPref("runDoNotOpenOutWin");
    UpdateField("runIn", true);

    var env = gPrefSvc.prefs.getStringPref("runEnv");
    gEnvView.SetEnvironmentStrings(env);
    UpdateField("env", true);
}


function LoadOptionsFromPart(part)
{
    if (part.hasAttribute("cwd")) {
        gDlg.cwdTextbox.setAttribute("value",
            part.getStringAttribute("cwd"));
        UpdateField("cwd", true);
    }
    if (part.hasAttribute("env")) {
        var env = unescapeWhitespace(part.getStringAttribute("env"), '\n');
        gEnvView.SetEnvironmentStrings(env);
        UpdateField("env", true);
    }
    if (part.hasAttribute("insertOutput")) {
        gDlg.insertOutputWidget.setAttribute("checked",
            part.getBooleanAttribute("insertOutput"));
        UpdateField("insert-output", true);
    }
    if (part.hasAttribute("operateOnSelection")) {
        gDlg.opOnSelWidget.setAttribute("checked",
            part.getBooleanAttribute("operateOnSelection"));
        UpdateField("operate-on-selection", true);
    }
    if (part.hasAttribute("doNotOpenOutputWindow")) {
        gDlg.doNotOpenOutWinWidget.setAttribute("checked",
            part.getBooleanAttribute("doNotOpenOutputWindow"));
        UpdateField("do-not-open-output-window", true);
    }
    if (part.hasAttribute("runIn")) {
        var runIn = part.getStringAttribute("runIn");
        var menuitems = gDlg.runInMenulist.firstChild.childNodes;
        for (var i=0; i < menuitems.length; i++) {
            var menuitem = menuitems[i];
            //XXX Beware, this might not be getting the actual value. May need
            //    menuitem.value instead.
            if (menuitem.getAttribute("value") == runIn) {
                gDlg.runInMenulist.selectedIndex = i;
            }
        }
        UpdateField("runIn", true);
    }
}


function ToggleCheck(name)
{
    var id = name + "-checkbox";
    var widget = document.getElementById(id);
    widget.checked = !widget.checked;
    UpdateField(name);
}


// Do the proper UI updates for a user change.
//  "field" (string) indicates the field to update.
//  "initializing" (boolean, optional) indicates that the dialog is still
//      initializing so some updates, e.g. enabling the <Apply> button, should
//      not be done.
function UpdateField(field, initializing /* =null */)
{
    // Only take action if there was an actual change. Otherwise things like
    // the <Alt-A> shortcut when in a textbox will cause a cycle in reenabling
    // the apply button.
    var changed = false;

    switch (field) {
    case "name":
        var name = gDlg.nameTextbox.value;
        if (!(field in gCache) || name != gCache[field]) {
            changed = true;
            gCache[field] = name;
            if (gMode == "properties") {
                window.title = "'"+name+"' Properties";
            }
            if (!initializing && name != gDlg.commandTextbox.value) {
                gNameTracksCommand = false;
            }
            gDlg.nameLabel.value = name;
        }
        break;

    case "command":
        var command = gDlg.commandTextbox.value;
        if (!(field in gCache) || command != gCache[field]) {
            changed = true;
            gCache[field] = command;
            if (gNameTracksCommand) {
                gDlg.nameTextbox.setAttribute("value", command);
                gDlg.nameLabel.setAttribute("value", command);
                UpdateField("name", initializing);
            }
        }
        if (command) {
            gDlg.okButton.setAttribute("disabled", "false");
        } else {
            gDlg.okButton.setAttribute("disabled", "true");
        }
        break;

    case "runIn":
        var runIn = gDlg.runInMenulist.value;
        if (!(field in gCache) || runIn != gCache[field]) {
            changed = true;
            gCache[field] = runIn;
        }
        if (runIn == "command-output-window") {
            gDlg.doNotOpenOutWinWidget.setAttribute("disabled", "false");
        } else {
            gDlg.doNotOpenOutWinWidget.setAttribute("disabled", "true");
        }
        break;

    case "insert-output":
        var insertOutput = gDlg.insertOutputWidget.checked;
        if (!(field in gCache) || insertOutput != gCache[field]) {
            changed = true;
            gCache[field] = insertOutput;
            if (gDlg.insertOutputWidget.checked) {
                gDlg.runInMenulist.setAttribute("disabled", "true");
                gDlg.doNotOpenOutWinWidget.setAttribute("disabled", "true");
            } else {
                gDlg.runInMenulist.setAttribute("disabled", "false");
                UpdateField("runIn", initializing);
            }
        }
        break;
    case "operate-on-selection":
        var operateOnSelection = gDlg.opOnSelWidget.checked;
        if (!(field in gCache) || operateOnSelection != gCache[field]) {
            changed = true;
            gCache[field] = operateOnSelection;
        }
        break;
    case "cwd":
        var cwd = gDlg.cwdTextbox.value;
        if (!(field in gCache) || cwd != gCache[field]) {
            changed = true;
            gCache[field] = cwd;
        }
        break;
    case "env":
        var env = gEnvView.GetEnvironmentStrings();
        if (!(field in gCache) || env != gCache[field]) {
            changed = true;
            gCache[field] = env;
        }
        break;
    case "do-not-open-output-window":
        var doNotOpenOutputWindow = gDlg.doNotOpenOutWinWidget.checked;
        if (!(field in gCache) || doNotOpenOutputWindow != gCache[field]) {
            changed = true;
            gCache[field] = doNotOpenOutputWindow;
        }
        break;
    case "shortcut":
        changed = true;
        break;
    }
    
    if (!initializing && changed) {
        gDlg.applyButton.setAttribute("disabled", "false");
    }
}


function OK()
{
    // Validate.
    if (! gDlg.commandTextbox.value) {
        alertDialog("You must specify a command string.");
        gDlg.commandTextbox.focus();
        return false;
    }

    // Apply changes and exit.
    if (!Apply()) {
        return false;
    }
    window.arguments[0].retval = "OK";
    opener.focus(); // must explicitly ensure that opener comes to foreground
    window.close();
    return true;
}


function Apply()
{
    // The keybinding needs the commandparam to be set before the application would ever work.
    gDlg.keybinding.commandParam = gDlg.nameTextbox.value;
    var retval = gDlg.keybinding.apply(); // This may return false if a keybinding is partially entered
    if (!retval) return retval;
    
    if (gDlg.applyButton.hasAttribute('disabled') && gDlg.applyButton.getAttribute("disabled")=='true') return true;

    // Update the command part with info from the UI.
    gPart.value = gDlg.commandTextbox.value;
    gPart.setStringAttribute('name', gDlg.nameTextbox.value);
    gPart.setStringAttribute('cwd', gDlg.cwdTextbox.value);
    var env = gEnvView.GetEnvironmentStrings();
    gPart.setStringAttribute('env', escapeWhitespace(env));
    gPart.setBooleanAttribute('insertOutput', gDlg.insertOutputWidget.checked);
    gPart.setBooleanAttribute('operateOnSelection', gDlg.opOnSelWidget.checked);
    gPart.setBooleanAttribute('doNotOpenOutputWindow', gDlg.doNotOpenOutWinWidget.checked);
    gPart.setStringAttribute('runIn', gDlg.runInMenulist.value);
    
    // Update the wrapper around the part (item) if it was passed in.
    if (gItem) {
        gItem.name = gPart.getStringAttribute("name");
        opener.invalidateItem(gItem);
    }

    gDlg.applyButton.setAttribute("disabled", "true");
    return retval;
}


function Cancel()
{
    window.arguments[0].retval = "Cancel";
    window.close();
}


// Get the shortcut string from the menuitem widget and append it to
// the current command string.
// XXX Should really insert and the current insertion point.
function InsertShortcut(shortcutItemWidget)
{
    var shortcut = shortcutItemWidget.getAttribute("shortcut");
    var commandTextbox = document.getElementById("command-textbox");
    var currentCommand = commandTextbox.value;
    //XXX Should add the shortcut at the current insertion point in the textbox
    //    and replace the selection if there is one.
    commandTextbox.setAttribute("value", currentCommand+shortcut);

    UpdateField('command');
}


// Browse for a directory for the start dir.
function BrowseForCwd()
{
    // from window-functions.js
    var cwd = BrowseForDir(gDlg.cwdTextbox.value);
    if (cwd) {
        gDlg.cwdTextbox.value = cwd;
        UpdateField('cwd');
    }
}



//---- The methods for the environment outliner area.

function _UpdateSortIndicators(sortId)
{
    var sortedColumn = null;

    // Set the sort indicator on the column we are sorted by.
    if (sortId) {
        sortedColumn = document.getElementById(sortId);
        if (sortedColumn) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // Remove the sort indicator from all the columns except the one we are
    // sorted by.
    var currCol = document.getElementById("env").firstChild;
    while (currCol) {
        while (currCol && currCol.localName != "outlinercol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


function Env_OnLoad()
{
    // Should be called in the onload handler for the window containing the
    // find results tab XUL.
    gEnvView = Components.classes['@activestate.com/koRunEnvView;1']
        .createInstance(Components.interfaces.koIRunEnvView);
    var envOutliner = document.getElementById("env");
    envOutliner.boxObject
               .QueryInterface(Components.interfaces.nsIOutlinerBoxObject)
               .view = gEnvView;
}


function Env_OnClick(event)
{
    // c.f. mozilla/mailnews/base/resources/content/threadPane.js
    var t = event.originalTarget;
    var index = gEnvView.selection.currentIndex;
    var rowCount = gEnvView.rowCount;
    //dump("Env_OnClick(): start, index="+index+", rowCount="+rowCount+"\n");

    // single-click on a column
    if (t.localName == "outlinercol") {
        //dump("Env_OnClick: single-click on a column...");
        if (rowCount > 0) {
            //dump("run sort\n");
            Env_Sort(t.id);
        } else {
            //dump("ignored\n");
        }
    }

    // double-click in the outliner body
    else if (event.detail == 2 && t.localName == "outlinerbody") {
        //dump("Env_OnClick: double-click in the outliner body...");
        if (0 <= index && index < rowCount) {
            //dump("run edit\n");
            Env_Edit();
        } else {
            //dump("ignored\n");
        }
    }

    // single-click in the outliner body
    else if (event.detail == 1 && t.localName == "outlinerbody") {
        //dump("Env_OnClick: single-click in the outliner body...");
        if (0 <= index && index < rowCount) {
            //dump("enable buttons\n");
            document.getElementById("env-edit-button").disabled = false;
            document.getElementById("env-delete-button").disabled = false;
        } else {
            //dump("ignored\n");
        }
    }
}


function Env_Sort(columnID)
{
    gEnvView.Sort(columnID);
    _UpdateSortIndicators(columnID);
}


function Env_OnKeyPress(event)
{
    var index = gEnvView.selection.currentIndex;
    var rowCount = gEnvView.rowCount;
    //dump("Env_OnKeyPress(): start, index="+index+", rowCount="+rowCount+
    //     ", keyCode="+event.keyCode+"\n");

    if (event.keyCode == 13 /* <Enter> */) {
        //dump("Env_OnKeyPress(): <Enter> key...");
        if (0 <= index && index < rowCount) {
            //dump("run edit and cancel bubble\n");
            Env_Edit();
            event.cancelBubble = true;
        } else {
            //dump("ignored\n");
        }
    }
    return true;
}


// Add a new environment variable.
function Env_New()
{
    // Disable the other buttons and unselect any current variable while the
    // edit gDlg is up.
    document.getElementById("env").currentIndex = -1;
    document.getElementById("env-edit-button").disabled = true;
    document.getElementById("env-delete-button").disabled = true;

    var obj = new Object();
    obj.name = "";
    obj.value = "";
    window.openDialog("chrome://komodo/content/pref/editenvironvar.xul",
                      "Environment Variable",
                      "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                      obj);
    if (obj.res == "cancel") { // Cancelled.
        return;
    }
    var addIt = "Yes";
    if (gEnvView.Have(obj.name)) {
        addIt = yesNoDialog("There is already a value for '" +
                            obj.name + "'. Overwrite it?", "no");
    }
    if (addIt == "Yes") {
        gEnvView.Set(obj.name, obj.value);

        // Select the new variable.
        var index = gEnvView.Index(obj.name);
        gEnvView.selection.currentIndex = index;
        gEnvView.selection.select(index);
        document.getElementById("env-edit-button").disabled = false;
        document.getElementById("env-delete-button").disabled = false;

        UpdateField('env');
    }
}


// Edit the currently selected environment variable row.
function Env_Edit()
{
    var index = document.getElementById("env").currentIndex;
    var rowCount = gEnvView.rowCount;
    if (index < 0 || rowCount <= index) {
        return;
    }

    var obj = new Object();
    obj.name = gEnvView.GetVariable(index);
    obj.value = gEnvView.GetValue(index);
    window.openDialog("chrome://komodo/content/pref/editenvironvar.xul",
                      "Environment Variable",
                      "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                      obj);
    if (obj.res == "cancel") { // Cancelled.
        return;
    }
    gEnvView.Set(obj.name, obj.value);

    // Select the editted variable.
    index = gEnvView.Index(obj.name);
    gEnvView.selection.currentIndex = index;
    gEnvView.selection.select(index);
    document.getElementById("env-edit-button").disabled = false;
    document.getElementById("env-delete-button").disabled = false;

    UpdateField('env');
}


// Delete the currently selected environment variable row.
function Env_Delete()
{
    var index = document.getElementById("env").currentIndex;
    var rowCount = gEnvView.rowCount;
    if (index < 0 || rowCount <= index) {
        return;
    }

    gEnvView.Delete(gEnvView.selection.currentIndex);

    // Select the previous variable.
    index = gEnvView.selection.currentIndex;
    rowCount = gEnvView.rowCount;
    if (rowCount > 0 && index >= rowCount) {
        index = rowCount - 1;
        gEnvView.selection.currentIndex = index;
        gEnvView.selection.select(index);
    }

    if (index >= 0 && index < rowCount) {
        document.getElementById("env-edit-button").disabled = false;
        document.getElementById("env-delete-button").disabled = false;
    } else {
        document.getElementById("env-edit-button").disabled = true;
        document.getElementById("env-delete-button").disabled = true;
    }

    UpdateField('env');
}

