/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

// -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
//
// Assitance for Komodo's Run Command feature set.
//

//---- globals

var _gRunSvc = Components.classes["@activestate.com/koRunService;1"]
               .getService(Components.interfaces.koIRunService);



//---- internal utility routines

function _IsWordCharacter(ch)
{
    return /^\w/.test(ch);
}


function _Run_UpdateMRU(command, cwd, env, insertOutput, operateOnSelection,
                        doNotOpenOutputWindow, runIn)
{
    var encodedCommand = _gRunSvc.Encode(command, cwd, env, insertOutput,
                                         operateOnSelection,
                                         doNotOpenOutputWindow, runIn);
    MRU_add("run-commandMru", encodedCommand, true /* case-sensitive */);
}


// Keep a list of processes spawned by the RunCommand system. This is necessary
// to ensure that there are no hangs when Komodo closes because some process is
// still running. Before closing Komodo all process must be terminated.
function _Run_RegisterProcess(editor, command, process)
{
    //dump("_Run_RegisterProcess(command='"+command+"', process='"+
    //     process+"')\n");
    
    // XXX There is some timing issue with process termination. For quick
    //     commands (e.g. 'echo hi'), the process may be a zombie before it
    //     gets here. We MUST, however, maintain a reference to it, at least
    //     until the next time around because some related things need to clean
    //     up before the ProcessProxy object destructs. Otherwise Komodo will
    //     hang. Don't know the details.
    _Run_UnregisterZombieProcess(editor);

    var p = {};
    p['command'] = command;
    p['process'] = process;
    editor.gRunProcesses[editor.gRunProcesses.length] = p;
}


// Remove any processes that have terminated from the list.
function _Run_UnregisterZombieProcess(editor)
{
    var newList = [];
    for (var i=0; i < editor.gRunProcesses.length; i++) {
        var p = editor.gRunProcesses[i]['process'];
        try {
            p.wait(0);
            //dump("_Run_UnregisterZombieProcess: zombie: index="+i+
            //     ", command='"+editor.gRunProcesses[i]['command']+"'\n");
        } catch (ex) {
            // XXX Could getLastError here to determine if the wait failed
            //     because of WAIT_TIMEOUT or WAIT_FAILED. Should probably not
            //     ignore the latter, but punting on that for now.
            newList[newList.length] = editor.gRunProcesses[i];
        }
    }
    editor.gRunProcesses = newList;
}



//---- support classes

// Subclass the MRUAutoCompleteSession in mru.js to handle the special encoding
// of the command that we do to inline the run options.
function RunMRUAutoCompleteSession(widget) {
    MRUAutoCompleteSession.apply(this, [widget]);
};

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
RunMRUAutoCompleteSession.prototype = new MRUAutoCompleteSession();
RunMRUAutoCompleteSession.prototype.constructor = RunMRUAutoCompleteSession;

RunMRUAutoCompleteSession.prototype.decodeMRUString = function(s) {
    var commandObj = new Object();
    var cwdObj = new Object();
    var envObj = new Object();
    var insertOutputObj = new Object();
    var operateOnSelectionObj = new Object();
    var doNotOpenOutputWindowObj = new Object();
    var runInObj = new Object();
    _gRunSvc.Decode(s, commandObj, cwdObj, envObj,
                    insertOutputObj, operateOnSelectionObj,
                    doNotOpenOutputWindowObj, runInObj);

    return commandObj.value;
}


// koIRunTerminationListener implementation whose only job is to call
// notify on child termination.
// Note: This is psychotic how much code you need to do this. Do I have
//       to have a separate file for this? See
//       http://lxr.mozilla.org/seamonkey/source/xpcom/sample/nsSample.js
function _koRunTerminationListener() { }
_koRunTerminationListener.prototype = {
    init: function (editor, command) {
        this._editor = editor;
        this._command = command;
    },
    onTerminate: function (retval) {
        //dump("_koRunTerminationListener::onTerminate(retval="+retval+")\n");
        this._editor.RunOutput_EndSession(retval);
        var msg = "'" + this._command + "' returned " + retval + ".";
        this._editor.StatusBar_AddMessage(msg, "run_command", 3000,
                                          retval ? 1 : 0);
    },
    QueryInterface: function (iid) {
        if (!iid.equals(Components.interfaces.koIRunTerminationListener) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }
}

var _koRunTerminationListenerModule = {
    myCID: Components.ID("{6EB8049A-ED64-4959-8D6E-0ED16C3BF9EA}"),
    myProgID: "@activestate.com/koIRunTerminationListener;1",

    firstTime: true,
    registerSelf: function (compMgr, fileSpec, location, type) {
        if (this.firstTime) {
            this.firstTime = false;
            throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
        }
        compMgr = compMgr.QueryInterface(
            Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.myCID,
                                        "Sample JS Component",
                                        this.myProgID, 
                                        fileSpec,
                                        location,
                                        type);
    },
    
    getClassObject: function (compMgr, cid, iid) {
        if (!cid.equals(this.myCID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return this.myFactory;
    },

    myFactory: {
        createInstance: function (outer, iid) {
            if (outer != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            return (new _koRunTerminationListener()).QueryInterface(iid);
        }
    },

    canUnload: function(compMgr) {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec) {
    return _koRunTerminationListenerModule;
}



//---- public interface

function Run_BuildRecentCommandsMenu(popupWidget)
{
    var globalPrefs = Components.classes["@activestate.com/koPrefService;1"]
                      .getService(Components.interfaces.koIPrefService).prefs;

    var mruPrefName = "run-commandMru";
    var mruList = globalPrefs.hasPref(mruPrefName) ?
                  globalPrefs.getPref(mruPrefName) : null;

    // clear the current list
    while (popupWidget.childNodes.length != 0)  {
        var child = popupWidget.firstChild;
        popupWidget.removeChild(child);
    }    

    // if the MRU is empty then say so
    var itemWidget = null;
    if (!mruList || mruList.length == 0) {
        itemWidget = document.createElement("menuitem");
        itemWidget.setAttribute("label", "(Empty)");
        itemWidget.setAttribute("disabled", "true");
        popupWidget.appendChild(itemWidget);
    }
    
    // otherwise, populate the menupopup
    else {
        for(var i = 0; i < mruList.length; i++) {
            itemWidget = document.createElement("menuitem");

            // special case: if there are no options then strip the leading
            // "[] " for display to the user (XXX this presumes knowledge of
            // the encoding scheme)
            var encodedCommand = mruList.getStringPref(i);
            var displayCommand = encodedCommand;
            if (encodedCommand.substring(0,3) == "[] ") {
                displayCommand = encodedCommand.substring(3,
                                   encodedCommand.length);
            }
            itemWidget.setAttribute("label", (i+1) + " " + displayCommand);

            // escape char that will get interpreter twice (XXX not confident
            // that this covers all cases)
            encodedCommand = encodedCommand.replace('\\', '\\\\', "g");
            encodedCommand = encodedCommand.replace("'", "\\'", "g");
            encodedCommand = encodedCommand.replace('"', '\\"', "g");
            var handler = "Run_RunEncodedCommand(window, '"
                          + encodedCommand + "');";
            itemWidget.setAttribute("oncommand", handler);

            if (i <= 8) {
                itemWidget.setAttribute("accesskey", ""+(i+1));
            } else if (i == 9) {
                itemWidget.setAttribute("accesskey", "0");
            }

            popupWidget.appendChild(itemWidget);
        }
    }
}


function Run_RunEncodedCommand(editor, encodedCommand)
{
    // decode the command string and then run it
    var commandObj = new Object();
    var cwdObj = new Object();
    var envObj = new Object();
    var insertOutputObj = new Object();
    var operateOnSelectionObj = new Object();
    var doNotOpenOutputWindowObj = new Object();
    var runInObj = new Object();
    _gRunSvc.Decode(encodedCommand, commandObj, cwdObj, envObj,
                    insertOutputObj, operateOnSelectionObj,
                    doNotOpenOutputWindowObj, runInObj);

    return Run_RunCommand(editor, commandObj.value, cwdObj.value, envObj.value,
                          insertOutputObj.value, operateOnSelectionObj.value,
                          doNotOpenOutputWindowObj.value, runInObj.value);
}


// Run the given command.
//  "editor" is the main editor window JS object.
//  "command" is the command string to spawn.
//  "cwd" is the directory to start the child process in (null to inherit from
//      the parent).
//  "env" is an encoded string of environment changes to make for the child
//      (null to inherit from the parent). Encoding scheme:
//              "VAR1=value1\nVAR2=value2\n..."
//  "insertOutput" (boolean) specifies whether to insert the command's output
//      into the current insertion point.
//  "operateOnSelection" (boolean) specifies whether to use the current
//      selection as input to the given command.
//  "doNotOpenOutputWindow" (boolean) specifies that the Command Output window
//      should *not* be opened.
//  "runIn" (string) indentifies where the command should be run, e.g. in a new
//      console window, in the command output pane (this is only meaningful if
//      the ocmmand will be run interactively). See the koIPart_command IDL for
//      a description of the exact strings to use.
//
// Returns true iff the command was launched successfully.
//
function Run_RunCommand(editor, command, cwd, env, insertOutput,
                        operateOnSelection, doNotOpenOutputWindow, runIn)
{
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"]
                       .getService(Components.interfaces.koILastErrorService);

    var view = editor.gLastFocusedViewManager.getCurrentView();
    var scimoz = null;
    if (view) {
        scimoz = view._scimoz;
    }
    
    // Ensure that it is reasonable to run this command given the current state
    // of Komodo.
    if (operateOnSelection) {
        if (!view) {
            alertDialog("Cannot 'operate on selection' because there is " +
                        "no current file.");
            return false;
        } else if (scimoz.selText == "") {
            alertDialog("Cannot 'operate on selection' because the current " +
                        "file does not have a selection.");
            return false;
        }
    }
    if (insertOutput && !view) {
        alertDialog("Cannot 'insert output' because there is no current " +
                    "file in which to insert.");
        return false;
    }

    // Interpolate the command string.
    var filename = null;
    if (view) {
        var url = view.url;
        filename = displayNameFromURL(url);
    }
    var word = null;
    if (scimoz) {
        var selection = scimoz.selText;
        // Parse out the current word under the cursor:
        //  The original code used scimoz.getLine, which appears completely
        //  broken.
        var origSelStart, origSelEnd;
        origSelStart = scimoz.selectionStart;
        origSelEnd = scimoz.selectionEnd;
        scimoz.wordLeftExtend();
        var wordBegin = scimoz.selectionStart;
        scimoz.wordRightExtend();
        var wordEnd = scimoz.selectionEnd; // Need to trim whitespace from this.
        scimoz.selectionStart = wordBegin;
        word = scimoz.selText;
        scimoz.selectionStart = origSelStart;
        scimoz.selectionEnd = origSelEnd;
        var endPos = 0;
        while (_IsWordCharacter(word.charAt(endPos))) {
            endPos++;
        }
        word = word.substr(0, endPos);
    }
    try {
        var interpolated = _gRunSvc.Interpolate(command, filename, word,
                                                selection);
    } catch (e) {
        alertDialog("Error running command: " + lastErrorSvc.getLastErrorMessage());
        return false;
    }

    // Run the command string.
    var retval = null;
    var termListener = null;
    var process = null;
    var input = null;
    var output = null;
    var error = null;
    var errmsg = null;
    if (operateOnSelection && view && scimoz.selText != "") {
        input = scimoz.selText;
    }

    if (insertOutput) {
        // Start the command and monitor it. If the process is taking a long
        // time to complete then pop up a dialog to wait for its termination
        // giving the user the option to abort the command by killing the
        // process.
        //dump("XXX RunAndNotify: about to start '"+interpolated+"'\n");
        try {
            process = _gRunSvc.RunAndNotify(interpolated, cwd, env, input);
        } catch (ex) {
            //dump("XXX RunAndNotify: '"+interpolated+"' failed to start\n");
            errmsg = lastErrorSvc.getLastErrorMessage();
            if (! errmsg) {
                errmsg = "Unknown error running: '"+interpolated+"'";
            }
            alertDialog(errmsg);
            return false;
        }
        //dump("XXX RunAndNotify: '"+interpolated+"' started normally\n");

        var cancelled = false;
        try {
            process.wait(2);  // Give the command a little bit of time to run.
            //dump("XXX RunAndNotify: child finished within 2s\n");
        } catch (ex) {
            // XXX Could getLastError here to determine if the wait failed
            //     because of WAIT_TIMEOUT or WAIT_FAILED. Should probably not
            //     ignore the latter, but punting on that for now.
            //dump("XXX RunAndNotify: 2s passed, child has NOT finished\n");
            var obj = new Object();
            obj.process = process;
            obj.command = command;
            openWindowUniqueInstance(
                "chrome://komodo/content/run/waitfortermination.xul", 
                "komodo_waitfortermination",
                "chrome,close=no,modal=yes", obj);
            if (obj.retval == "killed") { // Cancelled.
                cancelled = true;
            }
        }

        if (! cancelled) {
            //dump("XXX RunAndNotify: get output and error\n");
            output = process.readStdout();
            error = process.readStderr();
            //dump("XXX RunAndNotify: finished getting output and error\n");
        }

    } else {
        if (runIn == "command-output-window") {
            try {
                editor.RunOutput_StartSession(interpolated);
            } catch (ex) {
                alertDialog(ex);
                return false;
            }
            termListener = new _koRunTerminationListener();
            termListener.init(editor, interpolated);
            try {
                process = _gRunSvc.RunInTerminal(interpolated, cwd, env,
                    editor.RunOutput_GetTerminal(), termListener, input);
            } catch (ex) {
                errmsg = lastErrorSvc.getLastErrorMessage();
                if (! errmsg) {
                    errmsg = "Unknown error running: '"+interpolated+"'";
                }
                alertDialog(errmsg);
                editor.RunOutput_EndSession(-1);
                return false;
            }
            _Run_RegisterProcess(editor, interpolated, process);
            editor.RunOutput_SetProcessHandle(process);
            if (! doNotOpenOutputWindow) {
                editor.RunOutput_Show(editor);
            }
        } else if (runIn == "new-console") {
            try {
                _gRunSvc.Run(interpolated, cwd, env, true, input);
            } catch (ex) {
                errmsg = lastErrorSvc.getLastErrorMessage();
                if (! errmsg) {
                    errmsg = "Unknown error running: '"+interpolated+"'";
                }
                alertDialog(errmsg);
                return false;
            }
        } else if (runIn == "no-console") {
            try {
                _gRunSvc.Run(interpolated, cwd, env, false, input);
            } catch (ex) {
                errmsg = lastErrorSvc.getLastErrorMessage();
                if (! errmsg) {
                    errmsg = "Unknown error running: '"+interpolated+"'";
                }
                alertDialog(errmsg);
                return false;
            }
        } else {
            throw("Unexpected 'runIn' value: " + runIn + "\n");
        }
    }
    _Run_UpdateMRU(command, cwd, env, insertOutput, operateOnSelection,
                   doNotOpenOutputWindow, runIn);

    // Put focus back on previous view. SciMoz's undo action stuff requires
    // this and even if not inserting output, it is nice to have focus back in
    // the file being editted.
    if (view) {
        view.manager.setFocus();
    }

    // Insert the returned output if requested and select it.
    if (insertOutput && view && output) {
        scimoz.beginUndoAction();
        scimoz.hideSelection(true);
        var selStart = scimoz.selectionStart;
        scimoz.replaceSel(output);
        scimoz.selectionStart = selStart;
        scimoz.selectionEnd = selStart + output.length;
        scimoz.hideSelection(false);
        scimoz.endUndoAction();
    }

    // Raise an alert dialog if there was error output.
    if (error) {
        alertDialog("The command returned the following error output:", true,
                    false, null, null, error);
    }
    return true;
}


// Prepare to close Komodo. Return false if cannot yet close Komodo.
// If there are processes still running then must wait for or kill them.
// Note: 'gRunProcesses' is defined in komodo.js, therefore this method must
//       only be called from the komodo.xul window.
function Run_CanClose()
{
    var i;
    // If there are any still running processes, then ask the user if they want
    // to kill those processes (call kill on them and return true) or cancel
    // and wait for those processes to finish normally (return false).
    _Run_UnregisterZombieProcess(window);
    if (gRunProcesses.length > 0) {
        var commands = "";
        for (i=0; i < gRunProcesses.length; i++) {
            commands += gRunProcesses[i]['command'] + '\n';
        }
        var result = alertDialog("The following commands are still running "+
                                 "and must terminate before Komodo can "+
                                 "exit. Would you like Komodo to terminate "+
                                 "these commands?",
                                 false, false, "Terminate Processes", null,
                                 commands);
        if (result == "Terminate Processes") {
            try {
                for (i=0; i < gRunProcesses.length; i++) {
                    //dump("Run_CanClose(): killing '"+
                    //     gRunProcesses[i]['command']+"'\n");
                    gRunProcesses[i]['process'].kill(-1);
                }
            } catch (ex) {
                logError("Error killing running process when closing Komodo:"+
                         ex);
                //XXX Should this be reported to the user?
                return false;
            }
        } else if (result == "Cancel") {
            return false;
        } else {
            throw("Unexpected return value from alert dialog in "+
                  "Run_CanClose: '"+result+"'.\n");
        }
    }
    return true;
}

