/* Copyright (c) 2000-2001 ActiveState Tool Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo Regular Expression Debugger */

function assert(expression) {
    if(!expression) {
        throw "Regex Debugger Assertion!";
    }
}

//---- globals

// References to DOM noes in the dialog.
var dlg = new Object();
var gGlobalPrefs = null;

// Scintillas
var gRxSciRegex;
var gRxSciTestString;
var gRxSciRegexEventist;
var gRxSciTestStringEventist;

//const gRxSciRegexFontSize = 12;

// Cached copies of the regex and test string to detect and
// eliminate horrific recursive onModified() calls.
var gRxOldRegex;
var gRxOldTestString;

// Regex info components.
var gRxNodeHighlighter;
var gRxNodeTipGenerator;
var gRxRegexEvaluator;

// The regex evaluation manager component.
var gRxEvalMan;

// Do we need to re-analyze the regex because the user modified
// the regex or test string box?
var gRxDirty;

// An interval timer to keep track the need to update
// the state of the evaluation manager.
var gRxEvalManTimer;

// An object which maintains the current state index and allows
// us to increment it and decrement it without overflowing the
// evaluation manager's state list.
var gRxStateIndexManager;

// Has onLoad been run yet?
var gRxInitialised = false;

// Own Window Rx
var gRxOwnWindow = false;

// UI Elements
var rxNodeTipText;

// Controllers.
var gRxModifierItemsController = null;
var gRxItemsController = null;
var gRxEvalControlItemsController = null;

var rxSuccessLight = null;
var rxFailureLight = null;
var rxClass = null;

// Scintilla styling constants.
var NH_STYLE_BREAK = 1;
var NH_STYLE_CHILD = 2;
var NH_STYLE_PARENT = 3;

function regexSciMozEventist() {}
regexSciMozEventist.prototype = {
    onStyleNeeded: function(position) {
        return false;
    },
    onCharAdded: function(ch) {
        return false;
    },
    onSavePointReached: function() {
        return false;
    },
    onSavePointLeft: function() {
        return false;
    },
    onModifyAttemptRO: function() {
        return false;
    },
    onKey: function(ch, modifiers) {
        return false;
    },
    onDoubleClick: function() {
        return false;
    },
    onUpdateUI: function() {
        return false;
    },
    onModified: function(position, modificationType, text, length,
                                 linesAdded, line, foldLevelNow, foldLevelPrev)
    {
        if(gRxSciRegex.text != gRxOldRegex) {
            gRxOldRegex = gRxSciRegex.text;
            gRxDirty = true;
            return true;
        }
        return false;
    },
    onMacroRecord: function(message, wParam, lparam) { 
        return false;
    },
    onMarginClick: function(modifiers, position, margin) {
        return false;
    },
    onNeedShown: function(position, length) {
        return false;
    },
    onPosChanged: function(position) {
        highlightNodeByOffset(position, NH_STYLE_PARENT);
        rxUpdateShortcutMenu(); 
        return false;
    },
    // nsISupports interface
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.ISciMozEvents))
        throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

function testStringSciMozEventist() {}
testStringSciMozEventist.prototype = new regexSciMozEventist();
testStringSciMozEventist.prototype.constructor = testStringSciMozEventist;
testStringSciMozEventist.prototype.onModified = function(position, modificationType, text, length,
                                 linesAdded, line, foldLevelNow, foldLevelPrev) {
    if(gRxSciTestString.text != gRxOldTestString) {
        gRxOldTestString = gRxSciTestString.text;
        gRxDirty = true;
        return true;
    }
    return false;
}
testStringSciMozEventist.prototype.onPosChanged = function(position) {return false;}


function RxItemsController() {
    //dump("RxItemsController starting up...\n");
}
RxItemsController.prototype = {
    supportsCommand: function(command) {
        switch(command) {
            case 'cmd_loadTestString': 
            case 'cmd_rxRefresh': 
            case 'cmd_rxHelp': 
            return true;
            default: break;
        }
        return false;
    },
    isCommandEnabled: function(command) {
        switch(command) {
            case 'cmd_loadTestString': 
            case 'cmd_rxRefresh': 
            case 'cmd_rxHelp': return true;
            default: break;
        }
        return false;
    },
    doCommand: function(command) {
        switch(command) {
            case 'cmd_loadTestString': this.load_file(); break;
            case 'cmd_rxRefresh':
            gRxDirty = true;
            break;
            case 'cmd_rxHelp': this.open_help(); break;
        }
        return true;
    },
    load_file: function() {
        var fpOpen = Components.classes["@mozilla.org/filepicker;1"].
        createInstance(Components.interfaces.nsIFilePicker);
        fpOpen.init(window, "Open File", 
                    Components.interfaces.nsIFilePicker.modeOpen);
        _append_all_filter(fpOpen);
        _append_other_filters(fpOpen);
        if (fpOpen.show() == Components.interfaces.nsIFilePicker.returnOK) {
            // don't return fpOpen.file -- let the caller access
            // url and local-file properties of fpOpen
            var file = new File();
            file.open(fpOpen.file.path, 'r');
            gRxSciTestString.text = file.read();
            file.close();
        }
    },
    open_help: function()  {
      func_helpHelp('chrome://komodo/content/doc/komodo/komodo-doc-regex.html')
    }
}

function setupRxShortcutMenu() {
    var menu = document.getElementById("rxShortcutMenuPopup");

    // Get a list of children from gRxNodeHighlighter
    var tokens = {};
    var tokenCount = {};
    var descriptions = {};
    var ranges = {};
    var rangeCount = {};
    var ok = gRxNodeHighlighter.getAllTokens(tokenCount,tokens,descriptions,rangeCount,ranges);
    if (!ok) {
        logError("Failed to populate shortcut menu.");
        return;
    }

    var mi = document.createElement("menuitem");
    mi.setAttribute("id", "rxShortcutMenuItemNoItems");
    mi.setAttribute("label", "No valid tokens.");
    mi.setAttribute("hidden", false);
    menu.appendChild(mi);
    for (var i=0; i<tokenCount.value; i++) {
        mi = document.createElement("menuitem");
        mi.setAttribute("id", "rxShortcutMenuItem" + i);
        mi.setAttribute("label", tokens.value[i] + ": " +descriptions.value[i]);
        mi.setAttribute("startOffset", ranges.value[2*i]);
        mi.setAttribute("endOffset", ranges.value[2*i + 1]);
        mi.setAttribute("token", tokens.value[i]);
        mi.setAttribute("oncommand", "onRxShortcutMenu(event)");
        menu.appendChild(mi);
    }

    rxUpdateShortcutMenu();
}

var prevStart = -1;
var prevEnd = -1;
function rxUpdateShortcutMenu(event)  {
    var validItems = {};
    var validCount = {};
    var start = gRxSciRegex.selectionStart;
    var end = gRxSciRegex.selectionEnd;
    var id;

    if (start != prevStart || end != prevEnd) {
        var ok = gRxNodeHighlighter.getInsertableTokens(gRxSciRegex.text,
                             gRxModifierItemsController.modifiers,
                             start,
                             end,
                             validCount,
                             validItems
                             );
        var trueCount = false;
        for (var i=0; i<validCount.value; i++) {
            id = "rxShortcutMenuItem" + i;
            SetMenuItemVisibility(id, validItems.value[i]);
            trueCount |= validItems.value[i];
        }
        if (trueCount)
            SetMenuItemVisibility("rxShortcutMenuItemNoItems", false);
        else 
            SetMenuItemVisibility("rxShortcutMenuItemNoItems", true);
        prevStart = start;
        prevEnd = end;
    }
}

// Show/Hide a menu item
// Parameters - 1. menuitem id 2. Boolean to show/hide
function SetMenuItemVisibility(id, showItem)
{
    var item = document.getElementById(id);
    
    if (item) 
        item.setAttribute("hidden", !showItem);
    else
        logError("HideMenuItem::item id="+id+" not found\n");
}

function onRxShortcutMenu(event) {
    var mi = event.target;
    var token = mi.getAttribute("token");
    var startOffset = parseInt(mi.getAttribute("startOffset"));
    var endOffset  = parseInt(mi.getAttribute("endOffset"));
    var oldstart = gRxSciRegex.selectionStart;
    var oldend = gRxSciRegex.selectionEnd;
    
    if (gRxSciRegex.selText)
        gRxSciRegex.replaceSel(token);
    else
        gRxSciRegex.insertText(oldstart, token);
    
    gRxSciRegex.selectionStart = oldstart + startOffset;
    gRxSciRegex.selectionEnd = oldstart + endOffset;
}

function RxModifierItemsController() {
    //logDebug("RxModifierItemsController starting up...");
    var mods = ['g', 'm', 'o', 'i', 's', 'x'];
    this.mods = {};
    this.commands = {};
    for (x in mods) {
        var prefName = "rxMod" + mods[x].toUpperCase();
        var cmdName = "cmd_regexMod" + mods[x];
        var checked = false;
        if (gGlobalPrefs.hasBooleanPref(prefName)) {
            checked = gGlobalPrefs.getBooleanPref(prefName);
        }
        this.commands["cmd_regexMod" + mods[x]] = checked;
        this.mods["cmd_regexMod" + mods[x]] = mods[x];
    }
    this.modifiers = this.buildModString();
}
RxModifierItemsController.prototype = {
    supportsCommand: function(command) {
        return this.commands[command] != null;
    },
    isCommandEnabled: function(command) {
        return this.commands[command] == true;
    },
    doCommand: function(command) {
        //logDebug(command);
        if(this.commands[command] == true) {
            this.commands[command] = false;
        } else {
            this.commands[command] = true;
        }

        this.modifiers = this.buildModString();
        koRequestUpdateCommands("rx-modifiers");
        gRxDirty = true;
    },
    onEvent: function(event) {
        //logDebug(event);
    },
    buildModString: function() {
        var modString = "";
        for(command in this.commands) {
            if(this.commands[command] == true)
                modString += this.mods[command];
        }
        //logDebug(modString);
        return modString;
    }
};

// Evaluation control items controller.
function RxEvalControlItemsController() {
    //logDebug("RxEvalControlItemsController starting up...");
}
RxEvalControlItemsController.prototype = {
    supportsCommand: function(command) {
        switch(command) {
        case 'cmd_runBackward':
        case 'cmd_stepBack':
        case 'cmd_stepForward':
        case 'cmd_runForward':
            return true;
            break;

        default:
            break; // fall through
        }
        return false;
    },

    isCommandEnabled: function(command) {
        var retval = false;
        switch(command) {
        case 'cmd_runBackward':
            retval = gRxStateIndexManager.canDecIndex();
            return retval;
            break;

        case 'cmd_stepBack':
            retval = gRxStateIndexManager.canDecIndex();
            return retval;
            break;

        case 'cmd_stepForward':
            retval = gRxStateIndexManager.canIncIndex();
            return retval;
            break;

        case 'cmd_runForward':
            retval = gRxStateIndexManager.canIncIndex();
            return retval;
            break;
        }
        return retval;
    },

    doCommand: function(command) {
        switch(command) {
        case 'cmd_runBackward':
            gRxStateIndexManager.
            setIndex(gRxEvalMan.
                        runBackwardUntilBreakpoint(gRxStateIndexManager.currentIndex()));
            //dump("cmd_runBackward: new index is "
            //         + gRxStateIndexManager.currentIndex() + "\n");
            updateEvalState();
            break;

        case 'cmd_stepBack':
            if(gRxStateIndexManager.canDecIndex()) {
                gRxStateIndexManager.decIndex();
                //dump("cmd_stepBack: new index is "
                //       + gRxStateIndexManager.currentIndex() + "\n");
                updateEvalState();
            }
            break;

        case 'cmd_stepForward':
            if(gRxStateIndexManager.canIncIndex()) {
                gRxStateIndexManager.incIndex();
                //dump("cmd_stepForward: new index is "
                //       + gRxStateIndexManager.currentIndex() + "\n");
                updateEvalState();
            }
            break;

        case 'cmd_runForward':
            gRxStateIndexManager.
                        setIndex(gRxEvalMan.
                        runForwardUntilBreakpoint(gRxStateIndexManager.currentIndex()));
            //dump("cmd_runForward: new index is "
            //         + gRxStateIndexManager.currentIndex() + "\n");
            updateEvalState();
            break;
        }

        return true;
    },

    onEvent: function(event) {}
}

function RxEvalStateIndexManager(evalMan) {
    //dump("RxEvalStateIndexManager starting up...\n");
    this.evalMan = evalMan;
    this.reset();
}
RxEvalStateIndexManager.prototype = {
    reset: function() {
        this.stateListIndex = 0;
    },
    
    canIncIndex: function() {
        if (!this.evalMan.runFinished) {
            window.setTimeout('koRequestUpdateCommands("rx-eval")', 200);
            return true;
        }
        return this.evalMan.stateCount &&
        this.evalMan.stateCount > this.stateListIndex + 1;
    },
    
    canDecIndex: function() {
        return this.evalMan.stateCount &&
        this.stateListIndex > 0;
    },
    
    currentIndex: function() { return this.stateListIndex; },
    
    incIndex: function() {
        if(this.evalMan.stateCount > this.stateListIndex) {
            this.stateListIndex++;
        }
    },
    
    setIndex: function(index) {
        this.stateListIndex = index;
    },
    
    decIndex: function() {
        if(this.stateListIndex > 0) {
            this.stateListIndex--;
        }
    }
}

function setRxTrafficLights(status) {
    // Set the traffic lights to red or green.
    if (status) {
        rxSuccessLight.setAttribute('class', 'rxSuccessActive');
        rxFailureLight.setAttribute('class', 'rxFailureIdle');
    } else {
        rxFailureLight.setAttribute('class', 'rxFailureActive');
        rxSuccessLight.setAttribute('class', 'rxSuccessIdle	');
    }
}

function RxFileDrop() {}
RxFileDrop.prototype = {
    Observe : function (subject ,topic, fname) {
        if (topic == 'SciMoz:FileDrop') {
            var scimoz = subject.QueryInterface(Components.interfaces.ISciMoz);
            if (document.getElementById("sci-teststring").ISciMoz == scimoz || document.getElementById("sci-regex").ISciMoz == scimoz) {
                var os = Components.classes['@activestate.com/koOs;1'].getService(Components.interfaces.koIOs);
              scimoz.text = os.readfile(fname);
            }
        }
    }
}


var gDrag_and_drop_listener;

function RxInit() {
    // Initialise UI Elements
    rxNodeTipText = document.getElementById("rxNodeTipText");
    rxSuccessLight = document.getElementById("rxLightSuccess");
    rxFailureLight = document.getElementById("rxLightFailure");
    
    gRxSciRegex = document.getElementById('sci-regex').ISciMoz;
    var sci_regex = document.getElementById('sci-regex');
    sci_regex.init();
    sci_regex.ISciMoz.viewWS = 1;
    
    gRxSciTestString = document.getElementById('sci-teststring').ISciMoz;
    var sci_teststring = document.getElementById('sci-teststring');
    sci_teststring.init();
    sci_teststring.ISciMoz.viewWS = 1;
    // Add a listener for drag/drop operations.
    gDrag_and_drop_listener = new RxFileDrop();
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);
    // This observer is never removed, but it doesn't matter - very light-weight and no cycles.
    observerSvc.AddObserver(gDrag_and_drop_listener, "SciMoz:FileDrop");    
    

    rxSetMatchoutView(document.getElementById('rx-matchout'),gRxMatchoutView);

    // Initialize the regex entry widget.
    gRxSciRegexEventist = new regexSciMozEventist();
    gRxSciRegex.hookEvents(gRxSciRegexEventist);
    gRxSciRegex.lexer = gRxSciRegex.SCLEX_CONTAINER;
    gRxSciRegex.setCommandUpdateTarget(window);

    gRxSciRegex.hScrollBar = false;
    gRxSciRegex.setMarginWidthN(1, 0);

    // Setup the regex entry boxes with appropriate styling.
    var styleApplierRegex = Components.classes["@activestate.com/koScintillaStyleApplier;1"].createInstance();
    styleApplierRegex.init(gRxSciRegex, "Rx", null);
    styleApplierRegex.applyStyles()
  
    var styleApplierTest = Components.classes["@activestate.com/koScintillaStyleApplier;1"].createInstance();

    styleApplierTest.init(gRxSciTestString, "Rx", null);
    styleApplierTest.applyStyles();

    // Hook the test string Scintilla up to its eventist.
    gRxSciTestStringEventist = new testStringSciMozEventist();
    gRxSciTestString.hookEvents(gRxSciTestStringEventist);
    gRxSciTestString.lexer = gRxSciTestString.SCLEX_CONTAINER;
    gRxSciTestString.setCommandUpdateTarget(window);

    gRxSciTestString.hScrollBar = false;
    gRxSciTestString.setMarginWidthN(1, 0);

    // Load the regex back-end components.
    rxClass = Components.classes['@activestate.com/koRxRegexInfo;1'].createInstance();
    gRxNodeHighlighter = rxClass.QueryInterface(Components.interfaces.koIRxNodeHighlighter);
    assert(gRxNodeHighlighter != null);

    gRxNodeTipGenerator = rxClass.QueryInterface(Components.interfaces.koIRxNodeTipGenerator);
    assert(gRxNodeTipGenerator != null);

    gRxRegexEvaluator = rxClass.QueryInterface(Components.interfaces.koIRxRegexEvaluator);
    assert(gRxRegexEvaluator != null);

    rxClass = Components.classes['@activestate.com/koRxEvalManager;1'].createInstance();
    gRxEvalMan = rxClass.QueryInterface(Components.interfaces.koIRxEvalManager);
    assert(gRxEvalMan != null);

    gRxStateIndexManager = new RxEvalStateIndexManager(gRxEvalMan);

    // Queue up an eval of the current regex.
    gRxEvalManTimer = window.setInterval(updateUIState, 100);

    rxInstallObservers();
    rxInstallControllers();

    // Build Shortcuts menu
    setupRxShortcutMenu();


    //logDebug("Finished loading Rx window.");
}


function _rxSetSplitterState(splitterWidget, collapsed)
{
    if (collapsed) {
        splitterWidget.setAttribute('state', 'collapsed');
    } else {
        splitterWidget.removeAttribute('state');
    }
}


function RxOnLoad(rx_sci_moz) {
    dlg.window = document.getElementById('komodo_rx');
    dlg.regexArea = document.getElementById("rx-regex-area");
    dlg.matchVarsArea = document.getElementById("rx-match-vars-area");
    dlg.matchVarsSplitter = document.getElementById("rx-match-vars-splitter");
    dlg.modG = document.getElementById("regexModg");
    dlg.modM = document.getElementById("regexModm");
    dlg.modI = document.getElementById("regexModi");
    dlg.modS = document.getElementById("regexMods");
    dlg.modX = document.getElementById("regexModx");

    // Get the preferences
    gGlobalPrefs = Components.classes["@activestate.com/koPrefService;1"].
                   getService(Components.interfaces.koIPrefService).prefs;

    if (typeof(rx_sci_moz) != 'undefined' && rx_sci_moz) {
        // only need to do this once during a Komodo session
        if (!gRxInitialised) {
            RxInit();
            gRxInitialised = true;
        }
        if (rx_sci_moz.selText) {
            gRxSciRegex.text = rx_sci_moz.selText;
            if(gRxSciTestString.textLength == 0) {
                gRxSciTestString.text = "";
            }
        }
        else if (gGlobalPrefs.hasPref("rx-default-regex") &&
                 gGlobalPrefs.hasPref("rx-default-teststring")) {
                    
            if (gRxSciRegex.textLength == 0) {
                gRxSciRegex.text = gGlobalPrefs.getStringPref("rx-default-regex");
            }
            if (gRxSciTestString.textLength == 0) {
                gRxSciTestString.text = gGlobalPrefs.getStringPref("rx-default-teststring");
            }
        }
    } else {
        RxInit();
        var sci = window.arguments[0];
        if (sci.selText) {
            gRxSciRegex.text = sci.selText;
        } else
        if (gGlobalPrefs.hasPref("rx-default-regex") && gGlobalPrefs.hasPref("rx-default-teststring")) {
            gRxSciRegex.text = gGlobalPrefs.getStringPref("rx-default-regex");
            gRxSciTestString.text = gGlobalPrefs.getStringPref("rx-default-teststring");
        }
        gRxInitialised = true;
        gRxOwnWindow = true;
    }
    gRxSciRegex.emptyUndoBuffer();
    gRxSciTestString.emptyUndoBuffer();
    
    // Set window layout.
    var x = gGlobalPrefs.getLongPref("rxScreenX");
    var y = gGlobalPrefs.getLongPref("rxScreenY");
    var w = gGlobalPrefs.getLongPref("rxOuterWidth");
    var h = gGlobalPrefs.getLongPref("rxOuterHeight");
    var regexAreaHeight = gGlobalPrefs.getLongPref("rxRegexAreaHeight");
    var matchVarsAreaHeight = gGlobalPrefs.getLongPref("rxMatchVarsAreaHeight");
    // XXX Should perhaps validate size/position values to ensure on screen
    //     as done in kombo.js::setupWin().
    dlg.window.setAttribute("screenX", x);
    dlg.window.setAttribute("screenY", y);
    dlg.window.setAttribute("width", w);
    dlg.window.setAttribute("height", h);
    dlg.regexArea.setAttribute("height", regexAreaHeight);
    dlg.matchVarsArea.setAttribute("height", matchVarsAreaHeight);
    _rxSetSplitterState(dlg.matchVarsSplitter,
        gGlobalPrefs.getBooleanPref("rxMatchVarsAreaCollapsed"));
    
    document.getElementById('sci-regex').focus();
    // "create" event already fired.
    // Fire a new "load" event our controllers recognize
    koRequestUpdateCommands("load");
    koRequestUpdateCommands("rx-modifiers");
}

function RxOnUnload() {
    if (gRxInitialised)  {
        gRxSciRegex.unhookEvents(gRxSciRegexEventist);
        gRxSciRegexEventist = null;
        gRxSciTestString.unhookEvents(gRxSciTestStringEventist);
        gRxSciTestStringEventist = null;

        rxRemoveControllers();
        rxRemoveObservers();

        gRxNodeHighlighter = null;
        gRxNodeTipGenerator = null;
        gRxRegexEvaluator = null;
        gRxEvalMan = null;
    }

    // Save window layout.
    gGlobalPrefs.setLongPref("rxScreenX", window.screenX);
    gGlobalPrefs.setLongPref("rxScreenY", window.screenY);
    gGlobalPrefs.setLongPref("rxOuterHeight", window.outerHeight);
    gGlobalPrefs.setLongPref("rxOuterWidth", window.outerWidth);
    gGlobalPrefs.setLongPref("rxRegexAreaHeight",
                              dlg.regexArea.getAttribute("height"));
    gGlobalPrefs.setLongPref("rxMatchVarsAreaHeight",
                              dlg.matchVarsArea.getAttribute("height"));
    gGlobalPrefs.setBooleanPref("rxMatchVarsAreaCollapsed", 
        dlg.matchVarsSplitter.getAttribute("state") == "collapsed");

    // Save current regex and test string as defaults.
    if (gRxSciRegex.textLength != 0) {
        gGlobalPrefs.setStringPref("rx-default-regex", gRxSciRegex.text);
    }
    if (gRxSciTestString.textLength != 0) {
        gGlobalPrefs.setStringPref("rx-default-teststring",
                                    gRxSciTestString.text);
    }

    // Save regex modifiers
    gGlobalPrefs.setBooleanPref("rxModG", dlg.modG.checked);
    gGlobalPrefs.setBooleanPref("rxModM", dlg.modM.checked);
    gGlobalPrefs.setBooleanPref("rxModI", dlg.modI.checked);
    gGlobalPrefs.setBooleanPref("rxModS", dlg.modS.checked);
    gGlobalPrefs.setBooleanPref("rxModX", dlg.modX.checked);

    opener.focus();
}

// Update the UI in response to an event in the evaluation manager, such
// as moving to the next state in the evaluation state list.
function updateEvalState() {

    //logDebug("Updating UI in response to a change in the evaluation manager.");
    // Current state will contain an array of state variables.
    var matchOffsets = {};
    var matchOffsetsCount = {};
    var nodeId = {};
    var startOfMatch = {};
    var currentPos = {};
    var closestOpen = {};

    try {
        gRxEvalMan.getNodeAt(gRxStateIndexManager.currentIndex(),
                                       nodeId,
                                       startOfMatch,
                                       currentPos,
                                       closestOpen,
                                       matchOffsetsCount,
                                       matchOffsets);
    }
    catch(e) {
        // Okay, so there isn't an active node.
        logDebug("Failed to retrieve state for index " + gRxStateIndexManager.currentIndex());
        logDebug("  because... " + e);
        return;
    }

    // Set the group match variables.
    gRxMatchoutView.matchData = matchOffsets.value;
    gRxMatchoutView.setRowCount(matchOffsetsCount.value / 2);

    // Highlight the current match ($&)
    scinClearHighlights(gRxSciTestString, NH_STYLE_PARENT);
    scinHighlight(gRxSciTestString, NH_STYLE_PARENT, startOfMatch.value, currentPos.value - 1);

    // Try to highlight the current node.
    highlightNodeById(nodeId.value, NH_STYLE_PARENT, false);

    // Update the evaluation manager control buttons.
    koRequestUpdateCommands("rx-eval");
}

function getNodeIdFromOffset(offset) {
    var nodeId = null;
    
    try {
        nodeId = gRxNodeHighlighter.getNodeId(gRxSciRegex.text,
                                                gRxModifierItemsController.modifiers,
                                                offset);
    } catch(e) { }

    return nodeId;
}

function highlightNodeByOffset(offset, style) {
    var nodeId = getNodeIdFromOffset(offset);
    if(nodeId != null)
        highlightNodeById(nodeId, style, true);
}

function setNodeTipText(statustext) {
    rxNodeTipText.setAttribute("label", statustext);
    rxNodeTipText.setAttribute("tooltiptext", statustext);
}

function setNodeTipById(nodeId) {
    try {
        var nodeTip = gRxNodeTipGenerator.getNodeTip(gRxSciRegex.text,
                                                 gRxModifierItemsController.modifiers,
                                                 nodeId);
        setNodeTipText(nodeTip);
    } catch(e) {
        logDebug("Failed to generate node tip for node id " + nodeId + "\n");
    }
}

function highlightNodeById(nodeId, style, highlightGroupMatches) {
    var a = {};
    var b = {};
    try {
        scinClearAllHighlights(gRxSciRegex);
        if (!nodeId) return;
        gRxNodeHighlighter.getNodeRange(gRxSciRegex.text,
                                    gRxModifierItemsController.modifiers,
                                    nodeId, 0, a, b);

        scinHighlight(gRxSciRegex, style, a.value, b.value);

        gRxNodeHighlighter.getChildNodesRange(gRxSciRegex.text,
                                          gRxModifierItemsController.modifiers,
                                          nodeId, a, b);
        if(!(a.value == 0 && b.value == 0)) {
            scinHighlight(gRxSciRegex, NH_STYLE_CHILD, a.value, b.value);
        }

        // That's all fine and dandy; but if it's a group, we do more:
        // 1. Highlight the other group
        // 2. Highlight the areas in the target string that matched this
        //    group.

        var gid = gRxNodeHighlighter.getGroup(gRxSciRegex.text, 
                                              gRxModifierItemsController.modifiers,
                                              nodeId);
        if (gid != 0) {
            var mid = gRxNodeHighlighter.getMatchingNode(gRxSciRegex.text,
                                                         gRxModifierItemsController.modifiers,
                                                         nodeId);
            gRxNodeHighlighter.getNodeRange(gRxSciRegex.text,
                                            gRxModifierItemsController.modifiers,
                                            mid, 0, a, b);
            scinHighlight(gRxSciRegex, style, a.value, b.value);

            // This feature may want to be in a preference:
            var globalMode = (gRxModifierItemsController.modifiers.indexOf('g') >= 0);
            var matchOffsetsCount = {};
            var matchOffsets = {};
            var groupCount = {};
            var errorString = {};
            var matched = gRxRegexEvaluator.
                evalRegex(gRxSciRegex.text,
                          gRxModifierItemsController.modifiers,
                          gRxSciTestString.text,
                          groupCount,
                          matchOffsetsCount,
                          matchOffsets,
                          errorString);

            if(highlightGroupMatches) {
                scinClearAllHighlights(gRxSciTestString);
                for (var i=2*gid;
                     i<matchOffsetsCount.value; 
                     i+=2*groupCount.value) {
                    var start = matchOffsets.value[i];
                    var end = matchOffsets.value[i+1];
                    if (start != -1 && end != -1 && start <= end)
                        scinHighlight(gRxSciTestString, style, start, end);
                }
            }
        }
    }
    catch(e) {
        logDebug("highlightNodeById: Can't highlight node " + nodeId + "\n");
        return;
    }

    setNodeTipById(nodeId);
}

// Clear all the highlights from a Scintilla.
function scinClearAllHighlights(scin) {
    scin.clearDocumentStyle();
}

// Clear all the highlights from a Scintilla which are
// in a particular style.
function scinClearHighlights(scin, style) {
    if(scin.length > 0) {
        scin.startStyling(0, style);
        scin.setStyling(scin.length, 0);
    }
}

// Highlight some part of a Scintilla widget with some style.
function scinHighlight(scin, style, startOffset, endOffset) {
    // Don't bother highlighting if the Scintilla has zero length or
    // if the end and start offsets are the same.
    if(scin.length == 0) {
        return;
    }

    if(startOffset > endOffset) {
        var tempOffset = startOffset;
        startOffset = endOffset;
        endOffset = tempOffset;
    }

    //    logDebug("Highlighting from " + startOffset + " to " + endOffset +
    //             " with style " + style + " in scin " + scin);
    
    // Adjust the starting and ending offsets to prevent 
    var scinLength = scin.length;
    if(startOffset > scinLength - 1) {
        startOffset = scinLength - 1;
    }
    if(startOffset < 0) {
        startOffset = 0;
    }

    if(endOffset > scinLength - 1) {
        endOffset = scinLength;
    }
    if(endOffset < 0) {
        endOffset = 0;
    }
    
    scin.startStyling(startOffset, style);
    scin.setStyling(endOffset - startOffset + 1, style);
}

// Once in a while, check to see whether the user has modified either
// the regular expression string or the test string.
// If either of these have been modified, as indicated by the
// gRxDirty flag, then reset the evaluation manager and trigger
// a command update of the regex evaluation control commands.
function updateUIState() {
    if (!gRxDirty) return;
    
    gRxDirty = false;
    gRxEvalMan.initEval(gRxSciRegex.text,
                      gRxSciTestString.text,
                      gRxModifierItemsController.modifiers);
    gRxStateIndexManager.reset();

    // Give the evaluation manager some time to gather a state list.
    window.setTimeout('koRequestUpdateCommands("rx-eval");', 100);

    // If instant mode is enabled, update the match state.
    window.setTimeout('updateMatches();', 0);

    // If linting is enabled, update the lints.
    window.setTimeout('updateLinting();', 0);
}

var gRxOldHoverPos = -1;
var gRxHoverPos = -1;
var gRxOldNodeTipNodeId = null;

function onSciRegexMouseMove(event) {
    var x = event.clientX - event.target.boxObject.x;
    var y = event.clientY - event.target.boxObject.y;
    gRxHoverPos = gRxSciRegex.positionFromPoint(x, y);
    
    if(gRxOldHoverPos != gRxHoverPos) {
        var nodeId = getNodeIdFromOffset(gRxHoverPos);
        if (!nodeId) return;
        if(gRxHoverPos >= 0 && nodeId != gRxOldNodeTipNodeId) {
            highlightNodeById(nodeId, NH_STYLE_PARENT, true);
            gRxOldNodeTipNodeId = nodeId;
        }
        gRxOldHoverPos = gRxHoverPos;
    }
}

function rxIsVisible() {
    var windowManager = Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);
    var rxWindow = windowManager.getMostRecentWindow("komodo_rx");    
    return rxWindow; 
}

function rxToggle() {
    var rxBox = document.getElementById("rx-box");
    var rxSplitter = document.getElementById("rx-content-splitter");
    var scimoz = document.getElementById("scin").ISciMoz;
    // determine from prefs whether to open in separate window or not
    var ownWindow = prefs.getBooleanPref("uiRxSeparateWindow");
    // try to get a handle on an already open komodo workspace
    const nsIWindowMediator = Components.interfaces.nsIWindowMediator;
    const MEDIATOR_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=window-mediator";
    var windowManager = Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);
    var rxWindow = windowManager.getMostRecentWindow("komodo_rx");
    
    // Handle change of pref when rx open
    if (rxWindow && !ownWindow) {
        rxWindow.close();
        return;
    }
    if(rxSplitter && rxSplitter.getAttribute('collapsed') == 'false' && ownWindow) {
        rxBox.setAttribute('collapsed', 'true');
        rxSplitter.setAttribute('collapsed', 'true');
        return;
    }

    if (ownWindow) {
        if (rxWindow) {
            rxWindow.close();
        }
        else {
            openWindowUniqueInstance('chrome://komodo/content/rx/koRx.xul',
                                     'komodo_rx',
                                     'chrome,resizable=no,dialog=no,close=yes,dependent=no',
                                     scimoz);
        }
    }
    else if (rxBox) {
        if(rxSplitter.getAttribute('collapsed') == 'false') {
            rxBox.setAttribute('collapsed', 'true');
            rxSplitter.setAttribute('collapsed', 'true');
        }
        else {
            RxOnLoad(scimoz);  // From koRxOverlay.js
            rxBox.setAttribute('collapsed', 'false');
            rxSplitter.setAttribute('collapsed', 'false');
        }
    }
}

function rxToggleAdvancedMode(element) {
    var rxAdvancedBox = document.getElementById("rx-advanced-box");

    if (rxAdvancedBox.getAttribute("collapsed") == "true") {
        rxAdvancedBox.setAttribute("collapsed", "false");
    } else {
        rxAdvancedBox.setAttribute("collapsed", "true");
    }
    if (gRxOwnWindow)
        window.sizeToContent();
}

function updateLinting() {
  // Perform syntax error linting (crude!)
    var errorString = {};
    if(!gRxRegexEvaluator.isValidRegex(gRxSciRegex.text,
                                     gRxModifierItemsController.modifiers,
                                     errorString)) {
        gRxSciRegex.indicSetStyle(2, gRxSciRegex.INDIC_SQUIGGLE);
        gRxSciRegex.indicSetFore(2, 0x0000ff);
        scinHighlight(gRxSciRegex, gRxSciRegex.INDIC2_MASK,
                    0, gRxSciRegex.length - 1);

        setRxTrafficLights(false);
        setNodeTipText(errorString.value);
    } else {
        scinClearHighlights(gRxSciRegex, gRxSciRegex.INDIC2_MASK);
    }
}

// Perform a match against the test string and update the UI
// in response to the match.
function updateMatches() {
    scinClearAllHighlights(gRxSciTestString);

    var matchOffsetsCount = {};
    var matchOffsets = {};
    var groupCount = {};
    var errorString = {};
    var matched = gRxRegexEvaluator.
        evalRegex(gRxSciRegex.text,
                  gRxModifierItemsController.modifiers,
                  gRxSciTestString.text,
                  groupCount,
                  matchOffsetsCount,
                  matchOffsets,
                  errorString);

    if(matched) {
        setRxTrafficLights(true);
        if (groupCount.value == 1)
            setNodeTipText("Match succeeded: 1 group");
        else
            setNodeTipText("Match succeeded: " + groupCount.value + " groups");

        if(matchOffsetsCount.value > 1) {
            scinHighlight(gRxSciTestString,
                          NH_STYLE_PARENT,
                          matchOffsets.value[0],
                          matchOffsets.value[1]);
            gRxMatchoutView.matchData = matchOffsets.value.slice(2);
            gRxMatchoutView.groupCount = groupCount.value;
            gRxMatchoutView.setRowCount((matchOffsetsCount.value - 2) / 2);
        }
        else {
            logError("Insufficient match offsets.");
            logError("matchOffsetsCount: " + matchOffsetsCount.value);
            logError("matchOffsets:      " + matchOffsets.value);
        }
    }
    else {
        setRxTrafficLights(false);
        setNodeTipText("No matches found");
    }
}

// The view object for the match variables outliner widget.
var gRxMatchoutView = ({
    // nsIOutlinerView
    rowCount : 0,
    getRowProperties : function(i, prop) {
        // prop.AppendElement(this.smile);
    },
    getColumnProperties : function(index, prop) {},
    getCellProperties : function(index, prop) {},
    isContainer : function(index) {return false;},
    isSeparator : function(index) {return false;},
    outliner : null,
    setOutliner : function(out) { this.outliner = out; },
    getCellText : function(i, col) {
        var globalMode =
            (gRxModifierItemsController.modifiers.indexOf('g') >= 0);
        var text = "";
        switch(col){
        case "rx-matchout-var":
            if(globalMode) {
                var m = Math.round(i / this.groupCount + 0.5);
                var v = (i % this.groupCount) + 1;
                text = "$" + v + " (match " + m + ")";
            }
            else {
                text = "$" + (i + 1);
            }
            break;
        case "rx-matchout-value":
            text = gRxSciTestString.text.
                substring(this.matchData[2*i],
                          this.matchData[2*i + 1] + 1);
            break;
        default:
            text = "XXX in " + col + " and " + i;
        }
        return text;
    },
    performAction : function(action) {
    },
    cycleHeader : function(index) {
    },
    selectionChanged : function() {
        if (this.selection.currentIndex >= 0 && this.matchData
            && this.matchData.length > 2 * this.selection.currentIndex) {
            var startOffset = this.matchData[2 * this.selection.currentIndex];
            var endOffset = this.matchData[2 * this.selection.currentIndex + 1];
            gRxSciTestString.gotoPos(startOffset);
            scinClearAllHighlights(gRxSciTestString);
            scinHighlight(gRxSciTestString, NH_STYLE_PARENT, startOffset,
                          endOffset);
        }
    },
    
    // Private stuff
    matchData : null,
    groupCount : 0,
    setRowCount : function(rowCount) {
        this.rowCount = rowCount;
        this.outliner.rowCountChanged(0, this.rowCount);
        this.outliner.invalidate();
    }
});
 
function rxSetMatchoutView(outliner, v) {
    outliner.boxObject.QueryInterface(Components.interfaces.nsIOutlinerBoxObject).view = v;
}

var rxObserver = {
    Observe: function() {
        var v = viewManager.getCurrentView();
        if (rxIsVisible() && !v) {
            rxToggle();
        }
    }
}

function rxInstallObservers() {
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                                            getService(Components.interfaces.nsIObserverService);

    observerSvc.AddObserver(rxObserver, "view_closed");
}

function rxRemoveObservers() {
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                                            getService(Components.interfaces.nsIObserverService);

    observerSvc.RemoveObserver(rxObserver, "view_closed");
}

function rxInstallControllers() {
    gRxEvalControlItemsController = new RxEvalControlItemsController();
    window.controllers.insertControllerAt(0, gRxEvalControlItemsController);
    
    gRxItemsController = new RxItemsController();
    window.controllers.insertControllerAt(0, gRxItemsController);
    
    gRxModifierItemsController = new RxModifierItemsController();
    window.controllers.insertControllerAt(0, gRxModifierItemsController);
}

function rxRemoveControllers() {
    window.controllers.removeController(gRxModifierItemsController);
    window.controllers.removeController(gRxItemsController);
    window.controllers.removeController(gRxEvalControlItemsController);
    //  window.controllers.removeController(gRxTokenMenuController);

    gRxModifierItemsController = null;
    gRxItemsController = null;
    gRxEvalControlItemsController = null;
}

function rxUpdateCommands()
{
    // Ensure any buffer sensitive commands are updated.
    koRequestUpdateCommands("buffer-focus");
    koRequestUpdateCommands("debug");
     // The selection may have changed.
    koRequestUpdateCommands("select");
    // The save and undo state of the file will be different.
    koRequestUpdateCommands("save");
    koRequestUpdateCommands("undo");
}

