/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* Web Services Setup Wizard - panel */

//---- globals

var _gPanelName = "wsSetup_ChooseInstallation";
var _gLanguage2Interpreter = {
    "Perl": "perl",
    "PHP": "php",
    "Python": "python"
    }
var _gFindingInterps = false;
var _gOsPath = Components.classes["@activestate.com/koOsPath;1"].getService();
var _gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].getService();



//---- internal support routines

function _CreateTreeItem(label)
{
    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    var treecell = document.createElementNS(XUL_NS, 'treecell');
    treecell.setAttribute('label', label);
    var treerow = document.createElementNS(XUL_NS, 'treerow');
    treerow.appendChild(treecell);
    var treeitem = document.createElementNS(XUL_NS, 'treeitem');
    treeitem.appendChild(treerow);
    return treeitem;
}


function _InterpPathToInstallPath(lang, interpreterPath)
{
    var installationPath = null;
    if (interpreterPath) {
        switch (lang) {
            case "Perl":
                installationPath = _gOsPath.dirname(_gOsPath.dirname(
                    interpreterPath));
                break;
            default:
                if (_gInfoSvc.platform.substring(0,3) == "win") {
                    installationPath = _gOsPath.dirname(interpreterPath);
                } else {
                    installationPath = _gOsPath.dirname(_gOsPath.dirname(
                        interpreterPath));
                }
                break;
        }
    }
    return installationPath;
}


function _UpdateInstallation(installation)
{
    // Persist.
    parent.wd_installation = installation;

    // Display the installation selection.
    var installWidget = document.getElementById("installation");
    if (installation == null) {
        installWidget.value = "";
    } else {
        installWidget.value = installation;
    }

    // Pass to the setup service.
    var wsSetupSvc = parent.wsSetupSvcs[parent.wd_language];
    wsSetupSvc.SetInstallationPath(installation);
}



//---- Wizard Infrastructure interface routines

function validate()
{
    // Validate this page.
    // - Ask the setup service if the setup will work in this
    //   installation.
    var wsSetupSvc = parent.wsSetupSvcs[parent.wd_language];
    var problem = wsSetupSvc.ReasonCannotSetup_GivenInstallationPath();
    if (problem) {
        alertDialog(problem,    // Message to display
                    true,   // Okay button is the default
                    true,   // hide the cancel button
                    "OK",   // Okay button label
                    "");   // Cancel button label
        return false;
    } else {
        return true;
    }
}


//---- interface for wsSetup_ChooseInstallation.xul

function OnLoad()
{
    // Give a nice header name for this pane.
    var headerWidget = parent.document.getElementById("header");
    headerWidget.setAttribute("title", "Choose Installation");
    headerWidget.setAttribute("description",
        "Select which language and installation to setup.");

    // Fill in the data from the global set stored in the parent
    // namespace.
    var langList = document.getElementById("language-list");
    if (parent.wd_language) {
        // find the menulist entry for this language
        var items = langList.firstChild.childNodes;
        for (var i=0; i < items.length; i++) {
            var item = items[i];
            if (item.getAttribute("label") == parent.wd_language) {
                langList.selectedItem = item;
                break;
            }
        }
        // ... and select that language
        UpdateLanguage(parent.wd_language);
    } else {
        // default to the first one in the list
        langList.selectedIndex = 0;
        UpdateLanguage(langList.selectedItem.getAttribute("label"));
    }
    
    parent.wizardManager.onPageLoad(_gPanelName);
}


function UpdateLanguage(lang)
{
    // Persist the language selection.
    var oldLang = parent.wd_language;    
    parent.wd_language = lang;
    
    // Get a setup service if necessary.
    if (parent.wsSetupSvcs[lang] == null) {
        var wsSetupSvc = Components.classes['@activestate.com/ko'
                                            + lang + 'WSSetupService;1'].
                         getService();
        parent.wsSetupSvcs[lang] = wsSetupSvc;
    }

    // setup "avail-install-caption"
    var captionWidget = document.getElementById("avail-install-caption");
    captionWidget.setAttribute("label", "Select " + lang
                               + " installation to setup");
    
    // setup "avail-install-text"
    var textWidget = document.getElementById("avail-install-text");
    textWidget.setAttribute("value", lang
                            + " installations found on your system:");
    
    // setup "avail-install-list"
    var listWidget = document.getElementById("avail-install-list");
    // - remove any existing items and add a "finding..." one
    _gFindingInterps = true;
    while (listWidget.hasChildNodes()) {
        listWidget.removeChild(listWidget.childNodes[0]);
    }
    listWidget.appendChild(_CreateTreeItem(
        "Finding available "+lang+" installation..."));
    // - get a list of installations
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
                   getService(Components.interfaces.koISysUtils);
    var availInterps = new Array();
    availInterps = sysUtils.WhichAll(_gLanguage2Interpreter[lang],
                                     new Object());
    // - populate the tree listing them
    listWidget.removeChild(listWidget.childNodes[0]); // "Finding..."
    if (availInterps.length != 0) {
        for (var i = 0; i < availInterps.length; i++) {
            listWidget.appendChild(
                _CreateTreeItem(
                _InterpPathToInstallPath(lang, availInterps[i])));
        }
    }
    _gFindingInterps = false;

    // If there is no installation setting or a new language was
    // selected then pick a reasonable default.
    if (!parent.wd_installation || lang != oldLang) {
        var languageSvc = Components.classes[
            "@activestate.com/koLanguage?language=" + lang + ";1"].
            getService(Components.interfaces.koILanguage);
        var interpreterPath = languageSvc.getLanguageService(
            Components.interfaces.koIInterpreterLanguageService).
            interpreterPath;
        // This interpreterPath is quoted from PHP (XXX I think that that is
        // is a bug in koPHPLanguage.py)
        var len = interpreterPath.length;
        if ((interpreterPath[0] == '"' || interpreterPath[0] == "'")
            &&
            (interpreterPath[len-1] == '"' || interpreterPath[len-1] == "'"))
        {
            interpreterPath = interpreterPath.substring(1, len-1);
        }
        parent.wd_installation = _InterpPathToInstallPath(lang,
            interpreterPath);
    }
    _UpdateInstallation(parent.wd_installation);
}


function UpdateInstallation_OnSelection()
{
    // User selected one of the "found installations".
    if (! _gFindingInterps) {
        // Load the default installation textfield with the selection
        var tree = document.getElementById("avail-install-tree");
        if (tree.selectedItems.length > 0) {
            var selection = tree.selectedItems[0];
            //          treeitem . treerow  . treecell 
            var value = selection.firstChild.firstChild
                        .getAttribute("label");
            _UpdateInstallation(value);
        }
    }
}


function UpdateInstallation_OnKey()
{
    // User typed into the installation field.
    // - Unselect any tree item.
    var tree = document.getElementById("avail-install-tree");
    tree.clearItemSelection();
    
    // - Persist the value.
    var installWidget = document.getElementById("installation");
    _UpdateInstallation(installWidget.value);
}


function UpdateInstallation_OnBrowse()
{
    var installation = BrowseForDir();
    if (installation != null) {
        _UpdateInstallation(installation);
    }
}
