// Copyright (c) 2000-2001 ActiveState Tool Corporation.
// See the file LICENSE.txt for licensing information.

const DEBUG = false; /* set to true to enable debug messages */

const DIALOGPROXY_CONTRACTID     = "@activestate.com/asDialogProxy;1";
const DIALOGPROXY_CID        = Components.ID("{57002286-C22C-4e42-BCFC-DE91AF24B5CA}");
const APPSHELL_SERV_CONTRACTID  = "@mozilla.org/appshell/appShellService;1";

const nsIAppShellService    = Components.interfaces.nsIAppShellService;
const nsISupports           = Components.interfaces.nsISupports;
const nsIFactory            = Components.interfaces.nsIFactory;
const asIDialogProxy         = Components.interfaces.asIDialogProxy;
const nsIInterfaceRequestor = Components.interfaces.nsIInterfaceRequestor
const nsIDOMWindow          = Components.interfaces.nsIDOMWindow;


function asDialogProxy()
{
    this.mParentWindow = null;
}

asDialogProxy.prototype = {
  init: function(parent) {
    this.mParentWindow = parent;
  },
    
  _getParentWindow: function() {
    var parent;
    try {
      if (this.mParentWindow) {
        parent = this.mParentWindow;
      } else if (typeof(window) == "object" && window != null) {
        parent = window;
      } else {
        try {
          var appShellService = Components.classes[APPSHELL_SERV_CONTRACTID].getService(nsIAppShellService);
          parent = appShellService.hiddenDOMWindow;
        } catch(ex) {
          debug("Can't get parent.  xpconnect hates me so we can't get one from the appShellService.\n");
          debug(ex + "\n");
        }
      }
    } catch(ex) { debug("no window access\n"); }
    this.mParentWindow = parent;
    return parent;
  },

  alert: function(prompt) {
      return this.alertEx(prompt, true, true, "OK", "Cancel");
  },
  
  alertEx: function(prompt, okIsDefault, hideCancel, okLabel, cancelLabel) {
    var parent = this._getParentWindow();
    try {
        var obj = new Object ();
        // "prompt" is the string displayed in the dialog. (must be defined)
        obj.prompt = prompt;
        obj.res = null;  // response
        // "okIsDefault" indicates that <Return> will fire the "Ok" button.
        // (defaults to true)
        if (typeof(okIsDefault) == "undefined") {
            obj.okIsDefault = true;
        } else {
            obj.okIsDefault = okIsDefault;
        }
        // "hideCancel" indicates if the <<Cancel>> button should be shown
        // (defaults to true, if "okIsDefault" is false this must be
        // true)
        if (!obj.okIsDefault) {
            obj.hideCancel = false;
        } else {
            if (typeof(hideCancel) == "undefined") {
                obj.hideCancel = true;
            } else {
                obj.hideCancel = hideCancel;
            }
        }
        // "okLabel" and "cancelLabel" are the button text strings
        if (typeof(okLabel) == "undefined") {
            obj.okLabel = "OK";
        } else {
            obj.okLabel = okLabel;
        }
        if (typeof(cancelLabel) == "undefined") {
            obj.cancelLabel = "Cancel";
        } else {
            obj.cancelLabel = cancelLabel;
        }

        parent.openDialog("chrome://komodo/content/commdlg/okcancel.xul",
                          "Komodo",
                          "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
                          obj);

        // Return the result, which is the text of the button clicked
        return obj.res;    
    } catch(ex) { dump("unable to open alert dialog\n" + ex + "\n"); }

    return "";
  },

  authenticate: function(title, prompt, loginname, allowAnonymous, allowPersist) {
    /* NOTE: if you change this usage, also change it in window_functions.js */
    var parent = this._getParentWindow();
    try {
        var obj = new Object ();
        obj.title = title;
        obj.user = loginname;
        obj.message = prompt;
        obj.allowAnonymous = allowAnonymous;
        obj.allowPersist = allowPersist;
        parent.openDialog("chrome://komodo/content/commdlg/authenticate.xul",
            "komodo_authenticate",
            "chrome,titlebar,modal=yes",
            obj);
        if (obj.retval) {
            var res = obj.user + ":" + obj.pwd;
            return res;
        }
        return "";
    } catch(ex) { dump("unable to open auth dialog\n" + ex + "\n"); }

    return "";
  },

  prompt: function(prompt, defaultResponse, okLabel, cancelLabel) {
    var parent = this._getParentWindow();
    try {
      obj = new Object();
      obj.prompt = prompt;
      obj.defaultResponse = defaultResponse;
      obj.okLabel = okLabel;
      obj.cancelLabel = cancelLabel;
      parent.openDialog
      ("chrome://komodo/content/commdlg/prompt.xul",
          "Komodo",
          "chrome,dialog=yes,close=yes,dependent=yes,modal=yes",
          obj);
      return obj.res;
    } catch(ex) { dump("unable to open prompt dialog\n" + ex + "\n"); }

    return "";
  },
  
  open: function(url, name, flags, obj) {
      var parent = this._getParentWindow();
      try {
          return parent.open(url, name, flags, obj);
      } catch (e) {
          logError(e);
          // Should set lastError here
      }
      return null;
  },
  
  openDialog: function(url, name, flags, obj) {
      var parent = this._getParentWindow();
      try {
          return parent.openDialog(url, name, flags, obj);
      } catch (e) {
          logError(e);
          // Should set lastError here
      }
      return null;
  },

  eval_: function(code) {
      var parent = this._getParentWindow();
      try {
          return parent.eval(code);
      } catch (e) {
          logError(e);
          // Should set lastError here
      }
      return null;
  },

  QueryInterface: function(iid) {
    if (!iid.equals(asIDialogProxy) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

/* module foo */

var dialogProxyModule = new Object();

dialogProxyModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    //debug("registering (all right -- a JavaScript module!)");
    compMgr.registerComponentWithType(DIALOGPROXY_CID, "JS Dialog Proxy Component",
                                      DIALOGPROXY_CONTRACTID, fileSpec, location,
                                      true, true, type);
}

dialogProxyModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(DIALOGPROXY_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return dialogProxyFactory;
}

dialogProxyModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}
    
/* factory object */
var dialogProxyFactory = new Object();

dialogProxyFactory.createInstance =
function (outer, iid) {
    //debug("CI: " + iid);
    //debug("IID:" + asIDialogProxy);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new asDialogProxy()).QueryInterface(iid);
}

/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return dialogProxyModule;
}
