# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

import urllib
import urlparse
import re
import xpcom
import xpcom.components
from StringIO import StringIO
from koLogger import *

_scheme="kodebugger"

class koSimpleStringInputStream:
    _com_interfaces_ = [xpcom.components.interfaces.nsIInputStream,
                        xpcom.components.interfaces.koIInputStreamHack]

    nonBlocking=0

    def __init__(self, data):
        self.data=StringIO(data.encode("UTF-8"))

    def close( self ):
        pass
    def available( self ):
        return self.data.len-self.data.pos

    def readStr( self, amount):
        return self.data.read(amount)

    read=readStr

    def get_observer( self ):
        raise "Not implemented"
    def set_observer( self, param0 ):
        raise "Not implemented"

class koSimpleStringChannel:
    """Note that this class only implements what we need in the short-term.
    Weird stuff like asynch can wait. """
    _com_interfaces_ = [xpcom.components.interfaces.nsIChannel]
    # If this object needs to be registered, the following 2 are also needed.
    #_reg_clsid_ = "{ddd88366-fd18-4090-9ec8-ea26062c07dd}"
    #_reg_contractid_ = "The.Object.Name"

    def __init__(self, url, data):
        self.uri=url
        self.name=url
        self.originalURI=url
        self.data=data

    # the new mozilla way (.9)
    def open(self):
        return koSimpleStringInputStream(self.data)

    def asyncOpen(self):
        return None

    # the old mozilla way (.8)
    #def get_localFile( self ):
    #    return None
    #
    #def openInputStream( self ):
    #    return koSimpleStringInputStream(self.data)
    #
    #def openOutputStream( self ):
    #    return None

_prefix="%s://"%_scheme
_template=_prefix+"%s/%s"

class koDebuggerURI:
    _com_interfaces_ = [xpcom.components.interfaces.koIDebuggerURI]
    _reg_clsid_ = "{d3f90cc1-5c7f-4764-a3e5-2a43c7a811cc}"
    _reg_contractid_ = "@activestate.com/koDebuggerURI"

    scheme=_scheme

    def init(self, uri):
        assert uri.startswith(_prefix)
        rest=uri[len(_prefix):]
        if rest.find("/")>=0:
            engine_name,self.path = rest.split("/", 1)
            self.path = urllib.unquote(self.path)
        else:
            engine_name = rest
            self.path = ""
        self.engine_name = engine_name.lower()

        self.pattern = re.compile("(.*?)://(.*?)/(.*)")
        # put it back together for retrieval as a unit
        self.spec=_template%( self.engine_name, self.path)
        return 

    def newURI(self, uri):
        "Create a new one based on this one."
        #Unobfuscation: generates kodebugger://<lang>/uri
        newobj = self.__class__()
        newBaseString = _template % (self.engine_name, uri)
        newobj.init(newBaseString)
        return newobj

    def getBaseURI(self, uri):
        "get the base uri from a kodebugger: uri"
        originalProtocol, baseuri = urllib.splittype( uri )
        if originalProtocol == _scheme:
            prot,lang,baseuri = self.pattern.search(uri).groups()
            return baseuri
        return uri

class koDebuggerProtocolHandler:
    _com_interfaces_ = [xpcom.components.interfaces.koIDebuggerProtocolHandler]
    _reg_clsid_ = "{c987776e-c55a-421b-a350-eb16618d6e3a}"
    _reg_contractid_ = "@mozilla.org/network/protocol;1?name=%s"%_scheme

    scheme=_scheme

    defaultPort=9966
    instantiated=0
    counter=0
    protocolFlags = 0
    
    def __init__(self):
        self.engines={}
        assert koDebuggerProtocolHandler.instantiated==0, "Should be singleton"
        koDebuggerProtocolHandler.instantiated=1

    def registerEngine(self, debugEngine, friendly_name="dbgengine"):
        # koDebuggerProtocolHandler.counter=koDebuggerProtocolHandler.counter+1
        # name="%s_%s"%(friendly_name.lower(),koDebuggerProtocolHandler.counter)
        name = friendly_name.lower()
        try:
            self.engines[name.lower()]=debugEngine
        except KeyError:
            logError( "No such engine %s"% name.lower())
        base=koDebuggerURI()
        base.init(_prefix+name.lower())
        debugEngine.baseURI=base

#        logDebug("registering debug engine as %s"%debugEngine.baseURI.spec)

    def newURI(self, newuri, base):
        if base is None:
            base = koDebuggerURI()
            base.init(newuri)
            return base
        else:
            return base.newURI(newuri)

    def newChannel(self, url):
        url_obj = koDebuggerURI()
        url_obj.init(url.spec)
        # Done by koDebuggerURI() constructor
        # url_obj.engine_name = url_obj.engine_name.lower()
        engine = self.engines[url_obj.engine_name]
        rc=koSimpleStringChannel(url, engine.getSourceCode(url_obj.path))
        return rc


class koDebuggerToFileResourceConverter:
    _com_interfaces_ = [xpcom.components.interfaces.koIResourceConverter]
    _reg_clsid_ = "{33C92FC6-279B-4abf-BC07-B476CDDF91AD}"
    _reg_contractid_ = "@activestate.com/koDebuggerToFileResourceConverter;1"

    def convert(self, source, target, url):
        assert url.startswith(source)

        path = url[len(source):]
        #XXX strip the leading slash from Linux paths but not from
        #XXX UNC paths
        if path.startswith("/") and not path.startswith("//"):
            result = target + path[1:]
        else:
            result = target + path
        return result
