#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

import types
from xpcom import components, _xpcom
from koLogger import logError, logDebug, logWarning
import indexer, os, threading

class koIndexer:
    _com_interfaces_ = [components.interfaces.koIIndexer]
    _reg_clsid_ = "{71878320-D69C-4e8f-AB88-9D070157E0FC}"
    _reg_contractid_ = "@activestate.com/koIndexer;1"
    _reg_desc_ = "Komodo Indexing Service"

    def __init__(self):
        self._index = index = indexer.ZPickleIndexer()
        dirs = components.classes['@activestate.com/koDirs;1'].getService()
        self.dbname = dirs.userDataDir + '/docs.db';
        # Create a proxy to the observer service, so that:
        # a) we don't have to worry if the service itself is thread-safe
        # b) the notification handlers are thread-safe
        svc = components.classes["@mozilla.org/observer-service;1"].getService(components.interfaces.nsIObserverService)
        self.observer_svc_proxy = _xpcom.getProxyForObject(None, components.interfaces.nsIObserverService, svc, _xpcom.PROXY_ALWAYS | _xpcom.PROXY_ASYNC)

    def load_index(self):
        self._index.load_index(self.dbname)
        
    def add_file(self, filename):
        # for some reason this doesn't work. 
        self._index.add_file(filename, ftype='text/html')

    def add_files(self, dirnames, filepattern, recursive, notify_message):
        if notify_message:
            t = threading.Thread(target=self._add_files_and_notify,
                            args=(dirnames, filepattern, recursive, notify_message))
            t.start()
        else:
            for dirname in dirnames.split(';'):
                self._index.add_files(dir=dirname, pattern=filepattern, descend=recursive)

    def _add_files_and_notify(self, dirnames, filepattern, recursive, notify_message):
        for dirname in dirnames.split(';'):
            self._index.add_files(dir=dirname, pattern=filepattern, descend=recursive)
        self.observer_svc_proxy.Notify(None, notify_message, "");

    def purge_index(self):
        if os.path.exists(self.dbname+'!'):
            try:
                os.remove(self.dbname+'!')
            except OSError, exc:
                logError("Couldn't erase file: " + self.dbname+'!' + ' ' + str(exc))
                return
        self._index.load_index(self.dbname)

    def save_index(self):
        self._index.save_index(self.dbname)

    def has_index(self):
        return os.path.exists(self.dbname+'!');
        
    def find(self, words):
        # _index.find() returns a dictionary:
        #   { <linenum> : <filename>\t<title>,
        #     <linenum> : <filename>\t<title>, 
        #     ... }
        # iff there are hits. If there are no hits the return value is '0'.
        # (Any empty dictionary would have been nice.)
        hits = self._index.find(words.split())
        if type(hits) == types.DictType:
            # need to filter out the hits corresponding to the webservices
            # "template.html" file
            # XXX need to move the template.html out of the doc area to clean
            # this hack.
            for k,v in hits.items():
                if v.endswith('%%%TITLE%%%'):
                    del hits[k]
            return '\n'.join(hits.values())
        else:
            return ''

if __name__ == '__main__':
    dirs = components.classes['@activestate.com/koDirs;1'].createInstance(components.interfaces.koIDirs)
    x = koIndexer()
    x.load_index()
    print x.find('ListSubscribe')

