#!python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

# Initialization Services for Komodo
#
# XXX The WebServices init stuff should just be part of a koWSService.init()
#     or the like. This module is for init stuff that must happen very early.

import sys
import os

from xpcom import components, nsError, ServerException
from koLogger import logError, logWarning, logDebug

import locale


#---- globals

_gSampleWebservices = [
    ('TemperatureService.wsdl', 
     'http://www.xmethods.net/sd/TemperatureService.wsdl',
     'TemperatureService', 
     'Returns current temperature in a given U.S. zipcode',
     'tempSvc'),
    ('BabelFishService.wsdl', 
     'http://www.xmethods.net/sd/BabelFishService.wsdl',
     'BabelFish',
     'Translates text of up to 5k in length, between a variety of languages',
     'babelfish'),
    ('ITimeService.wsdl', 
     'http://www.lemurlabs.com/projects/soap/itime/ITimeService.wsdl',
     'InternetTimeService',
     'Returns current Internet time', 
     'internetTime'),
    ('PSDBService.wsdl', 
     'http://soaptest.activestate.com:8080/PSDBService.wsdl',
     'PerlPackageDatabase',
     'Perl Package State Database Query Service', 
     'perlPackageDB')
     ]


#---- support routines

# The following imports, two globals and two methods are used by the import
# timing stuff, which is enabled by uncommenting a line in the initialize()
# call of koInitService
import __builtin__, time
_old_import = __builtin__.__import__
_imported_modules = {}
def _timed_import(name, *args, **kw):
    t1 = time.clock()
    mod = _old_import(name, *args, **kw)
    dt = time.clock() - t1
    realname = mod.__name__
    if not _imported_modules.has_key(name):
        _imported_modules[name] = dt
    else:
        _imported_modules[name] += dt
    _report_times()
    return mod

def _report_times():
    items = _imported_modules.items()
    items.sort(lambda i,j: cmp(i[1], j[1]))
    log = open("import_times.txt", "w")
    for module, time in items:
        print >>log, '%4.4f\t%s' % (time, module)
    log.close()

from progressbar import progressStep
def _progress_bar_import(name, *args, **kw):
    if name not in sys.modules:
        progressStep("Loading "+name+'...', 1)
    mod = _old_import(name, *args, **kw)
    return mod

#---- main component implementation

class KoInitService:
    _com_interfaces_ = [components.interfaces.koIInitService]
    _reg_clsid_ = "{BAECC764-52AC-46dc-9428-D23F05247818}"
    _reg_contractid_ = "@activestate.com/koInitService;1"
    _reg_desc_ = "Komodo Init Service"

    def __init__(self):
        self._isInitialized = 0

    def setPythonImportHook(self):
        # Comment this out to turn on timing of imports
        #__builtin__.__import__ = _timed_import
        __builtin__.__import__ = _progress_bar_import

    def setPlatformErrorMode(self):
        if sys.platform.startswith("win"):
            try:
                # Stop Windows displaying a dialog when removable
                # media is accessed, but not available
                # (eg, when a file from a floppy is on the MRU,
                # but no floppy is in the drive)
                import win32api, win32con
                win32api.SetErrorMode(win32con.SEM_FAILCRITICALERRORS)
            except ImportError:
                logError("Could not import win32api/win32con to set the "\
                         "Win32 Error Mode.")

    def _setencoding(self,encoding=None):
        try:
            if not encoding:
                encoding = locale.getdefaultlocale()
            if encoding is not None:
                reload(sys)
                sys.setdefaultencoding(encoding[1])
                del sys.setdefaultencoding
                return encoding
        except ValueError, e:
            # XXX Also getting this error occassionaly (as reported on
            # Komodo-beta). Should handle that.
            #    LookupError: unknown encoding
            if encoding:
                logError('Unsupported encoding for locale %s.%s'%\
                         (encoding[0],encoding[1]))
            else:
                logError('Unable to set encoding for locale %s'%\
                         os.getenv('LANG'))
        except LookupError, e:
            # windows sets codepages into the locale without
            # the leading 'cp' that python wants.  If the encoding
            # we received is a number, try cp# and see if it works.
            # eg. 1252 should be turned into cp1252
            try:
                isint = int(encoding[1])
                if sys.platform.startswith("win") and isint:
                    encoding[1] = 'cp'+encoding[1]
                    return self._setencoding(encoding)
            except ValueError,e:
                # not an int
                pass
            logDebug('Unsupported encoding for locale %s.%s'%\
                     (encoding[0],encoding[1]))
        except TypeError, e:
            if not sys.platform.startswith("win") and os.getenv("LANG") == "C":
                return ("en_US","iso8859-1")
            else:
                logDebug('Unable to set encoding for locale %s'%\
                            os.getenv('LANG'))
        except Exception, e:
            logDebug('Unable to determine system locale settings')
        return None
    
    def _setlocale(self):
        try:
            locale.setlocale(locale.LC_ALL,'')
        except:
            logDebug('Unable to set encoding for locale %s'%os.getenv('LANG'))
            return None
        return self._setencoding()

    def _removeEuroLocale(self):
        envvars = ['LC_ALL','LC_CTYPE','LANG','LANGUAGE']
        for var in envvars:
            orig_lang = os.getenv(var)
            if orig_lang and orig_lang.lower().endswith('@euro'):
                lang = orig_lang[:orig_lang.find('@euro')]
                os.environ[var] = lang
        
    def setEncoding(self):
        self._startup_locale = self._setlocale()
        if sys.platform.startswith("win"):
            # XXX - this is evil and should be dropped ASAP.
            # Changing the default encoding could impact other code.
            # Python 2.2 will have this support for all file system functions,
            # so as soon as that version is in Komodo, this MUST DIE.

            # On Windows, default python <=2.1 chokes on Unicode file
            # names.  If we change the encoding to "Latin-1", we can
            # at least support European chars transparently.
            # (and if we use "mbcs" we should be able to use
            # _any_ language supported by Windows, latin or otherwise.)
            reload(sys)
            sys.setdefaultencoding("mbcs")
            del sys.setdefaultencoding
            logDebug("encoding set to '%s'" % sys.getdefaultencoding())

        else: # Platforms other than Windows
            # Set the default encoding to that appropriate for the
            # current locale, except with UTF-8 support
            if not self._startup_locale:
                # XXX In python 2.2, setlocale fails if
                # the LANG envt variable has @euro at the end.
                # It's a bug in Python 2.2
                self._removeEuroLocale()
                self._startup_locale = self._setlocale()
            if not self._startup_locale:
                logDebug("Unable to determine the current locale "+\
                            "settings, please set LANG environment "+\
                            "variable. Defaulting to en_US.iso8859-1")
                # set the locale to something that we can
                # least let komodo startup and run with
                self._startup_locale = ['en_US','iso8859-1']
            elif self._startup_locale[1].lower() == 'utf':
                # RH 8 likes to use 'utf' rather than UTF-8,
                # but it will accept UTF-8
                self._startup_locale[1]='UTF-8'
            new_locale = self._startup_locale[0]+'.UTF-8'
            
            # We have to set pythons encoding to utf-8 so that
            # we can have some hope of interpreting file paths
            # that are written in different encodings, and to
            # make GTK show UTF8 fonts correctly.  for instance
            # saving a file with a latin-1 filename, but starting
            # komodo with encoding ISO8859-1.  If we do not do
            # this, and a latin-1 filename is in mru, komodo fails
            # to load
            
            # XXX this will break on older unix systems that do not
            # support UTF-8 -- it would be nice to detect this and
            # let the user know.
            try:
                # this bit is for file I/O
                reload(sys)
                sys.setdefaultencoding('utf-8')
                del sys.setdefaultencoding
            except Exception,e:
                logWarning('cannot set encoding to utf-8: %s'%e)
            
            # this bit is the GTK important bit
            # if it's not done, GTK won't display correctly
            try:
                locale.setlocale(locale.LC_CTYPE,new_locale)
            except Exception,e:
                logWarning("cannot reset LOCALE set to %s : %s" % (new_locale,e))

    def getStartupLocale(self):
        return "%s.%s" % \
                (self._startup_locale[0],self._startup_locale[1])

    def initProcessUtils(self):
        import koprocessutils
        koprocessutils.initialize()

    def initWS(self):
        # Set up for web services
        import WShelper
        dirs = components.classes["@activestate.com/koDirs;1"].getService()
        wsdir = os.path.join(dirs.userDataDir, "webservices")
        if not os.path.exists(wsdir):
            try:
                os.mkdir(wsdir)
            except Exception, e: # This sucks but it's not enough reason to stop Komodo
                logError("Exception creating directory '%s': %r [%r]" % (wsdir, e, e.args))
        # XXX all this needs to be fixed.
        wscachedir = os.path.join(wsdir, "cache")
        if not os.path.exists(wscachedir):
            try:
                os.mkdir(wscachedir)
            except Exception, e: # This sucks but it's not enough reason to stop Komodo
                logError("Exception creating directory '%s': %r [%r]" % (wscachedir, e, e.args))
                
        WShelper.SetCacheDir(wscachedir) 

    def finishInitialization(self):
        pass
        #XXX Creation of sample web services just causes headaches,
        #    So skip it. This used to be called after the pref service had
        #    started up.
        #bookmarkMgr = components.classes['@activestate.com/koBookmarkManager;1'].getService()
        #prefs = components.classes['@activestate.com/koPrefService;1'].getService().prefs
        #infoSvc = components.classes['@activestate.com/koInfoService;1'].getService()
        #keypref = 'installed_ws_samples' + infoSvc.version;
        #if not prefs.hasBooleanPref(keypref):
        #    # Setup sample web services
        #    import WShelper
        #    
        #    dirs = components.classes["@activestate.com/koDirs;1"].getService()
        #    templatedir = os.path.join(dirs.installDir, "WSDLs")
        #    for (fname, url, name, description, objectname) in _gSampleWebservices:
        #        origpath = os.path.join(templatedir, fname)
        #        try:
        #          WShelper.CacheWSDLFromLocalFile(origpath, url)
        #        except EnvironmentError:
        #          logWarning("Couldn't create web service: " + name)
        #          continue
        #        try:
        #          bookmark = bookmarkMgr.open(os.path.join(bookmarkMgr.rootDirectory,
        #                                      os.path.splitext(fname)[0]+'.asbk'))
        #          bookmark.url = url
        #          bookmark.shortname = objectname
        #          bookmark.description = description
        #          ok, errmsg = bookmark.save(name, 1)
        #        except EnvironmentError:
        #          logWarning("Couldn't save web service: " + name)
        #          continue
        #        if not ok:
        #          logWarning("Couldn't save web service: " + name + " " + errmsg)
        #          continue
        #    
        #    prefs.setBooleanPref(keypref, 1)
            


