from xpcom import nsError, ServerException, components
from xpcom.server.enumerator import SimpleEnumerator

import invocationutils

class koIInvocationInformation:
    _com_interfaces_ = components.interfaces.koIInvocationInformation
    _reg_clsid_ = "{f448f503-2b3a-4f63-abf4-99a842628e22}"
    _reg_contractid_ = "@activestate.com/InvocationInformation;1"
    def __init__(self):
        self.name = self.description = self.contractid = self.chrome_category = self.chrome_instance = None

class koIInvocationRegistryService:
    _com_interfaces_ = components.interfaces.koIInvocationRegistryService
    # If this object needs to be registered, the following 2 are also needed.
    _reg_clsid_ = "{1a67c86f-f330-4952-a415-94c0586bde01}"
    _reg_contractid_ = "@activestate.com/InvocationRegistryService;1"
    def __init__(self):
        self.invocation_to_info_map = None
    def _loadInvocations(self):
        assert self.invocation_to_info_map is None, "Already loaded!"
        self.invocation_to_info_map = {}
        registry, key = invocationutils._GetInvocationsRegistryAndKey()
        enum = registry.enumerateSubtrees(key)
        enum.first()
        while not enum.isDone():
            item = enum.currentItem(components.interfaces.nsIRegistryNode)
            info = koIInvocationInformation()
            name = item.name
            info.name = name
            info.description = registry.getString(item.key, "Description")
            info.contractid = registry.getString(item.key, "ContractID")
            info.chrome_category = registry.getString(item.key, "chrome_category")
            info.chrome_instance = registry.getString(item.key, "chrome_instance")
            self.invocation_to_info_map[name] = info
            enum.next()

    def getInvocationInformation( self, name ):
        # Result: koIInvocationInformation
        # In: name: string
        if self.invocation_to_info_map is None:
            self._loadInvocations()
        try:
            return self.invocation_to_info_map[name]
        except KeyError:
            return None

    def enumerateInvocations( self ):
        # Result: nsISimpleEnumerator
        # return the list sorted by description (as that is how the UI is likely to want it)
        if self.invocation_to_info_map is None:
            self._loadInvocations()
        vals = self.invocation_to_info_map.values()
        vals.sort( lambda i1, i2: cmp(i1.description, i2.description) )
        return SimpleEnumerator(vals)

if __name__=='__main__':
    svc = components.classes["@activestate.com/InvocationRegistryService;1"].getService()
    enum = svc.enumerateInvocations()
    while enum.hasMoreElements():
        item = enum.getNext()
        print "Have invocation", item.name, item.description
