#!/usr/bin/env python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

"""
Exposes some of Python's os module to other languages.
"""

import os
import stat
from xpcom import components, nsError, ServerException, COMException

lastErrorSvc = None

class koOsPath:
    _com_interfaces_ = [components.interfaces.koIOsPath]
    _reg_clsid_ = "{5f25f8f1-c0aa-4d2f-9519-e2832cedd296}"
    _reg_contractid_ = "@activestate.com/koOsPath;1"
    _reg_desc_ = "Makes os.path available outside of Python"

    def basename(self, path):
        return os.path.basename(path)

    def dirname(self, path):
        return os.path.dirname(path)

    def join(self, path1, path2):
        return os.path.join(path1, path2)

    def joinlist(self, paths):
        return os.path.join(*paths)

    def withoutExtension(self, path):
        return os.path.splitext(path)[0]
    
    def getExtension(self, path):
        return os.path.splitext(path)[1]

    def exists(self, path):
        return os.path.exists(path)
    
    def isfile(self, path):
        return os.path.isfile(path)
    
    def isdir(self, path):
        return os.path.isdir(path)

    
class koOs:
    _com_interfaces_ = [components.interfaces.koIOs]
    _reg_clsid_ = "{3348b94f-f27e-4819-a689-74978024d106}"
    _reg_contractid_ = "@activestate.com/koOs;1"
    _reg_desc_ = "Makes os available outside of Python"

    _path = None

    def __init__(self):
        global lastErrorSvc
        lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
               .getService(components.interfaces.koILastErrorService)

    def get_sep(self):
        return os.sep

    def get_path(self):
        if self._path is None:
            self._path = components.classes["@activestate.com/koOsPath;1"] \
                         .getService()
        return self._path

    def getenv(self, env):
        return os.getenv(env,'')
    
    def chdir(self, path):
        os.chdir(path)

    def getcwd(self):
        return os.getcwd()

    def system(self, command):
        return os.system(command)

    def startfile(self, command):
        os.startfile(command)

    def readfile(self, path):
        return open(path).read()

    def writefile(self, path, data):
        return open(path, 'wb').write(data)

    def mkdir(self, path):
        try:
            os.mkdir(path)
        except Exception, ex:
            lastErrorSvc.setLastError(0, str(ex))
            raise ServerException(nsError.NS_ERROR_INVALID_ARG, str(ex))

    def stat(self, filename):
        try:
            # See http://bugs.activestate.com/show_bug.cgi?id=21252.
            # """stat'ing C:\core\http\clicknames\htdocs\top.phtml returned
            # -1 for the m_time. This from Python's os.stat which uses
            # Windows' _stati64, which makes no mention of error cases
            # for m_time.
            #
            # Proposed workaround: Catch this in koOs::stat method -- if
            # m_time and/or c_time is -1 then consider this an error and
            # default it to a_time. If *a_time* is also -1 then raise an
            # exception (i.e. we will really have to figure out what is
            # going on.) """
            retval = list( os.stat(filename)[:10] )
            if retval[stat.ST_ATIME] == -1:
                raise OSError("stat() returned -1 access time for '%s'"\
                              % filename)
            if retval[stat.ST_MTIME] == -1:
                retval[stat.ST_MTIME] = retval[stat.ST_ATIME]
            if retval[stat.ST_CTIME] == -1:
                retval[stat.ST_CTIME] = retval[stat.ST_ATIME]
            return retval
        except OSError, ex:
            lastErrorSvc.setLastError(0, str(ex))
            raise ServerException, (nsError.NS_ERROR_FILE_NOT_FOUND, str(ex))


