#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.


from xpcom import components
from xpcom.server.enumerator import *
from koLanguageServiceBase import KoCompletionLanguageService
import os, sys, traceback, re
import tempfile
import string
import threading, thread
import time
import convertURL

import WShelper

# XXX This should be centralized, used in koPerlAutoComplete
def caseIndependentSort(something, other):
    something, other = string.lower(something), string.lower(other)
    return cmp(something, other)
             

wordbreaks = "_" + string.uppercase + string.lowercase + string.digits

# > is not the best solution, -> is the actual deliminator for
# objects
variablespec  = '$.>"'

global_spec = "_global_"

# filesparsed contains all files parsed
filesparsed={}
# needfile contains a list of files included by the file that is the key
needfile={}
# infile contains a list of files that the key file is included in 
infile={}
# classindex tells us what file a class definition is contained in
classindex={}
# functionindex tells us what file a function is defined in
functionindex={}

#config observer class to update php config info

class KoPHPCompletion(KoCompletionLanguageService):
    _com_interfaces_ = [components.interfaces.koICompletionLanguageService]
    _reg_desc_ = "PHP Calltip/AutoCompletion Service"
    _reg_clsid_ = "{9BC17B87-140A-4da4-8730-A6EC9F6953E7}"
    _reg_contractid_ = "@activestate.com/koPHPCompletionLanguageService;1"
    
    _globalFuncs = None
    _retiptype = None
    _retipargs = None
    php_config = None
    
    def __init__(self):
        # set up config observer
        if not self.php_config:
            php_lang = components.classes["@activestate.com/koLanguage?language=PHP;1"].getService(components.interfaces.koILanguage)
            self.php_config = php_lang.getLanguageService(components.interfaces.koIInterpreterLanguageService)
            
        self.triggers = None
        self.triggersCallTip = '(),'
        self._braceCount = 0
        self._scintilla = None
        self._functext = ''
        self._ok = 0
        self._lastfuncnames = []
        self._lastcompletion = None
        self.currentFile = ""
        self.currentFileText = None
        self.doInclude = 0
        self._lastscan = -1
        self.scanLock = threading.Lock()
        if not self._globalFuncs:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
              getService(components.interfaces.koIDirs)
            self._fname = os.path.join(koDirs.installDir,"php","phpfunctips.txt")
            threading.Thread(target = self._background, args=()).start()
        
    def _background(self, *args):
        self._retiptype = re.compile(r'^^([a-zA-Z]*)\s.*?')
        self._retipargs = re.compile(r'^.*?(\(.*?\))$')
        self.load_funcs()
        
    def load_funcs(self):
        # read PHP function definitions
        try:
            inputfile = open(self._fname,"r")
            lines = inputfile.readlines()
            inputfile.close()
            i=0
            theRe = re.compile(r'^[a-zA-Z]*\s(.*?)\(')
            self._globalFuncs = PHPFile(global_spec)
            for line in lines:
                line=line.strip()
                tmatch=re.search(theRe,line)
                if tmatch:
                    self._globalFuncs.functions[tmatch.group(1)] = (line, 0)
            self._ok = 1
            #print "PHP NumFunctions: ",len(self.funcnames)
        except:
            #print "PHP Failed loading function list"
            pass

    def _scanFileCompletion(self):
        if not self.scanLock.acquire(0): return
        PHPLexer(self.currentFile, self.currentFileText, self.doInclude, self.php_config.includePath)
        self.currentFileText = ""
        self.scanLock.release()
        
    def scanFileCompletion(self, filename):
        
        # prevent multiple scans at the same time
        if filename:
            self.currentFile = convertURL.LocalPathFromURL(filename)
        else:
            # XXX ok for beta, need to deal with multiple new files
            self.currentFile = "__newfile__"
        self.doInclude = 1
        self.currentFileText = ""
        threading.Thread(target = self._scanFileCompletion, args=()).start()
    
    def scanBufferCompletion(self, scimoz, filename):
        # prevent multiple scans at the same time
        self._scintilla = scimoz
        lastfile = self.currentFile
        if filename:
            # get local file system path so we can include files correctly
            self.currentFile = convertURL.LocalPathFromURL(filename)
            
            # allow filename to be used anyway, this allows remote files
            # to be correctly parsed, we only do this on parsing the buffer
            if not self.currentFile:
                self.currentFile = filename
        else:
            # XXX ok for beta, need to deal with multiple new files
            self.currentFile = "__newfile__"

        self.doInclude = 1
        self.currentFileText = scimoz.text
        # only do this if we actually have something to parse
        if self.currentFileText:
            # delay always, to speed typing
            now = time.clock()
            if lastfile == self.currentFile and \
               filesparsed.has_key(self.currentFile) and \
               now - self._lastscan < 1:
                #print "aborting lexical scan due to timer now=",now," lastscan=",self._lastscan," diff ",now - self._lastscan
                self._lastscan  = now
                return
            self._lastscan = now

            threading.Thread(target = self._scanFileCompletion, args=()).start()
    
    def _getClassTipArgs(self, funcname, classname):
        if classindex.has_key(classname):
            _file = filesparsed[classindex[classname]]
            cl = _file.classes[classname]
            if cl.functions.has_key(funcname):
                return cl.functions[funcname][0]
            # check parent classes
            nest = 0
            while cl.extends and classindex.has_key(cl.extends):
                _file = filesparsed[classindex[cl.extends]]
                cl = _file.classes[cl.extends]
                if cl.functions.has_key(funcname):
                    return cl.functions[funcname][0]
                # protect against endless loop
                nest = nest + 1
                if nest > 3: break
        return None
        
    # get the function arguments
    def gettipargs(self, funcname, classname):
        # check the global namespace
        if not classname:
            # check the global built in php functions
            if self._globalFuncs.functions.has_key(funcname):
                return self._globalFuncs.functions[funcname][0]
            # check the user namespace
            if functionindex.has_key(funcname):
                _file = filesparsed[functionindex[funcname]]
                if _file.functions.has_key(funcname):
                    return _file.functions[funcname][0]
                else:
                    return None
            # check for class constructors
            return self._getClassTipArgs(funcname, funcname)
        else:
            return self._getClassTipArgs(funcname, classname)
        return None

    def _combineItems(self, items, inc_items):
        if len(items):
            for item in inc_items:
                #print "removing ",repr(item)
                try:
                    items.remove(item)
                except ValueError, e:
                    pass
        items.extend(inc_items)
        return items
        
    def _getvariables(self, filename, name, curline):
        # first, recurse the included files
        items = []
        if needfile.has_key(filename):
            for file in needfile[filename]:
                inc_items = self._getvariables(file, name, curline)
                #print "found %d vars in %s"%(len(inc_items),file)
                items = self._combineItems(items, inc_items)
        
        if filesparsed.has_key(filename):
            _file = filesparsed[filename]
            varnames = _file.variables.keys()
            inc_items = [key for key in varnames if key.lower().startswith(name.lower()) \
                         and (filename != self.currentFile or _file.variables[key][1] <= curline)]
            items = self._combineItems(items, inc_items)
        return items

    def getvariables(self, name, curline):
        items = self._getvariables(self.currentFile, name, curline)
        items.sort(caseIndependentSort)
        return string.join(items, chr(self.completionSeparator)), 1
    
    # given a linenumber, returns the classname at that line
    #currentclass is always in currentfile!
    def getCurrentClassName(self, lineno):
        if filesparsed.has_key(self.currentFile):
            _file = filesparsed[self.currentFile]
            classes = _file.classes.keys()
            for classname in classes:
                if lineno > _file.classes[classname].linestart and \
                   lineno < _file.classes[classname].lineend:
                    return classname
        return None

    # given a classname, returns the members
    def _getClassMembers(self, classname):
        items = []
        if classindex.has_key(classname):
            fn = classindex[classname];
            if filesparsed.has_key(fn):
                _file = filesparsed[fn]
                if _file.classes.has_key(classname):
                    cl = _file.classes[classname]
                    items = cl.functions.keys()
                    items.extend(cl.members.keys())
                    # if we subclassed, get parents info also
                    if cl.extends:
                        pitems = self._getClassMembers(cl.extends)
                        # remove overridden functions
                        items = self._combineItems(pitems, items)
        return items

    def getClassMembers(self, classname):
        items = self._getClassMembers(classname)
        items.sort(caseIndependentSort)
        return string.join(items, chr(self.completionSeparator)), 1
    
    # given a variable name, returns the class name
    def getClassName(self, varname, filename=None):
        if not filename: filename = self.currentFile
        if filesparsed.has_key(filename):
            _file = filesparsed[filename]
            if _file.variables.has_key(varname):
                # get the classname, exit if not a class variable
                return _file.variables[varname][0]
            #look in included files to find variable
            if needfile.has_key(filename):
                for fn in needfile[filename]:
                    cn = self.getClassName(varname,fn)
                    if cn: return cn
        return None


    def _getclasses(self, filename):
        # first, recurse the included files
        items = []
        if needfile.has_key(filename):
            for file in needfile[filename]:
                inc_items = self._getclasses(file)
                items = self._combineItems(items, inc_items)
        
        if filesparsed.has_key(filename):
            _file = filesparsed[filename]
            inc_items = _file.classes.keys()
            items = self._combineItems(items, inc_items)
        return items
    
    # get a sorted list of the classes available
    def getclasses(self):
        items = self._getclasses(self.currentFile)
        items.sort(caseIndependentSort)
        return string.join(items, chr(self.completionSeparator)), 1

    def _getfunctions(self, filename):
        # first, recurse the included files
        items = []
        if needfile.has_key(filename):
            for file in needfile[filename]:
                inc_items = self._getfunctions(file)
                items = self._combineItems(items, inc_items)
        
        if filesparsed.has_key(filename):
            _file = filesparsed[filename]
            inc_items = _file.functions.keys()
            items = self._combineItems(items, inc_items)
        return items
    
    # get the available completions
    def getmethods(self, name):
        items = []
        if self._lastcompletion is not None and \
            self._lastcompletion == name[:len(self._lastcompletion)]:
            items = [key for key in self._lastfuncnames if key.lower().startswith(name.lower())]

            #if same list, skip doing it.
            if self._scintilla.autoCActive() and \
               (len(items) == len(self._lastfuncnames)): 
                #print "PHP keeping current autocomplete"
                return None, 0
        
        if (len(items)==0):
            funcnames = self._getfunctions(self.currentFile)
            _functions = [key for key in funcnames if key.lower().startswith(name.lower())]
            funcnames = self._globalFuncs.functions.keys()
            items = [key for key in funcnames if key.lower().startswith(name.lower())]
            items.extend(_functions)
            items.sort(caseIndependentSort)

        self._lastcompletion = name
        self._lastfuncnames = items
        return string.join(items, chr(self.completionSeparator)), 1

    def StartCallTip(self, ch, scimoz):
        if not self._ok: return
        s = self._scintilla = scimoz
        selStart, selEnd = s.selectionStart, s.selectionEnd
        if selEnd == selStart and selStart > 0:
            style = s.getStyleAt(selStart - 1)
            if style == s.SCE_HPHP_COMMENT or style == s.SCE_HPHP_COMMENTLINE:
                return
            
            if s.callTipActive():
                if ch == ')':
                    self._braceCount = self._braceCount - 1
                    if (self._braceCount < 1):
                        s.callTipCancel()
                elif ch == '(':
                    self._braceCount = self._braceCount + 1
                else:
                    self._ContinueCallTip()
            elif s.autoCActive():
                if ch == '(':
                    self._braceCount = self._braceCount + 1
                    self._StartCallTip()
                elif ch == ')':
                    self._braceCount = self._braceCount - 1
            else:
                if ch == '(':
                    self._braceCount = 1
                    self._StartCallTip()

    def DoCallTip(self, scimoz):
        if not self._ok: return
        self._scintilla = scimoz
        self._StartCallTip()

    def AutoComplete(self, ch, scimoz):
        if not self._ok or scimoz.autoCActive(): return

        self._scintilla = scimoz

        style = scimoz.getStyleAt(scimoz.currentPos-1) & 127
        #print "keyPressed style: ",style," char: ", scimoz.text[scimoz.currentPos-1]
        #prevstyle = scimoz.getStyleAt(scimoz.currentPos-2) & 127
        #print "keyPressed prevstyle: ",prevstyle," char: ", scimoz.text[scimoz.currentPos-2]
        if style == scimoz.SCE_HPHP_DEFAULT:
            self._StartAutoComplete()
        elif style == scimoz.SCE_HPHP_VARIABLE or \
            style == scimoz.SCE_HPHP_HSTRING_VARIABLE:
            self._VariableAutoComplete()
        else:
            char = scimoz.text[scimoz.currentPos-1]
            if (char == ">" or char == ":") and \
                (style == scimoz.SCE_HPHP_OPERATOR or \
                (scimoz.currentPos > 3 and scimoz.getStyleAt(scimoz.currentPos-3) & 127 == scimoz.SCE_HPHP_HSTRING_VARIABLE)):
                pchar = scimoz.text[scimoz.currentPos-2]
                if pchar == "-":
                    # class completion!  varname->function
                    self._ClassAutoComplete()
                elif pchar == ":":
                    # class completion!  classname::function
                    self._ClassAutoComplete(1)

    def _ClassAutoComplete(self, explicit = 0):
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        linelen, lastline = s.getCurLine()
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        pos = curpos - s.positionFromLine(curline)-2
        if pos <= 0 or pos > linelen: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)
        #print "PHP word is: [%s]"%(text)

        if text=="this" or text == "parent":
            # find the current class name
            text = self.getCurrentClassName(curline)
            #print "PHP classname: [%s]"%(text)
        elif not explicit:
            text = self.getClassName(text)
            
        if text=="": 
            if s.autoCActive(): s.autoCCancel()
            return
            
        completions, cancel = self.getClassMembers(text)
        if completions:
            #print "PHP Completions: ",completions
            s.autoCShow(0, completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()
        
    def _VariableAutoComplete(self):
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        text = self._GetPrevWord()
        #print "PHP text: [%s]"%(text)
        if len(text) < 2 or text=="": 
            if s.autoCActive(): s.autoCCancel()
            return

        completions, cancel = self.getvariables(text, curline)
        #print "PHP Completions: ",repr(completions)
        if completions:
            s.autoCShow(len(text), completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()
    
    def _StartAutoComplete(self):
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        linelen, lastline = s.getCurLine()
        #print "looking for new in [%s]"%(lastline)
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        pos = curpos - s.positionFromLine(curline)
        if pos <= 0 or pos > linelen: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)
        #print "PHP text: [%s]"%(text)
        if text == "" and lastline[(pos-1)]==" ":
            pos = pos - 1
            #print "char just typed [%s]"%(lastline[pos])
            text = lastline[:pos]
            text = self._GetPrevWordFromText(text)+" "
        
        if len(text) < 3 or text == "": 
            if s.autoCActive(): s.autoCCancel()
            return
        
        # new class completion
        if text == "new ":
            completions, cancel = self.getclasses()
            cpos = 0
        else:
            completions, cancel = self.getmethods(text)
            cpos = len(text)
        if completions:
            #print "PHP Completions: ",completions
            s.autoCShow(cpos, completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()

    def _StartCallTip(self):
        tip = None
        cname = None
        s = self._scintilla
        linelen, lastline = s.getCurLine()
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        pos = curpos - s.positionFromLine(curline)-1
        if pos <= 0: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)

        # we have the function name, is this a class member?
        ctext = lastline[:(pos - len(text))]
        pos = len(ctext)-1
        #print "CALLTIP: ",ctext
        # at least 4 chars required: $a->
        if pos > 2 and ctext[pos]==">" and ctext[pos-1]=="-":
            ctext = lastline[:pos-1]
            ctext = self._GetPrevWordFromText(ctext)
            if ctext == "this":
                cname = self.getCurrentClassName(curline)
            else:
                cname = self.getClassName(ctext)
        # at least 3 chars required for explicit class calls "a::"
        elif pos > 1 and ctext[pos]==":" and ctext[pos-1]==":":
            ctext = lastline[:pos-1]
            #print "CALLTIP text: ",ctext
            cname = self._GetPrevWordFromText(ctext)
            #print "CALLTIP: class ",cname

        if text:
            #print "PHP CallTip on: ", text, " in ",cname
            tip = self.gettipargs(text, cname)
            #print "Call Tip: ",repr(tip)
            if tip:
                self._funcdef = tip 
                self._scintilla.callTipShow(curpos-1, tip)
                self._ContinueCallTip()
        
    def _ContinueCallTip(self):
        s = self._scintilla
        start = 0
        l, current = s.getCurLine()
        funcdef = self._funcdef

        if not funcdef: return

        commas = current.count(',')
        if funcdef.find('(') != -1: # XXX hack
            while (start < len(funcdef) and funcdef[start] != '('):
                start = start + 1
        if (start < len(funcdef) and funcdef[start] == '('):
            start = start + 1
        while (start < len(funcdef) and commas > 0) :
            if (funcdef[start] == ',' or funcdef[start] == ')'):
                commas = commas - 1
            start = start + 1
        if (start < len(funcdef) and (funcdef[start] == ',' or funcdef[start] == ')')):
            start = start + 1
        end = start
        if (end < len(funcdef)):
            end = end + 1
        while (end < len(funcdef) and funcdef[end] != ',' and funcdef[end] != ')'):
            end = end + 1
        s.callTipSetHlt(start, end)



    def _GetPrevWord(self, bAllowCalls = 0):
        s = self._scintilla
        linelen, lastline = s.getCurLine()
        curpos = s.selectionStart
        linestart = s.positionFromLine(s.lineFromPosition(curpos))
        pos = curpos - linestart
        fulltext = lastline[:pos]

        index = len(fulltext)-1
        wordbreaks_use = wordbreaks
        if bAllowCalls: wordbreaks_use = wordbreaks_use + "()[]"
        while index>=0:
            char = fulltext[index]
            if char not in wordbreaks_use: break
            index = index-1
        # check preceeding character, dont calltip on variables
        if index >=0:
            char = fulltext[index]
            #if char in variablespec: return ""
        word = fulltext[index+1:]
        return word

    def _GetPrevWordFromText(self, text):
        index = len(text)-1
        wordbreaks_use = wordbreaks
        while index>=0:
            char = text[index]
            if char not in wordbreaks_use: break
            index = index-1
        # check preceeding character, dont calltip on variables
        if index >=0:
            char = text[index]
            #if char in variablespec: return ""
        word = text[index+1:]
        return word



re_newlines = re.compile("(\r\n|\r|\n)")
re_comment_line = re.compile("(//.*|#.*)")
re_comment_block = re.compile(r'((?:/\*.*?\*/))',re.S|re.M)
if sys.version_info[:2] == (2,0):
    re_string = re.compile(r'^(?<=(?<!\\)"|\').*?((?<!\\)\1)',re.S|re.M)
else:
    re_string = re.compile(r'((?<!\\)"|\').*?((?<!\\)\1)',re.S|re.M)
re_string_delim = re.compile(r'((?:(?<!\\)"|\'))')
re_classvar = re.compile(r'(?:\s*\$([\w_]*?)\s*=\s*new\s*([\w_]*?)(?:\(.*?\)|\s*)\s*);',re.I|re.M)
re_variable = re.compile(r'(?:\s*\$([\w_]*?)(?:(?:\s*=.*?)|\s*));',re.I|re.M)    
re_include = re.compile(r'\s*(?:require|include|require_once|include_once)\s*[\(|\s]*[\"|\']\s*(.*?)\s*[\"|\'][\)|\s]*;',re.I)
re_block_begin = re.compile("{")
re_block_end = re.compile("}")
re_function = re.compile(r'(?:function\s+(?:[&]*?)\s*([\w_]*?)\s*(\(.*?\)))',re.I|re.S)
re_class = re.compile(r'((?<=\s)class\s+([\w_]*)\s*(?:(?:extends\s+([\w_]*)\s*)|\s*))',re.I|re.S)
re_classmember = re.compile(r'(?:var\s+\$([\w_]*?)\s*[=|;])',re.I)
re_php_start = re.compile("<\?php")
re_php_end = re.compile("\?>")
re_label = re.compile("([a-zA-Z_][a-zA-Z0-9_]*)")
re_lnum = re.compile("([0-9]+)")
re_whitespace = re.compile("[ \t]+")
re_any_char = re.compile("(.)") # |[\n]
re_webservice = re.compile(r'(?:\s*\$([\w_]*?)\s*=\s*WebService\:\:ServiceProxy\s*(?:\(\s*[\'\"]+(.*?)[\'\"]+\s*(?:,\s*[\'\"]+(.*?)[\'\"]+)?\s*\)\s*)\s*);',re.I)
SKIPTOK = 0x01 # don't consider this a token that is to be considered a part of the grammar, like '\n'
MAPTOK = 0x02  # use the token associated with the pattern when it matches
EXECFN= 0x04   # execute the function associated with the pattern when it matches
USETXT = 0x08  # if you match a single character and want its ascii value to be the token

# Lexer class borrowed from the PyLRd project,
# http://starship.python.net/crew/scott/PyLR.html
class Lexer:

    eof = -1

    def __init__(self):
        self.tokenmap = {}
        self.prgmap = {}
        self.prglist = []
        self.lasttok = -1
        self.text = ""
        self.textindex = 0

    def nexttok(self):
        self.lasttok = self.lasttok + 1
        return self.lasttok

    def settext(self, t):
        self.text = t
        self.textindex = 0

    def addmatch(self, prg, func=None, tokname="", flags=MAPTOK|EXECFN):
        self.prglist.append(prg)
        tok = -2
        if not func:
            flags = flags & ~EXECFN
        if not tokname:
            flags = flags & ~MAPTOK
        if flags & MAPTOK:
            self.tokenmap[tokname] = tok = self.nexttok()
        else:
            tok = self.nexttok()
        self.prgmap[prg] = tok, flags, func

    def scan(self):
        for prg in self.prglist:
            mo = prg.match(self.text, self.textindex)
            if not mo: 
                continue
            self.textindex = self.textindex + len(mo.group(0))
            tmpres = mo.group(0)
            t, flags, fn = self.prgmap[prg]
            if flags & EXECFN:
                tmpres = apply(fn, (mo,))
            if flags & USETXT:
                t = ord(mo.group(0)[0])
            return (t, tmpres)
        if self.textindex >= len(self.text):
            return (self.eof, "")
        raise "Syntax Error in lexer"


class PHPClass:
    def __init__(self, name, extends, lineno, depth):
        self.name = name
        self.extends = extends
        self.linestart = lineno
        self.lineend = 0
        self.depth = depth
        self.members = {} # declared class variables
        self.variables = {} # all variables used in class
        self.functions = {}
    
    def dump(self):
        # dump our contents to human readable form
        print "CLASS ", self.name,
        if self.extends: print " EXTENDS ",self.extends,
        print

        print "Members:"
        members = self.members.keys()
        for var in members:
            print "    var ",var," line ",self.members[var]
        functions = self.functions.keys()
        for fn in functions:
            print "    function ",fn, self.functions[fn],";"

        print "variables:"
        variables = self.variables.keys()
        for var in variables:
            print "    var ",var," on line ",self.variables[var]

class PHPFile:
    def __init__(self, filename):
        self.filename = filename
        self.functions = {} # functions declared in file
        self.classes = {} # classes declared in file
        self.variables = {} # all variables used in file

    def dump(self):
        # dump our contents to human readable form
        print "FILE ", self.filename,
        print

        print "classes:"
        classes = self.classes.keys()
        for cs in classes:
            self.classes[cs].dump()

        print "functions:"
        functions = self.functions.keys()
        for fn in functions:
            print "    function ",fn, self.functions[fn],";"

        print "variables:"
        variables = self.variables.keys()
        for var in variables:
            print "    var ",var," on line ",self.variables[var]



class PHPLexer:

    def __init__(self, filename, data="", include=1, include_path=""):
            
        self.lex_matches = [
            ('newline', re_newlines, self.incline, EXECFN|SKIPTOK),    
            ('comment_line', re_comment_line,None,SKIPTOK),
            ('comment_block', re_comment_block,self.doMultiLineBlock,EXECFN|SKIPTOK),
            ('include',re_include,self.include_file,EXECFN|SKIPTOK),
            ('string',re_string,self.doMultiLineBlock,EXECFN|SKIPTOK),
            #('string',re_string_delim, self.toggleStringState, EXECFN|SKIPTOK),
            ('block_begin', re_block_begin, self.incBlock, EXECFN|SKIPTOK),    
            ('block_end', re_block_end, self.decBlock, EXECFN|SKIPTOK),    
            ('webservice', re_webservice, self.addWebService, EXECFN|SKIPTOK),
            ('classvar', re_classvar, self.addClassVar, EXECFN|SKIPTOK),
            ('variable', re_variable,self.addVariable,EXECFN|SKIPTOK),
            ('function', re_function, self.addFunction, EXECFN|SKIPTOK),
            ('class', re_class, self.addClass, EXECFN|SKIPTOK),
            ('classmember', re_classmember, self.addClassMember, EXECFN|SKIPTOK),
            ('php_start', re_php_start,self.toggleCodeState,EXECFN|SKIPTOK),
            ('php_end', re_php_end,self.toggleCodeState,EXECFN|SKIPTOK),
            ('label', re_label, None, 0),    
            ('lnum', re_lnum, None,  SKIPTOK),    
            ('whitespace', re_whitespace, None, SKIPTOK),    
            ('any_char', re_any_char, None, SKIPTOK),    
        ]
        
        self.lineno = 1
        self.phpcode = 0
        self._stringDelim = ""
        self.depth = 0
        self.classStack = []
        self.currentClass = None
        self.include_path = include_path
        if not filename:
            # set a filename so we can handle completion
            self.basedir = ""
            self.filename = "___newfile___"
        else:
            self.basedir = os.path.dirname(filename)
            self.filename = self.findfile(filename)
        
        self.l = Lexer()
        
        # add regexes to lexer
        for p in self.lex_matches:
            #print "adding ",p[0]
            flags = p[3]
            if not flags: flags = MAPTOK|EXECFN
            self.l.addmatch(p[1],p[2],p[0],flags)

        # class to hold information
        self.fileinfo = PHPFile(self.filename)
        
        # set up index
        needfile[self.filename] = []
        
        self.doincludes = include
        if data:
            self.scanData(data)
            filesparsed[self.filename]=self.fileinfo
        else:
            if self.scanFile():
                filesparsed[self.filename]=self.fileinfo
            else:
                # failed fail scan, make sure we remove
                # entry in case situation is fixed outside komodo
                # (ie.php.ini is updated with proper include_path
                if filesparsed.has_key(self.filename):
                    del filesparsed[self.filename]
                if needfile.has_key(self.filename):
                    del needfile[self.filename]
        

    def idfunc(self, m):
        if not self.phpcode: return
        #print "ID: ",m.group(0)
        return m.group(0)

    def incline(self, m):
        self.lineno = self.lineno + 1
        #print "line: ",self.lineno
        return ""

    def toggleStringState(self, m):
        if self.phpcode:
            #print "exiting code fragment"
            self.phpcode = 0
            self._stringDelim = m.group(0)
        elif self._stringDelim == m.group(0):
            #print "entering code fragment"
            self.phpcode = 1
        return ""
        
    def toggleCodeState(self, m):
        if self.phpcode:
            #print "exiting code fragment"
            self.phpcode = 0
        else:
            #print "entering code fragment"
            self.phpcode = 1
        return ""

    # parses included files
    def include_file(self, m):
        if not self.phpcode: return
        filename = self.findfile(m.group(1))

        # XXX Very simple prevention of include looping.  Realy should
        # recurse the indices to make sure we are not creating a loop
        if self.filename == filename: return ""
        
        # parse the included file
        if self.doincludes and not filesparsed.has_key(filename):
            # build new include path
            path = string.join([self.basedir, self.include_path], os.pathsep)
            PHPLexer(filename, "", self.doincludes, path)
            if not filesparsed.has_key(filename):
                return ""
        
        # add the included file to our list of included files
        try:
            needfile[self.filename].index(filename)
        except ValueError, e:
            needfile[self.filename].append(filename)
            
        # add this file to the infile list
        if not infile.has_key(filename):
            infile[filename] = []
        try:
            infile[filename].index(self.filename)
        except ValueError, e:
            infile[filename].append(self.filename)
            
        return ""

    def addWebService(self, m):
        if not self.phpcode: return
        var = m.group(1)
        url = m.group(2)
        hash = m.group(3)
        info = None
        classname = None
        #print "get webservice: var [%s] url[%s]"%(var,url)
        try:
            info = WShelper.Get_Completion_Info(url)
            if info.has_key("service_name"):
                classname = info["service_name"]+"_webservice"
        except Exception, e:
            # XXX This catch-all is here because WShelper does not return
            # well-defined exceptions is reponse to a WSDL file that it
            # cannot parse.

            observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                          .getService(components.interfaces.nsIObserverService)
            sm = components.classes["@activestate.com/koStatusMessage;1"]\
                 .createInstance(components.interfaces.koIStatusMessage)
            sm.category = "webservices_intellisense"
            sm.msg = 'AutoComplete is not available for "%s" [%r]' % (url, e.args) 
            sm.timeout = 5000
            sm.highlight = 1
            try:
               observerSvc.Notify(sm, 'status_message', None)
            except COMException, e:
               pass
            
            return None
                        
        #print "adding webservice: var [%s] ws [%s] url[%s]"%(var,classname,url)
        if classname and not self.fileinfo.classes.has_key(classname):
            ws_class = PHPClass(classname,"soapclient", self.lineno, self.lineno)
            # class methods are in info["methods"]
            functions = info["methods"].keys()
            for func in functions:
                tip = ""
                # build completion line from info["methods"][func]["in"|"out"]
                argout = info["methods"][func]["out"]
                for arg in argout:
                    if len(tip) > 1:
                        tip += ", "+arg
                    else:
                        tip += arg
                
                if tip: tip += " "
                tip += func
                
                argin = info["methods"][func]["in"]
                tip += "("
                tipin = ""
                for arg in argin:
                    if len(tipin) > 1:
                        tipin += ", "+arg
                    else:
                        tipin += arg
                tip += tipin+")"
                ws_class.functions[func] = (tip, self.lineno)
                
            # add the web service into our completion structure
            self.fileinfo.variables[var]=(classname, self.lineno)
            self.fileinfo.classes[classname]=ws_class
            classindex[classname]=self.fileinfo.filename
            #print "added ",classname, " on line ", self.lineno, " file ",self.fileinfo.filename
            
            
    # takes a block, and figures out how many lines it spans, to
    # keep lineno correct
    def doMultiLineBlock(self, m):
        if not self.phpcode: return
        nl = re_newlines.findall(m.group(0))
        #print "block start at lineno ",self.lineno
        self.lineno = self.lineno + len(nl)
        #print "block had ",len(nl)," lines, lineno is ", self.lineno

    def incBlock(self, m):
        if not self.phpcode: return
        self.depth = self.depth+1
        #print "depth at ", self.depth
        return ""

    def decBlock(self, m):
        if not self.phpcode: return
        self.depth = self.depth-1
        #print "depth at ", self.depth
        if self.currentClass and self.currentClass.depth == self.depth:
            #print "done with class ",self.currentClass.name, " at depth ",self.depth
            self.currentClass.lineend = self.lineno
            self.currentClass = self.classStack.pop()
        return ""

    def addFunction(self, m):
        if not self.phpcode: return
        #print "FUNC: %s%s on line %d"%(m.group(1),m.group(2), self.lineno)
        self.doMultiLineBlock(m)
        if self.currentClass:
            if not self.currentClass.functions.has_key(m.group(1)):
                self.currentClass.functions[m.group(1)] = (m.group(2), self.lineno)
        else:
            if not self.fileinfo.functions.has_key(m.group(1)):
                self.fileinfo.functions[m.group(1)] = (m.group(2), self.lineno)
                functionindex[m.group(1)]=self.fileinfo.filename
        return ""

    def addClass(self, m):
        if not self.phpcode: return
        if not self.fileinfo.classes.has_key(m.group(2)):
            # push the current class onto the class stack
            self.classStack.append(self.currentClass)
            # make this class the current class
            self.currentClass = PHPClass(m.group(2),m.group(3), self.lineno, self.depth)
            self.fileinfo.classes[m.group(2)]=self.currentClass
            #print "adding classindex[%s]=%s"%(m.group(2),self.fileinfo.filename)
            classindex[m.group(2)]=self.fileinfo.filename
            #print "CLASS: %s extends %s on line %d in %s at depth %d"% (m.group(2),m.group(3), self.lineno, self.filename, self.depth)
            self.doMultiLineBlock(m)
        else:
            # shouldn't ever get here
            pass
        return ""
    
    def addClassMember(self, m):
        if not self.phpcode: return
        if self.currentClass:
            if not self.currentClass.members.has_key(m.group(1)):
                self.currentClass.members[m.group(1)]=self.lineno
            #print "CLASSMBR: ", m.group(1)
        return ""
    
    def addClassVar(self, m):
        if not self.phpcode: return
        if not self.fileinfo.variables.has_key(m.group(1)):
            self.fileinfo.variables[m.group(1)]=(m.group(2), self.lineno)
        #print "CLASSVAR: ", m.group(1), m.group(2), " on line ", self.lineno
        return ""

    def addVariable(self, m):
        if not self.phpcode: return
        #print "VAR: %s on line %d"%(m.group(1), self.lineno)
        if self.currentClass:
            if not self.currentClass.variables.has_key(m.group(1)):
                self.currentClass.variables[m.group(1)] = ('', self.lineno)
        else:
            if not self.fileinfo.variables.has_key(m.group(1)):
                self.fileinfo.variables[m.group(1)] = ('', self.lineno)
        return ""

    def findfile(self, filename):
        if not os.path.exists(filename):
            # try the basedir for a relative include
            if self.basedir:
                fn = os.path.join(self.basedir, filename)
                fn = os.path.normpath(fn)
                if os.path.exists(fn):
                    return fn
            
            # try the include paths
            paths = self.include_path.split(os.pathsep)
            for path in paths:
                fn = os.path.join(path, filename)
                fn = os.path.normpath(fn)
                if os.path.exists(fn):
                    return fn
                
        return filename
    
    def scanData(self, data):
        #print "doing: ",data
        self.l.settext( string.strip(data))
            
        #res = []
        while 1:
            t, v = self.l.scan()
            #print "info: ",repr(t),repr(v)
            if t == self.l.eof:
                break
            #res.append((t, v))
        #print `res`
        
    def scanFile(self):
        #print "scanFile ",self.filename
        try:
            f = open(self.filename, "r")
            data = f.read()
            f.close()
        except IOError, e:
            print "unable to open ",self.filename
            return 0
        self.scanData(data)
        return 1



if __name__ == "__main__":
  filename = r'e:\php\pear\Mail.php'
  import time
  t1 = time.clock()
  PHPLexer(filename)
  print "parsing took ",time.clock()-t1

  #filesparsed[filename].dump()

  files = filesparsed.keys()
  for file in files:
    print "FILE: ",file
    filesparsed[file].dump()
