#!python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

from xpcom import components, nsError, ServerException
from koLintResult import *
from koLintResults import koLintResults
from xpcom.server.enumerator import *
import os, sys, re
import tempfile
import string
import process
import koprocessutils

# php error line format
# \nPHP ERROR NAME: error text in filename.php on line ##\n
# requires php.ini settings
#      display_errors	      = On
#      display_startup_errors = On
# php.ini must reside in os directory (c:\winnt)

class KoPHPCompileLinter:
    _com_interfaces_ = [components.interfaces.koILinter]
    _reg_desc_ = "Komodo PHP Linter"
    _reg_clsid_ = "{F6F8507C-21B0-4047-9E78-A648949B118F}"
    _reg_contractid_ = "@activestate.com/koPHPLinter;1"

    def __init__(self):
        self.sysUtils = components.classes["@activestate.com/koSysUtils;1"].\
            getService(components.interfaces.koISysUtils)
        self.prefService = components.classes["@activestate.com/koPrefService;1"].\
            getService(components.interfaces.koIPrefService)
        self.infoSvc = components.classes["@activestate.com/koInfoService;1"].\
                       getService(components.interfaces.koIInfoService)

    def _getInterpreter(self):
        if self.prefService.prefs.hasStringPref("phpDefaultInterpreter") and\
           self.prefService.prefs.getStringPref("phpDefaultInterpreter"):
            php = self.prefService.prefs.getStringPref("phpDefaultInterpreter")

            #XXX Is this still necessary now that we are using process.py?
            #    It existed because popen2 cannot handle unicode paths.
            try:
                php = str(php)
            except UnicodeError:
                raise ServerException, (nsError.NS_ERROR_NOT_AVAILABLE,
                        "Path to PHP interpreter cannot contain Unicode characters.") 
        else:
            php = self.sysUtils.Which("php")
        return php or None

    def _getInterpreterConfig(self):
        if self.prefService.prefs.hasStringPref("phpConfigFile") and\
           self.prefService.prefs.getStringPref("phpConfigFile"):
            iniFile = self.prefService.prefs.getStringPref("phpConfigFile")
        else:        
            iniFile = None
        return iniFile

    def _validversion(self, php):
        env = koprocessutils.getUserEnv()
        ini = self._getInterpreterConfig()
        if ini:
            env["PHPRC"] = ini
        argv = [php, '-d', 'activedebug.enabled=0', '-v']
        # PHP sets stdin, stdout to binary on it's end.  If we do not do
        # the same, some wierd things can happen, most notably with shell
        # comments that end with \r\n.
        p = process.ProcessOpen(argv, mode='b', env=env)
        lines = p.stdout.readlines()
        # For some reason, PHP linter causes an exception on close
        # with errno = 0, will investigate in PHP later.
        try:
            p.close()
        except IOError, e:
            if e.errno == 0:  
                pass
            else:
                raise

        try:
            if lines:
                version = lines[-1].split(".")
                # needs to be 4.0.5 or better, 4.0.4 has lint crash
                if string.atoi(version[0]) < 4:
                    return 0
                elif string.atoi(version[1]) > 0:
                    return 1
                elif string.atoi( re.match(r"(\d+)", version[2]).group() ) > 4:
                    return 1
            else:
                return 0
        except ValueError,e:
            return 0
            

    def lint(self, text, encoding, cwd):
        #print "----------------------------"
        #print "PHP Lint"
        #print text
        #print "----------------------------"
        php = self._getInterpreter()
        if php is None:
            # Exceptions are currently ignored by the LintManager but this
            # could be used later to give the user a useful error message.
            raise ServerException, (nsError.NS_ERROR_NOT_AVAILABLE,
                "Could not find a suitable PHP interpreter for linting.")

        if not self._validversion(php):
            # Exceptions are currently ignored by the LintManager but this
            # could be used later to give the user a useful error message.
            raise ServerException, (nsError.NS_ERROR_ILLEGAL_VALUE,
                "Could not find a suitable PHP interpreter for linting, need 4.0.5 or later.")

        # save php buffer to a temporary file
        phpfilename = tempfile.mktemp()
        fout = open(phpfilename, 'wb')
        fout.write(text)
        fout.close()

        try:
            argv = [php, '-d', 'display_errors=1',
                    '-d', 'display_startup_errors=1',
                    '-d', 'output_buffering=0',
                    '-d', 'activedebug.enabled=0',
                    '-q', '-l', phpfilename]
            env = koprocessutils.getUserEnv()
            ini = self._getInterpreterConfig()
            if ini:
                env["PHPRC"] = ini
            cwd = cwd or None
            # PHP sets stdin, stdout to binary on it's end.  If we do not do
            # the same, some wierd things can happen, most notably with shell
            # comments that end with \r\n.
            p = process.ProcessOpen(argv, mode='b', cwd=cwd, env=env)
            p.stdin.close()
            # XXX The relevant output is either on stdout or stderr. Dunno
            #     which. We should only be parsing over one of the two.
            lines = p.stdout.readlines() + p.stderr.readlines()
        finally:
            os.unlink(phpfilename)
            # For some reason, PHP linter causes an exception on close
            # with errno = 0, will investigate in PHP later.
            try:
                p.close()
            except IOError, e:
                if e.errno == 0:  
                    pass
                else:
                    raise
        
        results = koLintResults()
        if lines:
            datalines = re.split('\r\n|\r|\n',text)
            numLines = len(datalines)
            lines = [l for l in lines if l.find('error') != -1]
            
            for line in lines:
                #print line
                line = line.strip()
                # remove html from error output
                line = re.sub('<.*?>',"",line)
                lineText = line.rfind(' ')
                try:
                    lineNo = int(line[lineText+1:])
                except ValueError, e:
                    continue
                #print "Line Number: ", lineNo
                result = KoLintResult()
                # XXX error in FILENAME at line XXX
                m = re.match("(.*?)in.*?(on.*)",line)
                if m:
                    result.description = string.join(m.groups())
                else:
                    result.description = line
                result.lineStart = result.lineEnd = lineNo
                result.columnStart = 1
                result.columnEnd = len(datalines[result.lineEnd-1]) + 1
                result.severity = result.SEV_ERROR
                results.addResult(result)
        #print "----------------------------"
        return results

