# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# A simple XPCOM server which: accepts a string which is a line of
# python ending with the opening parentheses (but not including the
# parentheses) of a method call returns a documentation string
# suitable for a calltip this code only supports built-in modules and
# methods Much of this code was borrowed from Pythonwin's
# Pythonwin\pywin\idle\calltips.py

# modified from Brian Wing's code for VS7. [david ascher]

from xpcom import components

import re
import string
import sys
import types

import WShelper

class PerlCallTips:
    _com_interfaces_ = [components.interfaces.koICallTips]
    _reg_desc_ = "Perl CallTips"
    _reg_contractid_ = "@activestate.com/koPerlCallTips;1"
    _reg_clsid_ = "{cccefc6d-5821-48bd-96cd-b4ee61e7da0c}"

    def CallTips(self, source_line, text):
        word = self.get_word_at_cursor(source_line)
        arg_text = get_webservice_arg_text(word, text)
        return arg_text

    def get_word_at_cursor(self, source_line,
                           wordchars="_->:$%@" + string.uppercase + string.lowercase + string.digits):
        chars = str(source_line)
        i = len(chars)
        while i and chars[i-1] in wordchars:
            i = i-1
        return chars[i:]

def get_webservice_arg_text(word, text):
    (service, action) = rsplit(word, "->", 1)
    #XXX This Re can match a URL with mismatched quotes.
    match = re.search(re.escape(service) + r"\s*=\s*(WebService\s*::\s*ServiceProxy\s*->\s*new|SOAP::Lite->service)\s*\(\s*['\"]+([^'\"]+)['\"](\s*,\s*['\"][\da-f]+\s*['\"])?\s*\)", text)
    if not match:
        return None

    url = match.group(2)
    try:
        inArgs = WShelper.Get_In_Args_With_Type(url, action)
        outArgs = WShelper.Get_Out_Args_With_Type(url, action)
    except Exception, e:
        # XXX This catch-all is here because WShelper does not return
        # well-defined exceptions is reponse to a WSDL file that it
        # cannot parse.
        observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                      .getService(components.interfaces.nsIObserverService)
        sm = components.classes["@activestate.com/koStatusMessage;1"]\
             .createInstance(components.interfaces.koIStatusMessage)
        sm.category = "webservices_intellisense"
        sm.msg = 'Call tips are not available for this method [%r]' % (e.args) 
        sm.timeout = 5000
        sm.highlight = 1
        try:
           observerSvc.Notify(sm, 'status_message', None)
        except COMException, e:
           pass
            
        return None

    
    inArgString = ", ".join(map(make_arg_string, inArgs))
    if outArgs:
        outArgString = ", ".join(map(make_arg_string, outArgs))
        if len(outArgs) > 1:
            outArgString = "(%s)" % outArgString
    else:
        outArgString = "None"

    return "(%s) -> %s" % (inArgString, outArgString)


def make_arg_string(arg):
    """
    Return a string represnting an argument and its type
    """
    (name, type) = arg
    return "%s %s" % (type, name)


def rsplit(s, sep=" ", maxsplit=None):
    """
    Just like string.split(), but splits starting from the end of the
    string, rather than from the beginning.  Caveat: string.split()
    defaults to split on any whitespace.  Meeting that reqirement
    would be difficult, and is not worth the effort ATM.
    """
    if maxsplit == None:
        return s.split(sep)
    else:
        list = s.split(sep)
        return [sep.join(list[:-maxsplit])] + list[-maxsplit:]
