#!python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

import os, sys
from xpcom import components, ServerException
from koLanguageServiceBase import *
import autoinsert


class KoPerlWSInsertLanguageService(KoWSInsertLanguageService):
    def __init__(self):
        KoWSInsertLanguageService.__init__(self,
                                           autoinsert.PerlAutoInsert(),
                                           "<unset>", "<unset>", 0)
        self._prefs = components.classes["@activestate.com/koPrefService;1"]. \
                            getService(components.interfaces.koIPrefService). \
                            prefs;
    
    def insert(self, *args):
        module = self._prefs.getStringPref("perlWSModule")
        if module == "WebService":
            self._creationTemplate = "my $%s = WebService::ServiceProxy->new(\"%s\", \"%s\");"
            self._module = "WebService::ServiceProxy"
            self._useMD5 = 1
        elif module == "SOAP::Lite":
            self._creationTemplate = "my $%s = SOAP::Lite->service(\"%s\");"
            self._module = "SOAP::Lite"
            self._useMD5 = 0
        else:
            logError("Unknown value for pref perlWSModule: %r" % module);
            return
        
        apply(KoWSInsertLanguageService.insert, [self] + list(args))


class koPerlInterpreter(KoInterpreterLanguageServiceBase):
    _com_interfaces_ = [components.interfaces.koIInterpreterLanguageService]
    _reg_clsid_ = "{809d4f3e-d25f-4cb2-babb-b2032cca9b71}"
    _reg_contractid_ = "@activestate.com/koPerlInterpreter;1"
    _reg_desc_ = "Information about the default Perl interpreter"

    def __init__(self):
        self.prefService = components.classes[
            "@activestate.com/koPrefService;1"].getService();
        self.sysUtils = components.classes[
            "@activestate.com/koSysUtils;1"].getService();
        
    def get_interpreterPath(self):
        """Determine the Perl interpreter to use.
        Return value is the absolute path to the perl intepreter or None
        if an appropriate on could not be found.
        """
        # use pref'd default if one has been chosen
        perlExe = self.prefService.prefs.getStringPref("perlDefaultInterpreter")

        #XXX Is this still necessary now that we are using process.py?
        #    It existed because popen2 cannot handle unicode paths.
        try:
            perlExe = str(perlExe)
        except UnicodeError:
            raise ServerException, (nsError.NS_ERROR_NOT_AVAILABLE,
                                    "Path to Perl interpreter cannot contain Unicode characters.")

        if perlExe:
            # "wPerl.exe" on Windows causes problems, although at one point
            # this is what we encouraged users to use. We want to just use
            # "perl.exe" even if the preference states to use "wPerl.exe"
            if sys.platform.startswith("win"):
                dirName, baseName = os.path.split(perlExe)
                if os.path.splitext(baseName)[0].lower() == "wperl":
                    perlExe = os.path.join(dirName, "perl.exe")
                    if not os.path.isfile(perlExe):
                        perlExe = None
        if not perlExe:
            perlExe = self.sysUtils.Which("perl")
        if not perlExe or not os.path.isfile(perlExe):
            logError("No Perl interpreter could be found.");
            return None
        else:
            return perlExe

    
class koPerlLanguage(KoLanguageBase):
    name = "Perl"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{911E8F76-C8F9-46f2-A930-1F1693400FCB}"

    defaultExtension = ".pl"
    # XXX read url from some config file
    downloadURL = 'http://www.ActiveState.com/Products/ActivePerl'
    remoteDebuggerSupport = 1
    commentDelimiterInfo = { "line": [ "#" ]  }
    cookbookName = "Rx"
    cookbookUrl = "http://aspn.activestate.com/ASPN/Cookbook/Rx"
    variableIndicators = '$@%'
    namedBlockRE = "^[ \t]*?(sub\s+\w+|package\s+\w)"
    namedBlockDescription = 'Perl subs and packages'
    
    def getLanguageService(self, iid):
        return KoLanguageBase.getLanguageService(self, iid)

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_PERL)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 1
        return self._lexer

    def get_debugger(self):
        if self._debugger is None:
            self._debugger = KoDebuggerLanguageService(self.name)
        return self._debugger

    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koPerlCompileLinter;1", None)
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            self._methodcompletion = components.classes["@activestate.com/koPerlCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def get_cookbook(self):
        if self._cookbook is None:
            self._cookbook = KoCookbookLanguageService(
                self.cookbookName, self.cookbookUrl)
        return self._cookbook

    def get_inserter(self):
        if self._inserter is None:
            self._inserter = KoPerlWSInsertLanguageService()
        return self._inserter

    def get_interpreter(self):
        if self._interpreter is None:
            self._interpreter = koPerlInterpreter()
        return self._interpreter

    _keywords = ["abs",
                 "accept",
                 "alarm",
                 "and",
                 "atan2",
                 "BEGIN",
                 "bind",
                 "binmode",
                 "bless",
                 "caller",
                 "chdir",
                 "chmod",
                 "chomp",
                 "chop",
                 "chown",
                 "chr",
                 "chroot",
                 "close",
                 "closedir",
                 "cmp",
                 "connect",
                 "continue",
                 "cos",
                 "crypt",
                 "DATA",
                 "dbmclose",
                 "dbmopen",
                 "define",
                 "defined",
                 "delete",
                 "die",
                 "do",
                 "dump",
                 "each",
                 "else",
                 "elsif",
                 "END",
                 "endgrent",
                 "endhostent",
                 "endnetent",
                 "endprotoent",
                 "endpwent",
                 "endservent",
                 "eof",
                 "eq",
                 "eval",
                 "exec",
                 "exists",
                 "exit",
                 "exp",
                 "fcntl",
                 "fileno",
                 "flock",
                 "for",
                 "foreach",
                 "fork",
                 "format",
                 "formline",
                 "ge",
                 "getc",
                 "getgrent",
                 "getgrgid",
                 "getgrnam",
                 "gethostbyaddr",
                 "gethostbyname",
                 "gethostent",
                 "getlogin",
                 "getnetbyaddr",
                 "getnetbyname",
                 "getnetent",
                 "getpeername",
                 "getpgrp",
                 "getppid",
                 "getpriority",
                 "getprotobyname",
                 "getprotobynumber",
                 "getprotoent",
                 "getpwent",
                 "getpwnam",
                 "getpwuid",
                 "getservbyname",
                 "getservbyport",
                 "getservent",
                 "getsockname",
                 "getsockopt",
                 "glob",
                 "gmtime",
                 "goto",
                 "grep",
                 "gt",
                 "hex",
                 "if",
                 "import",
                 "include",
                 "index",
                 "int",
                 "ioctl",
                 "join",
                 "keys",
                 "kill",
                 "last",
                 "lc",
                 "lcfirst",
                 "le",
                 "length",
                 "link",
                 "listen",
                 "local",
                 "localtime",
                 "lock",
                 "log",
                 "lstat",
                 "lt",
                 "m",
                 "map",
                 "mkdir",
                 "msgctl",
                 "msgget",
                 "msgrcv",
                 "msgsnd",
                 "my",
                 "ne",
                 "new",
                 "next",
                 "no",
                 "not",
                 "oct",
                 "open",
                 "opendir",
                 "or",
                 "ord",
                 "our",
                 "pack",
                 "package",
                 "pipe",
                 "pop",
                 "pos",
                 "print",
                 "printf",
                 "prototype",
                 "push",
                 "q",
                 "qq",
                 "qr",
                 "qx",
                 "qw",
                 "quotemeta",
                 "rand",
                 "read",
                 "readdir",
                 "readline",
                 "readlink",
                 "readpipe",
                 "recv",
                 "redo",
                 "ref",
                 "rename",
                 "require",
                 "reset",
                 "return",
                 "reverse",
                 "rewinddir",
                 "rindex",
                 "rmdir",
                 "s",
                 "scalar",
                 "seek",
                 "seekdir",
                 "select",
                 "semctl",
                 "semget",
                 "semop",
                 "send",
                 "setgrent",
                 "sethostent",
                 "setnetent",
                 "setpgrp",
                 "setpriority",
                 "setprotoent",
                 "setpwent",
                 "setservent",
                 "setsockopt",
                 "shift",
                 "shmctl",
                 "shmget",
                 "shmread",
                 "shmwrite",
                 "shutdown",
                 "sin",
                 "sleep",
                 "socket",
                 "socketpair",
                 "sort",
                 "splice",
                 "split",
                 "sprintf",
                 "sqrt",
                 "srand",
                 "stat",
                 "study",
                 "sub",
                 "substr",
                 "symlink",
                 "syscall",
                 "sysopen",
                 "sysread",
                 "sysseek",
                 "system",
                 "syswrite",
                 "tell"
                 "telldir",
                 "tie",
                 "tied",
                 "time",
                 "times",
                 "tr",
                 "truncate",
                 "uc",
                 "ucfirst",
                 "umask",
                 "undef",
                 "unless",
                 "unlink",
                 "unpack",
                 "unshift",
                 "untie",
                 "until",
                 "use",
                 "utime",
                 "values",
                 "vec",
                 "wait",
                 "waitpid",
                 "wantarray",
                 "warn",
                 "while",
                 "write",
                 "xor",
                 "y"
                 ]






