# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# stolen from brian wing's autocompletecomserver, which was stolen from mark hammond's code

# AutoCompleteCOMServer.py - A simple COM server which:
# takes the entire edit buffer along with the current cursor position (number
# of characters from the beginning of the buffer) which is
# hopefully over the '.' of the user typing in "modulename."
# returns a list of methods defined in modulename
# Much of this code was borrowed from Pythonwin's Pythonwin\pywin\scintilla\view.py

from xpcom import components
import array
import string
import sys
import types
import struct
import re

import WShelper

wordbreaks = "._" + string.uppercase + string.lowercase + string.digits

class PythonAutoComplete:
    _com_interfaces_ = [components.interfaces.koIAutoComplete]
    _reg_desc_ = "Python AutoComplete Engine"
    _reg_contractid_ = "@activestate.com/koPyAutoComplete;1"
    _reg_clsid_ = "{CAC63DCB-68AA-4a35-8625-5358CB65F736}"

    def __init__(self):
      self._namespace = sys.modules.copy()

    def Exec(self, source_line):
        try:
            exec source_line in self._namespace
        except:
            pass

    def AutoComplete(self, pos, text):
        self.pos = pos
        self.text = str(text)
        try:
            return self._DoAutoComplete()
        finally:
            self.text = None
            self.pos = None

    def _DoAutoComplete(self):
        ob = self._GetObjectAtPos(bAllowCalls = 0)
        # If that failed, try and process call or indexing to get the object.
        if ob is None:
            ob = self._GetObjectAtPos(bAllowCalls = 1)
        if ob is not None:
            items = []

            try:
                items = items + dir(ob)
            except AttributeError:
                pass # object has no __dict__
            try:
                items = items + dir(ob.__class__)
            except AttributeError:
                pass
            # All names that start with "_" go!
            items = filter(lambda word: word[0]!='_', items)
            
            # The object may be a COM object with typelib support - lets see if we can get its props.
            # (contributed by Stefan Migowsky)
            try:
                # Get the automation attributes
                list = ob.__class__._prop_map_get_.keys()
                # See if there is an write only property 
                # could be optimized
                for i in ob.__class__._prop_map_put_.keys():
                    if i not in list:
                        list.append(i)
                # append to the already evaluated list
                items = items + list
            except AttributeError:
                pass

            if items:
                items.sort()
                attribute_types = []
                for item in items:
                    a_type = type(getattr(ob, item))
                    if a_type in [types.FunctionType, types.LambdaType, types.BuiltinFunctionType,
                                  types.InstanceType, types.UnboundMethodType, types.BuiltinMethodType,
                                  types.MethodType]:
                        attribute_types.append(1)
                    elif a_type in [types.ClassType, types.ModuleType]:
                        attribute_types.append(2)
                    elif a_type in [types.ComplexType, types.FloatType, types.FileType,
                                    types.BufferType, types.LongType, types.StringType,
                                    types.SliceType, types.FrameType, types.TupleType,
                                    types.ListType, types.CodeType, types.XRangeType,
                                    types.IntType]:
                        attribute_types.append(3)
                    else:
                        attribute_types.append(0)
                # in Mozilla we are ignoring the types for now. XXX
                # XXX The '\n' literal should really be taken from
                #     self.completionSeparator from koLanguageServiceBase
                return string.join(items, '\n')
        else:
            # Heuristics a-la AutoExpand
            # The idea is to find other usages of the current binding
            # and assume, that it refers to the same object (or at least,
            # to an object of the same type)
            # Contributed by Vadim Chugunov [vadimch@yahoo.com]
            left, right = self._GetWordSplit()
            if left=="": # Ignore standalone dots
                return None
            # We limit our search to the current class, if that
            # information is available
            try:
                list = re.findall(r"\b"+left+"\.\w+",self.text)
            except re.error:
                # parens etc may make an invalid RE, but this code wouldnt
                # benefit even if the RE did work :-)
                list = []
            prefix = len(left)+1
            unique = {}
            for li in list:
                unique[li[prefix:]] = 1

            items = filter(lambda word: word[:2]!='__' or word[-2:]!='__', unique.keys())
            # Ignore the word currently to the right of the dot - probably a red-herring.
            try:
                items.remove(right[1:])
            except ValueError:
                pass

            # Now look to see if the binding is associated with a web service
            try:
                #XXX what do we do for multiple occurences?
                #XXX This Re can match a URL with mismatched quotes.
                match = re.search(r"\b" + left + r"\s*=\s*WebService\.ServiceProxy\s*\(\s*['\"]+([^'\"]+)['\"](\s*,\s*['\"][\da-f]+\s*['\"])?\s*\)", self.text)
            except re.error:
                # parens etc may make an invalid RE, but this code wouldnt
                # benefit even if the RE did work :-)
                pass
            else:
                if match:
                    url = match.group(1)
                    try:
                        items += WShelper.Get_All_Methods(url)
                    except Exception, e:
                        # XXX This catch-all is here because WShelper does not return
                        # well-defined exceptions is reponse to a WSDL file that it
                        # cannot parse.

                        observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                                      .getService(components.interfaces.nsIObserverService)
                        sm = components.classes["@activestate.com/koStatusMessage;1"]\
                             .createInstance(components.interfaces.koIStatusMessage)
                        sm.category = "webservices_intellisense"
                        sm.msg = 'AutoComplete is not available for "%s" [%r]' % (url, e.args) 
                        sm.timeout = 5000
                        sm.highlight = 1
                        try:
                           observerSvc.Notify(sm, 'status_message', None)
                        except COMException, e:
                           pass
            
            if items:
                items.sort()
                self._removeDuplicates(items)
                # XXX The '\n' literal should really be taken from
                #     self.completionSeparator from koLanguageServiceBase
                return string.join(items, '\n')

    def _GetObjectAtPos(self, bAllowCalls = 0):
        left, right = self._GetWordSplit(bAllowCalls)
        if left: # It is an attribute lookup
            try:
                return eval(left, self._namespace)
            except:
                pass
        return None

    def _GetWordSplit(self, bAllowCalls = 0):
        limit = len(self.text)
        before = []
        after = []
        index = self.pos-1
        wordbreaks_use = wordbreaks
        if bAllowCalls: wordbreaks_use = wordbreaks_use + "()[]"
        while index>=0:
            char = self.text[index]
            if char not in wordbreaks_use: break
            before.insert(0, char)
            index = index-1
        index = self.pos
        while index<limit:
            char = self.text[index]
            if char not in wordbreaks_use: break
            after.append(char)
            index=index+1
        return string.join(before,''), string.join(after,'')

    def _removeDuplicates(self, list):
        """Remove duplicate items from a sorted list"""
        i = 1
        while i < len(list):
            if list[i-1] == list[i]:
                del list[i]
            i += 1

