# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# A simple XPCOM server which: accepts a string which is a line of
# python ending with the opening parentheses (but not including the
# parentheses) of a method call returns a documentation string
# suitable for a calltip this code only supports built-in modules and
# methods Much of this code was borrowed from Pythonwin's
# Pythonwin\pywin\idle\calltips.py

# modified from Brian Wing's code for VS7. [david ascher]

from xpcom import components

import re
import string
import sys
import types

import WShelper

class PythonCallTips:
    _com_interfaces_ = [components.interfaces.koICallTips]
    _reg_desc_ = "Scintilla Editor Wrapper"
    _reg_contractid_ = "@activestate.com/koPyCallTips;1"
    _reg_clsid_ = "{7AD68D2C-B39E-4f85-ACEF-881F178F8892}"

    def __init__(self):
      import sys, __main__
      self._namespace = sys.modules.copy()

    def CallTips(self, source_line, text):
        word = self.get_word_at_cursor(source_line)
        object = self.get_object_by_name(word)
        if object:
            arg_text = get_arg_text(object)
        else:
            arg_text = get_webservice_arg_text(word, text)
        return arg_text

    def Exec(self, source_line):
        try:
            exec source_line in self._namespace
        except:
            pass

    def get_word_at_cursor(self, source_line,
                           wordchars="._" + string.uppercase + string.lowercase + string.digits):
        chars = str(source_line)
        i = len(chars)
        while i and chars[i-1] in wordchars:
            i = i-1
        return chars[i:]

    def get_object_by_name(self, word):
        try:
            x = eval(word, self._namespace)
        except: # XXX catch all runtime exceptions, hence otherwise bad things happen in the linter, for reasons undetermined for now...
            return None
        return x

def _find_constructor(class_ob):
    # Given a class object, return a function object used for the
    # constructor (ie, __init__() ) or None if we can't find one.
    try:
        return class_ob.__init__.im_func
    except AttributeError:
        for base in class_ob.__bases__:
            rc = _find_constructor(base)
            if rc is not None: return rc
    return None

def get_arg_text(ob):
    # Get a string describing the arguments for the given object.
    argText = ""
    if ob is not None:
        argOffset = 0
        if type(ob)==types.ClassType:
            # Look for the highest __init__ in the class chain.
            fob = _find_constructor(ob)
            if fob is None:
                fob = lambda: None
            else:
                argOffset = 1
        elif type(ob)==types.MethodType:
            # bit of a hack for methods - turn it into a function
            # but we drop the "self" param.
            fob = ob.im_func
            argOffset = 1
        else:
            fob = ob
        # Try and build one for Python defined functions
        if type(fob) in [types.FunctionType, types.LambdaType]:
            try:
                realArgs = fob.func_code.co_varnames[argOffset:fob.func_code.co_argcount]
                defaults = fob.func_defaults or []
                defaults = list(map(lambda name: "=%s" % name, defaults))
                defaults = [""] * (len(realArgs)-len(defaults)) + defaults
                items = map(lambda arg, dflt: arg+dflt, realArgs,defaults)
                if fob.func_code.co_flags & 0x4:
                    items.append("...")
                if fob.func_code.co_flags & 0x8:
                    items.append("***")
                argText = string.join(items , ", ")
                argText = "(%s)" % argText
            except:
                pass
        # See if we can use the docstring
        if hasattr(ob, "__doc__") and ob.__doc__:
            pos = string.find(ob.__doc__, "\n")
            if pos<0 or pos>70: pos=70
            if not argText: argText = '(...)'
            if argText: argText = argText + "\n"
            argText = argText + ob.__doc__[:pos]
    return argText

def get_webservice_arg_text(word, text):
    (service, action) = rsplit(word, ".", 1)
    #XXX This Re can match a URL with mismatched quotes.
    match = re.search(r"\b" + service + r"\s*=\s*WebService\.ServiceProxy\s*\(\s*['\"]+([^'\"]+)['\"](\s*,\s*['\"][\da-f]+\s*['\"])?\s*\)", text)
    if not match:
        return None

    url = match.group(1)
    try:
        inArgs = WShelper.Get_In_Args_With_Type(url, action)
        outArgs = WShelper.Get_Out_Args_With_Type(url, action)
    except Exception, e:
        # XXX This catch-all is here because WShelper does not return
        # well-defined exceptions is reponse to a WSDL file that it
        # cannot parse.
        observerSvc = components.classes["@mozilla.org/observer-service;1"]\
                      .getService(components.interfaces.nsIObserverService)
        sm = components.classes["@activestate.com/koStatusMessage;1"]\
             .createInstance(components.interfaces.koIStatusMessage)
        sm.category = "webservices_intellisense"
        sm.msg = 'Call tips are not available for this method [%r]' % (e.args) 
        sm.timeout = 5000
        sm.highlight = 1
        try:
           observerSvc.Notify(sm, 'status_message', None)
        except COMException, e:
               pass
        
        return None

    
    inArgString = ", ".join(map(make_arg_string, inArgs))
    if outArgs:
        outArgString = ", ".join(map(make_arg_string, outArgs))
        if len(outArgs) > 1:
            outArgString = "(%s)" % outArgString
    else:
        outArgString = "None"

    return "(%s) -> %s" % (inArgString, outArgString)

def make_arg_string(arg):
    """
    Return a string represnting an argument and its type
    """
    (name, type) = arg
    return "%s %s" % (type, name)

def rsplit(s, sep=" ", maxsplit=None):
    """
    Just like string.split(), but splits starting from the end of the
    string, rather than from the beginning.  Caveat: string.split()
    defaults to split on any whitespace.  Meeting that reqirement
    would be difficult, and is not worth the effort ATM.
    """
    if maxsplit == None:
        return s.split(sep)
    else:
        list = s.split(sep)
        return [sep.join(list[:-maxsplit])] + list[-maxsplit:]

