from xpcom import components, ServerException
import sys
import keyword
from koLanguageServiceBase import *
import os

import autoinsert

class koPythonInterpreter(KoInterpreterLanguageServiceBase):
    _com_interfaces_ = [components.interfaces.koIInterpreterLanguageService]
    _reg_clsid_ = "{8ef788bf-82a5-49bd-a9d8-3c700498bbaa}"
    _reg_contractid_ = "@activestate.com/koPythonInterpreter;1"
    _reg_desc_ = "Information about the default Python interpreter"

    def __init__(self):
        self.prefService = components.classes["@activestate.com/koPrefService;1"].getService();
        
    def get_interpreterPath(self):
        sysUtils = components.classes["@activestate.com/koSysUtils;1"].getService()
        koInfoService = components.classes["@activestate.com/koInfoService;1"].getService();
        
        pythonExe = self.prefService.prefs.getStringPref("pythonDefaultInterpreter")
        if pythonExe:
            # "pythonw.exe" on Windows causes problems, although at one point
            # this is what we encouraged users to use. We want to just use
            # "python.exe" even if the preference states to use "pythonw.exe"
            if sys.platform.startswith("win"):
                dirName, baseName = os.path.split(pythonExe)
                if os.path.splitext(baseName)[0].lower() == "pythonw":
                    pythonExe = os.path.join(dirName, "python.exe")
                    if not os.path.isfile(pythonExe):
                        pythonExe = None
        if not pythonExe:
            pythonExe = ""
            if sys.platform.startswith("win"):
                if koInfoService.buildType == "release":
                    pythonExe = sysUtils.Which("python")
                else:
                    pythonExe = sysUtils.Which("python_d")
            else:
                pythonExe = sysUtils.Which("python")

        if not pythonExe or not os.path.isfile(pythonExe):
            return ''
        else:
            return pythonExe
    
class koPythonLanguage(KoLanguageBase):
    name = "Python"
    _reg_desc_ = "%s Language" % (name)
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{D90FF5C7-1FD4-4535-A0D2-47B5BDC3E7FE}"

    namedBlockRE = "^[ \t]*?(def\s+[^\(]+\([^\)]*?\):|class\s+[^:]*?:)"
    namedBlockDescription = 'Python functions and classes'
    defaultExtension = ".py"
    # XXX read url from some config file
    downloadURL = 'http://www.ActiveState.com/Products/ActivePython'
    remoteDebuggerSupport = 1
    commentDelimiterInfo = { "line": [ "#" ]  }
    cookbookName = "Python"
    cookbookUrl = "http://aspn.activestate.com/ASPN/Cookbook/Python"
    
    def get_debugger(self):
        if self._debugger is None:
            self._debugger = KoDebuggerLanguageService(self.name)
        return self._debugger

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_PYTHON)
            self._lexer.setKeywords(0, keyword.kwlist)
            self._lexer.setProperty('tab.timmy.whinge.level', '1')
            self._lexer.supportsFolding = 1
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService(
                "@activestate.com/koPythonLinter;1", None)
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            self._methodcompletion = components.classes["@activestate.com/koPyCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def get_cookbook(self):
        if self._cookbook is None:
            self._cookbook = KoCookbookLanguageService(
                self.cookbookName, self.cookbookUrl)
        return self._cookbook

    def get_inserter(self):
        if self._inserter is None:
            self._inserter = KoWSInsertLanguageService( \
                autoinsert.PythonAutoInsert(),
                "%s = WebService.ServiceProxy(\"%s\", \"%s\")",
                "WebService", 1)
        return self._inserter
    
    def get_interpreter(self):
        if self._interpreter is None:
            self._interpreter = koPythonInterpreter()
        return self._interpreter
    

