#!python
# Copyright (c) 2000-2001 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

# The implementation of the Komodo Run Command Service 

import os
import sys
import re
import tempfile
import threading
import process
import types
from xpcom import components, nsError, ServerException, COMException, _xpcom
from koLogger import logWarning, logError, logDebug


#---- globals

_gLastErrorSvc = None



#---- internal support stuff

class _NotifyOnTermination(threading.Thread):
    def __init__(self, process_, termListener, *args, **kwargs):
        threading.Thread.__init__(self, *args, **kwargs)
        self._termListener = termListener
        self._process = process_
    def run(self):
        retval = self._process.wait(self._process.INFINITE)
        self._termListener.onTerminate(retval)


class _KoRunProcessProxy(process.ProcessProxy):
    _com_interfaces_ = [components.interfaces.koIRunProcess]

    def wait(self, timeout):
        # Call the baseclass' wait() and set errors on koILastError as
        # appropriate.
        try:
            retval = process.ProcessProxy.wait(self, timeout)
        except process.ProcessError, ex:
            _gLastErrorSvc.setLastError(ex.errno, str(ex))
            raise
        return retval

    def readStdout(self):
        return self.stdout.read()
    def readStderr(self):
        return self.stderr.read()
            

class _OutlinerView:
    """A base implementation of the nsIOutlinerView.
    
    This should be subclassed to get any useful behaviour.
    """
    #XXX This should be placed in an 'outliner.py' or something with
    #    documentation and examples of use.
    _com_interfaces_ = [components.interfaces.nsIOutlinerView]
        
    def __init__(self, debug=0):
        self._debug = debug
        self.selection = None   # nsIOutlinerSelection

    def isSeparator(self, index):
        return 0
    
    def get_rowCount( self ):
        # Result: int32
        if self._debug:
            logDebug("get_rowCount()")
        return 0
    def getRowProperties( self, index, properties ):
        # Result: void - None
        # In: param0: int32
        # In: param1: nsISupportsArray
        if self._debug:
            logDebug("getRowProperties(%s, %s)" % (index, properties))
        pass
    def getCellProperties(self, row, colID, properties):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        # In: param2: nsISupportsArray
        if self._debug:
            logDebug("getCellProperties()")
        pass
    def getColumnProperties(self, colID, colElt, properties):
        # Result: void - None
        # In: param0: wstring
        # In: param1: nsIDOMElement
        # In: param2: nsISupportsArray
        if self._debug:
            logDebug("getColumnProperties(colID=%s, colElt=%s, props=%s)"\
                     % (colID, colElt, properties))
        pass
    def isContainer(self, index):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainer()")
        return 0
    def isContainerOpen(self, index):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainerOpen()")
        return 0
    def isContainerEmpty( self, param0 ):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("isContainerEmpty()")
        pass
    def isSorted( self ):
        # Result: boolean
        if self._debug:
            logDebug("isSorted()")
        return 0
    def canDropOn( self, param0 ):
        # Result: boolean
        # In: param0: int32
        if self._debug:
            logDebug("canDropOn()")
        return 0
    def canDropBeforeAfter( self, param0, param1 ):
        # Result: boolean
        # In: param0: int32
        # In: param1: boolean
        if self._debug:
            logDebug("canDropBeforeAfter()")
        return 0
    def drop( self, param0, param1 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: int32
        if self._debug:
            logDebug("drop()")
        pass
    def getParentIndex( self, param0 ):
        # Result: int32
        # In: param0: int32
        if self._debug:
            logDebug("getParentIndex()")
        #XXX huh?
        pass
    def hasNextSibling( self, param0, param1 ):
        # Result: boolean
        # In: param0: int32
        # In: param1: int32
        if self._debug:
            logDebug("hasNextSibling()")
        return 0
    def getLevel(self, index):
        # Result: int32
        # In: param0: int32
        if self._debug:
            logDebug("getLevel()")
        return 0
    def getCellText(self, row, colID):
        # Result: wstring
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("getCellText()")
            return "row: %s, col: %s" % (row, colID)
        else:
            return ""
    def setOutliner(self, outliner):
        # Result: void - None
        # In: param0: nsIOutlinerBoxObject
        #XXX may move to be an attribute.
        if self._debug:
            logDebug("setOutliner()")
        self._outliner = outliner
    def toggleOpenState(self, index):
        # Result: void - None
        # In: param0: int32
        if self._debug:
            logDebug("toggleOpenState()")
        pass
    def cycleHeader(self, colID, elt):
        # Result: void - None
        # In: param0: wstring
        # In: param1: nsIDOMElement
        if self._debug:
            logDebug("cycleHeader()")
        pass
    def selectionChanged(self):
        # Result: void - None
        if self._debug:
            logDebug("selectionChanged()")
        pass
    def cycleCell(self, row, colID):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("cycleCell()")
        pass
    def isEditable(self, row, colID):
        # Result: boolean
        # In: param0: int32
        # In: param1: wstring
        if self._debug:
            logDebug("isEditable()")
        return 0
    def setCellText( self, param0, param1, param2 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: wstring
        # In: param2: wstring
        if self._debug:
            logDebug("setCellText()")
        pass
    def performAction(self, action):
        # Result: void - None
        # In: param0: wstring
        if self._debug:
            logDebug("performAction(%s)" % action)
        pass
    def performActionOnRow(self, action, row):
        # Result: void - None
        # In: param0: wstring
        # In: param1: int32
        if self._debug:
            logDebug("performActionOnRow(%s, %s)" % (action, row))
        pass
    def performActionOnCell(self, action, row, colID):
        # Result: void - None
        # In: param0: wstring
        # In: param1: int32
        # In: param2: wstring
        if self._debug:
            logDebug("performActionOnCell(%s, %s, %s)" % (action, row, colID))
        pass



#---- run service components

class KoRunService:
    _com_interfaces_ = components.interfaces.koIRunService
    _reg_desc_ = "Run Service"
    _reg_clsid_ = "{8BE89689-2A82-433b-9D88-726737ED2547}"
    _reg_contractid_ = "@activestate.com/koRunService;1"

    def __init__(self):
        global _gLastErrorSvc
        _gLastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
                         .getService(components.interfaces.koILastErrorService)
        self._prefs = components.classes["@activestate.com/koPrefService;1"].\
            getService(components.interfaces.koIPrefService).prefs
        self._sysUtils = components.classes["@activestate.com/koSysUtils;1"].\
            getService(components.interfaces.koISysUtils)

        # Setup for sending status bar messages in sub-threads.
        self._observerSvc = components.classes["@mozilla.org/observer-service;1"].\
            getService(components.interfaces.nsIObserverService)
        self._observerSvcProxy = _xpcom.getProxyForObject(None,
            components.interfaces.nsIObserverService, self._observerSvc,
            _xpcom.PROXY_ALWAYS | _xpcom.PROXY_SYNC)
        self._sm = components.classes["@activestate.com/koStatusMessage;1"]\
            .createInstance(components.interfaces.koIStatusMessage)

        if sys.platform.startswith("win"):
            if os.environ.has_key("SHELL"):
                self.shell = os.environ["SHELL"]
            elif os.environ.has_key("ComSpec"):
                self.shell = os.environ["ComSpec"]
            else:
                #XXX Would be nice to get full path here so can use
                #    os.spawn* calls.
                self.shell = "command.com"
            self.shellArgs = "/C"
        else:
            self.shell = "xterm"
            self.shellArgs = "-e /bin/sh"
    
    def Encode(self, command, cwd, env, insertOutput,
               operateOnSelection, doNotOpenOutputWindow, runIn):
        """Encode the command (with all options) into a string.

        This will be displayed in the "Recent Commands" MRU so important
        information should be first. Plan:
            <command> <dict with options>
                - escape '{' and '}' in <command>
        """
        encoded = command.replace('{', '\\{')\
                         .replace('}', '\\}')
        # Add an option to the optdict if its value is other than the
        # default.
        optdict = {}
        if cwd:
            optdict['cwd'] = cwd
        if env:
            optdict['env'] = env
        if insertOutput:
            optdict['insertOutput'] = insertOutput
        if operateOnSelection:
            optdict['operateOnSelection'] = operateOnSelection
        if doNotOpenOutputWindow:
            optdict['doNotOpenOutputWindow'] = doNotOpenOutputWindow
        if runIn != 'command-output-window':
            optdict['runIn'] = runIn
        if optdict:
            encoded += ' ' + repr(optdict)
        return encoded

    def Decode(self, encoded):
        """Decode an encoded command (see .Encode()).
        
        Want to support old encoding style (from Komodo 1.x) for
        backward compatibility.  From the old docs the form is:
           [<options letters] <command>
        Options letters are (these are the same as thier accesskeys in
        run.xul):
           i == insertOutput 
           l == operateOnSelection
        """
        optdict = { # defaults
            'cwd': '',
            'env': '',
            'insertOutput': 0,
            'operateOnSelection': 0,
            'doNotOpenOutputWindow': 0,
            'runIn': 'command-output-window',
        }

        # First see if this is possibly an old-style encoding.
        oldStyleRe = re.compile(
            "^\[(?P<insertOutput>i)?(?P<operateOnSelection>l)?\] "\
            "(?P<command>.*)$")
        match = oldStyleRe.match(encoded)
        if match:
            if match.group('insertOutput') is not None:
                optdict['insertOutput'] = 1
            if match.group('operateOnSelection') is not None:
                optdict['operateOnSelection'] = 1
            command = match.group('command')

        # New-style encoding
        else:
            parts = encoded.split(' {', 1)
            if len(parts) == 1:
                command = parts[0]
                optdictstr = '{}'
            else:
                command, optdictstr = parts
                optdictstr = '{' + optdictstr
            command = command.replace('\\{', '{')\
                             .replace('\\}', '}')
            try:
                optupdate = eval(optdictstr)
            except SyntaxError, ex:
                # This likely means that we are trying to decode an old
                # style encoded command as if it were a new-style
                # encoded command and the encoded command includes '{'
                # and/or '}' chars.
                command = encoded
            else:
                optdict.update(optupdate)

        return (command, optdict['cwd'], optdict['env'],
                optdict['insertOutput'], optdict['operateOnSelection'],
                optdict['doNotOpenOutputWindow'], optdict['runIn'])

    def _ReplacePercent(self, matchObj):
        #print "_ReplacePercent: group=%r, map=%r"\
        #      % (matchObj.group(0), self._interpCodeMap)
        code = matchObj.group(0)[1:]
        replacement = self._interpCodeMap[code]
        if replacement is None:
            return matchObj.group(0)
        elif callable(replacement):
            return replacement()
        elif isinstance(replacement, Exception):
            raise replacement
        return self._interpCodeMap[code]

    def _FindInterpreterOnPath(self, interp):
        """Return a full path to the first occurence of the given interpreter
        on the path.

        Return the empty string if none is found.
        """
        if interp == "wish":
            # ActiveTcl does not put 'wish' in AppPaths but puts 'tclsh'
            # there.
            wish = self._sysUtils.Which("wish")
            if wish:
                return wish
            else:
                tclshs = self._sysUtils.WhichAll("tclsh")
                if sys.platform.startswith("win"):
                    wishBase = "wish.exe"
                else:
                    wishBase = "wish"
                for tclsh in tclshs:
                    wish = os.path.join(os.path.dirname(tclsh), wishBase)
                    if os.path.exists(wish):
                        return wish
                else:
                    return ""
        else:
            return self._sysUtils.Which(interp)

    def _GetInterpreter(self, interp, lang):
        """Return the full path to the given interpreter or raise a suitable
        ValueError.
        """
        code = "(%s)" % interp
        prefName = interp+"DefaultInterpreter"
        if self._prefs.hasStringPref(prefName):
            path = self._prefs.getStringPref(prefName)
        if not path:
            path = self._FindInterpreterOnPath(interp)
        
        if not path:
            raise ValueError("This command string includes '%%(%s)', but "\
                             "no '%s' interpreter could be found. You can "\
                             "configure this by visiting the %s "\
                             "preferences panel in Edit->Preferences."\
                             % (interp, interp, lang))
        elif ' ' in path:
            return '"' + path.replace('"', '\\"') + '"'
        else:
            return path

    def Interpolate(self, command, fileName, word, selection):
        """Interpolate certain special syntax in the command string.
            %%      '%' (i.e. this is how you escape a '%')
            %f      current file name (just base name)
            %F      current file name (full path)
            %d      directory of current file (just base name)
            %D      directory of current file (full path)
            %s      selection
            %S      URL-escaped selection
            %w      selection or word under cursor
            %W      URL-escaped selection or word under cursor
        """
        #print "KoRunService.Interpolate(command=%r, fileName=%r, word=%r, "\
        #      "selection=%r)" % (command, fileName, word, selection)
        # Define the interpolation mapping.
        #   This is a mapping from interpolation code to the replacement
        #   value in the command. The "replacement" may be one of the
        #   following:
        #      - None; '%<code>' is left as is
        #      - a string; replaces '%<code>' in the command
        #      - a ValueError; exception string will be passed back through
        #        XPCOM and displayed as an error to the user
        #      - a callable object which either returns None or a string, or
        #        raises a ValueError
        self._interpCodeMap = {
            '%': '%',
            'f': ValueError("The command string includes %f, but there is no current file"),
            'F': ValueError("The command string includes %F, but there is no current file"),
            'd': ValueError("The command string includes %d, but there is no current file"),
            'D': ValueError("The command string includes %D, but there is no current file"),
            'w': ValueError("The command string includes %w, but there is no selection or word under cursor"),
            'W': ValueError("The command string includes %W, but there is no selection or word under cursor"),
            's': ValueError("The command string includes %s, but there is no selection"),
            'S': ValueError("The command string includes %S, but there is no selection"),
            '(perl)':   lambda interp='perl',   lang='Perl':   self._GetInterpreter(interp, lang),
            '(php)':    lambda interp='php',    lang='PHP':    self._GetInterpreter(interp, lang),
            '(python)': lambda interp='python', lang='Python': self._GetInterpreter(interp, lang),
            '(tclsh)':  lambda interp='tclsh',  lang='Tcl':    self._GetInterpreter(interp, lang),
            '(wish)':   lambda interp='wish',   lang='Tcl':    self._GetInterpreter(interp, lang),
            }
        
        browser = self._prefs.getStringPref('browser')
        if browser:
            self._interpCodeMap['(browser)'] = browser
        else:
            if sys.platform.startswith('win'):
                self._interpCodeMap['(browser)'] = 'start'
            else:
                self._interpCodeMap['(browser)'] = ValueError("The command string includes %(browser), but there is no default browser set")

        if fileName is not None:
            self._interpCodeMap['f'] = os.path.basename(fileName)
            self._interpCodeMap['F'] = fileName
            self._interpCodeMap['d'] = os.path.basename(os.path.dirname(
                                        fileName))
            self._interpCodeMap['D'] = os.path.dirname(fileName)
        if word:
            self._interpCodeMap['w'] = word
            self._interpCodeMap['W'] = word
        if selection:
            self._interpCodeMap['w'] = selection
            self._interpCodeMap['s'] = selection
            import urllib
            self._interpCodeMap['S'] = urllib.quote_plus(selection)
            if selection:
                self._interpCodeMap['W'] = urllib.quote_plus(selection)
        # Do the interpolation (XXX This is not re-entrant.)
        # Want:  %(\(browser\)|[abc])
        # Allows:
        #        %(browser) or %a or %b
        codeReStr = "%%"
        singleCharKeys = [k for k in self._interpCodeMap.keys() if len(k)==1]
        multiCharKeys = [re.escape(k) for k in self._interpCodeMap.keys() if len(k)>1]
        assert multiCharKeys
        codeReStr = "%%([%s]|%s)" % (''.join(singleCharKeys),
                                     '|'.join(multiCharKeys))
        try:
            interpolated = re.sub(codeReStr, self._ReplacePercent, command)
        except ValueError, ex:
            lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"]\
                           .getService(components.interfaces.koILastErrorService)
            lastErrorSvc.setLastError(0, ex.args[0])
            raise ServerException(nsError.NS_ERROR_INVALID_ARG, ex.args[0])

        # Clean up and return.        
        del self._interpCodeMap
        #print "KoRunService.Interpolate: %r -> %r" % (command, interpolated)
        return interpolated

    def _getEnvDictFromString(self, env):
        """Convert this:
            "VAR1=value1\nVAR2=value2\n..."
        to this:
            {"VAR1": "value1", "VAR2": "value2", ...}
        """
        envDict = {}
        for piece in env.split('\n'):
            equalSign = piece.find('=')
            envDict[piece[:equalSign]] = piece[equalSign+1:]
        return envDict

    def _mergeEnvWithParent(self, envDict):
        """Merge the given environment variables with parent process' environ.

        Note: koIUserEnviron is used to avoid picking up the current
              Komodo process' environment changes from getting in the way.
        """
        userEnvSvc = components.classes["@activestate.com/koUserEnviron;1"].getService()
        userEnvDict = {}
        for piece in userEnvSvc.GetEnvironmentStrings():
            equalSign = piece.find('=')
            userEnvDict[piece[:equalSign]] = piece[equalSign+1:]
        userEnvDict.update(envDict)
        return userEnvDict

    def RunInTerminal(self, command, cwd, env, terminal, termListener,
                      input=None):
        """Run the given command connected to the given terminal.

        Optionally an "input" string can be provided. If it is then the
        terminal's stdin is *not* used. Instead the given input string
        is passed to the child and that pipe is closed.

        A termination listener, "termListener", can ge registered to
        receive notification when the child terminates and with what
        exit value.
        """
        #print "KoRunService.RunInTerminal(command=%r, cwd=%r, env=%r, "\
        #      "terminal=%r, termListener=%r, input=%r)"\
        #      % (command, cwd, env, terminal, termListener, input)
        if not cwd: # Allow cwd="".
            cwd = None
        if not env:
            env = ""
        envDict = self._getEnvDictFromString(env)
        envDict = self._mergeEnvWithParent(envDict)
        # The process library requires that all env strings be ASCII
        # or all Unicode. We'll just make them all Unicode for now.
        #XXX This unicode conversion may not be necessary since
        #    _SaferCreateProcess in process.py.
        uEnvDict = {}
        for key, val in envDict.items():
            #XXX Linux may have trouble with all unicode env
            #    strings. Perhaps have all ASCII on Linux.
            uEnvDict[unicode(key)] = unicode(val)
        envDict = uEnvDict

        if input:
            try:
                p = _KoRunProcessProxy(command, cwd=cwd, env=envDict,
                                       stdout=terminal.stdout,
                                       stderr=terminal.stderr)
            except process.ProcessError, ex:
                _gLastErrorSvc.setLastError(ex.errno, str(ex))
                raise ServerException(nsError.NS_ERROR_FAILURE, str(ex))
            p.stdin.write(input)
            p.stdin.close()
        else:
            try:
                p = _KoRunProcessProxy(command, cwd=cwd, env=envDict,
                                       mode='b',  # XXX need to expose this
                                       stdin=terminal.stdin,
                                       stdout=terminal.stdout,
                                       stderr=terminal.stderr)
            except process.ProcessError, ex:
                _gLastErrorSvc.setLastError(ex.errno, str(ex))
                raise ServerException(nsError.NS_ERROR_FAILURE, str(ex))

        # If the user provides a termination listener (interface
        # koIRunTerminationListener), then setup a thread to notify of
        # termination of the child.
        if termListener:
            _termListenerProxy = _xpcom.getProxyForObject(None,
                components.interfaces.koIRunTerminationListener, termListener,
                _xpcom.PROXY_ALWAYS | _xpcom.PROXY_SYNC)
            t = _NotifyOnTermination(p, _termListenerProxy)
            t.start()
        # XXX What if no termination listener is provided? Does the loss
        #     of 'p' reference cause premature deletion of the Process?
        #     This might be moot when (if?) we start passing back a
        #     koIRunProcess object holding the Process object. Nope, it
        #     would not be moot if the user did not assign the returned
        #     process to a variable. Perhaps a _NotifyOnTermination()
        #     should *always* be created, holding that reference. This
        #     then *requires* that all child processes must close before
        #     Komodo, which sucks for, say, TkBuilder (*does* this imply
        #     that?).
        return p

    def _WaitAndCleanUp(self, child, command, scriptFileName=None,
                        inputFileName=None):
        retval = child.wait()

        if scriptFileName:
            try:
                os.unlink(scriptFileName)
            except OSError, ex:
                logWarning("Could not remove temporary script file '%s': %s"\
                           % (scriptFileName, ex))
        if inputFileName:
            try:
                os.unlink(inputFileName)
            except OSError, ex:
                logWarning("Could not remove temporary input file '%s': %s"\
                           % (inputFileName, ex))
        
        # Put appropriate message on the status bar.
        self._sm.category = "run_command"
        self._sm.msg = "'%s' returned %s." % (command, retval)
        self._sm.timeout = 3000
        if retval == 0:
            self._sm.highlight = 0
        else:
            self._sm.highlight = 1
        try:
            self._observerSvcProxy.Notify(self._sm, 'status_message', None)
        except COMException, e:
            # do nothing: Notify sometimes raises an exception if (???)
            # receivers are not registered?
            pass

    def Run(self, command, cwd, env, console, input=None):
        """Run the command string in a new (console) window

        "console" is a boolean indicating whether the child should be
            spawned with a new console window or not.
        
        Optionally an "input" string can be provided. If it is then the
        command is run via "command < input", where the input is placed
        in a temporary file.
        """
        #print "KoRunService.Run(command=%r, cwd=%r, env=%r, "\
        #      "console=%r, input=%r)"\
        #      % (command, cwd, env, console, input)
        if not cwd: # Allow cwd="".
            cwd = None
        if not env:
            env = ""
        envDict = self._getEnvDictFromString(env)
        envDict = self._mergeEnvWithParent(envDict)
        # The process library requires that all env strings be ASCII
        # or all Unicode. We'll just make them all Unicode for now.
        #XXX This unicode conversion may not be necessary since
        #    _SaferCreateProcess in process.py.
        uEnvDict = {}
        for key, val in envDict.items():
            uEnvDict[unicode(key)] = unicode(val)
        envDict = uEnvDict

        # Keep a pure 'command' for reporting status to the user.
        actualCommand = command

        if input:
            # Put the input into a temp file.
            inputFileName = tempfile.mktemp()
            inputFile = open(inputFileName, "w")
            inputFile.write(input)
            inputFile.close()
            if ' ' in inputFileName:
                actualCommand += ' < "%s"' % inputFileName
            else:
                actualCommand += ' < %s' % inputFileName
        else:
            inputFileName = None

        if console:
            # A shell script that will be run is created.  The script
            # adds the "Press any key to continue..." feature and
            # ensures that the correct return value is returned.
            if sys.platform.startswith("win"):
                scriptFileName = tempfile.mktemp() + ".bat"
                bat = open(scriptFileName, "w")
                bat.write("@echo off\n")
                # Escape '%' once for being in a batch file plus once for
                # being on a "call ..." line.
                actualCommand = actualCommand.replace("%", "%%%%")
                bat.write("call %s\n" % actualCommand)
                bat.write("set KOMODO_COMMAND_ERRORLEVEL=%ERRORLEVEL%\n")
                bat.write("pause\n")
                # Must "exit" with the last return value to ensure that
                # cmd.exe will return that value
                #XXX does the correct retval actually get returned?
                bat.write("exit %KOMODO_COMMAND_ERRORLEVEL%\n")
                bat.close()
                #XXX does this work on Win9x???
            else:
                # Create a bash script that will be run.
                scriptFileName = tempfile.mktemp() + ".sh"
                script = open(scriptFileName, "w")
                script.write("#!/bin/sh\n")
                script.write("%s\n" % actualCommand)
                script.write("export KOMODO_COMMAND_RETVAL=$?\n")
                script.write("echo -n Press ENTER to continue . . .\n")
                script.write("read KOMODO_DUMMY\n")
                script.write("unset KOMODO_DUMMY\n")
                script.write("exit $KOMODO_COMMAND_RETVAL\n")
                script.close()
            actualCommand = "%s %s %s"\
                            % (self.shell, self.shellArgs, scriptFileName)
            #print "RUN: cmdLine is '%s'" % cmdLine
            #print "RUN: -------- %s is as follows -------" % scriptFileName
            #fin = open(scriptFileName, 'r')
            #for line in fin.readlines():
            #    print line,
            #fin.close()
            #print "RUN: ---------------------------------"
            flags = process.Process.CREATE_NEW_CONSOLE
        else:
            scriptFileName = None
            flags = 0

        try:
            child = process.Process(actualCommand, cwd=cwd, env=envDict,
                                    flags=flags)
        except process.ProcessError, ex:
            _gLastErrorSvc.setLastError(ex.errno, str(ex))
            raise ServerException(nsError.NS_ERROR_FAILURE, str(ex))

        # The return value is passed to the status bar when the child
        # terminate. A separate thread is created to handle that so
        # this call can return immediately.
        t = threading.Thread(target=self._WaitAndCleanUp,
                             kwargs={'child': child,
                                     'command': command,
                                     'scriptFileName': scriptFileName,
                                     'inputFileName': inputFileName})
        t.start()

    def RunAndCaptureOutput(self, command, cwd, env, input=None):
        """Run the command and return the output.

        Optionally an "input" string can be provided. If it is then
        the given input string is passed to the child and that pipe is
        closed.
        """
        #print "KoRunService.RunAndCaptureOutput(command=%r, cwd=%r, "\
        #      "env=%r, input=%r)" % (command, cwd, env, input)
        if not cwd: # Allow cwd="".
            cwd = None
        if not env:
            env = ""
        envDict = self._getEnvDictFromString(env)
        envDict = self._mergeEnvWithParent(envDict)
        # The process library requires that all env strings be ASCII
        # or all Unicode. We'll just make them all Unicode for now.
        #XXX This unicode conversion may not be necessary since
        #    _SaferCreateProcess in process.py.
        uEnvDict = {}
        for key, val in envDict.items():
            uEnvDict[unicode(key)] = unicode(val)
        envDict = uEnvDict

        try:
            child = process.ProcessProxy(command, cwd=cwd, env=envDict)
        except process.ProcessError, ex:
            _gLastErrorSvc.setLastError(ex.errno, str(ex))
            raise ServerException(nsError.NS_ERROR_FAILURE, str(ex))

        if input:
            child.stdin.write(input)
            child.stdin.close()
        retval = child.wait()
        output = child.stdout.read()
        error = child.stderr.read()
        child.close()

        return retval, output, error

    def _WaitAndNotify(self, child, command):
        retval = child.wait(child.INFINITE)

        # Notify any listener of child termination.
        try:
            #XXX Dunno if need to pass 'child' in the notification here.
            self._observerSvcProxy.Notify(child, 'run_terminated', command)
        except COMException, e:
            # Do nothing: Notify sometimes raises an exception if (???)
            # no receiver are registered?
            pass

    def RunAndNotify(self, command, cwd, env, input=None):
        """Run the given command, returning the process object
        immediately, and notify (via nsIObserver) when the process
        terminates.
        """
        #print "KoRunService.RunAndNotify(command=%r, cwd=%r, "\
        #      "env=%r, input=%r)" % (command, cwd, env, input)
        if not cwd: # Allow cwd="".
            cwd = None
        if not env:
            env = ""
        envDict = self._getEnvDictFromString(env)
        envDict = self._mergeEnvWithParent(envDict)
        # The process library requires that all env strings be ASCII
        # or all Unicode. We'll just make them all Unicode for now.
        #XXX This unicode conversion may not be necessary since
        #    _SaferCreateProcess in process.py.
        uEnvDict = {}
        for key, val in envDict.items():
            uEnvDict[unicode(key)] = unicode(val)
        envDict = uEnvDict

        try:
            child = _KoRunProcessProxy(command, cwd=cwd, env=envDict)
        except process.ProcessError, ex:
            _gLastErrorSvc.setLastError(ex.errno, str(ex))
            raise ServerException(nsError.NS_ERROR_FAILURE, str(ex))

        if input:
            child.stdin.write(input)
            child.stdin.close()

        t = threading 
        t = threading.Thread(target=self._WaitAndNotify,
                             kwargs={'child': child, 'command': command})
        t.start()
        return child


class KoRunEnvView(_OutlinerView):
    _com_interfaces_ = [components.interfaces.koIRunEnvView]
    _reg_clsid_ = "{0E3559E7-093D-47d1-ABC6-717FADB0D6C8}"
    _reg_contractid_ = "@activestate.com/koRunEnvView;1"
    _reg_desc_ = "Komodo Run Command Environment Variables Outliner View"

    def __init__(self):
        _OutlinerView.__init__(self, debug=0)
        self._data = {}
        # A list of the current variable names (i.e. self._data.keys())
        # giving the order in the table.
        self._order = []
        # A string key indicating by which column the env table is
        # sorted.
        self._sortedBy = None

    def get_rowCount(self):
        return len(self._data.keys())

    def getCellText(self, row, col):
        """Get the value for the specified cell.
        "row" is an index into the current order (self._order).
        "col" is an 'id' of the <outlinercol> element. Here
            "env-variable" or "env-value".
        """
        try:
            variable = self._order[row]
        except IndexError:
            logError("no %sth row of data" % row)
            return ""

        cell = None
        if col == "env-variable":
            cell = variable
        elif col == "env-value":
            cell = self._data[variable]
        else:
            logError("unknown run environment variables column id: '%s'" % col)
            return ""

        if type(cell) not in (types.StringType, types.UnicodeType):
            cell = str(cell)
        return cell

    def Set(self, variable, value):
        if sys.platform.startswith("win"):
            variable = variable.upper()
        if not self._data.has_key(variable):
            self._order.append(variable)
            self._sortedBy = None
            self._outliner.rowCountChanged(len(self._data)-2, 1)
        self._data[variable] = value
        self._outliner.invalidate()  #XXX invalidating too much here?

    def Delete(self, index):
        del self._data[self._order[index]]
        del self._order[index]
        self._outliner.invalidate()  #XXX invalidating too much here?

    def Have(self, variable):
        """Return true iff have a datum with the given variable name."""
        if sys.platform.startswith("win"):
            variable = variable.upper()
        return self._data.has_key(variable)

    def Index(self, variable):
        if sys.platform.startswith("win"):
            variable = variable.upper()
        try:
            return self._order.index(variable)
        except ValueError:
            return -1

    def GetVariable(self, index):
        return self._order[index]
    def GetValue(self, index):
        return self._data[self._order[index]]

    def Sort(self, sortBy):
        """Sort the current data by the given key. If already sorted by this
        key then reverse the sorting order."""
        if self._sortedBy == sortBy:
            self._order.reverse()
        else:
            if sortBy == 'env-variable':
                self._order.sort()
            elif sortBy == 'env-value':
                self._order.sort(lambda a,b,data=self._data,sortBy=sortBy:
                                    cmp(data[a], data[b])
                                )
            else:
                logError("Cannot sort results by: '%s'" % sortBy)
                return
        self._sortedBy = sortBy
        self._outliner.invalidate()

    def GetEnvironmentStrings(self):
        vars = ["%s=%s" % item for item in self._data.items()]
        return "\n".join(vars)

    def SetEnvironmentStrings(self, s):
        self._data = {}
        for envvar in s.split('\n'):
            if envvar: # skip empty lines
                var, val = envvar.split('=', 1)
                self._data[var] = val
        self._order = self._data.keys()
        self._sortedBy = None
        self._outliner.invalidate()

