# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components, COMException, ServerException, nsError
import string
from UserList import UserList

class SeqEnum(UserList):
    # make lazy at some point.
    def __init__(self, enum, iid):
        _data = []
        while enum.hasMoreElements:
            _data.append(enum.getNext().QueryInterface(iid))
        UserList.__init__(self, _data)

class koStack:
    _com_interfaces_ = [components.interfaces.koIStack]
    _reg_clsid_ = "{ABC8B07E-E690-4e47-90C7-F44670E12575}"
    _reg_contractid_ = "@activestate.com/stack;1"

    def __init__(self):
        self.listeners = {}
        self.listener_cookie = 0
        self._dirty = 0
        self._props = {}
        # we don't store things which are returned from the debugger's
        # enumContext methods, since those will be displayed anyway.
        # what we _do_ store are 1) watched variables
        # (context='watch') and 2) whether a variable in a context is
        # expanded or not
        self._frames = None
        self.watched_varNames = {}  # values are irrelevant

        # keys are tuples of (context, fullname), values are irrelevant
        self.expanded_vars = {}
        self.current_frame_index = 0
        
    def set_current_frame_index(self, index):
        self.current_frame_index = index # XXX need to do bounds-checking
        

    def init(self):
        self.dbg = components.classes["@activestate.com/debugger;1"]\
              .getService(components.interfaces.koIDebugger)
#        self.dbg_listener_cookie = self.dbg.addListener( self )
        self._frames = self.dbg.enumStackFrames()
        
    def close(self):
        self.dbg.removeListener(self.dbg_listener_cookie)

    def addListener(self, listener):
        print "addListener"
        cookie = self.listener_cookie = self.listener_cookie + 1
        self.listeners[cookie] = listener
        return cookie

    def removeListener(self, cookie):
        if not self.listeners.has_key(cookie):
            print "Asked to remove listener", cookie, " I don't have!", self.listeners.keys()
            raise ServerException(nsError.NS_ERROR_UNEXPECTED)
        del self.listeners[cookie]
        
    def onStateChanged(self, state, stateExtra):
        # the debugger is telling us that something changed.
        print "onStateChanges"
        self._dirty = 1
        for cookie, listener in self.listeners.items():
            if listener:
                listener.update()
            
    def getWatchedVars(self, frame_index):
        if self._dirty:
            try:
                self.init()
                rc = self._getWatchedVars(frame_index)
                if rc is not None:
                    return rc
            except COMException, why:
                # debugger not running, but we want to get the variables anyway
                vars = []
                for varname in self.watched_varNames.keys():
                    vars.append((varname, None))
                vars.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
                return map( lambda f:f[0], vars), map( lambda f:f[1], vars)
        return (), ()

    def _getWatchedVars(self, frame_index):
        vars = []
        temp_index = 0
        for frame_index in range(len(self._frames)):
            frame = self._frames[frame_index].QueryInterface(components.interfaces.koIDebugStackFrame)
            if temp_index == frame_index:
                for varname in self.watched_varNames.keys():
                    # If getting this variable in this stack frame
                    # fails for _any_ reason, we assume it is simply not a
                    # valid expression in this context.
                    try:
                        v = frame.getExpression(varname, 10)
##                        if v.type != 'Error': # XXX check details
                        vars.append((varname, v))
                    except COMException, why:
                        pass
                vars.sort(lambda a,b: cmp(a[0].lower(), b[0].lower()))
                return map( lambda f:f[0], vars), map( lambda f:f[1], vars)
            temp_index = temp_index + 1

    def addWatchedVarName(self, expr):
        self._dirty = 1
        self.watched_varNames[expr] = 1

    def removeWatchedVarName(self, expr):
#        assert self.watched_varNames.has_key(expr), "Removing non-watched variable!"
        if self.watched_varNames.has_key(expr):
            del self.watched_varNames[expr]
        if len(self.watched_varNames) == 0:
            self._dirty = 0

    def expandVar(self, context, fullname):
        self.expanded_vars[(context, fullname)] = 1

    def contractVar(self, context, fullname):
#        assert self.expanded_vars.has_key((context, fullname))
        if self.expanded_vars.has_key((context, fullname)):
            del self.expanded_vars[(context, fullname)]

    def isVarExpanded(self, context, fullname):
        return self.expanded_vars.has_key((context, fullname))


    def addDebugProperty(self, context, fullname, prop):
        self._props[(context, fullname)] = prop

    def lookupDebugProperty(self, context, fullname):
        if not self._props.has_key((context, fullname)):
            raise ServerException(nsError.NS_ERROR_UNEXPECTED, "I don't have the key!")
        return self._props[(context, fullname)]

    def removeDebugProperty(self, context, fullname):
        if not self._props.has_key((context, fullname)):
            raise ServerException(nsError.NS_ERROR_UNEXPECTED, "I don't have the key!")
        del self._props[(context, fullname)]
