#!/usr/bin/env python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# Launch a URL in the user's "default" browser.
#   We used to use "webbrowser.py" here but it does netscape magic and
#   allows for command line browsers, both of which can cause Komodo to hang.

from xpcom import components
from xpcom import components, nsError, ServerException
from koLogger import logWarning, logDebug, logError
import process, koprocessutils
import sys, os

class KoWebbrowser:
    _com_interfaces_ = [components.interfaces.koIWebbrowser]
    _reg_clsid_ = "{7B361CD6-4426-4d23-86D6-B6F8E0FCDA20}"
    _reg_contractid_ = "@activestate.com/koWebbrowser;1"
    _reg_desc_ = "Komodo Default Webbrowser Service"

    def open_new(self, url):
        """Open the given URL in the user's default browser.
        Return true iff successful.
        """
        prefs = components.classes["@activestate.com/koPrefService;1"].\
                getService(components.interfaces.koIPrefService).prefs
        if sys.platform.startswith("win") and prefs.getStringPref('browser') == '':
            try:
                os.startfile(url)
                return 1
            except WindowsError, e:
                # I don't know who is at fault here but if Netscape 6 (and
                # presumable, therefore, some versions of Mozilla) is set as
                # the default then (1) the browser successfully loads the
                # file *and* (2) this is raised:
                #    Traceback (most recent call last):
                #      File "C:\mozilla_source\moz.09apr.trentm.winnt.release.komodo5\mozilla\dist\WIN32_O.OBJ\bin\components\koWebbrowser.py", line 27, in open_new
                #        os.startfile(url)
                #    WindowsError: [Errno 2] The system cannot find the file specified: 'http://www.ActiveState.com/ASPN/Downloads/Komodo/NeedLicense?os=Windows2000v.5.0.2195&build=19520&version=1.1.0-devel'
                #
                if e.errno == 2:
                    return 1
                else:
                    return 0
            except:
                return 0
        sysUtils = components.classes["@activestate.com/koSysUtils;1"].\
            getService(components.interfaces.koISysUtils)
        try:
            env = koprocessutils.getUserEnv()
        except ServerException, ex:
            logError(str(ex))
            return 1
        browser = prefs.getStringPref('browser')
        try:
            process.Process("%s %s" % (browser, url), env=env)
            return 1
        except process.ProcessError, ex:
            logError(str(ex))
            return 0

    def _SameFile(self, fname1, fname2):
        return ( os.path.normpath(os.path.normcase(fname1)) ==\
            os.path.normpath(os.path.normcase(fname2)) )

    def get_possible_browsers(self):
        sysUtils = components.classes["@activestate.com/koSysUtils;1"].\
            getService(components.interfaces.koISysUtils)
        if sys.platform.startswith('win'):
            matches = sysUtils.WhichAll("iexplore") + \
                    sysUtils.WhichAll("mozilla") + \
                    sysUtils.WhichAll("opera") + \
                    sysUtils.WhichAll("netscape")
        else:
            matches = sysUtils.WhichAll("mozilla") + \
                    sysUtils.WhichAll("konqueror") + \
                    sysUtils.WhichAll("opera") + \
                    sysUtils.WhichAll("netscape") + \
                    sysUtils.WhichAll("netscape-communicator") + \
                    sysUtils.WhichAll("kfm")
        # check for duplicates
        matchesWithoutDups = []
        for i in range(len(matches)):
            for j in range(i+1, len(matches)):
                if self._SameFile(matches[i], matches[j]):
                    break
            else:
                matchesWithoutDups.append(matches[i])
        return matchesWithoutDups
