from xpcom import components, ServerException

from koLanguageServiceBase import *

class koXMLLanguage(KoLanguageBase):
    name = "XML"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{13CE88EE-3F2D-4ad4-8D33-CC8178E4BA26}"

    defaultExtension = ".xml"
    styleBits = 6
    commentDelimiterInfo = { "block": [ ("<!--", "-->") ]  }
    
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_XSLT)
            self._lexer.supportsFolding = 1
        return self._lexer

    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService( "@activestate.com/koXMLLinter;1", None)
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            self._methodcompletion = components.classes["@activestate.com/koXMLCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def getEncodingWarning(self, encoding):
        if not encoding.byte_order_marker:
            if encoding.python_encoding_name.startswith('utf-16') or encoding.python_encoding_name.startswith('ucs-'):
                return 'Including a signature (BOM) is recommended for "%s".' % encoding.friendly_encoding_name
            else:
                return ''
        else: # It's all good
            return ''
        
