#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components, _xpcom
from xpcom.server.enumerator import *
import os, sys
import string
from koLogger import logWarning, logDebug, logError
from  koXMLCompletionBase import koXMLCompletionBase, koNamespaceHandler, koDumbXMLNamespaceHandler
from koXMLStateMachineReader import state_machine_info
import xmllex

class koXSLTCompletion(koXMLCompletionBase):
    _reg_clsid_ = "{CC666E6B-7885-48bf-9D43-54EF1D009C5C}"
    _reg_contractid_ = "@activestate.com/koXSLTCompletionLanguageService;1"

    _machine_info = None

    def __init__(self):
        self._ok = 0
        koXMLCompletionBase.__init__(self)
        self.load_tags()

    def AutoComplete(self, ch, scimoz):
        if self._ok:
            return koXMLCompletionBase.AutoComplete(self, ch, scimoz)
        else:
            logError( "koXSLTCompletion could not load statemachine." )

    def load_tags(self):
        # read ML tag definitions
        # format: TAG ATTR ATTR ....
        try:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
              getService(components.interfaces.koIDirs)

            filename = os.path.join(str(koDirs.installDir),"xslt","state-machine.xml")

            koXSLTCompletion._machine_info = state_machine_info(filename)
            self._ok = 1
        except Exception, e:
            logError( "koXSLTCompletion could not load statemachine %s" % e )

class XSLTNamespaceHandler(koDumbXMLNamespaceHandler):
    namespace = "http://www.w3.org/1999/XSL/Transform"
    def __init__(self, scimoz, scitext, pos, tree):
        koDumbXMLNamespaceHandler.__init__(self, scimoz, scitext, pos, tree)
        self.parent = tree.current
        # prefer to let koXSLTCompletion initialize itself on first load
        # but if it hasn't been loaded, I'll do it myself here.
        if not koXSLTCompletion._machine_info:
            koXSLTCompletion().load_tags()

    def localnames(self, prefix):
        names = koXSLTCompletion._machine_info.\
               possible_children(xmllex.localname(xmllex.parent(self.parent)))

        if xmllex.namespaces(self.parent).get(prefix, None)==self.namespace:
            try:
                del names[names.index("#LITERAL")]
            except (KeyError, ValueError):
                pass
            return names
        else:
            return koDumbXMLNamespaceHandler.localnames(self, prefix)

    def tagnames(self):
        if self.parent and xmllex.namespace(self.parent)==self.namespace:
            names = koXSLTCompletion._machine_info.possible_children(
                        xmllex.localname(self.parent))
        else:
            names = koXSLTCompletion._machine_info.possible_children(
                        "#LITERAL")
        addliterals = "#LITERAL" in names
        prefix = xmllex.prefix_for(self.parent, self.namespace)
        if prefix:
            prefix += ":"
        else:
            prefix = ""
        names = [prefix + name
                     for name in names
                        if name != "#LITERAL"]

        if addliterals:
            xsltprefixes = [prefix 
                for prefix, namespc in xmllex.namespaces(self.parent).items()
                    if namespc==self.namespace]
                
            otherstuff = koDumbXMLNamespaceHandler.tagnames(self)
            for name in otherstuff:
                if (name not in names
                    and name.split(":")[0] not in xsltprefixes):
                    names.append(name)

        return names

    def attrs(self):
        namespace = xmllex.namespace(self.parent)
        if namespace!=self.namespace:
            logError("Don't know about attributes for handle %s" %
                     xmllex.tagname(self.parent))
            return []
        else:
            localname = xmllex.localname(self.parent)
            return koXSLTCompletion._machine_info.possible_attributes(localname)

    def values(self, attrname):
        namespace = xmllex.namespace(self.parent)
        if namespace!=self.namespace:
            logError("Don't know about attributes for handle %s" %
                     xmllex.tagname(self.parent))
            return []
        else:
            localname = xmllex.localname(self.parent)
            return koXSLTCompletion._machine_info.\
                   possible_attribute_values(localname, attrname)

koNamespaceHandler.handlers[XSLTNamespaceHandler.namespace] = XSLTNamespaceHandler
