from xpcom import components, ServerException
from koLanguageServiceBase import *
import os
from koLogger import logError, logWarning, logDebug

class koXSLTLanguage(KoLanguageBase):
    name = "XSLT"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{7F76A3CE-7FE1-4363-99EB-4AAAFA79BC88}"

    defaultExtension = ".xsl"
    styleBits = 6
    commentDelimiterInfo = { "block": [ ("<!--", "-->") ],
                             "line": [ ("<!--", "-->") ]}
    cookbookName = "XSLT"
    cookbookUrl = "http://aspn.activestate.com/ASPN/Cookbook/XSLT"

    namedBlockRE = r'(\<xsl:template.*?\>)'
    namedBlockDescription = 'XSL Templates'

    def get_debugger(self):
        if self._debugger is None:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
              getService(components.interfaces.koIDirs)
            dbgpy = os.path.join(koDirs.mozBinDir, "components" ,"koXSLTDebugger.py")
            if os.path.exists(dbgpy):
                self._debugger = KoDebuggerLanguageService(self.name)
            else:
                logWarning( 'Could not find "%s" for XSLT debugging.' % dbgpy )
                self._debugger = None
        return self._debugger
        

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_XSLT)
            self._lexer.supportsFolding = 1
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
              getService(components.interfaces.koIDirs)
            linterPath = os.path.join(koDirs.mozBinDir, "components" ,"koXSLTLinter.py")
            if os.path.exists(linterPath):
                self._linter = KoLinterLanguageService( "@activestate.com/koXSLTLinter;1", None)
            else:
                logWarning( 'Could not find "%s" for XSLT linting, using XML linter.' % linterPath )
                self._linter = KoLinterLanguageService( "@activestate.com/koXMLLinter;1", None)
        return self._linter

    def get_methodcompletion(self):
        if self._methodcompletion is None:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
                                  getService(components.interfaces.koIDirs)
            completionPath = os.path.join(koDirs.mozBinDir, "components" ,"koXSLTCompletion.py")
            if os.path.exists(completionPath):
                self._methodcompletion = components.classes["@activestate.com/koXSLTCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
            else:
                logWarning( 'Could not find "%s" for XSLT completion.' % completionPath )

        return self._methodcompletion

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    def get_cookbook(self):
        if self._cookbook is None:
            self._cookbook = KoCookbookLanguageService(
                self.cookbookName, self.cookbookUrl)
        return self._cookbook

    def getEncodingWarning(self, encoding):
        if not encoding.byte_order_marker:
            if encoding.python_encoding_name.startswith('utf-16') or encoding.python_encoding_name.startswith('ucs-'):
                return 'Including a signature (BOM) is recommended for "%s".' % encoding.friendly_encoding_name
            else:
                return ''
        else: # It's all good
            return ''