#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

import os
from xpcom import components
from koLintResult import *
from koLintResults import koLintResults
from koLogger import logWarning, logDebug, logError
import eol
import re
import process

PROXY_SYNC    = 0x0001  # acts just like a function call.
PROXY_ASYNC   = 0x0002  # fire and forget.  This will return immediately and you will lose all return information.
PROXY_ALWAYS  = 0x0004  # ignore check to see if the eventQ is on the same thread as the caller, and alway return a proxied object.


class KoXSLTCompileLinter:
    _com_interfaces_ = [components.interfaces.koILinter]
    _reg_desc_ = "Komodo XSLT Compile Linter"
    _reg_clsid_ = "{ae3f7974-78cd-43fc-a06d-99325911e619}"
    _reg_contractid_ = "@activestate.com/koXSLTLinter;1"

    initialized = 0

    def __init__(self):
        proxyManager = components.classes["@mozilla.org/xpcomproxy;1"].\
                            getService(components.interfaces.nsIProxyObjectManager)

        koDirs = components.classes["@activestate.com/koDirs;1"].\
            getService(components.interfaces.koIDirs)
        xmlLinter = components.classes["@activestate.com/koXMLLinter;1"].\
                            getService(components.interfaces.koILinter)

        self._koDirs = proxyManager.getProxyForObject(None,
                                                       components.interfaces.koIDirs,
                                                       koDirs,
                                                       PROXY_SYNC | PROXY_ALWAYS)
        self._xmlLinter = proxyManager.getProxyForObject(None,
                                                       components.interfaces.koILinter,
                                                       xmlLinter,
                                                       PROXY_SYNC | PROXY_ALWAYS)

    def lint(self, text, encoding, cwd):
        results = self._xmlLinter.lint(text, encoding, cwd)
        if results is not None:
            return results

        #XXX unfinished business
        # we never get to this point since the above lint ALWAYS returns
        # results.  Until XSLTSyntaxChecker is done, this is a GOOD THING(tm)
        
        lintResults = koLintResults()
     
        cwd = os.path.join(self._koDirs.installDir, 'xslt')
        cmd = './XSLTSyntaxChecker state-machine.xml'
        p = process.ProcessOpen(cmd, cwd=cwd)

        text = eol.convertToEOLFormat(text, eol.EOL_LF)
        lines = text.split('\n')
        p.stdin.write(text)
        p.stdin.close()

        resultPattern = re.compile('''Error #: """(.*?)""" \[(\d+)\] @ \((\d+), (\d+)\) => \((\d+), (\d+)\)''')
        for i in p.stdout.readlines():
            lintResult = KoLintResult()

            lintResult.description, severity,\
                lineStart, columnStart,\
                lineEnd, columnEnd = resultPattern.match(i).groups()                                              
          
            lintResult.lineStart = int(lineStart)
            lintResult.columnStart = int(columnStart)
            lintResult.lineEnd = int(lineEnd)
            lintResult.columnEnd = int(columnEnd)

            EXTENDFIRSTLINE = 0
            EXTENDLASTLINE = 1
            EXTENDOTHER = 2
            NOCHANGE = 3
         
            fixMethod = EXTENDOTHER 
            for i in range(lintResult.lineStart, lintResult.lineEnd + 1):
                if len(lines[i-1]) > 0:
                    # See if the column start is at the beginning of the line and is, therefore,
                    # not doing us any good
                    if i == lintResult.lineStart and lintResult.columnStart >= len(lines[i-1]):
                        fixMethod = EXTENDFIRSTLINE
                    elif i == lintResult.lineEnd:
                        # Extended the last line is prefered over extending the first line
                        if lintResult.lineStart == lintResult.lineEnd and \
                                lintResult.columnStart == lintResult.columnEnd or \
                                lintResult.columnEnd == 1:
                            fixMethod = EXTENDLASTLINE
                    else:
                        fixMethod = NOCHANGE
                        break
              
            if fixMethod == EXTENDFIRSTLINE:
                lintResult.columnStart = 1
            elif fixMethod == EXTENDLASTLINE:
                lintResult.columnEnd = len(lines[lintResult.lineEnd-1]) + 1
            elif fixMethod == EXTENDOTHER:
                i = lintResult.lineStart
                # Work backwards to extend the first non-blank line that we find
                while i > 0:
                    if len(lines[i-1]) > 0:
                        lintResult.lineStart = i
                        lintResult.columnStart = 1
                        break
                    i = i - 1

            if severity == 1: # XML_WARNING
                lintResult.severity = lintResult.SEV_WARNING
            else:             # XML_ERROR (and call it an error until proven otherwise)
                lintResult.severity = lintResult.SEV_ERROR

            lintResults.addResult( lintResult )
        p.stdout.close()
        p.close()
        return lintResults
