# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# command-line listener
# just reads from stdin, writes to stdout 
# instead of using OLE
#
# to talk to the jacket.
# 
# Originally written by Graham TerMarsch <gtermars@home.com> and
# Doug Lankshear <dougl@activestate.com>.
#
# Modified by Eric Promislow <ericp@activestate.com> to work with
# Komodo.
#
###############################################################################


use ASRemote::Jacket;
use strict;

# $ASRemote::Jacket::TraceFile = './jacket-trace.log';
# $ASRemote::Jacket::SessionTraceFile = './jacket-session.log';


my $rtval;
my $cmd;

my $jacket;
my $ldebug;
my ($App, $Sync);
    my $rv;

BEGIN {
    $|=1;
    print STDERR "Doing the komodo listener.\n" if $ldebug;
}

sub init1 {
    my $args = shift;
    my ($cmdPort, $IOPort, $showConsole, $isRemote) = split ':', $args;
    my %param = ( -port => $cmdPort );  #XXX definitely pass it in.
    %param = ( -perlsock => 1 ) if (defined $ENV{'PerlHost'} and
				    $ENV{'PerlHost'} ne '');

    $jacket = ASRemote::Jacket->new( %param );
    $jacket->{'IOPort'} = $IOPort;
    $jacket->{'showConsole'} = $showConsole;
    $jacket->{'isRemote'} = $isRemote;
    eval '$rv = $jacket->Connect();';
    if ($@) {
	print STDERR "Connect:$@\n";
	return "0";
    }
    print STDERR "Connect returned rv=$rv\n" if $ldebug;
    $ldebug = 0;

# Check which OLE is available to us
    my $OLD_OLE = 1;

    $rtval = 0;
    $cmd = '';

    return $rv ? "0" : "1";
}

sub IsBreakPoint($@) {
    my $here = shift;
    my @list = @_;
    # print STDERR "Breakpoints: " . join(" ", @list) . "\n";
    foreach (@list) {
	return 1 if ($here eq $_);
    }
    return 0;
}

sub SetBreakPoint($$) {
    my $jacket = shift;
    my $line = shift;

    ## This triggers a fault when we try to set the first breakpoint
    ## my %old = %{ $jacket->{'breakpoints'} };

    my $jacketName = $jacket->{'breakpoints'};
    my %old;
    ###my $tmp;
    ###if ($tmp) {

    if ($jacketName) {
	print STDERR "\$jacket->{'breakpoints'} => $jacketName\n";
	print STDERR "Old breakpoints: " . join(" ", keys(%old)) . "\n";
	%old = %{ $jacket->{'breakpoints'} };
    } else {
	print STDERR "No current jacketName\n";
	print STDERR "Call SetBreakPoint($line)\n";
	%old = {};
    }
    
    my $rtval = $jacket->SetBreakpoint( $line );
    print DB::OUT $jacket->{'error_text'} if ($rtval);

    my %new = %{ $jacket->{'breakpoints'} };
    print STDERR "New breakpoints: " . join(" ", keys(%new)) . "\n";
    foreach my $val (keys(%new)) {
        if (!defined($old{$val})) {
	    if ($val =~ /^(.+):(\d+)/o)
	    {
		$val = $2;
	    }
	    print STDERR "SetBreakPoint: listener returns $val\n";
	    return $val;
	}
    }
    return 0;
}

sub RemoveBreakPoint($$) {
    my $jacket = shift;
    my $line = shift;
    return 0 unless $jacket->{'breakpoints'};
    my %old = %{ $jacket->{'breakpoints'} };

    # print STDERR "Old breakpoints: " . join(" ", keys(%old)) . "\n";
    my $rtval = $jacket->RemoveBreakpoint( $line );
    print DB::OUT $jacket->{'error_text'} if ($rtval);

    my %new = %{ $jacket->{'breakpoints'} };
    # print STDERR "New breakpoints: " . join(" ", keys(%new)) . "\n";
    foreach my $val (keys(%old)) {
        if (!defined($new{$val})) {
	    if ($val =~ /^(.+):(\d+)/o)
	    {
		$val = $2;
	    }
	    return $val; 
	}
    }
    return 0;
}

# do_cmd (command) -- called from the C thing, drives the listener
# This is in a loop only as long as we need to update the status.
# Afterwards, it returns to the caller.

# Return values:
# 0 -- all finished
# 1 -- normal return
# 2 -- call UpdateStatus next time.

sub do_cmd
{
    ($cmd) = @_;
    chomp $cmd;
    my $updatestatus = 0;

    print STDERR "Processing command $cmd\n" if $ldebug;
    return "" if $jacket->{'terminated'};
    # tell ASPD where the source file is
#    print STDERR "SourceFile = " . $jacket->{'filename'} . "\n";
#    print STDERR "LineNumber = " . $jacket->{'linenum'} . "\n";
#    print STDERR "codeline = " . $jacket->{'codeline'} . "\n";

    if ($cmd eq 'PeerHost') {
	# get the socket client address
	return $jacket->{'peerhost'};
    } 
    elsif ($cmd eq 'PeerPort') {
	# get the socket client address
	return $jacket->{'peerport'};
    }
    if (!$jacket->{'debuggerInitialized'}) {
	if ($cmd =~ /^InitDebugger\s*(\S+)$/) {
	    my $args = $1;
	    my ($IOPort, $showConsole, $isRemote) = split ':', $args;
	    $jacket->{'IOPort'} = $IOPort;
	    $jacket->{'showConsole'} = $showConsole;
	    $jacket->{'isRemote'} = $isRemote;
	    $jacket->InitDebugger();
	    return $jacket->{'debuggerInitialized'};
	} else {
	    return "";
	}
    }
    if ($cmd eq "Quit") {
	# This isn't needed.
	return "";
    }
    elsif ($cmd eq "RemoveAllBreakpoints") {
	$rtval = $jacket->ClearBreakpoints();
	return $jacket->{'error_text'} if ($rtval);

	# notify window
    }
    elsif ($cmd eq "ListBreakPoints") {
	$rtval = $jacket->QQQ_ListBreakpoints();
	print STDERR "QQQ: listener.pl: ListBreakPoints returned $jacket->{'result'}\n";
	return $jacket->{'result'};
    }
    elsif (substr($cmd,0,19) eq "CanInsertBreakpoint") {
    }
   elsif ($cmd =~ /^InsertBreakpoint\s*(\S+)$/) {
	my $i = $1;
	$i = SetBreakPoint($jacket, $i);
	# print STDERR "InsertBreakpoint -> $i \n";
	return $i;
    }
    elsif ($cmd =~ /^RemoveBreakpoint\s*(\S+)$/) {
	my $i = $1;
	$i = RemoveBreakPoint($jacket, $i);
	# print STDERR "RemoveBreakPoint -> $i \n";
	return $i;
    }
    elsif ($cmd =~ /^ToggleBreakpoint\s*(\S+)$/) {
	my $i = $1;
	printf STDERR "Toggling bp at $i\n";
	if (IsBreakPoint("$jacket->{'filename'}:$i", keys %{ $jacket->{'breakpoints'} } )) {
	    printf STDERR "Yes, it's a bp.\n";
	    $i = RemoveBreakPoint($jacket, $i);
	    if ($i) {
		printf STDERR "and it's been removed.\n";
		return "RemoveBreakpoint $i";
	    }
	    else {
		printf STDERR "and we failed to remove it.\n";
	    }
	}
	else {
	    printf STDERR "No, it's not a bp.\n";
	    $i = SetBreakPoint($jacket, $i);
	    if ($i) {
		printf STDERR "and it's been set ($i).\n";
		return "InsertBreakpoint $i";
	    } else {
		printf STDERR "but it wasn't set.\n";
	    }
	}
    }
    elsif ($cmd eq "StepOver") {
	$rtval = $jacket->StepOver();
####	return $rtval ? "0" : "1";
    }
    elsif ($cmd eq "StepInto") {
	$rtval = $jacket->Step();
####	return $rtval ? "0" : "1";
    }
    elsif ($cmd eq "StepOut") {
	$rtval = $jacket->ReturnFromSub();
####	return $rtval ? "0" : "1";
    }
    elsif ($cmd eq "Continue") {
	print STDERR "Continuing..." if $ldebug;
	$rtval = $jacket->RunUntil();
####	return $rtval ? "0" : "1";
    }
    elsif (substr($cmd,0,11) eq "RunToCursor") {
	# grab line number
	my $i = int substr($cmd,12,9);
	$rtval = $jacket->RunUntil($i);
####	return $rtval ? "0" : "1";
    }
    elsif ($cmd eq "CallStack") {
	my $res;
	if ($res = $jacket->DumpCallStack()) {
	    print STDERR "Error getting the call stack: $res\n";
	    return "0";
	} else {
	    $res = $jacket->{'result'};
	    print STDERR "$cmd => $res\n" if $ldebug;
	    return $res;
	}
    }
    elsif ($cmd eq "UpdateStatus") {
	UpdateWatches($jacket);
	return "Line " . $jacket->{'linenum'} .
	    ", File " . $jacket->{'filename'};
    }
    elsif ($cmd eq "GetLineNumber") {
	return "" . $jacket->{'linenum'};
    }
    elsif ($cmd eq "GetFileName") {
	my $testFileName = $jacket->{'filename'};
# Let JavaScript deal with this
#	$testFileName =~ s/\(eval\s+\d+\)\[(.*):\d+\]/\1/;
#	print STDERR "GetFileName => $testFileName from " .
#	    $jacket->{'filename'} . '\n' if $ldebug;
	return $testFileName;
    }
    elsif (substr($cmd,0,16) eq "GetVariableValue") {
	my $varname = substr($cmd, 17);
	if ($varname !~ m/^[\$@%*]/) {
	    $varname = "\$$varname";
	}
	$jacket->QuickEval("$varname");
	my $res = $jacket->{'result'};
        print STDERR "$cmd ($varname) => $res\n" if $ldebug;
	return "$res";
    } elsif ($cmd =~ /^EvalExpression (.+)$/) {
	my $expn = $1;
	my $retval = $jacket->QuickEval("$expn");
	my $res = $jacket->{'result'};
        print STDERR "EvalExpression($expn) => ($retval, $res)\n" if $ldebug;
	return "$res";
    }
    # Returns a string which can be unpickled into a Python tuple
    # of strings.
    elsif ($cmd =~ /^EvalExpressions (.+)$/) {
	# Return a pickled python tuple of value, reference pairs for each space-separated Perl expression.
	# Note that the reference handling is not strictly correct.  This was done for compatability with
	# the existing code in koPerlDebugger.py.  References for the various types are:
	#    ARRAY, HASH: ''
	#    other:       ref($foo)

	#XXX Because we use a space-separated list of expressions, this is not a viable general solution.
	#XXX At this time, we are only passing variable names to this function, so spaces are ok as
	#XXX delimiters.
	my $exprs = quotemeta $1;

	my $cmd = <<'END';
do {
    my @__Komodo_vars__ = split ' ', !!EXPRESSIONS!!;
    my @__Komodo_retlist__;
    for my $__Komodo_loop__ (@__Komodo_vars__) {
	my $__Komodo_ref__ = eval qq{ref\\$__Komodo_loop__};
	if ($__Komodo_ref__ eq "ARRAY" || $__Komodo_ref__ eq "HASH") {
	    push @__Komodo_retlist__, $__Komodo_ref__, "";
	}
	else {
	    push @__Komodo_retlist__, eval qq{defined $__Komodo_loop__ ? overload::StrVal($__Komodo_loop__) : undef }, eval qq{ref$__Komodo_loop__};
	}
    }
    DB::ASRemote::pickle(@__Komodo_retlist__);
};
END
	$cmd =~ s/!!EXPRESSIONS!!/"$exprs"/;
	$cmd =~ s/\n/ /g;
	my $retval = $jacket->QuickEval($cmd);
	my $res = $jacket->{'result'};
	print STDERR "EvalExpressions($exprs) => ($retval, $res)\n" if $ldebug;
	return "$res";
    }
    elsif ($cmd =~ /^RelayToDebugger (.+)$/) {
	my $expn = $1;
	my $retval = $jacket->RelayToDebugger("$expn");
	my $res = $jacket->{'result'};
        print STDERR "RelayToDebugger($expn) => ($retval, $res)\n" if $ldebug;
	return "$res";
    }
    elsif (substr($cmd,0,15) eq "WatchExpression") {
	my $varname = substr($cmd, 16);
	return $jacket->AddWatchExpression("$varname");
    }
    elsif (substr($cmd,0,10) eq "DoWatchTip") {
	# note that the GUI requires a trailing '\n'
	my $watchexpr = substr($cmd,11);
	if ($watchexpr eq '')
	{
	    # Quick opt; nothing to display, return empty.  NOTE, though, that
	    # this is _NEEDED_ as otherwise the GUI halts waiting for a
	    # response to come back.
	}
	else
	{
	    # Evaluate the expression to get the result and send it back to the
	    # GUI.
	    $jacket->QuickEval( $watchexpr );
	}
    }
    elsif (substr($cmd,0,12) eq "DumpVariable") {
	# grab variable namecaller(1)caller(1)
	my $varname = substr($cmd,13);
	# remove variable symbol
	my $varnamechar = substr($varname,0,1);
	if ($varnamechar eq '$' or		# ' dcb -- just a comment to fix Emacs syntax coloring
	    $varnamechar eq '@' or
	    $varnamechar eq '%')
	{
	    $varname = substr($varname,1);
	}

	#todo: send to temporary file
    }
    elsif ($cmd eq "SourceFile") {
	my $base;
	($base = $jacket->{'filename'}) =~ s/^.*[\/\\]([^\/\\]+)$/$1/;

	# the dir we use for temp files depends on the os
	my $fallback_dir = $^O =~ /MSWin32/ ? 'c:' : '/tmp';
	my $local_file = ($ENV{'TEMP'} || $ENV{'TMP'} || $fallback_dir) . "/" . $$ . $base;
	$rtval = $jacket->GetScript( $jacket->{'filename'}, $local_file );
	print DB::OUT $jacket->{'error_text'} if ($rtval);
	$updatestatus = 1;

	open(FILE, "$local_file");
	my @dbline = <FILE>;
	close(FILE);
	unlink $local_file;

	return join ("", @dbline);
    }
    elsif ($cmd eq "UpdateWatches") {
	return UpdateWatches ($jacket);
    }
    elsif ($cmd eq "GetCurrentLine") {
	return $jacket->GetCurrentLine();
    }

    return "" if $jacket->{'terminated'};
    return "1";
}

sub term1 ()
{
# exit script.  Be sure to tell the Jacket to quit, though, otherwise
# the connection doesn't get shut down properly to the remote script.
    $jacket->Quit();

    undef $jacket;
}


sub UpdateWatches {
    my $jacket = shift;
    # Get the list of variables we are already watching
    my @was_watched;
    map { push( @was_watched, $_->{'name'} ); } @{$jacket->{'watched'}};

    # Get the new watch list
    my @watch_categories;
    my %watches;
    my $sanitized;
    foreach my $watch_category ( @watch_categories ) {
	foreach my $watch ( split(/\t/, $watch_category) ) {
	    ($sanitized = $watch) =~ s/([\.\$\*\+\(\)\]\[\?\{\}])/\\$1/g;

	    if ( (!@was_watched ||
		  grep( !/$sanitized/, @was_watched ) ==  @was_watched
                 ) &&
		!exists $watches{$watch}
               )
            {
	        $rtval = $jacket->SetWatch($watch);
                print DB::OUT $jacket->{'error_text'} if ($rtval);
            }
	    $watches{$watch} = 1;
	}
    }

    foreach my $watch ( @was_watched ) {
	if (!exists $watches{$watch})
        {
            $rtval = $jacket->RemoveWatch( $watch );
            print DB::OUT $jacket->{'error_text'} if ($rtval);
        }
    }

    if (0) {
	print STDERR ">>=========================================================\n";
	print STDERR Data::Dumper->Dump([ \@was_watched ], [ 'was_watched' ]);
	print STDERR Data::Dumper->Dump([ \@watch_categories ], [ 'watch_categories' ]);
	print STDERR Data::Dumper->Dump([ \@{$jacket->{'_watched'}} ], [ '_watched' ]);
	print STDERR Data::Dumper->Dump([ \@{$jacket->{'watched'}} ], [ 'watched' ]);
	print STDERR Data::Dumper->Dump([ \@{$jacket->{'_proximity'}} ], [ '_proximity' ]);
	print STDERR Data::Dumper->Dump([ \@{$jacket->{'proximity'}} ], [ 'proximity' ]);
    }
    my $watched;
    foreach my $watch_list (@watch_categories) {
	my @watch_list = split( /\t/, $watch_list );
	$watched = $jacket->{'watched'};
	my $replaced;
	my $watch_var;
	foreach $watch_var ( @watch_list ) {
	    $replaced = 0;
	    foreach my $item ( @{$watched} ) {
		if ($watch_var eq $item->{'name'}) {
		    $watch_var = $item->{'scalar'};
		    $watch_var =~ s!\n!\\n!g;
		    $watch_var =~ s!\t!\\t!g;
		    $watch_var =~ s![\x00-\x1f]!.!g;
		    if ($watch_var =~ /^\'(.*)\'$/) {
			$watch_var = $1;
		    }
		    $replaced = 1;
		    last;
		}		
	    }
	    $watch_var = '' unless $replaced;
	}
	$watch_list = join("\t", @watch_list);
    }
    return join( "\n", @watch_categories );
}
