#!python
# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.
#
# WSDL PARSER LIBRARY

import string, time, os, md5, types
import urllib, xmllib, shutil
from types import *
from cgi import escape

try:
    import sgmlop
except ImportError:
    sgmlop = None # accelerator not available

__version__ = "0.1"

PREFER_CACHE = 0x01 #if in cache, use it, otherwise fetch
FORCE_CACHE = 0x02 # only get from cache
FORCE_FETCH = 0x04 # only get from url
NO_CACHE = 0x08 # fetch, don't cache it!

class _DiskCacheManager:
    def __init__(self):
        self._dir = os.getenv("TEMP")  
        self._scriptCacheDir = os.getenv("WSDLCACHE")

    def setCacheDir(self, dir):
        self._dir = dir

    def getCacheFileName(self, url):
        assert self._dir
        return os.path.join(self._dir, md5.new(url).hexdigest() + ".wsdl")

    def getScriptCacheFileName(self, url):
        assert self._scriptCacheDir
        return os.path.join(self._scriptCacheDir, md5.new(url).hexdigest() + ".wsdl")
        
    def cacheWSDL(self, url):
        #XXX urlopen can't handle Unicode
        # urlretreive will not do anything if cacheFileName exists!
        # otherwise, it retrieves the file, and copies it to cacheFileName
        cacheFileName = self.getCacheFileName(url)
        #print "retreiving ",url, " copy to ",cacheFileName
        try:
            if os.path.exists(str(url)):
                shutil.copy(str(url), cacheFileName)
            else:
                urllib.urlretrieve(str(url), cacheFileName)
        except IOError, e:
            return None
        # now, copy the cached file to the script cache location
        if self._scriptCacheDir and os.path.exists(self._scriptCacheDir) and os.path.exists(cacheFileName):
            sc = self.getScriptCacheFileName(url)
            if not os.path.exists(sc):
                shutil.copy(cacheFileName, sc)
        return cacheFileName
        return None
    
    def removeCacheFile(self, url):
        cacheFileName = self.getCacheFileName(url)
        if os.path.exists(cacheFileName):
            try: os.remove(cacheFileName)
            except OSError: pass  # they may have screwed w/ the file themselves.
            if _memoryCache.has_key(url):
                del _memoryCache[url]
                
            # remove the script cache copy
            if self._scriptCacheDir and os.path.exists(self._scriptCacheDir):
                sc = self.getScriptCacheFileName(url)
                if os.path.exists(sc):
                    try: os.remove(sc)
                    except OSError: pass  # they may have screwed w/ the file themselves.
            
_diskCacheManager = _DiskCacheManager()
_memoryCache = {}


#---------------------------------------------------------------------
# soap globals
SOAP_XML_SCHEMA_VERSION = "http://www.w3.org/2001/XMLSchema"
SOAP_XML_SCHEMA_1999 = "http://www.w3.org/1999/XMLSchema"
SOAP_SCHEMA = "http://schemas.xmlsoap.org/wsdl/soap/"
SOAP_SCHEMA_ENCODING = "http://schemas.xmlsoap.org/soap/encoding/"
SOAP_ENVELOP = "http://schemas.xmlsoap.org/soap/envelope/"
SOAP_INTEROPORG = "http://soapinterop.org/xsd"


# --------------------------------------------------------------------
# XML parsers

if sgmlop:

    class FastParser:
	# sgmlop based XML parser.  this is typically 15x faster
	# than SlowParser...

	def __init__(self, target):

	    # setup callbacks
	    self.finish_starttag = target.start
	    self.finish_endtag = target.end
	    self.handle_cdata = target.data
            self.handle_data = target.data
            
	    # activate parser
	    self.parser = sgmlop.XMLParser()
	    self.parser.register(self)
	    self.feed = self.parser.feed
	    self.entity = {
		"amp": "&", "gt": ">", "lt": "<",
		"apos": "'", "quot": '"'
		}

	def close(self):
	    try:
		self.parser.close()
	    finally:
		self.parser = None # nuke circular reference

	def handle_entityref(self, entity):
	    # <string> entity
	    try:
		self.handle_data(self.entity[entity])
	    except KeyError:
		self.handle_data("&%s;" % entity)

else:
    FastParser = None

class SlowParser(xmllib.XMLParser):
    # slow but safe standard parser, based on the XML parser in
    # Python's standard library

    def __init__(self, target):
	self.unknown_starttag = target.start
	self.handle_cdata = target.data
	self.unknown_endtag = target.end
	xmllib.XMLParser.__init__(self)

def getparser(target):
    # get the fastest available parser, and attach it to an
    # unmarshalling object.  return both objects.
    if FastParser:
	return FastParser(target)
    return SlowParser(target)

class QName:
    def __init__(self, name):
        if ':' in name:
            s = name.split(':')
            self.name = s[1]
            self.ns = s[0]
        else:
            self.name = name
            self.ns = ''

class Parser:

    def __init__(self, wsdl = None):
	self._type = None
	self._stack = []
        self._marks = []
	self._data = []
	self._methodname = None
	self.append = self._stack.append

        self.position = 0
        self.depth = 0
        self.depth_array = []
        self.status = ''
        self.currentTag = ''
        self.currentElement =''
        self.currentOperation = ''
        self.currentPortType = ''
        self.schema = None
        self.schema_stack = []
        
        # wsdl information
        if not wsdl:
            self.wsdl = WSDL()
        else:
            self.wsdl = wsdl
        
    def _getFile(self, wsdl_file, cache=FORCE_FETCH):
        # read wsdl file, return it's contents
        if cache & FORCE_FETCH:
            _diskCacheManager.removeCacheFile(wsdl_file)

        # see comments in cacheWSDL
        #if not cache & FORCE_CACHE:
        cacheFileName = _diskCacheManager.cacheWSDL(wsdl_file)
        if not cacheFileName:
            return None

        f = open(cacheFileName)
        data = f.read()
        f.close()
        
        if data and not self.wsdl.md5digest:
            self.wsdl.md5digest = md5.new(data).hexdigest()

        #if cache & NO_CACHE:
        #    _diskCacheManager.removeCacheFile(url)
        #else:
        #    _memoryCache[url] = wsdl
        
        return data
        
    def parse(self, wsdl_file, cache=FORCE_CACHE):
        if wsdl_file:
            #print "getting ",wsdl_file
            # XXX implement caching
            wsdl_string  = self._getFile(wsdl_file)
            if not wsdl_string:
                return 0
            
            # parse the file now
            p = getparser(self)
            p.feed(wsdl_string)
            p.close()
        
    #
    # event handlers

    def start(self, name, attrs):
        # get element prefix
        qname = QName(name)
        self.currentTag = qname.name
        
        # find status, register data
        if self.status == 'types':
                
            parent_tag = None
            if len(self.schema_stack) > 0:
                parent_tag = self.schema_stack[len(self.schema_stack)-1]

            if qname.name == 'schema':
                self.schema = attrs['targetNamespace']
                if not self.schema: self.schema = self.wsdl.tns
                self.wsdl.complexTypes[self.schema] = {}
                
            elif qname.name == 'complexType':
                if parent_tag == 'schema':
                    if attrs.has_key('name'):
                        self.currentElement = attrs['name']
                        self.wsdl.complexTypes[self.schema][self.currentElement] = {}
                        if attrs.has_key('base'):
                            qn = QName(attrs['base'])
                            attrs['base'] = qn.name
                            attrs['baseNS'] = qn.ns
                        else:
                            attrs['base'] = 'Struct'
                        self.wsdl.complexTypes[self.schema][self.currentElement].update(attrs)
                    self.schemaStatus = 'complexType'
            
            elif qname.name == 'simpleType':
                if parent_tag == 'schema':
                    if attrs.has_key('name'):
                        self.currentElement = attrs['name']
                        self.wsdl.complexTypes[self.schema][self.currentElement] = {}
                        if attrs.has_key('base'):
                            qn = QName(attrs['base'])
                            attrs['base'] = qn.name
                            attrs['baseNS'] = qn.ns
                        else:
                            attrs['base'] = 'Struct'
                        self.wsdl.complexTypes[self.schema][self.currentElement].update(attrs)
                    self.schemaStatus = 'simpleType'
                
            elif qname.name == 'element':
                if attrs.has_key('type'):
                    qn = QName(attrs['type'])
                    attrs['type'] = qn.name
                    if qn.ns:
                        if self.wsdl.namespaces.has_key(qn.ns):
                            attrs['namespace'] = self.wsdl.namespaces[qn.ns]
                        else:
                            # XXX no namespace ?
                            pass
                if parent_tag == 'schema':
                    self.currentElement = attrs['name']
                    self.wsdl.complexTypes[self.schema][self.currentElement] = attrs
                    self.schemaStatus = 'complexType'
                else:    
                    if not self.wsdl.complexTypes[self.schema][self.currentElement].has_key('elements'):
                        self.wsdl.complexTypes[self.schema][self.currentElement]['elements'] = {}
                    if attrs.has_key('type') and \
                       self.wsdl.complexTypes[self.schema][self.currentElement].has_key('order') and \
                       self.wsdl.complexTypes[self.schema][self.currentElement]['order'] == 'sequence' and \
                       self.wsdl.complexTypes[self.schema][self.currentElement].has_key('base') and \
                       self.wsdl.complexTypes[self.schema][self.currentElement]['base'] == 'Array':
                        self.wsdl.complexTypes[self.schema][self.currentElement]['arrayType'] = attrs['type']
                    if attrs.has_key('name'):
                        self.wsdl.complexTypes[self.schema][self.currentElement]['elements'][attrs['name']] = attrs
                    elif attrs.has_key('ref'):
                        self.wsdl.complexTypes[self.schema][self.currentElement]['ref'] = attrs
                    
            elif qname.name == 'enumeration':
                if not self.wsdl.complexTypes[self.schema][self.currentElement].has_key('elements'):
                    self.wsdl.complexTypes[self.schema][self.currentElement]['enumeration'] = []
                else:
                    self.wsdl.complexTypes[self.schema][self.currentElement]['enumeration'].append(attrs)

            elif qname.name == 'complexContent':
                pass
            
            elif qname.name == 'restriction':
                self.wsdl.complexTypes[self.schema][self.currentElement]['restrictionBase'] = attrs['base']
                if attrs.has_key('base'):
                    qn = QName(attrs['base'])
                    self.wsdl.complexTypes[self.schema][self.currentElement]['base'] = qn.name
                    self.wsdl.complexTypes[self.schema][self.currentElement]['baseNS'] = qn.ns
                else:
                    self.wsdl.complexTypes[self.schema][self.currentElement]['base'] = 'Struct';
                
            elif qname.name == 'sequence':
                self.wsdl.complexTypes[self.schema][self.currentElement]['order'] = 'sequence'
                if not self.wsdl.complexTypes[self.schema][self.currentElement].has_key('type'):
                    self.wsdl.complexTypes[self.schema][self.currentElement]['type'] = 'Array'
                
            elif qname.name == 'all':
                self.wsdl.complexTypes[self.schema][self.currentElement]['order'] = 'all'
                if not self.wsdl.complexTypes[self.schema][self.currentElement].has_key('type'):
                    self.wsdl.complexTypes[self.schema][self.currentElement]['type'] = 'Struct'
                
            elif qname.name == 'attribute':
                self.wsdl.complexTypes[self.schema][self.currentElement]['attrs'] = attrs
                if attrs.has_key('ref'):
                    q = QName(attrs['ref'])
                    for k in attrs.keys():
                        kq = QName(k)
                        if k != 'ref' and q.name == kq.name:
                            vq = QName(attrs[k])
                            self.wsdl.complexTypes[self.schema][self.currentElement][q.name] = vq.name
                    self.wsdl.complexTypes[self.schema][self.currentElement]['attrs'] = attrs
                elif attrs.has_key('name'):
                    self.wsdl.complexTypes[self.schema][self.currentElement]['attrs'][attrs['name']] = attrs
             
            self.schema_stack.append(qname.name)
            
            
        elif self.status == 'message':
            if qname.name == 'part':
                if not self.wsdl.messages[self.currentMessage].has_key(attrs['name']):
                    self.wsdl.messages[self.currentMessage][attrs['name']] = {}
                    self.wsdl.messages[self.currentMessage][attrs['name']]['element'] = ''
                    self.wsdl.messages[self.currentMessage][attrs['name']]['type'] = ''
                    self.wsdl.messages[self.currentMessage][attrs['name']]['namespace'] = ''
                if attrs.has_key('type'):
                    qn = QName(attrs['type'])
                    self.wsdl.messages[self.currentMessage][attrs['name']]['type']=qn.name
                    self.wsdl.messages[self.currentMessage][attrs['name']]['namespace']=qn.ns
                elif attrs.has_key('element'):
                    self.wsdl.messages[self.currentMessage][attrs['name']]['element'] = attrs['element']
                    
                    
        elif self.status == 'portType':
            #switch($name) {
            if qname.name == 'operation':
                self.currentOperation = attrs['name'];
                if not self.wsdl.portTypes[self.currentPortType].has_key(attrs['name']):
                    self.wsdl.portTypes[self.currentPortType][attrs['name']] = {}
                if attrs.has_key('parameterOrder'):
                    self.wsdl.portTypes[self.currentPortType][attrs['name']]['parameterOrder'] = attrs['parameterOrder'];
                    
            elif qname.name == 'input' or qname.name == 'output':
                self.wsdl.portTypes[self.currentPortType][self.currentOperation][name]= attrs;
                qn = QName(attrs['message'])
                self.wsdl.portTypes[self.currentPortType][self.currentOperation][name]['message'] = qn.name
                self.wsdl.portTypes[self.currentPortType][self.currentOperation][name]['namespace'] = qn.ns
                
                
        elif self.status == 'binding':
            #switch($name) {
            if qname.name == 'binding' and qname.ns == self.soapns:
                self.wsdl.bindings[self.currentBinding].update(attrs);
                
            elif qname.name == 'operation' and qname.ns == self.soapns:
                self.wsdl.bindings[self.currentBinding]['operations'][self.currentOperation].update(attrs)
                
            elif qname.name == 'operation':
                if attrs.has_key('name'):
                    self.currentOperation = attrs['name'];
                    if not self.wsdl.bindings[self.currentBinding].has_key('operations'):
                        self.wsdl.bindings[self.currentBinding]['operations'] = {}
                    self.wsdl.bindings[self.currentBinding]['operations'][attrs['name']] = {};
                
            elif qname.name == 'input':
                self.opStatus = 'input';
                
            elif qname.name == 'body' and qname.ns == self.soapns:
                self.wsdl.bindings[self.currentBinding]['operations'][self.currentOperation][self.opStatus] = attrs;
                
            elif qname.name == 'output':
                self.opStatus = 'output';
                
                
        elif self.status ==  'service':
            if qname.name == 'port':
                self.currentPort = attrs['name'];
                if attrs.has_key('binding'):
                    qn = QName(attrs['binding'])
                    attrs['binding'] = qn.name
                    attrs['namespace'] = qn.ns
                self.wsdl.ports[attrs['name']] = attrs;
            elif qname.name == 'address' and qname.ns == self.soapns:
                self.wsdl.ports[self.currentPort]['location'] = attrs['location'];


        # set status
        if qname.name == 'import':
            if not self.wsdl.imports.has_key(attrs['namespace']):
                self.wsdl.imports[attrs['namespace']] = {}
            if attrs.has_key('location'):
                p = Parser(self.wsdl)
                p.parse(attrs['location'])
                self.wsdl.imports[attrs['namespace']]['location'] = attrs['location']
            self.wsdl.imports[attrs['namespace']]['namespace'] = attrs['namespace']
            self.currentImport = attrs['namespace']
            
        elif qname.name == 'types':
            self.status = 'types'
            
        elif qname.name == 'message':
            self.status = 'message'
            self.wsdl.messages[attrs['name']] = {}
            self.currentMessage = attrs['name']
            
        elif qname.name == 'portType':
            self.status = 'portType'
            self.wsdl.portTypes[attrs['name']] = {}
            self.currentPortType = attrs['name']
            
        elif qname.name == 'binding':
            if attrs.has_key('name'):
                self.status = 'binding'
                self.currentBinding = attrs['name']
                if not self.wsdl.bindings.has_key(attrs['name']):
                    self.wsdl.bindings[attrs['name']] = {}
                if attrs.has_key('type'):
                    qn = QName(attrs['type'])
                    self.wsdl.bindings[attrs['name']]['type'] = qn.name
                    self.wsdl.bindings[attrs['name']]['namespace'] = qn.ns
                
        elif qname.name == 'service':
            self.wsdl.services = attrs
            self.wsdl.serviceName = attrs['name']
            self.status = 'service'
            
        elif qname.name == 'definitions':
            self.wsdl.definition = attrs
            items = attrs.items()
            
            for name,value in items:
                qn = QName(name)
                if qn.ns:
                    self.wsdl.namespaces[qn.name] = value
                    self.wsdl.ns[value] = qn.name
                    
                elif name == 'targetNamespace':
                    self.wsdl.tns = value
                    
                if value.lower() == SOAP_SCHEMA:
                    self.soapns = qn.name
                    
            if qname.ns and qname.ns != self.wsdl.tns:
                if not self.wsdl.namespaces.has_key(qname.ns):
                    self.debug("WSDL Parse Error, no namespace for\n", qname.ns)
                    return
                self.wsdl.tns = qname.ns

    def data(self, data):
        if self.currentTag == 'documentation':
            if self.status ==  'service':
                self.wsdl.documentation += data
            if self.status ==  'portType' and self.currentPortType and self.currentOperation:
                if self.wsdl.portTypes[self.currentPortType][self.currentOperation].has_key(self.currentTag):
                    self.wsdl.portTypes[self.currentPortType][self.currentOperation][self.currentTag] += data
                else:
                    self.wsdl.portTypes[self.currentPortType][self.currentOperation][self.currentTag] = data
            if self.status ==  'binding':
                if self.wsdl.bindings[self.currentBinding].has_key(self.currentTag):
                    self.wsdl.bindings[self.currentBinding][self.currentTag] += data
                else:
                    self.wsdl.bindings[self.currentBinding][self.currentTag] = data
            if self.status ==  'message':
                if self.wsdl.messages[self.currentMessage].has_key(self.currentTag):
                    self.wsdl.messages[self.currentMessage][self.currentTag] += data
                else:
                    self.wsdl.messages[self.currentMessage][self.currentTag] = data
            if self.status ==  'operation':
                if self.wsdl.portTypes[self.currentPortType][self.currentOperation].has_key(self.currentTag):
                    self.wsdl.portTypes[self.currentPortType][self.currentOperation][self.currentTag] += data
                else:
                    self.wsdl.portTypes[self.currentPortType][self.currentOperation][self.currentTag] = data
        #elif self.currentTag ==  'message':
        #    if self.wsdl.messages[self.currentMessage].has_key('cdata'):
        #        self.wsdl.messages[self.currentMessage]['cdata'] += data
        #    else:
        #        self.wsdl.messages[self.currentMessage]['cdata'] = data

    def end(self, tag):
        #pos = self.depth_array[self.depth]
        #self.status = ''
        if tag == 'schema':
            self.schema = None
        if self.schema:
            if len(self.schema_stack) > 0:
                self.schema_stack.pop()
            if len(self.schema_stack) <= 1:
                # correct the type for sequences with multiple elements
                if self.currentElement \
                    and self.wsdl.complexTypes[self.schema][self.currentElement].has_key('type') \
                    and self.wsdl.complexTypes[self.schema][self.currentElement]['type'] == 'Array' \
                    and self.wsdl.complexTypes[self.schema][self.currentElement].has_key('elements') \
                    and len(self.wsdl.complexTypes[self.schema][self.currentElement]['elements']) > 1:
                        self.wsdl.complexTypes[self.schema][self.currentElement]['type'] = 'Struct';
                self.currentElement = None

    def handle_data(self, data):
        #print data
        pass
        
    def debug(self, data):
        print data
    
# -----------------------------------------
# wsdl class
# -----------------------------------------
class WSDL:
    def __init__(self):
        self.tns = None
        self.serviceName = ''
        self.documentation = ''
        self.definition = {}
        self.namespaces = {}
        self.complexTypes = {}
        self.messages = {}
        self.portTypes = {}
        self.bindings = {}
        self.ports = {}
        self.services = {}
        self.imports = {}
        self.ns = {}
        self.md5digest = ''
        
    def dump_item(self, dict, indent=0):
        if isinstance(dict, types.DictType):
            items = dict.items()
            for item in items:
                if indent:
                    print " "*indent,
                print item[0]+":"
                self.dump_item(item[1], indent+4)
        else:
            if indent:
                print " "*indent,
            print dict
            
    def dump(self):
        print "NameSpace:",self.tns
        if len(self.definition) > 0:
            print "Definition"
            self.dump_item(self.definition)
        if len(self.namespaces) > 0:
            print "NameSpaces"
            self.dump_item(self.namespaces)
        if len(self.complexTypes) > 0:
            print "ComplexTypes"
            self.dump_item(self.complexTypes)
        if len(self.messages) > 0:
            print "Messages"
            self.dump_item(self.messages)
        if len(self.portTypes) > 0:
            print "PortTypes"
            self.dump_item(self.portTypes)
        if len(self.bindings) > 0:
            print "Bindings"
            self.dump_item(self.bindings)
        if len(self.ports) > 0:
            print "Ports"
            self.dump_item(self.ports)
        if len(self.services) > 0:
            print "Services"
            self.dump_item(self.services)

_wsdls = {}
def ExtractWSDL(wsdlfile, cache=FORCE_CACHE):
    if not cache & FORCE_FETCH and \
       not cache & NO_CACHE and \
       _wsdls.has_key(wsdlfile):
        return _wsdls[wsdlfile]
    parser = Parser()
    parser.parse(wsdlfile,cache)
    _wsdls[wsdlfile] = parser.wsdl
    return parser.wsdl
    
def ReadWSDL(wsdlfile):
    linesep = os.linesep
    wsdl = ExtractWSDL(wsdlfile)
    wsdldigest = ''

    
    # Now print out the WSDL info in a readable form for programmers
    wsdldigest = wsdldigest + "----------------------------------------------------------" + linesep
    wsdldigest = wsdldigest + "Component" + linesep
    wsdldigest = wsdldigest + "\tName: " + wsdl.serviceName + linesep
    wsdldigest = wsdldigest + "\tDescription: " + wsdl.documentation + linesep
    wsdldigest = wsdldigest + linesep
    for port in wsdl.ports.keys():
        wsdldigest = wsdldigest + "URI" + linesep
        if wsdl.ports[port].has_key('location'):
            wsdldigest += "\URI: " + wsdl.ports[port]['location'] + linesep
        binding_name = wsdl.ports[port]['binding'] 
        wsdldigest = wsdldigest + linesep
        wsdldigest = wsdldigest + "Methods" + linesep
        wsdldigest = wsdldigest + "\tBinding -" + linesep
        for y in wsdl.bindings[binding_name]['operations'].keys():
            wsdldigest = wsdldigest + "\t\tname: " + y + "()" + linesep
            if wsdl.bindings[binding_name]['operations'][y].has_key('soapaction'):
                wsdldigest = wsdldigest + "\t\tsoapaction: " + wsdl.bindings[binding_name]['operations'][y]['soapaction'] + linesep
            if wsdl.bindings[binding_name]['operations'][y].has_key('documentation'):
                wsdldigest = wsdldigest + "\t\tdocumentation: " + wsdl.bindings[binding_name]['operations'][y]['documentation'] + linesep
            wsdldigest = wsdldigest + linesep
        binding_type = wsdl.bindings[binding_name]['type']
        wsdldigest = wsdldigest + "\tPrototype -" + linesep
        for y in wsdl.portTypes[binding_type].keys():
            wsdldigest = wsdldigest + "\t\t" + y + "()" + linesep
            wsdldigest = wsdldigest + "\t\tin" + linesep
            input_message = wsdl.portTypes[binding_type][y]['input']['message']
            if wsdl.messages[input_message].has_key('documentation'):
                wsdldigest += "\t\t\t" + wsdl.messages[input_message]['documentation'] + linesep
            for z in wsdl.messages[input_message].keys():
                wsdldigest = wsdldigest + "\t\t\t" + wsdl.messages[input_message][z]['type'] + "\t\t" + z + linesep
            wsdldigest = wsdldigest + "\t\tout" + linesep
            output_message = wsdl.portTypes[binding_type][y]['output']['message']
            if wsdl.messages[output_message].has_key('documentation'):
                wsdldigest += "\t\t\t" + wsdl.messages[output_message]['documentation'] + linesep
            for z in wsdl.messages[output_message].keys():
                wsdldigest = wsdldigest + "\t\t\t" + wsdl.messages[output_message][z]['type'] + "\t\t" + z + linesep
            if wsdl.bindings[binding_name]['operations'][y].has_key('soapaction'):
                wsdldigest = wsdldigest + "\t\tsoapaction" + linesep
                wsdldigest = wsdldigest + "\t\t\t" + wsdl.bindings[binding_name]['operations'][y]['soapaction'] + linesep
            wsdldigest = wsdldigest + linesep
    return wsdldigest

def WSDL_to_HTML(wsdlfile, user_description='', cache=FORCE_FETCH):
    linesep = os.linesep
    wsdl = ExtractWSDL(wsdlfile, cache)
    wsdldigest = ''
    # get the header
    
    
    # Now print out the WSDL info in a readable form for programmers
    wsdldigest += "<h1>Component Name: " + wsdl.serviceName + "</h1>" + linesep
    wsdldigest += "<p>WSDL: " + wsdlfile + "</p>" + linesep
    if user_description:
        wsdldigest += "<p>" + user_description + "</p>" + linesep
    else:
        wsdldigest += "<p>" + wsdl.documentation + "</p>" + linesep
    
    if len(wsdl.complexTypes) > 0:
        wsdldigest += "<h3>Types</h3>" + linesep
        for schema in wsdl.complexTypes.keys():
            for type in wsdl.complexTypes[schema].keys():
                if wsdl.complexTypes[schema][type].has_key('elements'):
                    wsdldigest +=  "<dt>struct %s%s" % (type,linesep)
                    for element in wsdl.complexTypes[schema][type]['elements'].values():
                        name = type = ''
                        if element.has_key('name'): name = element['name']
                        if element.has_key('type'): type = element['type']
                        wsdldigest +=  "<dd>%s %s</dd>%s" % (name, type,linesep)
                    wsdldigest += "</dt><br><br>" +linesep
                elif wsdl.complexTypes[schema][type].has_key('attrs'):
                    ref = QName(wsdl.complexTypes[schema][type]['attrs']['ref'])
                    keys = wsdl.complexTypes[schema][type]['attrs'].keys()
                    for k in keys:
                        if k.find(ref.name):
                            atype = QName(wsdl.complexTypes[schema][type]['attrs'][k])
                            break
                    wsdldigest +=  "<dt>%s %s %s</dt><br>%s" % (ref.name, type, atype.name,linesep)
                
        
    for port in wsdl.ports.keys():
        wsdldigest += "<h2>Port Name: " + port + "</h2>" + linesep
        if wsdl.ports[port].has_key('location'):
            wsdldigest += "<p>Endpoint: " + wsdl.ports[port]['location'] + "</p>" + linesep

        binding_name = wsdl.ports[port]['binding'] 
        wsdldigest += "Binding: " + binding_name + "<br>" + linesep

        wsdldigest += "<h3>Operations</h3>" + linesep

        for y in wsdl.bindings[binding_name]['operations'].keys():
            if y == 'documentation': continue
            #wsdldigest += "<dl><dt><b><tt class='function'>" + y + "</tt></b><br></dt>" + linesep

            binding_type = wsdl.bindings[binding_name]['type']
            #for y in wsdl.porttype[binding_type].operation.keys():
            wsdldigest += "<dl><dt><strong><code>" + y + "("

            input_message = output_message = None
            if wsdl.portTypes[binding_type][y].has_key('input'):
                input_message = wsdl.portTypes[binding_type][y]['input']['message']
            if wsdl.portTypes[binding_type][y].has_key('output'):
                output_message = wsdl.portTypes[binding_type][y]['output']['message']
            #self.wsdl.messages[self.currentMessage][attrs['name']]['element'] = attrs['element']
            args = ""
            if input_message:
                for z in wsdl.messages[input_message].keys():
                    if z != 'documentation':
                        if args:
                            args += ", IN " + wsdl.messages[input_message][z]['type'] + " " + z
                        else:
                            args = "IN " + wsdl.messages[input_message][z]['type'] + " " + z
            if output_message:
                for z in wsdl.messages[output_message].keys():
                    if z != 'documentation':
                        if args:
                            args += ", OUT " + wsdl.messages[output_message][z]['type'] + " " + z
                        else:
                            args = "OUT " + wsdl.messages[output_message][z]['type'] + " " + z
            wsdldigest += args + ")</code></strong><br><br>" + linesep
            if wsdl.bindings[binding_name]['operations'][y].has_key('soapaction'):
                wsdldigest += "<dd><strong>SOAPAction:</strong> " + wsdl.bindings[binding_name]['operations'][y]['soapaction'] + "</dd>" + linesep

            if wsdl.bindings[binding_name]['operations'][y].has_key('documentation'):
                wsdldigest += "<dd><p>" + wsdl.bindings[binding_name]['operations'][y]['documentation'] + "</p></dd>" + linesep
            elif wsdl.portTypes[binding_type][y].has_key('documentation'):
                wsdldigest += "<dd><p>" + wsdl.portTypes[binding_type][y]['documentation'] + "</p></dd>" + linesep
            
            if input_message and wsdl.messages[input_message].has_key('documentation'):
                wsdldigest += "<dd><p>" + wsdl.messages[input_message]['documentation'] + "</p></dd>" + linesep
            if output_message and wsdl.messages[output_message].has_key('documentation'):
                wsdldigest += "<dd><p>" + wsdl.messages[output_message]['documentation'] + "</p></dd>" + linesep
            wsdldigest += "</dl>" + linesep
            
    # get the footer
            
    return wsdldigest


def Get_Completion_Info(wsdlfile):
    # parse the wsdl
    wsdl = ExtractWSDL(wsdlfile)
    if not wsdl:
        print "unable to get wsdl ", wsdlfile
        return

    info = None    
    # we use the first port that has a soap address
    for port in wsdl.ports.keys():
        #print "looking at port ",port
        if wsdl.ports[port].has_key('location'):
            #print "creating structure for ", wsdl.service_name
            info = {}
            info["service_name"]=wsdl.serviceName
            info["methods"] = {}
            info["port_soapaddress"] = wsdl.ports[port]['location']
            info["port"]=port
            binding_name = wsdl.ports[port]['binding']
            
            # build the function list
            binding_type = wsdl.bindings[binding_name]['type']
            for func in wsdl.portTypes[binding_type].keys():
                input_message = wsdl.portTypes[binding_type][func]['input']['message']
                output_message = wsdl.portTypes[binding_type][func]['output']['message']
                
                # get the input arguments
                # we build the argument list to have "TYPE ARGNAME"
                
                arg_in_out = {}
                args = []
                for z in wsdl.messages[input_message].keys():
                    args.append(wsdl.messages[input_message][z]['type'] + " " + z)
 
                arg_in_out["in"] = args
                
                args = []
                for z in wsdl.messages[output_message].keys():
                    args.append(wsdl.messages[output_message][z]['type'] + " " + z)
                
                arg_in_out["out"] = args

                info["methods"][func]=arg_in_out

            break
    return info

    
def Get_All_Methods(wsdlfile):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']
    return wsdl.portTypes[binding_type].keys()
 
def Get_Url(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    Url = wsdl.ports[port]['location']
    return Url

def Get_SoapAction(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    return wsdl.bindings[binding_name]['operations'][methodname]['soapAction']

def Get_In_Namespace(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    return wsdl.namespaces[wsdl.bindings[binding_name]['namespace']]

def Get_In_Args(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']    
    input_message = wsdl.portTypes[binding_type][methodname]['input']['message']
    args = []
    return wsdl.messages[input_message].keys()

def Get_Out_Args(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']    
    output_message = wsdl.portTypes[binding_type][methodname]['output']['message']
    return wsdl.messages[output_message].keys()

def Get_In_Args_With_Type(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']    
    input_message = wsdl.portTypes[binding_type][methodname]['input']['message']
    args = []
    for z in wsdl.messages[input_message].keys():
          args.append((z, wsdl.messages[input_message][z]['type']))
    return args

def Get_Out_Args_With_Type(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']    
    output_message = wsdl.portTypes[binding_type][methodname]['output']['message']
    args = []
    for z in wsdl.messages[output_message].keys():
          args.append((z, wsdl.messages[output_message][z]['type']))
    return args

def Get_All_Args_With_Type(wsdlfile, methodname):
    wsdl = ExtractWSDL(wsdlfile)
    # XXX hack
    port = wsdl.ports.keys()[0]
    binding_name = wsdl.ports[port]['binding'] 
    binding_type = wsdl.bindings[binding_name]['type']
    output_message = wsdl.portTypes[binding_type][methodname]['output']['message']
    input_message = wsdl.portTypes[binding_type][methodname]['input']['message']
    args = []
    for z in wsdl.messages[output_message].keys():
          args.append((z, wsdl.messages[output_message][z]['type']))
    for z in wsdl.messages[input_message].keys():
          args.append((z, wsdl.messages[input_message][z]['type']))
    return args

def Get_Core_Info(wsdlfile, methodname):
    info = {}
    info['Url'] = Get_Url(wsdlfile, methodname)
    info['SoapAction'] = Get_SoapAction(wsdlfile, methodname)
    info['In_Namespace'] = Get_In_Namespace(wsdlfile, methodname)
    info['In_Args'] = Get_In_Args(wsdlfile, methodname)
    return info

def Get_MD5_Hash(wsdlfile):
    wsdl = ExtractWSDL(wsdlfile)
    if wsdl.md5digest is None:
        wsdl.md5digest = ''
        filename = _diskCacheManager.getCacheFileName(wsdlfile)
        file = open(filename, "rb")
        text = file.read()
        file.close()
        if text:
            wsdl.md5digest = md5.new(text).hexdigest()
    return wsdl.md5digest
    
def SetCacheDir(dir):
    _diskCacheManager.setCacheDir(dir)

def CacheWSDL(url):
    _diskCacheManager.cacheWSDL(url)

def CacheWSDLFromLocalFile(fileName, url):
    import shutil
    cacheFileName = _diskCacheManager.getCacheFileName(url)
    shutil.copyfile(fileName, cacheFileName)

def removeCacheWSDL(url):
    _diskCacheManager.removeCacheFile(url)


if __name__ == '__main__':
    # a wsdl with no import
    test = []
    #test.append('http://soap.4s4c.com/weblogs/subscribe.wsdl');
    #test.append('http://www.esynaps.com/WebServices/eSynapsSearch.asmx?WSDL');
    #test.append('http://www.richsolutions.com/richpayments/richpay.asmx?WSDL');
    #test.append('http://www.xml-webservices.net/services/entretainment/joke_server.asmx?WSDL');
    #test.append('http://www.aspxpressway.com/maincontent/webservices/piglatin.asmx?wsdl');
    #test.append('http://www.apache.org/~rubys/ApacheSoap.wsdl');
    #test.append('http://www.pocketsoap.com/services/ilab.wsdl')
    #test.append('http://mssoapinterop.org/stkV3/InteropB.wsdl')
    #test.append('http://www.xmethods.net/sd/BabelFishService.wsdl')
    #test.append('http://www.lemurlabs.com/projects/soap/itime/ITimeService.wsdl')
    #test.append('http://www.xmethods.net/sd/TemperatureService.wsdl')
    #test.append('http://soaptest.activestate.com:8080/PSDBService.wsdl')
    #test.append(r'E:\src\pear\SOAP\test\schematest_3.wsdl')

    #test.append('http://upload.eraserver.net/REFLECTIONIT/Html2Xml.asmx?WSDL')
    
    xmethods = 0
    if xmethods:
        # we at least parse all these, even if not entirely correctly yet
        ##xmethods list of url's
        test.append('http://www.codemechanisms.co.uk/WebServices/UNSPSC.asmx?WSDL')
        test.append('http://www.framewerks.com/WebServices/fWArticleService/fwArticles.asmx?WSDL')
        test.append('http://www.epionet.com/webservices/epiowebservices/livecitycams.wsdl')
        test.append('http://appserver.pepperzak.net/bankcode/BankCodeEJBHome/wsdl.jsp')
        test.append('http://64.78.60.122/GeoCash.asmx?WSDL')
        test.append('http://www.edelphi.com/scripts/KzAnyWho.exe/wsdl/IAnyWho')
        test.append('http://64.78.60.122/GeoPinPoint.asmx?WSDL')
        test.append('http://www.kirbyland.net/ws/wcav.asmx?wsdl')
        test.append('http://ww6.borland.com/webservices/BorlandBabel/BorlandBabel.exe/wsdl/IBorlandBabel')
        test.append('http://www.tankebolaget.se/scripts/NumToWords.dll/wsdl/INumToWords')
        test.append('http://ws.cdyne.com/emailverify/ev.asmx?wsdl')
        test.append('http://demo.eltegra.com/webservices/wsdl?app=CarPayment')
        test.append('http://www.verbix.com/cgi-bin/soapx.exe/wsdl/IGetInflection')
        test.append('http://www.tankebolaget.se/scripts/TBWS.exe/wsdl/IModulusCheck')
        test.append('http://www.tankebolaget.se/scripts/Haddock.exe/wsdl/IHaddock')
        test.append('http://www.orbitarium.com/schemas/ows.wsdl')
        test.append('http://www.esynaps.com/WebServices/MsProxy.asmx?WSDL')
        test.append('http://ws.cdyne.com/delayedstockquote/delayedstockquote.asmx?wsdl')
        test.append('http://ws.cdyne.com/psaddress/addresslookup.asmx?wsdl')
        # unable to retrieve
        #test.append('http://hosting001.vs.k2unisys.net/Weather/PDCWebService/WeatherServices.asmx?WSDL')
        test.append('http://soap.4s4c.com/weblogs/subscribe.wsdl')
        test.append('http://www.esynaps.com/WebServices/eSynapsSearch.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/financial/calculations.asmx?WSDL')
        test.append('http://www.richsolutions.com/richpayments/richpay.asmx?WSDL')
        test.append('http://upload.eraserver.net/REFLECTIONIT/Html2Xml.asmx?WSDL')
        test.append('http://www.alethea.net/webservices/LocalTime.asmx?WSDL')
        test.append('http://soap.systinet.net/ws/FreeDbService/')
        # unable to retrieve
        #test.append('http://webservices.juice.com:4646/temperature.wsdl')
        #test.append('http://webservices.juice.com:4646/snapshot.wsdl')
        #test.append('http://webservices.juice.com:4646/earnings.wsdl')
        test.append('http://www.xmethods.net/wsdl/query.wsdl')
        test.append('http://www.durrios.com/Finance.wsdl')
        test.append('http://www.danmarinescu.com/WebServices/ChessCGIServer.exe/wsdl/IBorlandChess')
        test.append('http://www.xml-webservices.net/services/entretainment/joke_server.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/time_server/clock.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/time_server/clock.asmx?WSDL')
        test.append('http://194.78.198.202/WebServices/BelgiumCities.asmx?WSDL')
        test.append('http://services.develop.co.za/GetLocalTime.asmx?WSDL')
        test.append('http://202.56.162.132/NearestTileDealer/NearestTileDealer.asmx?wsdl')
        test.append('http://202.56.162.132/HealthcareProviderSearch/HealthcareProviderSearch.asmx?wsdl')
        test.append('http://www.esynaps.com/WebServices/BusinessList.asmx?WSDL')
        test.append('http://www.aspxpressway.com/maincontent/webservices/piglatin.asmx?wsdl')
        test.append('http://www.esynaps.com/WebServices/eSynapsFeed.asmx?WSDL')
        test.append('http://www.mybubble.com:8080/mybubbleEntServer/MyBubbleSoapServices.wsdl')
        test.append('http://www.mybubble.com:8080/mybubbleEntServer/MyBubbleSoapServices.wsdl')
        test.append('http://www.mybubble.com:8080/mybubbleEntServer/MyBubbleSoapServices.wsdl')
        test.append('http://66.135.138.156:8080/glue/urn:LoanTermWorksheet.wsdl')
        test.append('http://66.135.138.156:8080/glue/urn:ExtraPaymentWorksheet.wsdl')
        test.append('http://66.135.138.156:8080/glue/urn:MonthlyPaymentWorksheet.wsdl')
        test.append('http://www.geographynetwork.com/geoservices/v1/PlaceFinder.wsdl')
        test.append('http://www.danmarinescu.com/WebServices/ChessCGIServer.exe/wsdl/IBorlandChess')
        test.append('http://www.esynaps.com/WebServices/NFLNews.asmx?WSDL')
        test.append('http://www.esynaps.com/WebServices/WhoIsService.asmx?WSDL')
        test.append('http://www.esynaps.com/WebServices/DailyDiblert.asmx?WSDL')
        test.append('http://demo.eltegra.com/webservices/wsdl?app=MortgageCalculator')
        test.append('http://www.alethea.net/webservices/ZipCode.asmx?WSDL')
        test.append('http://developerdays.com/cgi-bin/tempconverter.exe/wsdl/ITempConverter')
        test.append('http://inovagis.dcea.fct.unl.pt/services/GIServerWS.exe/wsdl/IGIServer')
        test.append('http://www.techmetrix.net:8080/wsdl/techservice.wsdl')
        test.append('http://www.mybubble.com:8080/mybubbleEntServer/MyBubbleSoapServices.wsdl')
        test.append('http://www.durrios.com/Bills.wsdl')
        test.append('http://www.marotz.se/scripts/zipinfo.exe/wsdl/ISwedishZipInfo')
        test.append('http://www.mybubble.com:8080/mybubbleEntServer/MyBubbleSoapServices.wsdl')
        test.append('http://www.xml-webservices.net/services/maths/calculator.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/conversions/unit_convert.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/directory/profile/user_account.asmx?WSDL')
        test.append('http://www.xml-webservices.net/services/messaging/smtp_mail/mailsender.asmx?WSDL')
        test.append('http://www.webappcabaret.com/sk/pub/sinosoap.wsdl')
        test.append('http://www.richsolutions.com/RichPayments/RichCardValidator.asmx?WSDL')
        test.append('http://www.west-wind.com/wconnect/soap/wwhelpservice.wsdl')
        test.append('http://www.xml-webservices.net/services/conversions/euro_convert/euro_conver.asmx?WSDL')
        test.append('http://transform.dataconcert.com/transform.wsdl')
        test.append('http://213.23.125.181:8080/christmas.wsdl')
        test.append('http://www.OneOutBox.com/wsdl/FreeFaxService.wsdl')
        test.append('http://www.drbob42.co.uk/cgi-bin/Euro42/wsdl/IEuro')
        test.append('http://HOSTING.MSUGS.CH/cheeso9/zips/ZipService.asmx?WSDL')
        test.append('http://hosting.msugs.ch/aravindc/BookPriceComparison.asmx?wsdl')
        test.append('http://www.foxcentral.net/foxcentral.wsdl')
        test.append('http://upload.eraserver.net/MOUNALISA/adictionary/arabicdictionary.asmx?WSDL')
        test.append('http://aspnet.lamaan.com/webservices/QuranVerse.asmx?WSDL')
        test.append('http://webservices.matlus.com/scripts/sessionservice.dll/wsdl/IMsSessionBrokerService')
        test.append('http://www.codebump.com/services/zipcodelookup.asmx?wsdl')
        test.append('http://www.soapengine.com/lucin/soapenginex/smsx.asmx?wsdl')
        test.append('http://www.cs.fsu.edu/~engelen/lu.wsdl')
        test.append('http://ws.xara.com/navbar/navbar.wsdl')
        test.append('http://www.abecas.com/webservices/wordservice.dll/wsdl/IWordHandler')
        test.append('http://www.jusufdarmawan.com/wsprimegenerator.exe/wsdl/IPrimeGenerator')
        test.append('http://www.forthlink.gr/TakisLib/TakisActiveLib.WSDL')
        test.append('http://213.23.125.181:8080/chemistry.wsdl')
        test.append('http://www.vbws.com/services/weatherretriever.asmx?WSDL')
        test.append('http://www.PerfectXML.net/WebServices/MusicTeachers/MusicTeachers.asmx?wsdl')
        test.append('http://upload.eraserver.net/circle24/autoloan.asmx?wsdl')
        test.append('http://www.codecube.net/services/xreonline.asmx?WSDL')
        test.append('http://213.23.125.181:8080/statistics.wsdl')
        test.append('http://www.ebi.ac.uk/xembl/XEMBL.wsdl')
        test.append('http://www.gxchart.com/webchart.wsdl')
        test.append('http://erwin.dstc.edu.au/GUIDDictionaryService/GUIDDictionary.asmx?wsdl')
        test.append('http://xmlrad.com/WSFindMP3Bin/WSFindMP3.dll/WSDL')
        test.append('http://www.dotnetmap.com/webservices/mapservice.asmx?wsdl')
        test.append('http://www.cs.fsu.edu/~engelen/magic.wsdl')
        test.append('http://www.PerfectXML.NET/WebServices/SalesRankNPrice/BookService.asmx?WSDL')
        test.append('http://erwin.dstc.edu.au/UDDIProxyService/UDDIProxy.asmx?wsdl')
        test.append('http://webservices.eraserver.net/zipcoderesolver/zipcoderesolver.asmx?WSDL')
        test.append('http://beta2.eraserver.net/webservices/mxchecker/mxchecker.asmx?WSDL')
        test.append('http://www.nims.nl/soap/oms.wsdl')
        test.append('http://ws.xara.com/graphicrender/render3d.wsdl')
        test.append('http://www.allesta.net:51110/webservices/wsdl/YahooUserPingService.xml')
        test.append('http://www.ebob42.com/cgi-bin/DrBobsClinic.exe/wsdl/IHeadline')
        test.append('http://www.ebob42.com/cgi-bin/NumberToWordsInDutch.exe/wsdl/IDutch')
        test.append('http://www.ebob42.com/cgi-bin/Romulan.exe/wsdl/IRoman')
        test.append('http://www.eBob42.com/cgi-bin/TicTacToe.exe/wsdl/ITicTacToe')
        test.append('http://www.xmlme.com/WSElectronics.asmx?WSDL')
        test.append('http://www.xmlme.com/WSVideoGames.asmx?WSDL')
        test.append('http://www.xmlme.com/WSSportingGoods.asmx?WSDL')
        test.append('http://www.xmlme.com/WSDailyXml.asmx?WSDL')
        test.append('http://www.xmlme.com/WSDailyNet.asmx?WSDL')
        test.append('http://www.xmlme.com/WSCustNews.asmx?WSDL')
        test.append(' http://www.capescience.com/webservices/airportweather/AirportWeather.wsdl')
        test.append('http://services.pagedownweb.com/ZipCodes.asmx?WSDL')
        test.append('http://soaptest.activestate.com:8080/PSDBService.wsdl')
        test.append('http://www.shinkatech.com/interop/NSLookup/NSLookup.wsdl')
        test.append('http://www.compkarori.com/wsdl/discordian.wsdl')
        test.append('http://www.capescience.com/webservices/polycalc/PolyCalcApp.wsdl')
        test.append('http://www.itera.com.br/soap/services/jabber/jabber.wsdl')
        test.append('http://webservices.matlus.com/scripts/WordFind.DLL/wsdl/IWordFind')
        test.append('http://webservices.matlus.com/scripts/whoiswebservice.dll/wsdl/IWhoIs')
        test.append('http://webservices.matlus.com/scripts/emailwebservice.dll/wsdl/IEmailService')
        test.append('http://www.shinkatech.com/interop/CurrencyConverter.wsdl')
        test.append('http://webservices.matlus.com/scripts/tutorialswebservice.dll/wsdl/IISAPITutorials')
        test.append('http://www.agnisoft.com/soap/mssoapmp3search.xml')
        test.append('http://www.kazoosoft.net/scripts/IKazooClasses.wsdl')
        # not retreivable
        #test.append('http://www.xmlbus.com:9010/xmlbus/container/Converter/ConverterService/ConverterPort')
        #test.append('http://www.xmlbus.com:9010/xmlbus/container/Electricity/ElectricityService/ElectricityPort')
        test.append('http://soap.systinet.net/ws/RpmFindService/')
        test.append('http://www.alanbushtrust.org.uk/soap/compositions.wsdl')
        test.append('http://www.pocketsoap.com/interop/manila.wsdl')
        test.append('http://www.SoapClient.com/xml/SQLDataSoap.WSDL')
        test.append('http://www.SoapClient.com/xml/SQLDataSoap.wsdl')
        test.append('http://www.SoapClient.com/xml/SQLDataSoap.WSDL')
        test.append('http://www.SoapClient.com/xml/SQLDataSoap.WSDL')
        test.append('http://www.xmethods.net/sd/2001/FedExTrackerService.wsdl')
        test.append('http://www.xmethods.net/sd/2001/BabelFishService.wsdl')
        test.append('http://www.xmethods.net/sd/2001/XMethodsFilesystemService.wsdl')
        test.append('http://www.xmethods.net/sd/2001/TemperatureService.wsdl')
        test.append('http://www.xmethods.net/sd/2001/BNQuoteService.wsdl')
        test.append('http://services.xmethods.net/soap/urn:xmethods-DomainChecker.wsdl')
        test.append('http://www.xmethods.net/sd/2001/CurrencyExchangeService.wsdl')
        test.append('http://www.xmethods.net/sd/2001/CATrafficService.wsdl')
        test.append('http://services.xmethods.net/soap/urn:xmethods-delayed-quotes.wsdl')
        test.append('http://www.xmethods.net/sd/2001/EBayWatcherService.wsdl')

    for url in test:
        print "<h1>%s</h1>" % url
        print WSDL_to_HTML(url)

    #url = "http://www.pocketsoap.com/services/ilab.wsdl"
    # a wsdl that imports
    #url = "http://www.apache.org/~rubys/ApacheSoap.wsdl"
    #url = r'E:\src\pear\SOAP\test\schematest.wsdl'
    #parser = Parser()
    #parser.parse(url)
    #wsdl = parser.wsdl
    #wsdl.dump()
    
    #print ReadWSDL(url)
    #print WSDL_to_HTML(url)
    #print repr(Get_All_Methods(url))
    #print Get_Url(url,'echoString')
    #print Get_SoapAction(url,'echoString')
    #print Get_In_Namespace(url,'echoString')
    #print Get_In_Args(url,'echoString')
    #print Get_Out_Args(url,'echoString')
    #print Get_In_Args_With_Type(url,'echoString')
    #print Get_Out_Args_With_Type(url,'echoString')
    #print Get_All_Args_With_Type(url,'echoString')
    #print Get_Core_Info(url,'echoString')
    #print Get_Completion_Info(url)
    