#!python
# Copyright (c) 2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

import os
from xpcom import components
from koLintResult import *
from koLintResults import koLintResults
from koLogger import logWarning, logDebug, logError
import eol
import codecs

def findRowsAndColumnsFromSortedOffsets(lines, offsets):
    currentLine = 0
    currentOffset = 0
    rowsAndColumns = []
    for i in offsets:
        while i > len(lines[currentLine]) + currentOffset:
            currentOffset = currentOffset + len(lines[currentLine]) + 1
            currentLine = currentLine + 1
        rowsAndColumns.append((currentLine+1, i - currentOffset + 1))     
    return rowsAndColumns

def generateUnicodeLintResults(unicodeBuffer, encoding):
    lintResults = koLintResults()
    uniformLineBuffer = eol.convertToEOLFormat(unicodeBuffer, eol.EOL_LF)

    # Encode character-by-character (a better average case
    # encoding time can be achieved by doing a binary search
    # for misencoded characters esp if we are willing to live
    # with a single lint result)
    errorStart = -1
    errors = []
    for i in xrange(0,len(uniformLineBuffer)):
        try:
            uniformLineBuffer[i].encode(encoding.python_encoding_name)
            if errorStart != -1:
                errors.append(errorStart)
                errors.append(i)
                errorStart = -1
        except UnicodeError:
            if errorStart == -1:
                errorStart = i

    if errorStart != -1:
            errors.append(errorStart)
            errors.append(len(uniformLineBuffer))
                
    if len(errors) > 0:
        unicodeLines = uniformLineBuffer.split('\n')

        errors = findRowsAndColumnsFromSortedOffsets(unicodeLines, errors)
        niceErrors = []

        # There was to be a better way to do this but my
        # primitive brain can't see it
        i = 0
        while i < len(errors) - 1:
            niceErrors.append((errors[i], errors[i+1]))
            i = i + 2

        for error in niceErrors:
            lintResult = KoLintResult()

            lintResult.severity = lintResult.SEV_ERROR
            lintResult.description = '''The highlighted characters cannot be represented using the current encoding ("%s"). '''\
                                     '''The file's encoding can be changed from the "File Properties and Settings" dialog.''' % encoding.short_encoding_name
            ((lintResult.lineStart, lintResult.columnStart), \
                    (lintResult.lineEnd, lintResult.columnEnd)) = error

            lintResults.addResult(lintResult)

        return lintResults
        
class KoEncodingLinter:
    def __init__(self, languageLinter = None):
        self.languageLinter = languageLinter
    
    def lint(self, buffer, encoding, cwd):
        # Note that the sematics of encoding are different for
        # this linter. The encoding is the encoding
        # that the buffer should be not the encoding that it
        # actually is (which is UTF-8)

        if encoding == 'utf-8':
            # XXX - shouldn't we combine the results from the
            # encoding and the real linter?
            # See bug 18631
            unicodeBuffer = codecs.utf_8_decode(buffer)[0]
            
            # Do a quick test before we go character-by-character
            try: 
                unicodeBuffer.encode(encoding.python_encoding_name)
            except UnicodeError:
                return generateUnicodeLintResults(unicodeBuffer, encoding)
    
            if encoding.byte_order_marker:
                unicodeBuffer = unichr(0xfeff) + unicodeBuffer
            
            data = unicodeBuffer.encode(encoding.python_encoding_name)
        else:
            data = buffer
            
        if self.languageLinter:
            return self.languageLinter.lint(data, encoding, cwd)
        else:
            return koLintResults()
