"""Base classes for languages and language services"""

import string
from xpcom import components
from koLogger import logError, logWarning, logDebug
import cgi, time, re, types, os, convertURL, eol

numWordLists=6 # from ScintillaBase.h

class KoDebuggerLanguageService:
    _com_interfaces_ = [components.interfaces.koIDebuggerLanguageService]

    _language = None
    _engine = None

    def __init__(self, language):
        self._language = language

        
    def get_engine(self):
        if self._engine is None:
            debugger = components.classes["@activestate.com/debugger;1"] \
                       .getService(components.interfaces.koIDebugger)
            self._engine = debugger.addEngine(self._language)
            
            handler = components \
                  .classes["@mozilla.org/network/protocol;1?name=kodebugger"] \
                  .getService(components.interfaces.koIDebuggerProtocolHandler)
            handler.registerEngine(self._engine, self._language)
        return self._engine

class KoCompletionLanguageService:
    _com_interfaces_ = [components.interfaces.koICompletionLanguageService]
    triggers = None
    completionSeparator = ord('\n')
    triggersCallTip = None
    
    def __init__(self):
        pass
    
    def scanFileCompletion(self, filename):
        pass
    def scanBufferCompletion(self, scimoz, filename):
        pass

class KoLexerLanguageService:
    _com_interfaces_ = [components.interfaces.koILexerLanguageService]

    def __init__(self):
        self._lexer = None
        self._keywords = []
        for i in range(numWordLists):
            self._keywords.append([])
        self._properties = {}
        self.supportsFolding = 0

    def setLexer(self, sciLexer):
        self._lexer = sciLexer

    def setKeywords(self, keywordSet, keywords):
        assert keywordSet >= 0 and keywordSet < numWordLists
        self._keywords[keywordSet] = keywords

    def setProperty(self, property, value):
        self._properties[property] = value

    def setCurrent(self, scimoz):
        if self._lexer is None:
            scimoz.lexer = scimoz.SCLEX_NULL
        else:
            scimoz.lexer = self._lexer

        for i in range(len(self._keywords)):
            scimoz.setKeyWords(i, " ".join(self._keywords[i]))

        for prop in self._properties.keys():
            scimoz.setProperty(prop, self._properties[prop])

class KoLinterLanguageService:
    """Just a way to get the contract IDs used by koLintManager"""
    _com_interfaces_ = [components.interfaces.koILinterLanguageService]

    def __init__(self, syntaxChecker, lintChecker):
        # this argument-laden constructor should go away --
        # not portable to other languages.
        self._syntaxChecker = syntaxChecker
        self._lintChecker = lintChecker

    def get_syntaxChecker(self):
        return self._syntaxChecker

    def get_lintChecker(self):
        return self._lintChecker

class KoPrettyPrinterLanguageService:
    _com_interfaces_ = [components.interfaces.koIPrettyPrinterLanguageService]

    def __init__(self):
        self._htmlStyleTags = {}
        self._styleManager = components.\
                             classes["@activestate.com/koStyleManager;1"].\
                             getService()
        
    def getStyleTags(self, language, styleNumber, defaultStyle, useColor):
        """ Returns a tuple containing the open and close HTML tags"""
        if self._htmlStyleTags.has_key(styleNumber):
            return self._htmlStyleTags[styleNumber]

        style = self._styleManager.getStyleByNumber("editor", language, styleNumber, 1)
        css = self._styleToCSS(style, defaultStyle, useColor)

        if css:
            opener = '<span style="%s">' % css
            closer = '</span>'
        else:
            opener = ''
            closer = ''

        self._htmlStyleTags[styleNumber]=(opener, closer)
        return (opener, closer)

    def getHTML(self, scimoz, title, language, style_bits):
        """ Returns the pretty-printed HTML form of the buffer. """
        regions = []
        self._htmlStyleTags = {}  # need to reset this in case they changed the color pref
        indicator_bits = 8 - style_bits
        
        defaultStyle = self._styleManager.getDefaultStyle("editor", 32)

        _globalPrefs = components.classes["@activestate.com/koPrefService;1"].\
                       getService(components.interfaces.koIPrefService).prefs
        useColor = _globalPrefs.getBooleanPref('print_useColor')
        useLineNos = _globalPrefs.getBooleanPref('print_showLineNos')
        self._wrapColumn = _globalPrefs.getLongPref('print_wrapColumn')
        self._fontSizeAdjust = _globalPrefs.getLongPref('print_fontSizeAdjust')
        self._fontSize = int(defaultStyle.size) + self._fontSizeAdjust
        # Format each region according to its style:
        from cStringIO import StringIO
        html = StringIO()
        html.write('<html>\n<head>\n<title>' + title + "</title>\n</head>\n")
        html.write("<body><pre style=\"" + self._styleToCSS(defaultStyle, None, useColor) + "\">")
        koDirs = components.classes["@activestate.com/koDirs;1"].\
            getService(components.interfaces.koIDirs)
        newlinePath = os.path.join(koDirs.installDir, "images" ,"newline.gif")
        if _globalPrefs.getBooleanPref('print_useNewline'):
            self._newlineURL = convertURL.LocalFileToURL(newlinePath)
        else:
            self._newlineURL = ''  # don't do newlines characters
        currentPos = 0
        lineNo = 0
        while 1:
            textlength = len(scimoz.text)
            lineStart = scimoz.positionFromLine(lineNo)
            lineNo += 1
            lineEnd = min(textlength, scimoz.positionFromLine(lineNo))
            self._addLogicalLine(html, defaultStyle, scimoz, lineStart, lineEnd, style_bits, language, lineNo,
                                 useColor, useLineNos)
            if lineEnd == textlength: break

        html.write('</pre></body></html>\n')
        text = html.getvalue()
        text = text.replace('\r', '\n')
        text = text.replace('\n\n', '\n')
        return text

    def _addLogicalLine(self, html, defaultStyle, scimoz, lineStart, lineEnd, style_bits, language, lineNo,
                        useColor, useLineNos):
        if useLineNos:
            html.write('<span style="font-size: %dpt; color:#770000;">%4d    </span>' % ((self._fontSize-3), lineNo))
            lineWidth = self._wrapColumn - 8
        else:
            lineWidth = self._wrapColumn
        big = lineEnd - lineStart > lineWidth
        while lineEnd - lineStart > lineWidth:
            self._addPhysicalLine(html, defaultStyle, scimoz.getStyledText(lineStart, lineStart+lineWidth), style_bits, language,
                                  useColor, useLineNos)
            lineStart += lineWidth
            if scimoz.getTextRange(lineStart, lineEnd).strip():
                if self._newlineURL:
                    html.write('<span><img src="'+self._newlineURL+'"/></span>')
                html.write('\n')
                if useLineNos:
                    html.write(r'<span style="font-size: %dpt;">        </span>' % (self._fontSize-3))
            else:
                html.write('\n')
        if scimoz.getTextRange(lineStart, lineEnd).strip() or not big:
            self._addPhysicalLine(html, defaultStyle, scimoz.getStyledText(lineStart, lineEnd), style_bits, language,
                                  useColor, useLineNos)

    def _addPhysicalLine(self, html, defaultStyle, buff, style_bits, language, useColor, useLineNos):
        regions = []
        mask = 0
        currentStyle = 0
        for bit in range(0, style_bits):
            mask |= 2**bit
        TXT, STY = 0, 1
        # Build a bunch of styled 'regions'
        for i in range(1, len(buff), 2):
            c = buff[i-1]
            s = ord(buff[i]) & mask
            if s != currentStyle or len(regions)==0:
                regions.append([[c], s])
                currentStyle = s
            else:
                # XXX this will probably cause problems on Mac files, but...
                if c != "\r": # ignore \r's, they just mess up printing.
                    regions[len(regions)-1][TXT].append(c)
        #XXX Work around Moz bug 104227.  Setting charsets in write
        #causes write to fail.  Reinstate the original once the Moz
        #bug is fixed.  Side effect: priting of docs including UTF-8
        #chars won't work right.
        #html.write('<html>\n<head>\n <META http-equiv="Content-Type" content="text/html; charset=UTF-8"> <title>' + title + "</title>\n</head>\n")
        for i in regions:
            styles = self.getStyleTags(language, i[STY], defaultStyle, useColor)
            line = (styles[0] + cgi.escape("".join(i[TXT])) + styles[1])
            html.write(line)

    def _styleToCSS(self, style, defaultStyle, useColor):
        """Return the CSS description of style, defined relative to
        defaultStyle.  If no defaultStyle is specified, unconditionally
        generate all properties.
        
        """
        cssSpec = {}
        if not useColor:
            if not defaultStyle or style.cssFore != defaultStyle.cssFore:
                cssSpec["color"] = "#000000"
        elif not defaultStyle or style.cssFore != defaultStyle.cssFore:
                cssSpec["color"] = style.cssFore
        if not defaultStyle or style.font != defaultStyle.font:
            cssSpec["font-family"] = style.font
        if not defaultStyle or style.size != defaultStyle.size:
            cssSpec["font-size"] = "%spt" % (str(int(style.size) + self._fontSizeAdjust))
        if not defaultStyle or style.italic != defaultStyle.italic:
            if style.italic:
                cssSpec["font-style"] = "italic"
            else:
                cssSpec["font-style"] = "normal"
        if not defaultStyle or style.bold != defaultStyle.bold:
            if style.bold:
                cssSpec["font-weight"] = "bold"
            else:
                cssSpec["font-weight"] = "normal"
        return self._makeCSSExpression(cssSpec)

    def _makeCSSExpression(self, spec):
        """Create a valid CSS expression out of a dictionary mapping CSS
        properties to CSS values.
        
        """
        props = []
        for property in spec.keys():
            props.append("%s: %s" % (property, spec[property]))
        return "; ".join(props)

#---- base commenting language service classes

class KoCommenterLanguageService:
    _com_interfaces_ = [components.interfaces.koICommenterLanguageService]

    def __init__(self, delimiterInfo):
        """'delimiterInfo' is a dictionary describing how comments are
        specified in the given language. The used keys are as follows:
            line:   an ordered list of prefixes to remove for line 
                    uncommenting, the first one is prefixed for line
                    commenting
            block:  an ordered list of (prefix, suffix) pair tuples
                    for block commenting and uncommenting
        If 'line' is defined then the language supports line commenting --
        Python and Perl. If 'block' is defined then the language supports
        block commenting -- XML. A language may support both -- C++. Which
        style is applied depends on the current selection in scimoz.
        """
        self.delimiterInfo = delimiterInfo
        pass

    def _applyChange(self, scimoz, startIndex, endIndex, replacement, kind):
        scimoz.hideSelection(1)
        selStart = scimoz.selectionStart
        selEnd = scimoz.selectionEnd

        # apply the commenting change
        scimoz.selectionStart = startIndex
        scimoz.selectionEnd = endIndex
        scimoz.replaceSel(replacement)

        # restore the selection and cursor position
        if kind == "line":
            if selStart != selEnd:
                startIndexLine = scimoz.lineFromPosition(startIndex)
                endIndexLine = scimoz.lineFromPosition(endIndex)
                scimoz.selectionStart = scimoz.positionFromLine(startIndexLine)
                scimoz.selectionEnd = scimoz.getLineEndPosition(endIndexLine)
            else:
                scimoz.selectionStart = selStart + len(prefix)
                scimoz.selectionEnd = selEnd + len(prefix)
        elif kind == "block":
            scimoz.selectionStart = selStart + len(prefix)
            scimoz.selectionEnd = selEnd + len(prefix)
        else:
            logError("unrecognized (un)commenting kind: '%s'" % kind)

        scimoz.hideSelection(0)

    def _commentLines(self, scimoz, startIndex, endIndex):
        """Comment the indexed lines."""
        selStart = scimoz.selectionStart
        selEnd = scimoz.selectionEnd
        if selStart != selEnd and scimoz.getColumn(selEnd) == 0:
            # if at the start of a line, work from the end of the previous
            # line
            workingEndIndex = scimoz.getLineEndPosition(
                scimoz.lineFromPosition(endIndex) - 1)
        else:
            workingEndIndex = endIndex
        original = scimoz.getTextRange(startIndex, workingEndIndex)
        prefix = self.delimiterInfo["line"][0]
        if type(prefix) == types.TupleType:
            prefix, suffix = prefix
        else:
            suffix = None

        # if the working text is all whitespace then abort
        if not re.search("\S", original, re.MULTILINE):
            return

        # find the maximum common whitespace offset of all the lines (comment
        # delimiters will be indented to this level)
        originalLines = original.split("\n")
        indentRe = re.compile("^(?P<indent>\s+).*$")
        if len(originalLines) > 0:
            indentMatch = indentRe.search(originalLines[0])
            if indentMatch:
                commonIndent = indentMatch.group("indent")
            else:
                commonIndent = ""
        else:
            commonIndent = ""
        for line in originalLines:
            indentMatch = indentRe.search(line)
            if indentMatch:
                indent = indentMatch.group("indent")
                while commonIndent:
                    if indent.startswith(commonIndent):
                        break
                    commonIndent = commonIndent[:-1]
                else:
                    break
            else:
                commonIndent = ""
                break

        # make the replacments
        replacementLines = []
        for line in originalLines:
            r = line[:len(commonIndent)] + prefix + line[len(commonIndent):]
            if suffix:
                for terminator in ("\r\n", "\n", "\r"):
                    if r.endswith(terminator):
                        length = len(terminator)
                        r = r[:-length] + suffix + r[-length]
                        break
                else:
                    r += suffix
            replacementLines.append(r)
        replacement = "\n".join(replacementLines)
        #logDebug("line comment: '%s' -> '%s'" % (original, replacement))

        if original != replacement:
            scimoz.hideSelection(1)
            startIndexLine = scimoz.lineFromPosition(startIndex)
            endIndexLine = scimoz.lineFromPosition(endIndex)
            selStartColumn = scimoz.getColumn(selStart)

            # apply the commenting change
            scimoz.selectionStart = startIndex
            scimoz.selectionEnd = workingEndIndex
            scimoz.replaceSel(replacement)

            # restore the selection and cursor position
            if selStart != selEnd:
                scimoz.selectionStart = scimoz.positionFromLine(startIndexLine)
                if endIndex == workingEndIndex:
                    scimoz.selectionEnd = scimoz.getLineEndPosition(
                        endIndexLine)
                else:
                    scimoz.selectionEnd = scimoz.positionFromLine(
                        endIndexLine)
            else:
                if selStartColumn <= len(commonIndent):
                    scimoz.selectionStart = selStart
                    scimoz.selectionEnd = selEnd
                else:
                    scimoz.selectionStart = selStart + len(prefix)
                    scimoz.selectionEnd = selEnd + len(prefix)
            scimoz.hideSelection(0)

    def _commentBlock(self, scimoz, startIndex, endIndex):
        """Comment the indexed block."""
        original = scimoz.getTextRange(startIndex, endIndex)
        prefix, suffix = self.delimiterInfo["block"][0]

        replacement = prefix + original + suffix
        #logDebug("block comment: '%s' -> '%s'" % (original, replacement))

        if original != replacement:
            scimoz.hideSelection(1)
            selStart = scimoz.selectionStart
            selEnd = scimoz.selectionEnd

            # apply the commenting change
            scimoz.selectionStart = startIndex
            scimoz.selectionEnd = endIndex
            scimoz.replaceSel(replacement)

            # restore the selection and cursor position
            scimoz.selectionStart = selStart + len(prefix)
            scimoz.selectionEnd = selEnd + len(prefix)
            scimoz.hideSelection(0)

    def _uncommentLines(self, scimoz, startIndex, endIndex):
        selStart = scimoz.selectionStart
        selEnd = scimoz.selectionEnd
        if selStart != selEnd and scimoz.getColumn(selEnd) == 0:
            # if at the start of a line, work from the end of the previous
            # line
            workingEndIndex = scimoz.getLineEndPosition(
                scimoz.lineFromPosition(endIndex) - 1)
        else:
            workingEndIndex = endIndex
        original = scimoz.getTextRange(startIndex, workingEndIndex)
        prefixes = self.delimiterInfo["line"]

        originalLines = original.split("\n")
        replacementLines = []
        lastIndent = lastPrefix = ""  # for restoring the position below
        for line in originalLines:
            for prefix in prefixes:
                if type(prefix) == types.TupleType:
                    prefix, suffix = prefix
                    commentRe = re.compile("^(\s*)(%s)(.*)(%s)(\s*)$"\
                                           % (prefix, suffix))
                else:
                    suffix = None
                    commentRe = re.compile("^(\s*)(%s)(.*)()()$" % prefix)
                commentMatch = commentRe.search(line)
                if commentMatch:
                    replacementLines.append(commentMatch.group(1) +
                                            commentMatch.group(3) +
                                            commentMatch.group(5))
                    lastIndent = commentMatch.group(1)
                    lastPrefix = commentMatch.group(2)
                    break
            else:
                replacementLines.append(line)
        replacement = "\n".join(replacementLines)
        #logDebug("line uncomment: '%s' -> '%s'" % (original, replacement))

        if original != replacement:
            scimoz.hideSelection(1)
            startIndexLine = scimoz.lineFromPosition(startIndex)
            endIndexLine = scimoz.lineFromPosition(endIndex)
            selStartColumn = scimoz.getColumn(selStart)

            # apply the commenting change
            scimoz.selectionStart = startIndex
            scimoz.selectionEnd = workingEndIndex
            scimoz.replaceSel(replacement)

            # restore the selection and cursor position
            if selStart != selEnd:
                scimoz.selectionStart = scimoz.positionFromLine(startIndexLine)
                if endIndex == workingEndIndex:
                    scimoz.selectionEnd = scimoz.getLineEndPosition(
                        endIndexLine)
                else:
                    scimoz.selectionEnd = scimoz.positionFromLine(
                        endIndexLine)
            else:
                if selStartColumn <= len(lastIndent):
                    scimoz.selectionStart = selStart
                    scimoz.selectionEnd = selEnd
                else:
                    scimoz.selectionStart = selStart - len(lastPrefix)
                    scimoz.selectionEnd = selEnd - len(lastPrefix)
            scimoz.hideSelection(0)

    def _uncommentBlock(self, scimoz, startIndex, endIndex):
        selStart = scimoz.selectionStart
        selEnd = scimoz.selectionEnd
        delimiters = self.delimiterInfo["block"]

        for prefix, suffix in delimiters:
            pStart, pEnd = startIndex-len(prefix), startIndex
            sStart, sEnd = endIndex, endIndex+len(suffix)
            p = scimoz.getTextRange(pStart, pEnd)
            s = scimoz.getTextRange(sStart, sEnd)
            #logDebug("block uncomment delimiter match if (%s==%s  "\
            #         "and  %s==%s)" % (p, prefix, s, suffix))
            if (p == prefix and s == suffix):
                scimoz.hideSelection(1)
                # remove the existing prefix and suffix (suffix first to get
                # indeces correct)
                scimoz.beginUndoAction()
                scimoz.selectionStart = sStart
                scimoz.selectionEnd = sEnd
                scimoz.replaceSel("")
                scimoz.selectionStart = pStart
                scimoz.selectionEnd = pEnd
                scimoz.replaceSel("")
                scimoz.endUndoAction()
                # restore the selection and cursor position
                scimoz.selectionStart = selStart - len(prefix)
                scimoz.selectionEnd = selEnd - len(prefix)
                scimoz.hideSelection(0)

    def _determineMethodAndDispatch(self, scimoz, workers):
        selStart = scimoz.selectionStart
        selEnd = scimoz.selectionEnd
        selStartColumn = scimoz.getColumn(selStart)
        selStartLine = scimoz.lineFromPosition(selStart)
        selStartLineEndPosition = scimoz.getLineEndPosition(selStartLine)
        selEndColumn = scimoz.getColumn(selEnd)
        selEndLine = scimoz.lineFromPosition(selEnd)
        selEndLineEndPosition = scimoz.getLineEndPosition(selEndLine)

        # determine preferred commenting method (if the selection is _within_
        # a line then block commenting is preferred)
        if selStart != selEnd:
            if (selStartColumn == 0 or selStart == selStartLineEndPosition) \
               and (selEndColumn == 0 or selEnd == selEndLineEndPosition):
                preferBlockCommenting = 0
            else:
                preferBlockCommenting = 1
        else:
            preferBlockCommenting = 0

        # do the commenting/uncommenting
        if preferBlockCommenting and self.delimiterInfo.get("block", None):
            workers["block"](scimoz, selStart, selEnd)
        elif self.delimiterInfo.get("line", None):
            textStart = scimoz.positionFromLine(selStartLine)
            if selStart != selEnd and scimoz.getColumn(selEnd) == 0:
                # if at the start of a line, then do NOT include that line
                textEnd = selEnd
            else:
                textEnd = scimoz.getLineEndPosition(selEndLine)
            workers["line"](scimoz, textStart, textEnd)
        elif self.delimiterInfo.get("block", None):
            # This lang only has block delimiters but the selection is such
            # that it looks like line commenting should be done. Sneak
            # the block delimiter tuple in as line delimiters.
            self.delimiterInfo["line"] = self.delimiterInfo["block"]
            textStart = scimoz.positionFromLine(selStartLine)
            if selStart != selEnd and scimoz.getColumn(selEnd) == 0:
                # if at the start of a line, then do NOT include that line
                textEnd = selEnd
            else:
                textEnd = scimoz.getLineEndPosition(selEndLine)
            workers["line"](scimoz, textStart, textEnd)
        else:
            logWarning("Could not comment code because no appropriate "\
                       "comment delimiter info exists for the current "\
                       "language.\n")

    def comment(self, scimoz):
        """Comment the current line or current selection."""
        commenters = {
            "line"  : self._commentLines,
            "block" : self._commentBlock
        }
        self._determineMethodAndDispatch(scimoz, commenters)

    def uncomment(self, scimoz):
        """Uncomment the current line or current selection."""
        uncommenters = {
            "line"  : self._uncommentLines,
            "block" : self._uncommentBlock
        }
        self._determineMethodAndDispatch(scimoz, uncommenters)



class KoWSInsertLanguageService:
    _com_interfaces_ = [components.interfaces.koIWSInsertLanguageService]

    sciEOLMappings = {
        components.interfaces.ISciMoz.SC_EOL_CRLF: eol.EOL_CRLF,
        components.interfaces.ISciMoz.SC_EOL_CR: eol.EOL_CR,
        components.interfaces.ISciMoz.SC_EOL_LF: eol.EOL_LF
        }

    def __init__(self, inserter, creationTemplate, module, useMD5):
        self._inserter = inserter
        self._creationTemplate = creationTemplate
        self._module = module
        self._useMD5 = useMD5

    def getScintillaLine(self, scin, lineno):
        (length, text) = scin.getLine(lineno)
        return text[:length]
    
    def insert(self, scin, position, objectName, url):
        lineno = scin.lineFromPosition(position)
        eolformat = self.sciEOLMappings[scin.eOLMode]

        if self._useMD5:
            import WShelper
            md5hash = WShelper.Get_MD5_Hash(url)
            statement = self._creationTemplate % (objectName, url, md5hash)
        else:
            statement = self._creationTemplate % (objectName, url)
            
        insertions = self._inserter.autoInsert(self._splitlines_with_trailing(scin.text),
                                               lineno,
                                               [statement],
                                               [self._module])
        linesInserted = 0
        for (insert_lineno, insert_lines) in insertions:
            scin_lineno = insert_lineno + linesInserted
            pos = scin.getLineEndPosition(scin_lineno)
            insert_text = eol.eolMappings[eolformat].join(insert_lines)

            lineText = self.getScintillaLine(scin, scin_lineno)
            insertedLineCount = len(insert_lines)
            insert_text = eol.eolMappings[eolformat] + insert_text
            scin.targetStart = pos
            scin.targetEnd = pos
            scin.replaceTarget(len(insert_text), insert_text)

            # set insertion point to end of the new content
            newPos = scin.getLineEndPosition(scin_lineno + insertedLineCount)
            scin.gotoPos(newPos)
            linesInserted += insertedLineCount
            
        line = scin.lineFromPosition(scin.currentPos)
        newPos = scin.positionFromLine(line + 1)
        scin.gotoPos(newPos)

    def _splitlines_with_trailing(self, s):

        """Same as splitlines() except we treat a trailing newline as
        an empty line.

        """
        lines = s.splitlines()
        if not s or s[-1] in ("\n", "\r"):
            lines.append('')
        return lines

class KoInterpreterLanguageServiceBase:
    _com_interfaces_ = [components.interfaces.koIInterpreterLanguageService]
    def get_interpreterPath(self):
        return ""
    def get_version(self):
        return ""
    def get_includePath(self):
        return ""
    def getConfiguration(self):
        return []
    
class KoLanguageBase:
    _com_interfaces_ = [components.interfaces.koILanguage]

    _lexer = None
    _linter = None
    _debugger = None
    _prettyprinter = None
    _style = None
    _methodcompletion = None
    _commenter = None
    _inserter = None
    _interpreter = None
    
    styleBits = 5
    indicatorBits = 2
    downloadURL = '' # location to download the language
    _cookbook = None
    remoteDebuggerSupport = 0 # does this language support remote debugging?
    variableIndicators = ''
    _svcdict = {
        components.interfaces.koILexerLanguageService: 'get_lexer',
        components.interfaces.koILinterLanguageService: 'get_linter',
        components.interfaces.koIDebuggerLanguageService: 'get_debugger',
        components.interfaces.koIPrettyPrinterLanguageService: 'get_prettyprinter',
        components.interfaces.koICompletionLanguageService: 'get_methodcompletion',
        components.interfaces.koICommenterLanguageService: 'get_commenter',
        components.interfaces.koIInterpreterLanguageService: 'get_interpreter',
        components.interfaces.koICookbookLanguageService: 'get_cookbook',
        components.interfaces.koIWSInsertLanguageService: 'get_inserter'
        }
    namedBlockRE = ''
    namedBlockDescription = ''
    
    def getLanguageService(self, iid):
        if self._svcdict.has_key(iid):
            return getattr(self, self._svcdict[iid])()
        return None

    # override these getters if you need to override the language services

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_NULL)
        return self._lexer

    def get_linter(self):
        return self._linter

    def get_debugger(self):
        return self._debugger

    def get_style(self):
        if self._style is None:
            self._style = KoStyleLanguageService()
        return self._style

    def get_prettyprinter(self):
        if self._prettyprinter is None:
            self._prettyprinter = KoPrettyPrinterLanguageService()
        return self._prettyprinter

    def get_methodcompletion(self):
        return self._methodcompletion

    def get_interpreter(self):
        return self._interpreter
    
    def get_commenter(self):
        return self._commenter

    def get_cookbook(self):
        return self._cookbook

    def get_inserter(self):
        return self._inserter
    
    def getEncodingWarning(self, encoding):
        if not encoding.encoding_info.ascii_superset:
            return '"%s" is not a recommended encoding for %s. Using ASCII or an ASCII superset is recommended.'\
                    % (encoding.friendly_encoding_name, self.name)
        elif encoding.byte_order_marker:
            return 'Including a signature (BOM) is not recommended for %s.' % self.name
        else: # It's all good
            return ''
    
class KoCookbookLanguageService:
    _com_interfaces_ = [components.interfaces.koICookbookLanguageService]

    def __init__(self, Name, Url):
        self._name = Name
        self._url = Url

    def get_cookbookName(self):
        return self._name

    def get_cookbookUrl(self):
        return self._url
