# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

# A simple base class for objects that want to handle Scintilla events.

# NOTE a component - just provides the base implementations of
# the methods
# DONT FORGET to add ISciMozEvents to your _com_interfaces_!!

class SciMozEvents:

    def onStyleNeeded( self, num):
        # Result: void - None
        # In: param0: int32
        pass
    def onCharAdded( self, char ):
        # Result: void - None
        # In: char: int32
        pass
    def onSavePointReached( self ):
        # Result: void - None
        pass
    def onSavePointLeft( self ):
        # Result: void - None
        pass
    def onModifyAttemptRO( self ):
        # Result: void - None
        pass
    def onKey( self, param0, param1 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: int32
        pass
    def onDoubleClick( self ):
        # Result: void - None
        pass
    def onUpdateUI( self ):
        # Result: void - None
        pass
    def onModified( self, position, modificationType, text,
                    linesAdded, line, foldLevelNow, foldLevelPrev ):
        # Result: void - None
        # In: position: int32
        # In: modificationType: int32
        # In: text: string_s
        # param3 is a hidden parameter
        # In: linesAdded: int32
        # In: line: int32
        # In: foldLevelNow: int32
        # In: foldLevelPrev: int32
        pass
    def onMacroRecord( self, param0, param1, param2 ):
        # Result: void - None
        # In: param0: int32
        # In: param1: int32
        # In: param2: int32
        pass
    def onMarginClick( self, modifiers, position, margin ):
        # Result: void - None
        # In: modifiers: int32
        # In: position: int32
        # In: margin: int32
        pass
    def onNeedShown( self, position, length):
        # Result: void - None
        # In: position: int32
        # In: length: int32
        pass
        
    def onPosChanged( self, position):
        # Result: void - None
        # In: position: int32
        pass

class SciMozEventDelegator(SciMozEvents):
    """Delegates all events to self.event_delegate - which you must setup yourself"""
    def onStyleNeeded( self, val):
        if self.event_delegate is not None:
            return self.event_delegate.onStyleNeeded(val)

    def onSavePointReached( self ):
        if self.event_delegate is not None:
            return self.event_delegate.onSavePointReached()
    def onSavePointLeft( self ):
        if self.event_delegate is not None:
            return self.event_delegate.onSavePointLeft()
    def onModifyAttemptRO( self ):
        if self.event_delegate is not None:
            return self.event_delegate.onModifyAttemptRO()

    def onKey( self, param0, param1 ):
        if self.event_delegate is not None:
            return self.event_delegate.onKey(param0, param1)

    def onDoubleClick( self ):
        if self.event_delegate is not None:
            return self.event_delegate.onDoubleClick()
    def onUpdateUI( self ):
        if self.event_delegate is not None:
            return self.event_delegate.onUpdateUI()
    def onModified( self, param0, param1, param2, param3, param4, param5, param6, param7 ):
        if self.event_delegate is not None:
            return self.event_delegate.onModified( param0, param1, param2, param3, param4, param5, param6, param7 )
    def onMacroRecord( self, param0, param1, param2 ):
        if self.event_delegate is not None:
            return self.event_delegate.onMacroRecord( param0, param1, param2 )
    def onMarginClick( self, param0, param1, param2 ):
        if self.event_delegate is not None:
            return self.event_delegate.onMarginClick( param0, param1, param2 )
    def onNeedShown( self, position, length ):
        if self.event_delegate is not None:
            return self.event_delegate.onNeedShown(position, length)
    def onPosChanged( self, position):
        if self.event_delegate is not None:
            return self.event_delegate.onPosChanged(position)
    def onZoom( self):
        if self.event_delegate is not None:
            return self.event_delegate.onZoom()
