# Copyright (c) 2000-2001 ActiveState Tool Corporation.
# See the file LICENSE.txt for licensing information.

import marshal # for arg types (efficient!)
import pickle # for exceptions
import socket
import string
import sys
import urlparse

error = socket.error # Insulate our caller from the underlying transport.

class _Method:
    # some magic to bind an XML-RPC method to an RPC server.
    # supports "nested" methods (e.g. examples.getStateName)
    def __init__(self, send, name):
        self.__send = send
        self.__name = name
    def __getattr__(self, name):
        return _Method(self.__send, "%s.%s" % (self.__name, name))
    def __call__(self, *args):
        return self.__send(self.__name, args)

class Server:
    """ This is called a server, but it is actually a representation of
        a server for the client-side. This is xmlrpclib terminology. """
    def __init__(self, url):
        scheme, addr, path, params, query, frag = urlparse.urlparse(url)
        assert scheme=="http"
        assert path==params==query==frag==""
        self._host,port = string.split(addr, ":")
        self._port = int(port)
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock.connect((self._host,self._port))

    def __repr__(self):
        return (
            "<Server proxy for %s%s>" %
            (self.__host, self.__handler)
            )

    __str__ = __repr__

    def __getattr__(self, name):
        # magic method dispatcher
        return _Method(self.__request, name)

    def __request(self, methodname, params):
        "call a method on the remote server"
        len_of_length_raw = len(marshal.dumps((0xFFFFFFFF)))

        request = marshal.dumps((methodname, params))
        self._sock.send(marshal.dumps((len(request))))
        self._sock.send(request)

        response_length_raw = ""
        while len(response_length_raw) < len_of_length_raw:
            _response = self._sock.recv(len_of_length_raw-len(response_length_raw))
            if len(_response) < 1: raise socket.error, "KORPC no data on socket"
            response_length_raw = response_length_raw + _response
                                  
        response_length = marshal.loads(response_length_raw)
        response_data = ""
        while len(response_data) < response_length:
            _response = self._sock.recv(response_length-len(response_data))
            if len(_response) < 1: raise socket.error, "KORPC no data on socket"
            response_data = response_data + _response
            
        exception = response = None
        if response_data:
            (exception,response)=marshal.loads(response_data)
        if exception:
            (exc_type, exc_value)=pickle.loads(exception)
            raise exc_type, exc_value

        return response

import xmlrpcserver

class RequestHandler(xmlrpcserver.RequestHandler):
    def handle(self):
        len_of_length_raw = len(marshal.dumps((0xFFFFFFFF)))
        
        length_raw = ""
        while len(length_raw) < len_of_length_raw:
            _response = self.request.recv(len_of_length_raw-len(length_raw))
            if len(_response) < 1:
                self.server.quit=1
                return
            length_raw = length_raw + _response

        length = marshal.loads(length_raw)

        data = ""
        while len(data) < length:
            _response = self.request.recv(length-len(data))
            if len(_response) < 1:
                self.server.quit=1
                return
            data = data + _response
            
        if not data:
            self.server.quit=1
            return
        methodname, args = marshal.loads(data)
        exception = rc = None
        try:
            rc = self.call(methodname, args)
        except:
            exception=pickle.dumps((sys.exc_type, sys.exc_value))
        response = marshal.dumps((exception, rc))
        self.request.send(marshal.dumps((len(response))))
        self.request.send(marshal.dumps((exception,rc)))

    def setup(self):
        pass
    def finish(self):
        pass

class FastServer:
    """ A fast server. This code does not subclass from socket server
        because it uses a single connection instead of one per message.
 
        But it uses the SocketServer model.

        Perhaps it *could* subclass from SocketServer if you override the
        right methods but that would take a little bit of research.
        """

    quit=0
    def __init__(self, handler, port):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.bind(('', port))
        self._handler = handler
        self._conn = None

    def getsockname(self):
        return self.socket.getsockname()
 
    def serve_once(self):
        self.socket.listen(1)
        conn,addr = self.socket.accept()
        while not self.quit:
            handler = self._handler(conn,addr,self)

    def handle_request(self):
        if not self._conn:
            self.socket.listen(1)
            self._conn,self._addr = self.socket.accept()
        self._handler(self._conn,self._addr,self)

    def serve_forever(self):
        self.socket.listen(1)
        while 1:
            conn,addr = self.socket.accept()
            while not self.quit:
                # See handle_request - why do we take the result and again call that?
                handler = self._handler(conn,addr,self)

