#!/usr/bin/env python
# Copyright (c) 2000-2002 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

"""
    Return success (0) iff the given XPCOM component can be created.
    Otherwise return failure (non-zero).
    
    Usage:
        python _isregistered.py <progid>

    Note: You should not run this directly. Run "./install.sh" to
    install Komodo.
"""

import os
import sys
import getopt

from xpcom import components, _xpcom, nsError, ServerException, COMException


#---- internal support routines

def _isregistered(progid):
    inst = None
    try:
        inst = components.classes[progid].createInstance()
    except COMException:
        pass
    if inst:
        #print "'%s' IS registered" % progid
        return 1
    else:
        #print "'%s' is NOT registered" % progid
        return 0


#---- main line

def main(argv):
    # Process options.
    try:
        optlist, args = getopt.getopt(argv[1:], 'h', ['help'])
    except getopt.GetoptError, msg:
        sys.stderr.write("error: %s. Your invocation was: %s\n" % (msg, argv))
        sys.stderr.write("see 'python _isregistered.py --help'\n")
        return 1
    for opt, optarg in optlist:
        if opt in ('-h', '--help'):
            print __doc__
            return 0

    # Process arguments.
    if len(args) != 1:
        sys.stderr.write("error: Incorrect number of arguments: %s\n" % args)
        sys.stderr.write("see 'python _isregistered.py --help'\n")
        return 1

    progid = args[0]
    isregistered = _isregistered(progid)
    # Invert the return value because 0==success, non-zero==failure.
    return not isregistered


if __name__ == "__main__":
    sys.exit( main(sys.argv) )
